const config = require('config');
const randomize = require('randomatic');
const fastify = require('fastify');
const swagger = require('fastify-swagger');
const helmet = require('fastify-helmet');
const cors = require('fastify-cors');
const compress = require('fastify-compress');
const { VERSION } = require('./utils/consts');
const { writeToCache, deleteFromCache } = require('./utils/cache');
const healthRoutes = require('./routes/health');
const systemRoutes = require('./routes/system');
const awsRoutes = require('./routes/aws');
const sgwsRoutes = require('./routes/sgws');
const azureRoutes = require('./routes/azure');
const gcpRoutes = require('./routes/gcp');
const zapiRoutes = require('./routes/zapi');
const getLogger = require('./utils/logger');
const { setRequestId, setServiceRequestId, getLocalStorage } = require('./utils/async-local-storage');
const { sendAuditRecord } = require('./operations/audit-operations');

const API_PREFIX_PATH = '/account/:accountId/providers/cloudmanager_tiering/api';
const REQUEST_ID_HEADER = 'x-request-id';
const PORT = config.get('app-port');

const HOST = '0.0.0.0'; // when deploying to a Docker, and potentially other, containers, it is advisable to listen on 0.0.0.0

const logger = getLogger();
const accessLogger = getLogger('access');

process.on('unhandledRejection', (reason, p) => logger.error('Unhandled Rejection at:', p, 'reason:', reason));

process.on('uncaughtException', err => logger.error('Uncaught exception was thrown', err));

function exit(err = undefined) {
    if (err) {
        logger.error(err);
    }

    throw new Error('Something bad happened!!!');
}

async function start() {
    logger.info('Starting server...');

    // const documentationAddress = await getPublicIpAddress();
    const documentationAddress = HOST;
    const documentationPort = process.env.ENV_CT_BUILD_DOCKERIZED ? 8010 : PORT;
    logger.info('Documentation host and port:', { documentationAddress, documentationPort });

    const app = fastify({
        requestIdHeader: REQUEST_ID_HEADER,
        trustProxy: true,
        genReqId: () => randomize('Aa0', 8),
        ajv: {
            customOptions: {
                removeAdditional: false
            }
        }
    })
        .register(helmet)
        .register(cors)
        .register(compress)
        .register(swagger, {
            routePrefix: '/documentation',
            swagger: {
                info: {
                    title: 'API docs - CloudTiering Agent',
                    version: VERSION
                },
                host: `${documentationAddress}:${documentationPort}`,
                schemes: ['http'],
                consumes: ['application/json'],
                produces: ['application/json']
            },
            exposeRoute: true
        })
        .register((instance, opts, done) => {
            healthRoutes(instance);
            systemRoutes(instance);
            done();
        })
        .register((instance, opts, next) => {
            instance.addHook('onRequest', (request, reply, done) => {
                // register the request so we will not allow upgrade while requests are still in the air
                // note, that we don't count health and system requests
                writeToCache(request.id, request.req.url);
                done();
            });
            awsRoutes(instance);
            sgwsRoutes(instance);
            azureRoutes(instance);
            gcpRoutes(instance);
            zapiRoutes(instance);
            next();
        }, { prefix: API_PREFIX_PATH })
        .addHook('onRequest', (request, reply, done) => {
            if (request.req.method === 'DELETE') {
                delete request.headers['content-length'];
            }
            done();
        })
        .addHook('preHandler', (request, reply, done) => {
            if (request.headers['send-audit'] === 'true' && !process.env.IS_SIMULATOR) {
                sendAuditRecord(request, reply, true);
            }
            getLocalStorage().run(new Map(), () => {
                setRequestId(request.headers['x-request-id']);
                setServiceRequestId(request.headers['x-service-request-id']);
                done();
            });
        })
        .addHook('onSend', async (request, reply, payload) => {
            reply.header(REQUEST_ID_HEADER, request.id);
            // we dont want to cache error response in the backend
            if (reply.statusCode === 200) {
                reply.header('Cache-Control', 'public,max-age=300,smax-age=300');
            }
            if (request.headers['send-audit'] === 'true' && !process.env.IS_SIMULATOR) {
                await sendAuditRecord(request, reply, false, payload);
            }
        })
        .addHook('onResponse', (request, reply, done) => {
            deleteFromCache(request.id);
            accessLogger.info(`[${request.req.method}] [${request.req.url}] [${Math.round(reply.getResponseTime())}] [${reply.statusCode}]`);
            done();
        })
        .setErrorHandler((error, request, reply) => {
            logger.error('Received error status:', error.status || error.code, 'and error message:', error.message, 'on request:', request.req.method, request.req.url);
            if (error.message.includes('Timeout while accessing Redis')) {
                error.message = 'Timeout';
            }
            reply.send(error);
        });

    app.ready(err => {
        if (err) {
            exit(err);
        }
        logger.info('Plugins loaded!!!');
    });

    app.listen(PORT, HOST, err => {
        if (err) {
            exit(err);
        }
        logger.info(`Server listening to ${HOST} on port: ${PORT}`);
        logger.info(`Server version: ${VERSION}, node-version: ${process.version}, mode: ${process.env.ENV_CT_BUILD_MODE}, build: ${process.env.ENV_CT_BUILD_TC}, git: ${process.env.ENV_CT_BUILD_GIT}`);
    });
}

start();
