const { StorageAccounts, StorageManagementClient } = require('@azure/arm-storage');
const getCredentials = require('../cloud-manager/credentials-operations');
const { getConfigIdentity } = require('../cloud-manager/config-operations');
const { setProxy } = require('../cloud-manager/config-operations');
const getLogger = require('../../utils/logger');
const { AZURE } = require('../../utils/consts');

const logger = getLogger();

async function getStorageAccounts() {
    logger.debug('Getting storage accounts');
    const credentials = await getCredentials(AZURE);
    await setProxy(AZURE);
    const { subscription } = await getConfigIdentity();
    const client = new StorageManagementClient(credentials, subscription);

    return new StorageAccounts(client);
}

async function listStorageAccounts() {
    logger.info('Listing storage accounts:');
    const storageAccounts = await getStorageAccounts();
    return storageAccounts.list();
}

async function listStorageAccountsByResourceGroup(resourceGroup) {
    logger.info('Listing storage accounts by resource group:', { resourceGroup });
    const storageAccounts = await getStorageAccounts();

    return storageAccounts.listByResourceGroup(resourceGroup);
}

async function createStorageAccount(resourceGroup, kind, skuName, location, storageAccount, accessTier) {
    logger.info('Creating storage account:', {
        resourceGroup,
        kind,
        skuName,
        location,
        storageAccount,
        accessTier
    });
    const storageAccounts = await getStorageAccounts();

    await storageAccounts.create(resourceGroup, storageAccount, {
        kind, accessTier, location, sku: { name: skuName }
    });
}

async function listStorageAccountKeys(resourceGroup, storageAccount) {
    logger.info('Listing storage account keys:', { resourceGroup, storageAccount });
    const storageAccounts = await getStorageAccounts();

    return storageAccounts.listKeys(resourceGroup, storageAccount);
}

async function updateStorageAccountTierLevel(resourceGroup, storageAccount, accessTier) {
    logger.info('Updating storage account tier level:', {
        resourceGroup,
        storageAccount,
        accessTier
    });
    const storageAccounts = await getStorageAccounts();

    return storageAccounts.update(resourceGroup, storageAccount, { accessTier });
}

module.exports = {
    listStorageAccounts,
    listStorageAccountsByResourceGroup,
    createStorageAccount,
    listStorageAccountKeys,
    updateStorageAccountTierLevel
};
