const createError = require('http-errors');
const { TokenCredentials } = require('@azure/ms-rest-js');
const gotInstance = require('../../utils/got');
const { getConfigIdentity } = require('./config-operations');
const {
    CREDENTIALS_SERVICE_SERVER_ADDRESS, AWS, AZURE, GCP
} = require('../../utils/consts');
const getLogger = require('../../utils/logger');

const logger = getLogger();

async function getAwsCredentialsFromOnprem() {
    logger.debug('Getting AWS credentials from ONPREM machine');
    const { ids: accountsIds } = await gotInstance.get(`${CREDENTIALS_SERVICE_SERVER_ADDRESS}/credentials/aws/accounts`);
    if (accountsIds.length) {
        const [accountId] = accountsIds;
        const credentialsIdentity = await gotInstance.get(`${CREDENTIALS_SERVICE_SERVER_ADDRESS}/credentials/aws/account/${accountId}`);
        return {
            accessKeyId: credentialsIdentity.accessKeyId,
            secretAccessKey: credentialsIdentity.secretAccessKey
        };
    }
    throw createError(404, 'No aws credentials were found');
}

async function getCredentials(provider) {
    logger.debug(`Getting ${provider} credentials`);

    const configIdentity = await getConfigIdentity();
    if (configIdentity.location === 'OnPrem' || (configIdentity.location === 'AWS' && !configIdentity.hasRole)) {
        return getAwsCredentialsFromOnprem();
    }

    const identity = await gotInstance.get(`${CREDENTIALS_SERVICE_SERVER_ADDRESS}/credentials/${provider.toLowerCase()}/identity`);

    switch (provider) {
        case AWS:
            return {
                accessKeyId: identity.accessKeyId,
                secretAccessKey: identity.secretAccessKey,
                sessionToken: identity.session
            };
        case AZURE:
            return new TokenCredentials(identity.token);
        case GCP:
            return identity.token;
        default:
            throw new Error(`Unable to get credentials, unknown provider name: ${provider}`);
    }
}

module.exports = getCredentials;
