const { renderFile } = require('ejs');
const convert = require('xml-js');
const path = require('path');
const {
    get, has, isArray, set
} = require('lodash');
const escapeHtml = require('escape-html');
const zapiRequest = require('./zapiClient');
const { ZAPI_REQUEST_DIR, NONE } = require('../../utils/consts');
const { getOntapCredentials } = require('../cloud-manager/zapi-credentials-operations');
const { updateResponse } = require('../../utils/utils');

const MAX_RECORDS = 500;

function parseJson(obj) {
    Object.keys(obj).forEach(key => {
        if (has(obj, `${key}._text`)) {
            obj[key] = obj[key]._text;
            return obj;
        }
        return parseJson(obj[key]);
    });
}

async function renderZapiRequest(zapiCall, object = {}) {
    const fileName = path.resolve(ZAPI_REQUEST_DIR, zapiCall);

    const rendered = await renderFile(fileName, object, { async: true });

    const converted = convert.xml2js(rendered, {
        instructionHasAttributes: true, compact: true, ignoreComment: true, spaces: 4, nativeType: true
    });

    const iteratorNodeName = Object.keys(converted.netapp).find(key => key !== '_attributes' && key.endsWith('-iter'));

    if (iteratorNodeName) {
        set(converted, `netapp.${iteratorNodeName}.max-records._text`, MAX_RECORDS);
        return convert.js2xml(converted, { compact: true, ignoreComment: true, spaces: 4 });
    }
    return rendered;
}

async function describeOntapVersion(publicId, ip, userName, password) {
    const request = await renderZapiRequest('describe-ontap-version.ejs');
    const onPremCredentials = { ip, userName, password };
    if (publicId) {
        Object.assign(onPremCredentials, await getOntapCredentials(publicId));
    }

    const response = await zapiRequest(onPremCredentials.userName, onPremCredentials.password, onPremCredentials.ip, request);
    const versionObj = response['version-tuple']['system-version-tuple'];

    return {
        generation: versionObj.generation._text,
        major: versionObj.major._text,
        minor: versionObj.minor._text
    };
}

async function describeSvms(publicId, svmName) {
    let tag;
    const svms = [];
    do {
        const request = await renderZapiRequest('describe-svm.ejs', { svmName, tag });
        const {
            userName, password, ip
        } = await getOntapCredentials(publicId);
        const response = await zapiRequest(userName, password, ip, request);

        tag = response['next-tag'] ? response['next-tag']._text : undefined;
        updateResponse(response['attributes-list']['vserver-info']).forEach(vserver => {
            svms.push({
                svmName: get(vserver, 'vserver-name._text'),
                svmState: get(vserver, 'state._text'),
                svmType: get(vserver, 'vserver-type._text'),
                svmSubType: get(vserver, 'vserver-subtype._text'),
                language: get(vserver, 'language._text'),
                allowedProtocols: isArray(get(vserver, 'allowed-protocols.protocol')) ? vserver['allowed-protocols'].protocol.map(protocol => get(protocol, '_text')) : get(vserver, 'allowed-protocols.protocol._text'),
                allowedAggregates: isArray(vserver['aggr-list']) ? vserver['aggr-list'].map(aggr => get(aggr, 'aggr-name._text')) : get(vserver, 'aggr-list.aggr-name._text')
            });
        });
    } while (tag);

    return svms;
}

async function describeObjectStoreConfig(publicId) {
    const request = await renderZapiRequest('describe-object-store-config.ejs');
    const {
        userName, password, ip
    } = await getOntapCredentials(publicId);
    const response = await zapiRequest(userName, password, ip, request);

    return updateResponse(get(response, 'attributes-list.aggr-object-store-config-info', [])).map(objectStoreConfig => ({
        objectStoreName: objectStoreConfig['object-store-name']._text,
        s3Name: objectStoreConfig['s3-name']._text,
        usedSpace: objectStoreConfig['used-space']._text,
        server: objectStoreConfig.server._text,
        providerType: objectStoreConfig['provider-type']._text
    }));
}

async function describeSystemInfo(publicId) {
    const request = await renderZapiRequest('describe-system-info.ejs');
    const {
        userName, password, ip
    } = await getOntapCredentials(publicId);
    const response = await zapiRequest(userName, password, ip, request);

    return updateResponse(response['attributes-list']['node-details-info']).map(nodeDetailsInfo => ({
        systemId: nodeDetailsInfo['node-system-id']._text,
        systemVersion: nodeDetailsInfo['product-version']._text,
        serialNumber: nodeDetailsInfo['node-serial-number']._text,
        nodeName: nodeDetailsInfo.node._text
    }));
}

async function describeObjectStores(publicId) {
    const request = await renderZapiRequest('describe-object-stores.ejs');
    const {
        userName, password, ip
    } = await getOntapCredentials(publicId);
    const response = await zapiRequest(userName, password, ip, request);

    return updateResponse(get(response, 'attributes-list.object-store-information', [])).map(objectStore => ({
        aggregateName: objectStore.aggregate._text,
        name: objectStore['object-store-name']._text,
        availabilityState: objectStore['object-store-availability']._text,
        usedSpace: objectStore['used-space']._text,
        tieringFullnessThreshold: get(objectStore, 'tiering-fullness-threshold._text')
    }));
}

async function describeUsers(ip, userName, password) {
    let tag;
    const users = [];
    do {
        const request = await renderZapiRequest('describe-users.ejs', { tag });
        const response = await zapiRequest(userName, password, ip, request);

        tag = response['next-tag'] ? response['next-tag']._text : undefined;
        updateResponse(response['attributes-list']['security-login-account-info']).forEach(userObj => {
            users.push({
                userName: userObj['user-name']._text,
                application: userObj.application._text,
                authMethod: userObj['authentication-method']._text,
                role: userObj['role-name']._text,
                svmName: userObj.vserver._text
            });
        });
    } while (tag);

    return users;
}

async function describeAggregates(publicId) {
    const request = await renderZapiRequest('describe-aggregates.ejs');
    const {
        userName, password, ip
    } = await getOntapCredentials(publicId);
    const response = await zapiRequest(userName, password, ip, request);

    return updateResponse(response['attributes-list']['aggr-attributes']).map(aggr => ({
        name: get(aggr, 'aggregate-name._text'),
        state: get(aggr, 'aggr-raid-attributes.state._text'),
        isRoot: get(aggr, 'aggr-raid-attributes.is-root-aggregate._text'),
        isComposite: get(aggr, 'aggr-raid-attributes.is-composite._text'),
        isInactiveDataReportingEnabled: get(aggr, 'is-inactive-data-reporting-enabled._text', false),
        isObjectStoreAttachEligible: get(aggr, 'is-object-store-attach-eligible._text'),
        totalCapacity: get(aggr, 'aggr-space-attributes.size-total._text'),
        usedCapacity: get(aggr, 'aggr-space-attributes.size-used._text'),
        tierInactiveUserDataPercent: get(aggr, 'aggr-space-attributes.performance-tier-inactive-user-data-percent._text'),
        tierInactiveUserData: get(aggr, 'aggr-space-attributes.performance-tier-inactive-user-data._text')
    }));
}

async function describeClusterInfo(publicId, ip, userName, password) {
    const request = await renderZapiRequest('describe-cluster-info.ejs');

    const onPremCredentials = { ip, userName, password };
    if (publicId) {
        Object.assign(onPremCredentials, await getOntapCredentials(publicId));
    }

    const response = await zapiRequest(onPremCredentials.userName, onPremCredentials.password, onPremCredentials.ip, request);
    return {
        clusterName: response.attributes['cluster-identity-info']['cluster-name']._text,
        clusterUuid: response.attributes['cluster-identity-info']['cluster-uuid']._text,
        serialNumber: response.attributes['cluster-identity-info']['cluster-serial-number']._text
    };
}

async function describeClusterVolumesFootprint(publicId) {
    let tag;
    const attributesVolumeFootprintObjects = [];

    do {
        const request = await renderZapiRequest('describe-cluster-volumes-footprint.ejs', { tag });
        const {
            userName, password, ip
        } = await getOntapCredentials(publicId);
        const response = await zapiRequest(userName, password, ip, request);

        updateResponse(response['attributes-list']['footprint-info']).forEach(footprintInfo => {
            const obj = {
                name: footprintInfo.volume._text,
                svmName: footprintInfo.vserver._text,
                performanceTierFootprint: get(footprintInfo, 'volume-blocks-footprint-bin0._text'),
                capacityTierFootprint: get(footprintInfo, 'volume-blocks-footprint-bin1._text')
            };
            attributesVolumeFootprintObjects.push(obj);
        });
        tag = response['next-tag'] ? response['next-tag']._text : undefined;
    } while (tag);

    return attributesVolumeFootprintObjects;
}

async function describeDisks(publicId) {
    const request = await renderZapiRequest('describe-disks.ejs');
    const {
        userName, password, ip
    } = await getOntapCredentials(publicId);
    const response = await zapiRequest(userName, password, ip, request);

    return updateResponse(response['attributes-list']['storage-disk-info']).map(disk => ({
        diskName: get(disk, 'disk-name._text'),
        diskRaidInfoPosition: get(disk, 'disk-raid-info.position._text'),
        diskOwnershipInfoOwnerNodeName: get(disk, 'disk-ownership-info.owner-node-name._text'),
        diskRaidInfoDiskAggregateInfoAggregateName: get(disk, 'disk-raid-info.disk-aggregate-info.aggregate-name._text'),
        diskRaidInfoContainerType: get(disk, 'disk-raid-info.container-type._text'),
        diskRaidInfoDiskOutageInfoReason: get(disk, 'disk-raid-info.disk-outage-info.reason._text'),
        diskPathsDiskPathInfo: updateResponse(disk['disk-paths.disk-path-info']).map(diskPathInfo => ({
            vmdiskHypervisorFileName: get(diskPathInfo, 'vmdisk-hypervisor-file-name._text'),
            initiatorPort: get(diskPathInfo, 'initiator-port._text'),
            node: get(diskPathInfo, 'node._text')
        })),
        diskInventoryInfoVirtualMachineDiskInfo: updateResponse(disk['disk-inventory-info.virtual-machine-disk-info._text']).map(virtualMachineDiskInfo => ({
            vmdiskObjectName: get(virtualMachineDiskInfo, 'vmdisk-object-name._text'),
            vmdiskStorageAccount: get(virtualMachineDiskInfo, 'vmdisk-storage-account._text'),
            vmdiskContainerName: get(virtualMachineDiskInfo, 'vmdisk-container-name._text')
        })),
        diskRaidInfoPhysicalBlocks: get(disk, 'disk-raid-info.physical-blocks._text')
    }));
}

async function describeLicenses(publicId, svmName, volumeName) {
    const request = await renderZapiRequest('describe-licenses.ejs', {
        svmName,
        volumeName
    });
    const {
        userName, password, ip
    } = await getOntapCredentials(publicId);
    const response = await zapiRequest(userName, password, ip, request);

    return updateResponse(response.licenses['license-v2-info']).map(license => ({
        package: get(license, 'package._text'),
        description: get(license, 'description._text'),
        expirationTime: get(license, 'expiration-time._text')
    }));
}

async function describeLicenseV2(publicId) {
    const request = await renderZapiRequest('describe-license-v2.ejs');
    const {
        userName, password, ip
    } = await getOntapCredentials(publicId);
    const response = await zapiRequest(userName, password, ip, request);

    return updateResponse(get(response, 'attributes-list.license-v2-capacity-info', [])).map(info => ({
        package: get(info, 'package._text'),
        currentCapacity: get(info, 'current-capacity._text'), // GB
        maxCapacity: get(info, 'max-capacity._text'), // GB
        serialNumber: get(info, 'serial-number._text')
    }));
}

async function describeSvmVolumesWithFilter(publicId, svmName, volumeName) {
    const {
        userName, password, ip
    } = await getOntapCredentials(publicId);
    let tag;
    const volumes = [];
    do {
        const request = await renderZapiRequest('describe-svm-volumes-with-filter.ejs', {
            svmName,
            volumeName,
            tag
        });
        const response = await zapiRequest(userName, password, ip, request);
        tag = response['next-tag'] ? response['next-tag']._text : undefined;

        if (get(response, 'attributes-list.volume-attributes')) {
            updateResponse(response['attributes-list']['volume-attributes']).forEach(volume => {
                volumes.push({
                    name: get(volume, 'volume-id-attributes.name._text'),
                    svmName: get(volume, 'volume-id-attributes.owning-vserver-name._text'),
                    uuid: get(volume, 'volume-id-attributes.instance-uuid._text'),
                    usedSize: get(volume, 'volume-space-attributes.size-used._text'),
                    sizeTotal: get(volume, 'volume-space-attributes.size._text'),
                    state: get(volume, 'volume-state-attributes.state._text'),
                    parentVolumeName: get(volume, 'volume-clone-attributes.volume-clone-parent-attributes.name._text'),
                    isVserverRoot: get(volume, 'volume-state-attributes.is-vserver-root._text'),
                    aggregateName: get(volume, 'volume-id-attributes.containing-aggregate-name._text'),
                    volumeType: get(volume, 'volume-id-attributes.type._text'),
                    creationTime: get(volume, 'volume-id-attributes.creation-time._text'),
                    tieringPolicy: get(volume, 'volume-comp-aggr-attributes.tiering-policy._text', NONE),
                    tierInactiveUserData: get(volume, 'volume-space-attributes.performance-tier-inactive-user-data._text'),
                    tierInactiveUserDataPercent: get(volume, 'volume-space-attributes.performance-tier-inactive-user-data-percent._text'),
                    physicalUsed: get(volume, 'volume-space-attributes.physical-used._text'),
                    flexgroupMSid: get(volume, 'volume-id-attributes.flexgroup-msid._text'),
                    flexgroupConstituentIndex: get(volume, 'volume-id-attributes.flexgroup-index._text'),
                    snapshotsUsedSize: get(volume, 'volume-space-attributes.size-used-by-snapshots._text')
                });
            });
        }
    } while (tag);
    return volumes;
}

async function modifyColdDataReporting(publicId, aggregateName, enable) {
    const request = await renderZapiRequest(
        'modify-cold-data-reporting.ejs', { aggregateName, enable }
    );
    const {
        userName, password, ip
    } = await getOntapCredentials(publicId);
    try {
        await zapiRequest(userName, password, ip, request);
    } catch (err) {
        // Prior to version 9.5, ONTAP doesn't tell if IDR is on/off so we try to turn it on anyway buy we don't want to throw error if it is already on
        if (err.message.includes('inactive data reporting already turned on')) {
            return { status: 'passed' };
        }
        throw err;
    }
}

async function describeNodes(publicId, ip, userName, password) {
    let tag;
    const nodes = [];
    do {
        const request = await renderZapiRequest('describe-nodes.ejs', { tag });
        const onPremCredentials = { ip, userName, password };
        if (publicId) {
            Object.assign(onPremCredentials, await getOntapCredentials(publicId));
        }

        const response = await zapiRequest(onPremCredentials.userName, onPremCredentials.password, onPremCredentials.ip, request);

        tag = response['next-tag'] ? response['next-tag']._text : undefined;
        updateResponse(response['attributes-list']['node-details-info']).forEach(node => {
            nodes.push({
                nodeName: get(node, 'node._text'),
                model: get(node, 'node-model._text'),
                isNodeHealthy: get(node, 'is-node-healthy._text'),
                isClusterEligible: get(node, 'is-node-cluster-eligible._text')
            });
        });
    } while (tag);

    return nodes;
}

async function describeNodeSystemImages(publicId, nodeName) {
    let tag;
    const systemNodeImages = [];
    do {
        const request = await renderZapiRequest('describe-node-system-images.ejs', {
            tag,
            nodeName
        });
        const {
            userName, password, ip
        } = await getOntapCredentials(publicId);
        const response = await zapiRequest(userName, password, ip, request);

        tag = response['next-tag'] ? response['next-tag']._text : undefined;
        updateResponse(response['attributes-list']['system-image-attributes']).forEach(image => {
            systemNodeImages.push({
                image: image.image._text,
                isCurrent: image['is-current']._text,
                isDefault: image['is-default']._text,
                version: image.version._text
            });
        });
    } while (tag);

    return systemNodeImages;
}

async function describeSvmLifs(publicId, svmName) {
    let tag;
    const lifs = [];
    do {
        const request = await renderZapiRequest('describe-svm-lifs.ejs', {
            tag,
            svmName
        });
        const {
            userName, password, ip
        } = await getOntapCredentials(publicId);
        const response = await zapiRequest(userName, password, ip, request);

        tag = response['next-tag'] ? response['next-tag']._text : undefined;
        updateResponse(response['attributes-list']['net-interface-info']).forEach(lif => {
            lifs.push({
                interfaceName: get(lif, 'interface-name._text'),
                address: get(lif, 'address._text'),
                netmask: get(lif, 'netmask._text'),
                netmaskLength: get(lif, 'netmask-length._text'),
                port: get(lif, 'home-port._text'),
                node: get(lif, 'home-node._text'),
                status: get(lif, 'operational-status._text'),
                firewallPolicy: get(lif, 'firewall-policy._text'),
                svmName: get(lif, 'vserver._text'),
                role: get(lif, 'role._text'),
                failoverPolicy: get(lif, 'failover-policy._text'),
                ipSpace: get(lif, 'ipspace._text')
            });
        });
    } while (tag);

    return lifs.flat();
}

async function attachObjectStore(publicId, aggregateName, objectStoreName, checkOnly, allowFlexgroup) {
    const request = await renderZapiRequest('attach-object-store.ejs', {
        aggregateName, objectStoreName, allowFlexgroup, checkOnly
    });
    const {
        userName, password, ip
    } = await getOntapCredentials(publicId);

    try {
        await zapiRequest(userName, password, ip, request);
        return {
            isAggregateRestricted: false,
            reason: null
        };
    } catch (err) {
        // in case the check only is true we should not throw an error
        if (checkOnly) {
            return {
                isAggregateRestricted: true,
                reason: err.message
            };
        }
        throw err;
    }
}

async function createObjectStoreConfig(publicId, objectStoreName, server, objectStoreProviderType, s3Name, azureAccount, azurePrivateKey, keys, ipspace, port, checkOnly) {
    const request = await renderZapiRequest('object-store-config-create.ejs', {
        objectStoreName,
        server,
        objectStoreProviderType,
        s3Name,
        azureAccount,
        azurePrivateKey,
        keys,
        ipspace,
        port,
        checkOnly
    });
    const {
        userName, password, ip
    } = await getOntapCredentials(publicId);

    return zapiRequest(userName, password, ip, request);
}

async function clusterVolumes(publicId) {
    let tag;
    const volumes = [];
    do {
        const request = await renderZapiRequest('cluster-volumes.ejs', { tag });
        const {
            userName, password, ip
        } = await getOntapCredentials(publicId);
        const response = await zapiRequest(userName, password, ip, request);

        tag = response['next-tag'] ? response['next-tag']._text : undefined;
        parseJson(updateResponse(response['attributes-list']['volume-attributes']));
        volumes.push(updateResponse(response['attributes-list']['volume-attributes']));
    } while (tag);

    return volumes.flat().map(vol => ({
        volumeCompAggrAttributes: {
            tieringPolicy: vol['volume-comp-aggr-attributes']['tiering-policy']
        },
        volumeIdAttributes: {
            containingAggregateName: vol['volume-id-attributes']['containing-aggregate-name'],
            creationTime: vol['volume-id-attributes']['creation-time'],
            instanceUUID: vol['volume-id-attributes']['instance-uuid'],
            name: vol['volume-id-attributes'].name,
            owningVserverName: vol['volume-id-attributes']['owning-vserver-name'],
            type: vol['volume-id-attributes'].type
        },
        volumeSpaceAttributes: {
            performanceTierInactiveUserData: vol['volume-space-attributes']['performance-tier-inactive-user-data'],
            performanceTierInactiveUserDataPercent: vol['volume-space-attributes']['performance-tier-inactive-user-data-percent'],
            physicalUsed: vol['volume-space-attributes']['physical-used'],
            size: vol['volume-space-attributes'].size,
            sizeUsed: vol['volume-space-attributes']['size-used'],
            spaceGuarantee: vol['volume-space-attributes']['space-guarantee']
        },
        volumeStateAttributes: {
            isVserverRoot: vol['volume-state-attributes']['is-vserver-root'],
            state: vol['volume-state-attributes'].state
        }
    }));
}

async function modifyVolume(publicId, volumeName, svmName, tieringPolicy) {
    const request = await renderZapiRequest('modify-volume.ejs', {
        tieringPolicy, volumeName, svmName
    });
    const {
        userName, password, ip
    } = await getOntapCredentials(publicId);

    return zapiRequest(userName, password, ip, request);
}

async function applyLicenseV2(publicId, licenseObj) {
    const license = escapeHtml(JSON.stringify(licenseObj));
    const request = await renderZapiRequest('apply-license-v2.ejs', { license });
    const {
        userName, password, ip
    } = await getOntapCredentials(publicId);
    return zapiRequest(userName, password, ip, request);
}

async function modifyVolumeMinimumCoolingDays(publicId, volumeName, svmName, days) {
    const command = `set diag -confirmations off; volume modify -volume ${volumeName} -vserver ${svmName} -tiering-minimum-cooling-days ${days}`;
    const request = await renderZapiRequest('system-cli-request.ejs', { command });
    const {
        userName, password, ip
    } = await getOntapCredentials(publicId);
    const response = await zapiRequest(userName, password, ip, request);

    return {
        attributes: { status: response._attributes.status },
        cliOutput: response['cli-output']._text,
        cliResultValue: response['cli-result-value']._text
    };
}

async function modifyTieringFullnessThreshold(publicId, aggregateName, objectStoreName, fullnessThreshold) {
    const command = `set diag -confirmations off; storage aggregate object-store modify -aggregate ${aggregateName} -object-store-name ${objectStoreName} -tiering-fullness-threshold ${fullnessThreshold}%`;
    const request = await renderZapiRequest('system-cli-request.ejs', { command });
    const {
        userName, password, ip
    } = await getOntapCredentials(publicId);
    const response = await zapiRequest(userName, password, ip, request);

    return {
        attributes: { status: response._attributes.status },
        cliOutput: response['cli-output']._text,
        cliResultValue: response['cli-result-value']._text
    };
}

async function describeVolumesMinimumCoolingDays(publicId) {
    const command = 'set diag; volume show -fields tiering-minimum-cooling-days';
    const request = await renderZapiRequest('system-cli-request.ejs', { command });
    const {
        userName, password, ip
    } = await getOntapCredentials(publicId);
    const response = await zapiRequest(userName, password, ip, request);
    return {
        attributes: { status: response._attributes.status },
        cliOutput: response['cli-output']._text,
        cliResultValue: response['cli-result-value']._text
    };
}

module.exports = {
    describeOntapVersion,
    describeSvms,
    describeObjectStoreConfig,
    describeSystemInfo,
    describeObjectStores,
    describeUsers,
    describeAggregates,
    describeClusterInfo,
    describeClusterVolumesFootprint,
    describeDisks,
    describeLicenses,
    describeLicenseV2,
    describeSvmVolumesWithFilter,
    modifyColdDataReporting,
    describeNodes,
    describeNodeSystemImages,
    describeSvmLifs,
    attachObjectStore,
    createObjectStoreConfig,
    clusterVolumes,
    modifyVolume,
    applyLicenseV2,
    modifyVolumeMinimumCoolingDays,
    describeVolumesMinimumCoolingDays,
    renderZapiRequest,
    modifyTieringFullnessThreshold
};
