const config = require('config');
const got = require('got');
const ms = require('ms');
const { get } = require('lodash');
const createError = require('http-errors');
const Agent = require('agentkeepalive');
const getLogger = require('./logger');

const logger = getLogger('got');
const agent = new Agent();

const gotInstance = got.extend({
    retry: {
        limit: config.get('got.retries'),
        methods: ['GET', 'POST', 'PUT', 'DELETE', 'OPTIONS', 'TRACE']
    },
    timeout: {
        lookup: ms(config.get('got.lookup-timeout')),
        connect: ms(config.get('got.connect-timeout')),
        response: ms(config.get('got.response-timeout'))
    },
    resolveBodyOnly: true,
    responseType: 'json',
    mutableDefaults: true,
    agent: {
        http: agent
    },
    hooks: {
        beforeRequest: [
            request => logger.info(`${request.method} request to: ${request.url.origin}${request.url.pathname}  body:`, request.body)
        ],
        afterResponse: [
            response => {
                logger.info(`${response.req.method} response status: ${response.statusCode} from: ${response.url}  body:`, response.body, 'took', get(response, 'timings.phases.total'));
                return response;
            }
        ],
        beforeError: [
            error => {
                logger.error('Received error status:', get(error, 'response.status', get(error, 'response.statusCode', error.code || 500)), 'from:', get(error, 'response.url'), 'message', get(error, 'response.body'), 'Raw error:', error, 'took', get(error, 'timings.phases.total'));
                if (error.response) {
                    // applicable for ParseError, HTTPError, got.MaxRedirectsError
                    return createError(error.response.statusCode, error.response.body);
                }

                return createError(error.code, error.message);
            }
        ],
        beforeRetry: [
            (options, error, retryCount) => logger.info(`Before retry number: ${retryCount} on request to: ${options.url}, received Error: ${error.code}, ${error.name}`)
        ]
    }
});

module.exports = gotInstance;
