// eslint-disable-next-line node/no-unpublished-require
require('jest-xml-matcher');
const zapiRequest = require('../../../src/lib/zapi/zapiClient');
const { getOntapCredentials } = require('../../../src/lib/cloud-manager/zapi-credentials-operations');
const { setProxy } = require('../../../src/lib/cloud-manager/config-operations');

const {
    describeNodes, describeUsers, describeSvms, renderZapiRequest, describeObjectStores
} = jest.requireActual('../../../src/lib/zapi/zapi');
jest.mock('../../../src/lib/zapi/zapiClient');
jest.mock('../../../src/lib/cloud-manager/zapi-credentials-operations');
jest.mock('../../../src/lib/cloud-manager/config-operations');

describe('ZAPI functions', () => {
    beforeEach(() => {
        zapiRequest.mockClear();
        getOntapCredentials.mockClear();
    });

    it('describeObjectStores - may not contain fullness-threshold', async () => {
        const objectStoreZapiResponse = {
            _attributes: {
                status: 'passed'
            },
            'attributes-list': {
                'object-store-information': [
                    {
                        aggregate: {
                            _text: 'aggregate1'
                        },
                        'object-store-name': {
                            _text: 'objectStore1'
                        },
                        'object-store-availability': {
                            _text: 'available'
                        },
                        'tiering-fullness-threshold': {
                            _text: '50'
                        },
                        'used-space': {
                            _text: '30'
                        }
                    },
                    {
                        aggregate: {
                            _text: 'aggregate2'
                        },
                        'object-store-name': {
                            _text: 'objectStore2'
                        },
                        'object-store-availability': {
                            _text: 'available'
                        },
                        'used-space': {
                            _text: '30'
                        }
                    }]
            },
            'num-records': {
                _text: 1
            }
        };
        getOntapCredentials.mockReturnValueOnce({
            userName: 'USER',
            password: 'PASSWORD',
            clusterManagementIp: '1.1.1.1',
            proxyAgent: ''
        });
        zapiRequest.mockReturnValueOnce(objectStoreZapiResponse);

        const result = [{
            aggregateName: 'aggregate1',
            name: 'objectStore1',
            availabilityState: 'available',
            tieringFullnessThreshold: '50',
            usedSpace: '30'
        }, {
            aggregateName: 'aggregate2',
            name: 'objectStore2',
            availabilityState: 'available',
            usedSpace: '30'
        }];
        expect(await describeObjectStores('clusterId')).toEqual(result);
        expect(zapiRequest).toHaveBeenCalledTimes(1);
    });

    it('describeSvms - no next-tag tag function should be called only once', async () => {
        const svmZapiResponse = {
            _attributes: {
                status: 'passed'
            },
            'attributes-list': {
                'vserver-info': [
                    {
                        'allowed-protocols': {
                            protocol: [
                                {
                                    _text: 'nfs'
                                },
                                {
                                    _text: 'cifs'
                                },
                                {
                                    _text: 'fcp'
                                },
                                {
                                    _text: 'iscsi'
                                }
                            ]
                        },
                        language: {
                            _text: 'c.utf_8'
                        },
                        state: {
                            _text: 'running'
                        },
                        'vserver-name': {
                            _text: 'svm_jlantz'
                        },
                        'vserver-subtype': {
                            _text: 'default'
                        },
                        'vserver-type': {
                            _text: 'data'
                        }
                    }]
            },
            'num-records': {
                _text: 1
            }
        };
        zapiRequest.mockReturnValueOnce(svmZapiResponse);
        getOntapCredentials.mockReturnValueOnce({
            userName: 'USER',
            password: 'PASSWORD',
            clusterManagementIp: '1.1.1.1',
            proxyAgent: ''
        });

        await describeSvms('clusterId');
        expect(zapiRequest).toHaveBeenCalledTimes(1);
    });

    it('describeSvms - next-tag tag function should be called twice', async () => {
        const firstZapiResponse = {
            _attributes: {
                status: 'passed'
            },
            'attributes-list': {
                'vserver-info': [
                    {
                        'aggr-list': {
                            'aggr-name': {
                                _text: 'aff_01_aggr1'
                            }
                        },
                        'allowed-protocols': {
                            protocol: [
                                {
                                    _text: 'nfs'
                                },
                                {
                                    _text: 'cifs'
                                },
                                {
                                    _text: 'fcp'
                                },
                                {
                                    _text: 'iscsi'
                                },
                                {
                                    _text: 'ndmp'
                                },
                                {
                                    _text: 'http'
                                }
                            ]
                        },
                        language: {
                            _text: 'c.utf_8'
                        },
                        state: {
                            _text: 'running'
                        },
                        'vserver-name': {
                            _text: 'AFF_SAN_DEFAULT_SVM'
                        },
                        'vserver-subtype': {
                            _text: 'default'
                        },
                        'vserver-type': {
                            _text: 'data'
                        }
                    },
                    {
                        'vserver-name': {
                            _text: 'Cluster'
                        },
                        'vserver-type': {
                            _text: 'system'
                        }
                    }
                ]
            },
            'next-tag': {
                _text: '<vserver-get-iter-key-td>\n        <key-0>svm_fabricpool</key-0>\n</vserver-get-iter-key-td>\n'
            },
            'num-records': {
                _text: 2
            }
        };
        const secondZapiResponse = {
            _attributes: {
                status: 'passed'
            },
            'attributes-list': {
                'vserver-info': [
                    {
                        'allowed-protocols': {
                            protocol: [
                                {
                                    _text: 'nfs'
                                },
                                {
                                    _text: 'cifs'
                                },
                                {
                                    _text: 'fcp'
                                },
                                {
                                    _text: 'iscsi'
                                }
                            ]
                        },
                        language: {
                            _text: 'c.utf_8'
                        },
                        state: {
                            _text: 'running'
                        },
                        'vserver-name': {
                            _text: 'svm_jlantz'
                        },
                        'vserver-subtype': {
                            _text: 'default'
                        },
                        'vserver-type': {
                            _text: 'data'
                        }
                    }]
            },
            'num-records': {
                _text: 1
            }
        };
        zapiRequest.mockReturnValueOnce(firstZapiResponse).mockReturnValueOnce(secondZapiResponse);
        getOntapCredentials.mockReturnValue({
            userName: 'USER',
            password: 'PASSWORD',
            clusterManagementIp: '1.1.1.1',
            proxyAgent: ''
        });
        await describeSvms('clusterId');
        expect(zapiRequest).toHaveBeenCalledTimes(2);
    });

    it('describeUsers - no next-tag tag function should be called only once', async () => {
        const usersZapiResponse = {
            _attributes: {
                status: 'passed'
            },
            'attributes-list': {
                'security-login-account-info': [
                    {
                        application: {
                            _text: 'ssh'
                        },
                        'authentication-method': {
                            _text: 'password'
                        },
                        'role-name': {
                            _text: 'vsadmin'
                        },
                        'user-name': {
                            _text: 'vsadmin'
                        },
                        vserver: {
                            _text: 'svm_arun'
                        }
                    }
                ]
            },
            'num-records': {
                _text: 1
            }
        };

        zapiRequest.mockReturnValueOnce(usersZapiResponse);
        setProxy.mockReturnValue(undefined);
        await describeUsers('1.1.1.1', 'admin', '123');
        expect(zapiRequest).toHaveBeenCalledTimes(1);
    });

    it('describeUsers - next-tag tag function should be called twice', async () => {
        const firstZapiResponse = {
            _attributes: {
                status: 'passed'
            },
            'attributes-list': {
                'security-login-account-info': [
                    {
                        application: {
                            _text: 'http'
                        },
                        'authentication-method': {
                            _text: 'password'
                        },
                        'role-name': {
                            _text: 'vsadmin'
                        },
                        'user-name': {
                            _text: 'vsadmin'
                        },
                        vserver: {
                            _text: 'AFF_SAN_DEFAULT_SVM'
                        }
                    }
                ]
            },
            'next-tag': {
                _text: '<security-login-get-iter-key-td>\n        <key-0>aff</key-0>\n        <key-1>admin</key-1>\n        <key-2>console</key-2>\n        <key-3>password</key-3>\n</security-login-get-iter-key-td>\n'
            },
            'num-records': {
                _text: 2
            }
        };
        const secondZapiResponse = {
            _attributes: {
                status: 'passed'
            },
            'attributes-list': {
                'security-login-account-info': [
                    {
                        application: {
                            _text: 'ssh'
                        },
                        'authentication-method': {
                            _text: 'password'
                        },
                        'role-name': {
                            _text: 'vsadmin'
                        },
                        'user-name': {
                            _text: 'vsadmin'
                        },
                        vserver: {
                            _text: 'svm_arun'
                        }
                    }
                ]
            },
            'num-records': {
                _text: 1
            }
        };
        getOntapCredentials.mockReturnValueOnce({
            userName: 'USER',
            password: 'PASSWORD',
            clusterManagementIp: '1.1.1.1',
            proxyAgent: ''
        });
        zapiRequest.mockReturnValueOnce(firstZapiResponse).mockReturnValueOnce(secondZapiResponse);
        await describeUsers('clusterId');
        expect(zapiRequest).toHaveBeenCalledTimes(2);
    });

    it('describeNodes - no next-tag tag function should be called only once', async () => {
        const nodesZapiResponse = {
            _attributes: {
                status: 'passed'
            },
            'attributes-list': {
                'node-details-info': [
                    {
                        'is-node-cluster-eligible': {
                            _text: true
                        },
                        'is-node-healthy': {
                            _text: true
                        },
                        node: {
                            _text: 'aff-01'
                        },
                        'node-model': {
                            _text: 'AFF8040'
                        },
                        'node-uptime': {
                            _text: 5864856
                        }
                    }
                ]
            },
            'num-records': {
                _text: 1
            }
        };
        zapiRequest.mockReturnValueOnce(nodesZapiResponse);
        getOntapCredentials.mockReturnValueOnce({
            userName: 'USER',
            password: 'PASSWORD',
            clusterManagementIp: '1.1.1.1',
            proxyAgent: ''
        });
        await describeNodes('clusterId');
        expect(zapiRequest).toHaveBeenCalledTimes(1);
    });

    it('describeNodes - next-tag tag function should be called twice', async () => {
        const firstZapiResponse = {
            _attributes: {
                status: 'passed'
            },
            'attributes-list': {
                'node-details-info': [
                    {
                        'is-node-cluster-eligible': {
                            _text: true
                        },
                        'is-node-healthy': {
                            _text: true
                        },
                        node: {
                            _text: 'aff-01'
                        },
                        'node-model': {
                            _text: 'AFF8040'
                        },
                        'node-uptime': {
                            _text: 5864856
                        }
                    }
                ]
            },
            'next-tag': {
                _text: '<security-login-get-iter-key-td>\n        <key-0>aff</key-0>\n        <key-1>admin</key-1>\n        <key-2>console</key-2>\n        <key-3>password</key-3>\n</security-login-get-iter-key-td>\n'
            },
            'num-records': {
                _text: 1
            }
        };
        const secondZapiResponse = {
            _attributes: {
                status: 'passed'
            },
            'attributes-list': {
                'node-details-info': [
                    {
                        'is-node-cluster-eligible': {
                            _text: true
                        },
                        'is-node-healthy': {
                            _text: true
                        },
                        node: {
                            _text: 'aff-01'
                        },
                        'node-model': {
                            _text: 'AFF8040'
                        },
                        'node-uptime': {
                            _text: 5864856
                        }
                    }
                ]
            },
            'num-records': {
                _text: 1
            }
        };
        getOntapCredentials.mockReturnValueOnce({
            userName: 'USER',
            password: 'PASSWORD',
            clusterManagementIp: '1.1.1.1',
            proxyAgent: ''
        });
        zapiRequest.mockReturnValueOnce(firstZapiResponse).mockReturnValueOnce(secondZapiResponse);
        await describeNodes('clusterId');
        expect(zapiRequest).toHaveBeenCalledTimes(2);
    });
});

describe('renderZapiRequest', () => {
    it('render describeOntapVersion', async () => {
        const request = await renderZapiRequest('describe-ontap-version.ejs', {});
        expect(request).toEqualXML('<netapp version="1.19" xmlns="http://www.netapp.com/filer/admin">\n    <system-get-version/>\n</netapp>');
    });

    it('render describeSvms with svmName and without tag', async () => {
        const request = await renderZapiRequest('describe-svm.ejs', {
            svmName: '1',
            tag: undefined
        });
        expect(request).toEqualXML(`<netapp xmlns="http://www.netapp.com/filer/admin" version="1.20">
            <vserver-get-iter>
            <desired-attributes>
            <vserver-info>
            <state/>
            <vserver-subtype/>
            <vserver-name/>
            <vserver-type/>
            <allowed-protocols>
            <protocol/>
            </allowed-protocols>
        <aggr-list>
            <aggr-name/>
        </aggr-list>
        <language/>
        </vserver-info>
    </desired-attributes>
    <query>
    <vserver-info>
        <vserver-name>1</vserver-name>
    </vserver-info>
    </query>
    <max-records>500</max-records>
    </vserver-get-iter>
    </netapp>`);
    });

    it('render describeSvms with svmName and with tag', async () => {
        const request = await renderZapiRequest('describe-svm.ejs', {
            svmName: '1',
            tag: `&lt;vserver-get-iter-key-td&gt;
        &lt;key-0&gt;svm_fabricpool&lt;/key-0&gt;
&lt;/vserver-get-iter-key-td&gt;`
        });
        expect(request).toEqualXML(`<netapp xmlns="http://www.netapp.com/filer/admin" version="1.20">
            <vserver-get-iter>
            <desired-attributes>
            <vserver-info>
            <state/>
            <vserver-subtype/>
            <vserver-name/>
            <vserver-type/>
            <allowed-protocols>
            <protocol/>
            </allowed-protocols>
        <aggr-list>
            <aggr-name/>
        </aggr-list>
        <language/>
        </vserver-info>
    </desired-attributes>
    <query>
    <vserver-info>
        <vserver-name>1</vserver-name>
    </vserver-info>
    </query>
    <tag>&amp;lt;vserver-get-iter-key-td&amp;gt;
        &amp;lt;key-0&amp;gt;svm_fabricpool&amp;lt;/key-0&amp;gt;
&amp;lt;/vserver-get-iter-key-td&amp;gt;</tag>
    <max-records>500</max-records>
    </vserver-get-iter>
    </netapp>`);
    });

    it('render describeSvms with tag and without svmName', async () => {
        const request = await renderZapiRequest('describe-svm.ejs', {
            tag: `&lt;vserver-get-iter-key-td&gt;
        &lt;key-0&gt;svm_fabricpool&lt;/key-0&gt;
&lt;/vserver-get-iter-key-td&gt;`
        });
        expect(request).toEqualXML(`<netapp xmlns="http://www.netapp.com/filer/admin" version="1.20">
            <vserver-get-iter>
            <desired-attributes>
            <vserver-info>
            <state/>
            <vserver-subtype/>
            <vserver-name/>
            <vserver-type/>
            <allowed-protocols>
            <protocol/>
            </allowed-protocols>
        <aggr-list>
            <aggr-name/>
        </aggr-list>
        <language/>
        </vserver-info>
    </desired-attributes>
    <query>
    <vserver-info/>
    </query>
    <tag>&amp;lt;vserver-get-iter-key-td&amp;gt;
        &amp;lt;key-0&amp;gt;svm_fabricpool&amp;lt;/key-0&amp;gt;
&amp;lt;/vserver-get-iter-key-td&amp;gt;</tag>
    <max-records>500</max-records>
    </vserver-get-iter>
    </netapp>`);
    });

    it('render describeObjectStoreConfig', async () => {
        const request = await renderZapiRequest('describe-object-store-config.ejs', {});
        expect(request).toEqualXML(`<netapp xmlns="http://www.netapp.com/filer/admin" version="1.20">
    <aggr-object-store-config-get-iter>
        <desired-attributes>
            <aggr-object-store-config-info>
                <object-store-name/>
                <s3-name/>
                <used-space/>
                <provider-type/>
                <server/>
            </aggr-object-store-config-info>
        </desired-attributes>
        <max-records>500</max-records>
    </aggr-object-store-config-get-iter>
</netapp>`);
    });

    it('render describeSystemInfo', async () => {
        const request = await renderZapiRequest('describe-system-info.ejs', {});
        expect(request).toEqualXML(`<netapp version="1.19" xmlns="http://www.netapp.com/filer/admin">
    <system-node-get-iter>
        <desired-attributes>
            <node-details-info>
                <node-system-id/>
                <product-version/>
                <node-serial-number/>
                <node/>
            </node-details-info>
        </desired-attributes>
        <max-records>500</max-records>
    </system-node-get-iter>
</netapp>`);
    });

    it('render modifyColdDataReporting', async () => {
        const request = await renderZapiRequest('modify-cold-data-reporting.ejs', {
            aggregateName: 'aggr1',
            enable: true
        });
        expect(request).toEqualXML(`<netapp  xmlns="http://www.netapp.com/filer/admin" version="1.20">
    <aggr-set-option>
        <aggregate>aggr1</aggregate>
        <option-name>enable_cold_data_reporting</option-name>
        <option-value>true</option-value>
    </aggr-set-option>
</netapp>`);
    });

    it('render describeNodes without tag', async () => {
        const request = await renderZapiRequest('describe-nodes.ejs', { tag: undefined });
        expect(request).toEqualXML(`<netapp xmlns="http://www.netapp.com/filer/admin" version="1.20">
    <system-node-get-iter>
        <desired-attributes>
            <node-details-info>
                <node/>
                <node-model/>
                <is-node-healthy/>
                <is-node-cluster-eligible/>
            </node-details-info>
        </desired-attributes>
        <max-records>500</max-records>
    </system-node-get-iter>
</netapp>`);
    });

    it('render describeNodes with tag', async () => {
        const request = await renderZapiRequest('describe-nodes.ejs', {
            tag: `&lt;vserver-get-iter-key-td&gt;
        &lt;key-0&gt;svm_fabricpool&lt;/key-0&gt;
&lt;/vserver-get-iter-key-td&gt;`
        });
        expect(request).toEqualXML(`<netapp xmlns="http://www.netapp.com/filer/admin" version="1.20">
    <system-node-get-iter>
        <desired-attributes>
            <node-details-info>
                <node/>
                <node-model/>
                <is-node-healthy/>
                <is-node-cluster-eligible/>
            </node-details-info>
        </desired-attributes>
            <tag>&amp;lt;vserver-get-iter-key-td&amp;gt;
        &amp;lt;key-0&amp;gt;svm_fabricpool&amp;lt;/key-0&amp;gt;
&amp;lt;/vserver-get-iter-key-td&amp;gt;</tag>
        <max-records>500</max-records>
    </system-node-get-iter>
</netapp>`);
    });

    it('render describeUsers without tag', async () => {
        const request = await renderZapiRequest('describe-users.ejs', { tag: undefined });
        expect(request).toEqualXML(`<netapp xmlns="http://www.netapp.com/filer/admin" version="1.20">
    <security-login-get-iter>
        <desired-attributes>
            <security-login-account-info>
                <application/>
                <authentication-method/>
                <role-name/>
                <user-name/>
                <vserver/>
            </security-login-account-info>
        </desired-attributes>
        <max-records>500</max-records>
    </security-login-get-iter>
</netapp>`);
    });

    it('render describeUsers with tag', async () => {
        const request = await renderZapiRequest('describe-users.ejs', {
            tag: `&lt;vserver-get-iter-key-td&gt;
        &lt;key-0&gt;svm_fabricpool&lt;/key-0&gt;
&lt;/vserver-get-iter-key-td&gt;`
        });
        expect(request).toEqualXML(`<netapp xmlns="http://www.netapp.com/filer/admin" version="1.20">
    <security-login-get-iter>
        <desired-attributes>
            <security-login-account-info>
                <application/>
                <authentication-method/>
                <role-name/>
                <user-name/>
                <vserver/>
            </security-login-account-info>
        </desired-attributes> 
           <tag>&amp;lt;vserver-get-iter-key-td&amp;gt;
        &amp;lt;key-0&amp;gt;svm_fabricpool&amp;lt;/key-0&amp;gt;
&amp;lt;/vserver-get-iter-key-td&amp;gt;</tag>
        <max-records>500</max-records>
    </security-login-get-iter>
</netapp>`);
    });

    it('render describeNodeSystemImages without nodeName and without tag tag', async () => {
        const request = await renderZapiRequest('describe-node-system-images.ejs', { tag: undefined });
        expect(request).toEqualXML(`<netapp version="1.20" xmlns="http://www.netapp.com/filer/admin">
    <system-image-get-iter>
        <max-records>500</max-records>
    </system-image-get-iter>
</netapp>`);
    });

    it('render describeSvmLifs with svmName and tag', async () => {
        const request = await renderZapiRequest('describe-svm-lifs.ejs', {
            tag: `&lt;vserver-get-iter-key-td&gt;
        &lt;key-0&gt;svm_fabricpool&lt;/key-0&gt;
&lt;/vserver-get-iter-key-td&gt;`,
            svmName: '112'
        });
        expect(request).toEqualXML(`<netapp xmlns="http://www.netapp.com/filer/admin" vfiler="112" version="1.20">
        <net-interface-get-iter>
            <desired-attributes>
                <net-interface-info>
                    <address/>
                    <netmask/>
                    <netmask-length/>
                    <interface-name/>
                    <operational-status/>
                    <home-node/>
                    <ipspace/>
                    <vserver/>
                    <role/>
                </net-interface-info>
            </desired-attributes>
 <tag>&amp;lt;vserver-get-iter-key-td&amp;gt;
        &amp;lt;key-0&amp;gt;svm_fabricpool&amp;lt;/key-0&amp;gt;
&amp;lt;/vserver-get-iter-key-td&amp;gt;</tag>
            <max-records>500</max-records>
        </net-interface-get-iter>
    </netapp>`);
    });

    it('render describeSvmLifs without svmName and with tag', async () => {
        const request = await renderZapiRequest('describe-svm-lifs.ejs', {
            tag: `&lt;vserver-get-iter-key-td&gt;
        &lt;key-0&gt;svm_fabricpool&lt;/key-0&gt;
&lt;/vserver-get-iter-key-td&gt;`
        });
        expect(request).toEqualXML(`<netapp xmlns="http://www.netapp.com/filer/admin" version="1.20">
        <net-interface-get-iter>
            <desired-attributes>
                <net-interface-info>
                    <address/>
                    <netmask/>
                    <netmask-length/>
                    <interface-name/>
                    <operational-status/>
                    <home-node/>
                    <ipspace/>
                    <vserver/>
                    <role/>
                </net-interface-info>
            </desired-attributes>
 <tag>&amp;lt;vserver-get-iter-key-td&amp;gt;
        &amp;lt;key-0&amp;gt;svm_fabricpool&amp;lt;/key-0&amp;gt;
&amp;lt;/vserver-get-iter-key-td&amp;gt;</tag>
            <max-records>500</max-records>
        </net-interface-get-iter>
    </netapp>`);
    });

    it('render attachObjectStore with allowFlexgroup false', async () => {
        const request = await renderZapiRequest('attach-object-store.ejs', {
            aggregateName: 'aggr2', objectStoreName: 'objectStore', allowFlexgroup: false, checkOnly: true
        });
        expect(request).toEqualXML(`<netapp xmlns="http://www.netapp.com/filer/admin" version="1.20">
    <aggr-object-store-attach>
        <aggregate>aggr2</aggregate>
        <object-store-name>objectStore</object-store-name>
            <check-only>true</check-only>
    </aggr-object-store-attach>
</netapp>`);
    });

    it('render objectStoreConfigCreate aws provider', async () => {
        const request = await renderZapiRequest('object-store-config-create.ejs', {
            objectStoreName: 'OSC1',
            server: 's3.amazonaws.com',
            objectStoreProviderType: 'aws_s3',
            s3Name: 'bucket',
            keys: { accessKey: '111', secretKey: '222' },
            ipspace: 'Default',
            checkOnly: false
        });
        expect(request).toEqualXML(`<netapp xmlns="http://www.netapp.com/filer/admin" version="1.20">
    <aggr-object-store-config-create>
        <object-store-name>OSC1</object-store-name>
        <server>s3.amazonaws.com</server>
        <provider-type>aws_s3</provider-type>
        <s3-name>bucket</s3-name>
        <secret-password>222</secret-password>
        <access-key>111</access-key>
        <ipspace>Default</ipspace>
    </aggr-object-store-config-create>
</netapp>`);
    });

    it('render objectStoreConfigCreate sgws provider', async () => {
        const request = await renderZapiRequest('object-store-config-create.ejs', {
            objectStoreName: 'OSC1',
            server: 's3.amazonaws.com',
            objectStoreProviderType: 'sgws',
            s3Name: 'bucket',
            keys: { accessKey: '111', secretKey: '222' },
            ipspace: 'Default',
            checkOnly: false
        });
        expect(request).toEqualXML(`<netapp xmlns="http://www.netapp.com/filer/admin" version="1.20">
    <aggr-object-store-config-create>
        <object-store-name>OSC1</object-store-name>
        <server>s3.amazonaws.com</server>
        <provider-type>sgws</provider-type>
        <s3-name>bucket</s3-name>
        <secret-password>222</secret-password>
        <access-key>111</access-key>
        <is-certificate-validation-enabled>false</is-certificate-validation-enabled>
        <ipspace>Default</ipspace>
    </aggr-object-store-config-create>
</netapp>`);
    });

    it('render objectStoreConfigCreate azure provider', async () => {
        const request = await renderZapiRequest('object-store-config-create.ejs', {
            objectStoreName: 'OSC1',
            server: 'r2sc0lwwm52xfid.blob.core.windows.net',
            objectStoreProviderType: 'azure_cloud',
            s3Name: 'bucket',
            azureAccount: 'azure_account',
            azurePrivateKey: 'azure_key',
            ipspace: 'Default',
            checkOnly: false
        });
        expect(request).toEqualXML(`<netapp xmlns="http://www.netapp.com/filer/admin" version="1.20">
    <aggr-object-store-config-create>
        <object-store-name>OSC1</object-store-name>
        <server>r2sc0lwwm52xfid.blob.core.windows.net</server>
        <provider-type>azure_cloud</provider-type>
        <s3-name>bucket</s3-name>
        <azure-account>azure_account</azure-account>
        <azure-private-key>azure_key</azure-private-key>
        <ipspace>Default</ipspace>
    </aggr-object-store-config-create>
</netapp>`);
    });

    it('render objectStoreConfigCreate gcp provider', async () => {
        const request = await renderZapiRequest('object-store-config-create.ejs', {
            objectStoreName: 'OSC1',
            server: 'r2sc0lwwm52xfid.blob.core.windows.net',
            objectStoreProviderType: 'googlecloud',
            s3Name: 'bucket',
            keys: { secretKey: 'sec_key', accessKey: 'acc_key' },
            ipspace: 'Default',
            checkOnly: false
        });
        expect(request).toEqualXML(`<netapp xmlns="http://www.netapp.com/filer/admin" version="1.20">
    <aggr-object-store-config-create>
        <object-store-name>OSC1</object-store-name>
        <server>r2sc0lwwm52xfid.blob.core.windows.net</server>
        <provider-type>googlecloud</provider-type>
        <s3-name>bucket</s3-name>
        <secret-password>sec_key</secret-password>
        <access-key>acc_key</access-key>
        <ipspace>Default</ipspace>
    </aggr-object-store-config-create>
</netapp>`);
    });

    it('render clusterVolumes without tag', async () => {
        const request = await renderZapiRequest('cluster-volumes.ejs', { tag: undefined });
        expect(request).toEqualXML(`<netapp xmlns="http://www.netapp.com/filer/admin" version="1.20">
    <volume-get-iter>
        <desired-attributes>
            <volume-attributes>
                <volume-id-attributes>
                    <name/>
                    <owning-vserver-name/>
                    <containing-aggregate-name/>
                    <type/>
                    <creation-time/>
                    <flexgroup-msid/>
                    <flexgroup-index/>
                    <instance-uuid/>
                </volume-id-attributes>
                <volume-space-attributes>
                    <size/>
                    <size-used/>
                    <space-guarantee/>
                    <performance-tier-inactive-user-data/>
                    <performance-tier-inactive-user-data-percent/>
                    <physical-used/>
                </volume-space-attributes>
                <volume-state-attributes>
                    <is-vserver-root/>
                    <state/>
                </volume-state-attributes>
                <volume-comp-aggr-attributes>
                    <tiering-policy/>
                </volume-comp-aggr-attributes>
                <volume-qos-attributes>
                    <policy-group-name/>
                </volume-qos-attributes>
            </volume-attributes>
        </desired-attributes>
        <max-records>500</max-records>
    </volume-get-iter>
</netapp>`);
    });

    it('render modifyVolumes', async () => {
        const request = await renderZapiRequest('modify-volume.ejs', {
            tieringPolicy: 'auto',
            volumeName: 'vol1',
            svmName: 'svmName'
        });
        expect(request).toEqualXML(`<netapp xmlns="http://www.netapp.com/filer/admin" version="1.20">
    <volume-modify-iter>
        <attributes>
            <volume-attributes>
                <volume-comp-aggr-attributes>
                    <tiering-policy>auto</tiering-policy>
                </volume-comp-aggr-attributes>
            </volume-attributes>
        </attributes>
        <query>
            <volume-attributes>
                <volume-id-attributes>
                    <name>vol1</name>
                    <owning-vserver-name>svmName</owning-vserver-name>
                </volume-id-attributes>
            </volume-attributes>
        </query>
        <max-records>500</max-records>
    </volume-modify-iter>
</netapp>`);
    });
});
