const hyperid = require('hyperid');
const ms = require('ms');
const { Subscription } = require('../models/subscription');
const Message = require('../models/message');
const getLogger = require('../utils/logger/logger');

const logger = getLogger('asyncIterator');

async function getNextPromise(topic, accountId, subscriptionId) {
    const subscription = await Subscription.findOne({ subscriptionId }).exec();
    if (subscription) {
        const event = await Message.findOneAndRemove({
            topic,
            accountId,
            subscriptionId,
            payload: { $exists: true }
        }).exec();

        if (event) {
            logger.debug('getNextPromise', subscriptionId, topic, event.payload);
            return { value: event.payload, done: false };
        }
        await new Promise(resolve => setTimeout(() => resolve(), ms('30s')));
        return getNextPromise(topic, accountId, subscriptionId);
    }
    return { done: true };
}

async function getAsyncIterator(topic, accountId) {
    const subscriptionId = hyperid().uuid;
    await Subscription.create({
        topic,
        accountId,
        subscriptionId
    });

    const iterator = {
        next: () => getNextPromise(topic, accountId, subscriptionId),
        throw: async err => {
            logger.error(`Error on asyncIterator for triggerName ${topic} and ${accountId}`);
            await Subscription.deleteOne({ subscriptionId }).exec();
            await Message.deleteMany({ accountId, subscriptionId }).exec();
            throw new Error(err.message);
        },
        return: async () => {
            logger.info(`Subscription ${subscriptionId} disconnected`);
            await Subscription.deleteOne({ subscriptionId }).exec();
            await Message.deleteMany({ accountId, subscriptionId }).exec();
            return { value: undefined, done: true };
        }
    };

    return {
        [Symbol.asyncIterator]: () => iterator
    };
}

async function updateMessages(accountId, topic, payload) {
    const subscriptions = await Subscription.find({ accountId }).exec();
    if (subscriptions.length) {
        return Promise.all(subscriptions.map(subscription => {
            logger.debug('Creating message in messages collection', { accountId, topic, payload });
            return Message.create({
                accountId, topic, payload, subscriptionId: subscription.subscriptionId
            });
        }));
    }
}

module.exports = { getAsyncIterator, updateMessages };
