/*
 Each resolver contains 4 args:
1. parent: The object contains the result returned from the resolver on the parent field on in case of a top-level Query field, the rootValue passed from the serverConfiguration
2. args: an object  with the args passed into the field in the query
3. context : object shared by all resolversis used to contain per request state
4. info: This argument contains information about the execution state of the query, including the field name, path to the field from the root, and more.
*/
const { ApolloError } = require('apollo-server');
const { GraphQLScalarType } = require('graphql');
const { Kind } = require('graphql/language');
const Joi = require('@hapi/joi');
const {
    getClusterDetails,
    getIpSpaces, markAsDeletedOperation,
    getLazyClusters, getClusters
} = require('../operations/cluster-operations');
const { getSupportedVolumes } = require('../operations/volume-operations');
const { findLicenseInfo } = require('../operations/subscription-operations');
const { applyByolLicenseOperation } = require('../operations/ipa-operations');
const {
    listAzureStorageAccounts, createBucketOperation,
    changeTierLevelOperation, listBucketsOperation
} = require('../operations/buckets-operations');
const listResourceGroups = require('../operations/resource-group-operations');
const {
    findAccount, verifyClusterInWorkspace
} = require('../operations/multi-tenancy-operation');
const {
    UPDATE_LICENSE_INFO_EVENT
} = require('../utils/consts');
const { generateRequestId, isEmptyString, verifyRequestParams } = require('../utils/utils');
const HttpError = require('../utils/errors');
const { getRegions, getRegionByRegionCode } = require('../operations/regions-operations');
const {
    setupClusterOperation, modifyVolumeOperation
} = require('../operations/tiering-operations');
const { getAsyncIterator } = require('./mongo-async-iterator');
const getMyIterator = require('../utils/async-iterator');
const { verifyPermission } = require('../operations/check-permission-operations');
const { modifyAggregatesOperation, getAggregatesOperation } = require('../operations/aggregate-operations');
const {
    setUserToken,
    getLocalStorage, setUserPortalId, setAccountId, setRequestId, setUserName
} = require('../utils/async-local-storage');

const resolvers = {
    BucketG: {
        __resolveType(obj) {
            if (obj.resourceGroup && obj.storageAccount) {
                return 'AzureContainer';
            }
            if (obj.region) {
                return 'CloudBucket';
            }
            if (obj.name && !obj.region) {
                return 'SGBucket';
            }

            return null;
        }
    },
    Query: {
        clusters: (parent, args) => getClusters(args.agentId, args.workspaceId),
        lazyClusters: (parant, args) => getLazyClusters(args.agentId, args.workspaceId),
        cluster: async (parent, args) => {
            await verifyClusterInWorkspace(args.clusterId, args.workspaceId);
            return getClusterDetails(args.clusterId);
        },
        volumes: async (parent, args) => {
            const { clusterId, agentId, workspaceId } = args;
            await verifyClusterInWorkspace(clusterId, workspaceId);
            return getSupportedVolumes(clusterId, agentId);
        },
        aggregates: async (parent, args) => {
            const { clusterId, agentId, workspaceId } = args;
            await verifyClusterInWorkspace(clusterId, workspaceId);
            return getAggregatesOperation(clusterId, agentId);
        },
        regions: async (parent, args) => {
            const {
                regionCode, clusterId, agentId, cloudProvider
            } = args;
            return getRegions(cloudProvider, regionCode, clusterId, agentId);
        },
        region: async (parent, args) => {
            const {
                type, regionCode, cloudProvider
            } = args;
            return getRegionByRegionCode(type || cloudProvider, regionCode);
        },
        storageAccounts: async (parent, args) => {
            const { clusterId, agentId, resourceGroupName } = args;
            return listAzureStorageAccounts(clusterId, agentId, resourceGroupName);
        },
        resourceGroups: async (parent, args) => {
            const { clusterId, agentId } = args;
            return listResourceGroups(clusterId, agentId);
        },
        ipSpaces: async (parent, args) => {
            const { clusterId, agentId, workspaceId } = args;
            await verifyClusterInWorkspace(clusterId, workspaceId);
            return getIpSpaces(clusterId, agentId);
        },
        licenseInfo: async () => findLicenseInfo(),
        bucketsG: async (parent, args) => {
            const { request: { cloudProvider, ...rest } } = args;
            verifyRequestParams(cloudProvider, { ...rest });
            return listBucketsOperation(cloudProvider, { ...rest });
        }
    },
    Mutation: {
        setupClusterTieringG: async (parent, args) => {
            const {
                input: {
                    cloudProvider, ...rest
                }
            } = args;
            verifyRequestParams(cloudProvider, { ...rest });
            await setupClusterOperation(cloudProvider, { ...rest });
            return { status: 'success' };
        },
        setupClusterTieringG2: async (parent, args) => {
            const {
                input: {
                    cloudProvider, ...rest
                }
            } = args;
            verifyRequestParams(cloudProvider, { ...rest });
            return setupClusterOperation(cloudProvider, { ...rest });
        },
        modifyVolumes: async (parent, args) => {
            const {
                input: {
                    agentId, workspaceId,
                    clusterId, volumeProperties
                }
            } = args;
            await verifyClusterInWorkspace(clusterId, workspaceId);
            return modifyVolumeOperation(clusterId, volumeProperties, agentId);
        },
        modifyAggregates: async (parent, args) => {
            const {
                input: {
                    agentId, workspaceId,
                    clusterId, aggregatesProperties
                }
            } = args;
            await verifyClusterInWorkspace(clusterId, workspaceId);
            return modifyAggregatesOperation(clusterId, aggregatesProperties, agentId);
        },
        removeCluster: async (parent, args) => {
            const { input: { clusterId, agentId, workspaceId } } = args;
            await verifyClusterInWorkspace(clusterId, workspaceId);
            await markAsDeletedOperation(clusterId, agentId);
            return { status: 'success' };
        },
        verifyPermissions: async (parent, args) => {
            const {
                input: {
                    region, bucket, credentials, cloudProvider
                }
            } = args;
            await verifyPermission(cloudProvider, region, bucket, credentials);
            return { status: 'success' };
        },
        createBucketG: async (parent, args) => {
            const {
                input: { cloudProvider, ...rest }
            } = args;
            verifyRequestParams(cloudProvider, { ...rest });
            await createBucketOperation(cloudProvider, { ...rest });
            return { status: 'success' };
        },
        changeTierLevelG: async (parent, args) => {
            const {
                input: { cloudProvider, ...rest }
            } = args;
            verifyRequestParams(cloudProvider, { ...rest });
            await changeTierLevelOperation(cloudProvider, { ...rest });
            return { status: 'success' };
        },
        applyByolLicense: async (parent, args) => {
            const {
                input: {
                    serialNumber, clusterId, nssUserName, nssPassword, workspaceId
                }
            } = args;
            await verifyClusterInWorkspace(clusterId, workspaceId);
            if (isEmptyString(nssUserName) || isEmptyString(nssPassword)) {
                throw new HttpError('Invalid NSS credentials', 403);
            }
            await applyByolLicenseOperation(nssUserName, nssPassword, clusterId, serialNumber);
            return { status: 'success' };
        }
    },
    Subscription: {
        licenseInfo: {
            subscribe: (_, args) => getAsyncIterator(UPDATE_LICENSE_INFO_EVENT, args.accountId)
        },
        clusterInfo: {
            subscribe: async (_, args, context) => getLocalStorage().run(new Map(), async () => {
                setUserToken(context.authorization);
                setUserPortalId(context.userPortalId);
                setRequestId(generateRequestId());
                setUserName(context.userName);
                const accountId = await findAccount(args.accountId);
                setAccountId(accountId);
                return getMyIterator(args.accountId, args.agentId, args.workspaceId, args.resourceType);
            })
        }
    },
    IpAddress: new GraphQLScalarType({
        name: 'IpAddress',
        description: 'Ip Address custom scalar type',
        parseValue(value) { // value from the client (ui not from playground)
            const { error } = Joi.string().ip({ version: ['ipv4'] }).validate(value, { error: 'Invalid Ip Address' }); // TypeError
            if (error) {
                throw new ApolloError(error.message, 400);
            }
            return value;
        },
        serialize(value) { // value sent to the client
            const { error } = Joi.string().ip({ version: ['ipv4'] }).validate(value, { error: 'Invalid Ip Address' }); // TypeError
            if (error) {
                throw new ApolloError(error.message);
            }
            return value;
        },
        parseLiteral(ast) {
            if (ast.kind === Kind.STRING) {
                const { error } = Joi.string().ip({ version: ['ipv4'] }).validate(ast.value, { message: 'Invalid Ip Address' }); // TypeError
                if (error) {
                    throw new ApolloError(error.message, 400);
                }
                return ast.value;
            }
            return null;
        }
    }),
    HostName: new GraphQLScalarType({
        name: 'HostName',
        description: 'Host Name custom scalar type',
        parseValue(value) { // value from the client (ui not from playground)
            const { error } = Joi.string().domain().validate(value, { error: 'Invalid FQDN' }); // TypeError
            if (error) {
                throw new ApolloError(error.message, 400);
            }
            return value;
        },
        serialize(value) { // value sent to the client
            const { error } = Joi.string().domain().validate(value, { error: 'Invalid FQDN' }); // TypeError
            if (error) {
                throw new ApolloError(error.message);
            }
            return value;
        },
        parseLiteral(ast) {
            if (ast.kind === Kind.STRING) {
                const { error } = Joi.string().domain().validate(ast.value, { message: 'Invalid FQDN' }); // TypeError
                if (error) {
                    throw new ApolloError(error.message, 400);
                }
                return ast.value;
            }
            return null;
        }
    })
};

module.exports = resolvers;
