const util = require('util');
const got = require('got');
const express = require('express');
const cors = require('cors');
const { isEmpty, get, isObject } = require('lodash');
const { ApolloServer } = require('apollo-server-express');
const { up } = require('migrate-mongo');
const helmet = require('helmet');
const compression = require('compression');
const http = require('http');
const config = require('config');
const mongoose = require('mongoose');
const resolvers = require('./graphql/resolvers');
const typeDefs = require('./graphql/schema');
const getLogger = require('./utils/logger/logger');
const initAgenda = require('./utils/agenda');
const {
    API_ROOT, DB_CONNECTION_STRING, JWR_TOKEN_TENANCY_AUDIENCE
} = require('./utils/consts');
const { setAccountId } = require('./utils/async-local-storage');
const jwtVerify = require('./utils/jwt');
const statisticsRouter = require('./routes/statistics');
const internalRoute = require('./routes/internal');
const healthRouter = require('./routes/health');
const saasSubscriptionWebHookRouter = require('./routes/saas-subscription-webhook');
const reactRouterFallbackMiddleware = require('./middlewares/react-router-fallback-middleware');
const continuationLocalStorageMiddleware = require('./middlewares/continuation-local-storage-middleware');
const findAccountAndUpdateContinuationToken = require('./middlewares/get-account-middleware');
const authenticationMiddleware = require('./middlewares/authentication-middleware');
const { createServiceToken } = require('./lib/auth0');
const authenticate = require('./operations/authentication-operations');
const { findAccount, findAccountFromArgs } = require('./operations/multi-tenancy-operation');
const HttpError = require('./utils/errors');
const { sendAuditGroupRecord } = require('./operations/audit-operations');
const { setSendAudit } = require('./utils/async-local-storage');
// override default break length which is 80
util.inspect.defaultOptions.breakLength = Infinity;

const app = express();
const logger = getLogger();
const graphqlLogger = getLogger('graphql');
const port = config.get('app-port');

process.on('unhandledRejection', (reason, p) => {
    // p is the promise that was rejected
    // reason is the reason the promise was rejected
    logger.info('Unhandled Rejection at:', p, 'reason:', reason);
});

process.on('uncaughtException', err => logger.info('Uncaught exception was thrown', err));

async function connectToMongo() {
    try {
        logger.info('Connecting to DB');
        await mongoose.connect(DB_CONNECTION_STRING, {
            useUnifiedTopology: true,
            useNewUrlParser: true,
            dbName: 'cloudTiering',
            poolSize: 100,
            useCreateIndex: true,
            useFindAndModify: false
        });
        logger.info('Connected to db');
        const db = mongoose.connection;

        // for debug
        db.on('error', error => logger.error('MONGO error', error))
            .on('disconnected', () => logger.error('MONGO disconnected'))
            .on('reconnectFailed', () => logger.error('MONGO reconnect failed'));

        // migrate
        const migrated = await up(db);
        migrated.forEach(fileName => logger.info('Migrated:', fileName));
        return db;
    } catch (err) {
        logger.error('Failed connecting to mongo', err);
        process.exit(-1); // eslint-disable-line no-process-exit
    }
}

function findFieldInADocumentObject(object, field) {
    if (object?.name?.value === field) {
        return object?.value?.value;
    }
    if (object?.value?.fields) {
        return object?.value?.fields.flatMap(fieldObj => findFieldInADocumentObject(fieldObj, field));
    }
}

function findFieldInAVariablesObject(object, field) {
    if (object[field]) {
        return [object[field]];
    }
    return Object.values(object).filter(value => isObject(value)).flatMap(obj => findFieldInAVariablesObject(obj, field));
}

async function startServer() {
    const httpServer = http.createServer(app);

    await Promise.all([createServiceToken()]);

    const db = await connectToMongo();

    // configure security helmet
    app.use(helmet({
        frameguard: false,
        contentSecurityPolicy: false
    }));
    app.use(compression());

    app.use(cors());

    // configure static assets middleware
    const staticAssetsMiddleware = express.static(config.get('paths.ui'));
    app.use(staticAssetsMiddleware);

    // always return index.html in case no resource found and it's not an API call
    app.use(reactRouterFallbackMiddleware());

    app.use(express.json());// json body-parser
    app.use(express.urlencoded({ extended: true }));// urlencoded body-parser qs module instead of query-string- assumption we will need a deep object

    app.use(continuationLocalStorageMiddleware);

    app.use(API_ROOT, healthRouter);
    app.use(API_ROOT, saasSubscriptionWebHookRouter);

    if (process.env.NODE_ENV !== 'production') {
        app.all(/api\/test.*/, async (req, res, next) => {
            const { headers, method } = req;

            const { authorization } = headers;

            try {
                if (!headers.forwardurl) {
                    const data = await got.post('https://127.0.0.1:8888/simulator/change-scenario', {
                        json: req.body,
                        headers: { authorization: authorization || '' },
                        resolveBodyOnly: true,
                        responseType: 'json'
                    });
                    return res.json(data);
                }
                const headersToSend = headers['x-user-token'] ? { authorization, 'x-user-token': headers['x-user-token'] } : { authorization };
                if (isEmpty(req.body)) { //  req.body is empty
                    const data = await got[method.toLowerCase()](headers.forwardurl, {
                        headers: headersToSend,
                        resolveBodyOnly: true,
                        responseType: 'json'
                    });
                    return res.json(data);
                }
                const data = await got[method.toLowerCase()](headers.forwardurl, {
                    json: req.body,
                    headers: headersToSend,
                    resolveBodyOnly: true,
                    responseType: 'json'
                });
                res.json(data);
            } catch (err) {
                next(err);
            }
        });
    }

    await initAgenda(db);

    app.use(authenticationMiddleware);
    app.use(findAccountAndUpdateContinuationToken);

    if (process.env.IS_SIMULATOR) {
        app.use(API_ROOT, require('./routes/simulator')); // eslint-disable-line global-require
    }

    app.use(API_ROOT, statisticsRouter);
    app.use(API_ROOT, internalRoute);

    const apollo = new ApolloServer({
        typeDefs,
        resolvers,
        requireResolversForResolveType: false,
        formatError: (error => {
            if (error.extensions?.code === 'INTERNAL_SERVER_ERROR') {
                error.extensions.code = error.extensions.exception?.status;
                if (error.extensions.code === 500) {
                    delete error.message;
                }
                error.extensions.message = error.message;
            }
            return error;
        }),
        context: (async ({ req, connection }) => {
            if (connection) {
                return connection.context;
            }
            if (!(req.body && req.body.operationName)) {
                const authorizationToken = req.headers.Authorization || req.headers.authorization || '';
                await authenticate(authorizationToken, req);
                const headerAccountId = req.headers['x-account-id'];
                const accountId = headerAccountId ? await findAccount(headerAccountId) : undefined;
                setAccountId(accountId);
                return { accountId };
            }
        }),
        subscriptions: {
            keepAlive: 10000,
            onConnect: async connectionParams => {
                const { authorization } = connectionParams;
                if (!authorization || authorization === '') {
                    logger.error('No token was provided on webSocket connection - unauthorized request', authorization);
                    throw new HttpError('UNAUTHORIZED', 401);
                }
                const payload = await jwtVerify(authorization, JWR_TOKEN_TENANCY_AUDIENCE);
                return { authorization, userPortalId: payload.sub, userName: payload['http://cloud.netapp.com/full_name'] };
            },
            onOperation: (message, params) => {
                logger.info('<<<<<<<<<<<<<<message', message, params);
            },
            onDisconnect: () => {
                logger.info('Disconnect websocket operation happened');
            }
        },
        mocks: false, // for mocks responses until the simulator will work
        introspection: true, // for playground in production mode
        playground: {
            settings: {
                'editor.theme': 'dark',
                'editor.reuseHeaders': true
            },
            tabs: [{
                endpoint: '/graphql',
                query: '{clusters { clusterId }}',
                name: 'query clusters',
                headers: { authorization: 'user-token' }
            }]
        },
        plugins: [() => ({
            requestDidStart: () => ({
                didResolveSource(requestContext) {
                    const { request: { query, variables } } = requestContext;
                    graphqlLogger.info('GraphQL request', JSON.stringify({
                        queryString: query.replace(/(\r\n|\n|\r)/gm, ''),
                        variables
                    }));
                    if (requestContext?.operation?.operation === 'mutation' && process.env.AUDIT_ACCESSKEY) {
                        setSendAudit(true);
                    }
                },
                async didResolveOperation(requestContext) {
                    // parse in order to find accountId - must be here after the server did parsing to a document
                    const name = requestContext.document.definitions[0].selectionSet.selections[0].name.value;
                    let accountIdParam;
                    if (name !== 'accounts') {
                        const variables = requestContext?.request?.variables;
                        accountIdParam = findFieldInAVariablesObject(variables, 'accountId').flat().find(obj => obj);
                        if (!accountIdParam) {
                            const requestArguments = requestContext.document.definitions[0].selectionSet.selections[0].arguments;
                            accountIdParam = requestArguments.map(arg => findFieldInADocumentObject(arg, 'accountId')).flat().find(obj => obj);
                        }
                        if (accountIdParam) {
                            await findAccountFromArgs(accountIdParam);
                        }
                    }
                },
                async executionDidStart(requestContext) {
                    if (get(requestContext, 'operation.operation') === 'mutation' && process.env.AUDIT_ACCESSKEY) {
                        await sendAuditGroupRecord(requestContext, true);
                        setSendAudit(true);
                    }
                },
                async willSendResponse(requestContext) { // send end audit here
                    graphqlLogger.info('GraphQL response', JSON.stringify(requestContext.response));
                    if (get(requestContext, 'operation.operation') === 'mutation' && process.env.AUDIT_ACCESSKEY) {
                        await sendAuditGroupRecord(requestContext, false);
                    }
                }
            })
        })]
    });

    apollo.installSubscriptionHandlers(httpServer);

    apollo.applyMiddleware({ app });

    httpServer.listen({ port }, () => {
        logger.info('🚀 Server ready at',
            `http://local.cloudtiering.io:${port}${apollo.graphqlPath}`);
    });
}

startServer();
