const Joi = require('@hapi/joi');
const {
    STANDARD, ARCHIVE, COLDLINE, NEARLINE, MULTI_REGION, REGIONAL, STANDARD_AWS, INTELLIGENT, ONE_ZONE_IA, STANDARD_IA,
    ALL, NONE, AUTO, SNAPSHOT_ONLY, BACKUP, SGWS, CLOUD_PROVIDER_TO_OCCM_PROVIDER, GCP, AWS, AZURE
} = require('../utils/consts');

/** *******************************************
 *          Response validations             *
 ******************************************** */

// TODO use one schema for AWS/GCP bucket response
const gcpBucket = Joi.object().keys({
    name: Joi.string().required(),
    region: Joi.string().required(),
    // storageClass: Joi.string().required()
    storageClass: Joi.string().required().valid(MULTI_REGION, REGIONAL, STANDARD, ARCHIVE, COLDLINE, NEARLINE)
});
const awsBucket = Joi.object().keys({
    name: Joi.string().required(),
    region: Joi.string().required()
});
const sgwsBucket = Joi.object().keys({
    name: Joi.string().required()
});
const azureSubscription = Joi.object().keys({
    subscriptionId: Joi.string().required(),
    displayName: Joi.string().required(),
    id: Joi.string().required(),
    state: Joi.string().required()
});
const azureRegion = Joi.object().keys({
    name: Joi.string().required(),
    displayName: Joi.string().required()
});
const azureStorageAccount = Joi.object().keys({
    name: Joi.string().required(),
    location: Joi.string().required(),
    accessTier: Joi.string(),
    kind: Joi.string().required()
});
const azureStorageAccountKey = Joi.object().keys({
    keyName: Joi.string().required(),
    value: Joi.string().required(),
    permissions: Joi.string()
});

const azureResourceGroupSchema = Joi.object().keys({
    name: Joi.string().required()
});

const nodeSystemImage = Joi.object().keys({
    image: Joi.string().required(),
    isCurrent: Joi.boolean().required(),
    isDefault: Joi.boolean().required(),
    version: Joi.string()
});

const objectStore = Joi.object().keys({
    usedSpace: Joi.number().required(),
    name: Joi.string().required(),
    availabilityState: Joi.string().required(),
    aggregateName: Joi.string().required(),
    tieringFullnessThreshold: Joi.number()
});
const aggregateSchema = Joi.object().keys({
    name: Joi.string().required(),
    state: Joi.string(),
    isRoot: Joi.boolean().allow(''),
    isComposite: Joi.boolean(),
    isInactiveDataReportingEnabled: Joi.boolean(),
    isObjectStoreAttachEligible: Joi.boolean(),
    totalCapacity: Joi.number(),
    usedCapacity: Joi.number(),
    tierInactiveUserData: Joi.number(),
    tierInactiveUserDataPercent: Joi.number()
});
const svm = Joi.object().keys({
    svmName: Joi.string(),
    svmType: Joi.string(),
    svmState: Joi.string(),
    svmSubType: Joi.string()
});
const node = Joi.object().keys({
    nodeName: Joi.string().required(),
    model: Joi.string(),
    isNodeHealthy: Joi.boolean(),
    isClusterEligible: Joi.boolean()
});
const volumeSchema = Joi.object().keys({
    name: Joi.string(),
    svmName: Joi.string(),
    usedSize: Joi.number(),
    sizeTotal: Joi.number(),
    state: Joi.string(),
    isVserverRoot: Joi.boolean(),
    aggregateName: Joi.string(),
    volumeType: Joi.string(),
    tieringPolicy: Joi.string().allow(null),
    tierInactiveUserData: Joi.number().allow(null),
    tierInactiveUserDataPercent: Joi.number().allow(null),
    physicalUsed: Joi.number(),
    flexgroupMSid: Joi.string(),
    flexgroupConstituentIndex: Joi.string(),
    snapshotsUsedSize: Joi.number()
});
const volumeFootprint = Joi.object().keys({
    name: Joi.string(),
    svmName: Joi.string(),
    performanceTierFootprint: Joi.number(),
    capacityTierFootprint: Joi.number()
});
const lif = Joi.object().keys({
    interfaceName: Joi.string(),
    address: Joi.string(),
    status: Joi.string(),
    ipSpace: Joi.string(),
    role: Joi.string().valid('undef', 'cluster_mgmt', 'intercluster', 'node_mgmt', 'data', 'cluster')
});
/** *******************************************
 *          Response validations             *
 ******************************************** */

const regionsResponse = Joi.array().items(Joi.string());
const gcpBucketsResponse = Joi.array().items(gcpBucket);
const awsBucketsResponse = Joi.array().items(awsBucket);
const sgwsBucketsResponse = Joi.array().items(sgwsBucket);
const azureSubscriptionsResponse = Joi.array().items(azureSubscription);
const azureRegionsResponse = Joi.array().items(azureRegion);
const azureStorageAccountsResponse = Joi.array().items(azureStorageAccount);
const azureStorageAccountKeysResponse = Joi.object().keys({ keys: Joi.array().items(azureStorageAccountKey) });
const azureContainersResponse = Joi.array().items(Joi.string());
const azureResourceGroupsResponse = Joi.array().items(azureResourceGroupSchema);
const systemImagesResponse = Joi.array().items(nodeSystemImage);
const objectStoresResponse = Joi.array().items(objectStore);
const aggregatesResponse = Joi.array().items(aggregateSchema);
const svmsResponse = Joi.array().items(svm);
const clusterInfoResponse = Joi.object().keys({
    clusterUuid: Joi.string().required(),
    clusterName: Joi.string().required()
});
const nodesResponse = Joi.array().items(node);
const volumesResponseSchema = Joi.array().items(volumeSchema);
const clusterVolumeFootprintResponse = Joi.array().items(volumeFootprint);
const volumeMinimumCoolingDaysResponse = Joi.object().keys({
    cliOutput: Joi.string()
});
const describeLifsResponse = Joi.array().items(lif);
const describeOntapVersionResponse = Joi.object().keys({
    generation: Joi.string(),
    major: Joi.string(),
    minor: Joi.string()
});
const describeObjectStoreConfigResponse = Joi.array().items(Joi.object().keys({
    objectStoreName: Joi.string(),
    s3Name: Joi.string(),
    usedSpace: Joi.number(),
    server: Joi.string(),
    providerType: Joi.string()
}));

const licenseStatusResponseSchema = Joi.array().items(Joi.object().keys({
    currentCapacity: Joi.number().required(),
    maxCapacity: Joi.number(), // not required because not all license have maxCapacity for example package select
    serialNumber: Joi.string().required(),
    package: Joi.string().required()
}));

/** *******************************************
 *          Request validations               *
 ********************************************* */

const modifyVolumeRequestSchema = Joi.object().keys({ tieringPolicy: Joi.string().valid(AUTO, NONE, SNAPSHOT_ONLY, ALL, BACKUP) });

const modifyVolumeMinimumCoolingDaysRequestSchema = Joi.object().keys({
    volumeName: Joi.string().required(),
    svmName: Joi.string().required(),
    days: Joi.number().required()
});
const modifyTieringFullnessThresholdRequestSchema = Joi.object().keys({
    aggregateName: Joi.string().required(),
    objectStoreName: Joi.string().required(),
    tieringFullnessThreshold: Joi.number().min(1).max(100).required()
});

const enforcementAttributesItem = Joi.object().keys({
    name: Joi.string(),
    metric: Joi.string(),
    msg: Joi.string(),
    operatingPolicy: Joi.string()
});

const applyLicenseRequestSchema = Joi.object().keys({
    statusResp: Joi.object().keys({
        serialNumber: Joi.string().required(),
        version: Joi.string().required(),
        product: Joi.string().required(),
        licenses: Joi.object().keys({
            capacity: Joi.number().required(),
            type: Joi.string().required(),
            package: Joi.string().required(),
            legacyKey: Joi.string(),
            HostID: Joi.string(),
            entitlementLastUpdated: Joi.string(),
            licenseScope: Joi.string(),
            licenseProtocol: Joi.string(),
            enforcementAttributes: Joi.array().items(enforcementAttributesItem),
            endDate: Joi.string(),
            startDate: Joi.string()
        }),
        statusCode: Joi.string(),
        message: Joi.string(),
        cmatID: Joi.string(),
        companyBPID: Joi.string(),
        siteBPID: Joi.string(),
        filter: Joi.string(),
        contractStart: Joi.string(),
        contractEnd: Joi.string()
    }),
    trackingId: Joi.string().required(),
    Signature: Joi.string().required()
});

const awsOrGcpKeys = Joi.object().keys({
    accessKey: Joi.string().required(),
    secretKey: Joi.string().required()
});

const createObjectStoreConfigRequestSchema = Joi.object().keys({
    objectStoreName: Joi.string().required(),
    objectStoreProviderType: Joi.string().required(),
    server: Joi.string().required(),
    s3Name: Joi.string().required(),
    ipSpace: Joi.string().required(),
    keys: Joi.when('objectStoreProviderType', {
        is: [CLOUD_PROVIDER_TO_OCCM_PROVIDER[GCP],
            CLOUD_PROVIDER_TO_OCCM_PROVIDER[SGWS],
            CLOUD_PROVIDER_TO_OCCM_PROVIDER[AWS]],
        then: awsOrGcpKeys
    }),
    port: Joi.when('objectStoreProviderType', {
        is: CLOUD_PROVIDER_TO_OCCM_PROVIDER[SGWS],
        then: Joi.number()
    }),
    checkOnly: Joi.boolean().required(),
    azureAccount: Joi.when('objectStoreProviderType', {
        is: [CLOUD_PROVIDER_TO_OCCM_PROVIDER[AZURE]],
        then: Joi.string().required()
    }),
    azurePrivateKey: Joi.when('objectStoreProviderType', {
        is: [CLOUD_PROVIDER_TO_OCCM_PROVIDER[AZURE]],
        then: Joi.string().required()
    })
});

const enableColdDataReportingRequestSchema = Joi.object().keys({
    enable: Joi.boolean().required()
});

const createBucketRequestSchema = Joi.object().keys({
    name: Joi.string().required(),
    region: Joi.string().required()
});
const createSgwsBucketRequestSchema = Joi.object().keys({
    name: Joi.string().required(),
    host: Joi.string().required(),
    port: Joi.number()
});

const putSgwsObjectRequestSchema = Joi.object().keys({
    host: Joi.string().required(),
    port: Joi.number(),
    keyBody: Joi.string().required().allow('')
});

const createContainerRequestSchema = Joi.object().keys({
    storageAccount: Joi.string().required(),
    storageAccountKey: Joi.string().required(),
    containerName: Joi.string().required()
});
const setGcpStorageClassRequestSchema = Joi.object().keys({
    bucket: Joi.string().required(),
    storageClass: Joi.string().required().valid(STANDARD, ARCHIVE, COLDLINE, NEARLINE)
});
const setAwsStorageClassRequestSchema = Joi.object().keys({
    bucket: Joi.string().required(),
    storageClass: Joi.string().required().valid(STANDARD_AWS, INTELLIGENT, ONE_ZONE_IA, STANDARD_IA)
});
const setAzureStorageClassRequestSchema = Joi.object().keys({
    resourceGroup: Joi.string().required(),
    storageAccount: Joi.string().required(),
    accessTier: Joi.string().required()
});
const attachObjectStoreSchema = Joi.object().keys({
    aggregateName: Joi.string().required(),
    objectStoreName: Joi.string().required().allow(''),
    checkOnly: Joi.boolean().required(),
    allowFlexgroup: Joi.boolean().required()
});

module.exports = {
    setAwsStorageClassRequestSchema,
    setGcpStorageClassRequestSchema,
    regionsResponse,
    gcpBucketsResponse,
    createBucketRequestSchema,
    gcpBucket,
    setAzureStorageClassRequestSchema,
    awsBucket,
    azureStorageAccountsResponse,
    azureContainersResponse,
    awsBucketsResponse,
    azureSubscriptionsResponse,
    azureRegionsResponse,
    azureStorageAccountKeysResponse,
    createContainerRequestSchema,
    nodesResponse,
    systemImagesResponse,
    svmsResponse,
    clusterInfoResponse,
    attachObjectStoreSchema,
    objectStoresResponse,
    volumeMinimumCoolingDaysResponse,
    aggregatesResponse,
    volumesResponseSchema,
    describeOntapVersionResponse,
    clusterVolumeFootprintResponse,
    describeLifsResponse,
    describeObjectStoreConfigResponse,
    modifyVolumeRequestSchema,
    enableColdDataReportingRequestSchema,
    modifyVolumeMinimumCoolingDaysRequestSchema,
    applyLicenseRequestSchema,
    licenseStatusResponseSchema,
    sgwsBucketsResponse,
    aggregateSchema,
    createSgwsBucketRequestSchema,
    sgwsBucket,
    volumeSchema,
    putSgwsObjectRequestSchema,
    createObjectStoreConfigRequestSchema,
    modifyTieringFullnessThresholdRequestSchema,
    azureResourceGroupsResponse
};
