const { escape } = require('querystring');
const {
    AZURE, STORAGE_CLASSES_NEW_AGENT,
    AGENT_LIST_AZURE_STORAGE_ACCOUNT_KEYS, AGENT_LIST_AZURE_CONTAINERS, AGENT_CHANGE_AZURE_STORAGE_ACCOUNT_TIER_LEVEL,
    AGENT_LIST_AZURE_REGIONS, AGENT_LIST_AZURE_ALL_STORAGE_ACCOUNTS, AGENT_LIST_AZURE_RESOURCE_GROUPS, AGENT_LIST_AZURE_STORAGE_ACCOUNTS
} = require('../../utils/consts');
const {
    createContainerRequestSchema,
    azureContainersResponse, setAzureStorageClassRequestSchema,
    azureStorageAccountsResponse, azureRegionsResponse, azureStorageAccountKeysResponse, azureResourceGroupsResponse
} = require('../../joi-schemas/agent-schemas');
const getLogger = require('../../utils/logger/logger');
const { gotInstanceForAgent } = require('../../utils/got-utils');
const { validateResponse, validateRequest } = require('../../utils/schema-validation');
const { createAgentPath, createHeadersForAgent } = require('../../utils/utils');

const logger = getLogger();

async function getAzureRegions(agentId) {
    const data = await gotInstanceForAgent.get(`${createAgentPath(AGENT_LIST_AZURE_REGIONS)}`, createHeadersForAgent(agentId));
    validateResponse(data, azureRegionsResponse);
    return data;
}

async function getAzureStorageAccounts(agentId, accountId, resourceGroupName) {
    logger.info('Get AZURE storage accounts', {
        agentId, accountId, resourceGroupName
    });
    const path = resourceGroupName ? `${createAgentPath(AGENT_LIST_AZURE_STORAGE_ACCOUNTS, accountId)}?resourceGroup=${resourceGroupName}` : `${createAgentPath(AGENT_LIST_AZURE_ALL_STORAGE_ACCOUNTS, accountId)}`;
    const data = await gotInstanceForAgent.get(path, createHeadersForAgent(agentId));
    validateResponse(data, azureStorageAccountsResponse);
    return data;
}

async function getAzureStorageAccountKeys(agentId, storageAccount, resourceGroup) {
    logger.info('Get AZURE storage accounts keys', {
        agentId, storageAccount, resourceGroup
    });
    const data = await gotInstanceForAgent.get(`${createAgentPath(AGENT_LIST_AZURE_STORAGE_ACCOUNT_KEYS)}?storageAccount=${storageAccount}&resourceGroup=${resourceGroup}`, createHeadersForAgent(agentId));
    validateResponse(data, azureStorageAccountKeysResponse);
    return data;
}

async function getAzureContainers(agentId, storageAccount, storageAccountKey) {
    logger.info('Get AZURE containers', { agentId, storageAccount, storageAccountKey });
    const data = await gotInstanceForAgent.get(`${createAgentPath(AGENT_LIST_AZURE_CONTAINERS)}?storageAccount=${storageAccount}&storageAccountKey=${escape(storageAccountKey)}`, createHeadersForAgent(agentId));
    validateResponse(data, azureContainersResponse);
    return data;
}

async function getAzureResourceGroups(agentId, accountId) {
    logger.info('Get AZURE resource groups', { agentId, accountId });
    const data = await gotInstanceForAgent.get(`${createAgentPath(AGENT_LIST_AZURE_RESOURCE_GROUPS, accountId)}`, createHeadersForAgent(agentId));
    validateResponse(data, azureResourceGroupsResponse);
    return data;
}

async function createAzureContainer(agentId, containerName, storageAccount, storageAccountKey) {
    logger.info('Create azure container', {
        agentId, storageAccount, storageAccountKey, containerName
    });
    const body = { storageAccount, storageAccountKey, containerName };
    validateRequest(body, createContainerRequestSchema);
    await gotInstanceForAgent.post(`${createAgentPath(AGENT_LIST_AZURE_CONTAINERS)}`, { json: body, ...createHeadersForAgent(agentId) });
}

async function setAzureStorageAccountTierLevel(agentId, resourceGroup, storageAccount, tierLevel) {
    logger.info('Change storage account tier level', {
        resourceGroup, storageAccount, tierLevel
    });
    const accessTier = STORAGE_CLASSES_NEW_AGENT[AZURE][tierLevel];
    const body = {
        resourceGroup, storageAccount, accessTier
    };
    validateRequest(body, setAzureStorageClassRequestSchema);
    return gotInstanceForAgent.put(`${createAgentPath(AGENT_CHANGE_AZURE_STORAGE_ACCOUNT_TIER_LEVEL)}`, {
        json: body,
        ...createHeadersForAgent(agentId)
    });
}

module.exports = {
    setAzureStorageAccountTierLevel,
    getAzureRegions,
    getAzureStorageAccounts,
    getAzureStorageAccountKeys,
    createAzureContainer,
    getAzureContainers,
    getAzureResourceGroups
};
