const { ApolloError } = require('apollo-server');
const AWS = require('aws-sdk');
const { GCP } = require('../../utils/consts');
const getLogger = require('../../utils/logger/logger');

const logger = getLogger();

const accessDenied = 'AccessDenied';

function getS3(provider, region, accessKeyId, secretAccessKey) {
    logger.info('Get S3', {
        provider, region, accessKeyId, secretAccessKey
    });
    return new AWS.S3({
        region, accessKeyId, secretAccessKey, endpoint: provider === GCP ? 'storage.googleapis.com' : undefined
    });
}

async function putObject(s3, bucket, key, body = 'cloudTiering test') {
    try {
        await s3.putObject({ Bucket: bucket, Key: key, Body: body }).promise();
    } catch (err) {
        logger.error(`Failed to PutObject operation on bucket ${bucket}`, err);
        if (err.code === accessDenied) {
            throw new ApolloError('No permission for PutObject operation', 403);
        }
        throw err;
    }
}

async function getObject(s3, bucket, key) {
    try {
        return await s3.getObject({ Bucket: bucket, Key: key }).promise();
    } catch (err) {
        logger.error(`Failed to commit GetObject operation on bucket ${bucket}`, err);
        if (err.code === accessDenied) {
            throw new ApolloError('No permission for GetObject operation', 403);
        }
        throw err;
    }
}

async function deleteObject(s3, bucket, key) {
    try {
        await s3.deleteObject({ Bucket: bucket, Key: key }).promise();
    } catch (err) {
        logger.error(`Failed to commit DeleteObject operation on bucket ${bucket}`, err);
        if (err.code === accessDenied) {
            throw new ApolloError('No permission for DeleteObject operation', 403);
        }
        throw err;
    }
}

async function upload(s3, bucket, key, bodyStream) {
    const params = {
        Bucket: bucket,
        Key: key,
        Body: bodyStream
    };
    return new AWS.S3.ManagedUpload({
        params,
        leavePartsOnError: false,
        service: s3
    }).promise();
}

module.exports = {
    getS3, deleteObject, getObject, putObject, upload
};
