const config = require('config');
const mongoose = require('mongoose');

const nssCredentialsSchema = new mongoose.Schema({
    nssUserName: { type: String, required: false },
    nssPassword: { type: String, required: false }
}, { _id: false });

const accountSchema = new mongoose.Schema({
    accountId: { type: String, required: true, index: { unique: true } },
    freeTrialExpiryDate: { type: Date, required: false },
    automaticExtension: { type: Boolean, required: false, default: false },
    nssCredentials: { type: nssCredentialsSchema, required: false }
}, {
    timestamps: true,
    versionKey: false
});

const Account = mongoose.model('Account', accountSchema);

async function findClustersToIncreaseLicense() {
    return Account.aggregate([
        {
            $match: {
                freeTrialExpiryDate: { $exists: true }
            }
        },
        {

            $lookup: {
                from: 'clusters',
                localField: 'accountId',
                foreignField: 'accountId',
                as: 'cluster'
            }

        },
        { $unwind: '$cluster' },
        {
            $match: {
                $and: [{ $or: [{ freeTrialExpiryDate: { $gt: new Date() } }, { 'cluster.subscriptionId': { $exists: true } }] },
                    { 'cluster.cloudProvider': { $exists: true } }, { 'cluster.clusterUuid': { $exists: true } }]
            }
        },
        {
            $project: {
                tieringCapacity: { $ifNull: ['$cluster.tieringLicense.maxCapacity', 0] },
                byolCapacity: { $ifNull: ['$cluster.byolLicense.maxCapacity', 0] },
                accountId: 1,
                agentId: '$cluster.agentId',
                clusterId: '$cluster.clusterId',
                currentCapacity: '$cluster.currentCapacity'
            }
        }, {
            $project: {
                maxCapacity: { $max: ['$byolCapacity', '$tieringCapacity'] },
                accountId: 1,
                clusterId: 1,
                agentId: 1,
                currentCapacity: 1
            }
        },
        {
            $project: {
                maxCapacity: 1,
                currentCapacity: 1,
                licenseCapacityRatio: { $cond: [{ $gt: ['$maxCapacity', 0] }, { $divide: ['$currentCapacity', '$maxCapacity'] }, 0] },
                accountId: 1,
                agentId: 1,
                clusterId: 1,
                _id: 0
            }
        },
        {
            $match: { licenseCapacityRatio: { $gte: config.get('increase-license-capacity-ratio') } }
        }
    ]);
}

module.exports = { Account, findClustersToIncreaseLicense };
