const mongoose = require('mongoose');
const { SGWS } = require('../utils/consts');
const getLogger = require('../utils/logger/logger');

const logger = getLogger();

const licenseSchema = new mongoose.Schema({
    serialNumber: { type: String },
    maxCapacity: { type: Number },
    changedAt: { type: Date, required: false }
}, { _id: false });

// unique : accountId, clusterId
const clusterSchema = new mongoose.Schema({
    accountId: { type: String, required: true },
    agentId: { type: String, required: false },
    clusterId: { type: String, required: true },
    clusterName: { type: String, required: false },
    cloudProvider: { type: String, required: false },
    clusterUuid: { type: String, require: false },
    byolLicense: { type: licenseSchema, required: false },
    tieringLicense: { type: licenseSchema, required: false },
    subscriptionId: { type: String, required: false },
    currentCapacity: { type: Number, default: 0 },
    deleted: { type: Boolean, default: false },
    lastSeen: { type: Date, required: true },
    agentLocation: { type: String, require: false },
    isEligibleForTiering: { type: Boolean },
    ontapVersion: { type: String },
    sendNotification: { type: Boolean, default: false }
}, {
    timestamps: true,
    versionKey: false
});

const Cluster = mongoose.model('Cluster', clusterSchema);

clusterSchema.index({
    accountId: 1, clusterId: 1
}, { unique: true });

async function collectChargingInfoFromDB() {
    return Cluster.aggregate([
        { $match: { subscriptionId: { $exists: true }, cloudProvider: { $exists: true, $nin: [SGWS] }, clusterUuid: { $exists: true } } },
        {
            $lookup: {
                from: 'accounts',
                localField: 'accountId',
                foreignField: 'accountId',
                as: 'account'
            }
        },
        { $unwind: '$account' },
        { $match: { 'account.freeTrialExpiryDate': { $lt: new Date() } } },
        {
            $project: {
                subscriptionId: 1,
                accountId: 1,
                clusterUuid: 1,
                currentCapacity: 1,
                lastSeen: 1,
                byolCapacity: { $ifNull: ['$byolLicense.maxCapacity', 0] }
            }
        },
        {
            $project: {
                subscriptionId: 1,
                accountId: 1,
                clusterUuid: 1,
                lastSeen: 1,
                chargedCapacity: { $subtract: ['$currentCapacity', '$byolCapacity'] }
            }
        }, { $match: { chargedCapacity: { $gt: 0 } } },
        { $sort: { lastSeen: -1 } },
        {
            $group: {
                _id: '$clusterUuid',
                chargedCapacity: { $first: '$chargedCapacity' },
                subscriptionId: { $first: '$subscriptionId' },
                accountId: { $first: '$accountId' }
            }
        },
        {
            $group: {
                _id: '$subscriptionId',
                accountsAndClusters: {
                    $push: {
                        accountId: '$accountId',
                        resourceId: '$_id',
                        val: '$chargedCapacity'
                    }
                }
            }
        },
        {
            $project: {
                subscriptionId: '$_id',
                _id: 0,
                accountsAndClusters: 1
            }
        }
    ]);
}

async function findLicenseInfoOfCluster(clusterId, accountId) {
    // comment- canceling enforcement
    const result = await Cluster.aggregate([
        { $match: { clusterId, accountId } },
        {
            $lookup: {
                from: 'accounts',
                localField: 'accountId',
                foreignField: 'accountId',
                as: 'account'
            }
        },
        { $unwind: '$account' },
        {
            $project: {
                clusterId: 1,
                clusterUuid: 1,
                currentCapacity: 1,
                freeTrialExpiryDate: '$account.freeTrialExpiryDate',
                provider: { $cond: { if: { $not: ['$cloudProvider'] }, then: '$agentLocation', else: '$cloudProvider' } },
                hasBYOL: { $cond: { if: { $not: ['$byolLicense'] }, then: false, else: true } },
                freeTrialStatus: {
                    $cond: {
                        if: { $not: ['$account.freeTrialExpiryDate'] },
                        then: 'PRE',
                        else: {
                            $cond: {
                                if: { $lte: [new Date(), '$account.freeTrialExpiryDate'] },
                                then: 'STARTED',
                                else: 'EXPIRED'
                            }
                        }

                    }
                },
                byolCapacity: { $ifNull: ['$byolLicense.maxCapacity', 0] },
                subscriptionStatus: {
                    $cond: {
                        if: { $not: ['$subscriptionId'] },
                        then: 'unsubscribed',
                        else: 'subscribed'
                    }
                }
            }
        },
        {
            $project: {
                clusterId: 1,
                clusterUuid: 1,
                licensedCapacity: { $subtract: ['$byolCapacity', '$currentCapacity'] },
                freeTrialStatus: 1,
                hasBYOL: 1,
                subscriptionStatus: 1,
                provider: { $cond: { if: { $eq: ['$provider', 'ONPREM'] }, then: 'AWS', else: '$provider' } },
                byolCapacity: 1,
                currentCapacity: 1
            }
        },
        {
            $project: {
                shouldEnforce: {
                    $cond: {
                        if: {
                            $not: [{
                                $or: [{ $eq: ['$subscriptionStatus', 'subscribed'] }, { $in: ['$freeTrialStatus', ['STARTED', 'PRE']] },
                                    { $gt: ['$licensedCapacity', 0] }]
                            }]
                        },
                        then: true,
                        else: false

                    }
                },
                freeTrialStatus: 1,
                hasBYOL: 1,
                subscriptionStatus: 1,
                currentCapacity: 1,
                provider: 1,
                byolCapacity: 1,
                _id: 0
            }
        }
    ]);
    logger.info('licenseInfo of cluster', { clusterId, accountId }, result);
    return result[0];
}

module.exports = {
    Cluster, collectChargingInfoFromDB, findLicenseInfoOfCluster
};
