const { ApolloError } = require('apollo-server');
const {
    round, get, isNil, groupBy
} = require('lodash');
const {
    SAVING_OPPORTUNITY_ESTIMATION_RATIO, ON_STATUS, ONTAP_VERSIONS_FEATURES_SUPPORT
} = require('../utils/consts');
const aggregatesPropertiesSchema = require('../joi-schemas/aggregates-schemas');
const { validateSchema } = require('../utils/schema-validation');
const { getAggregatesByAggregateNames, getAggregates, getOntapVersion } = require('./agent-zapi-operations');
const { getAgentIdByClusterId } = require('./multi-tenancy-operation');
const { modifyTieringFullnessThreshold, enableColdDataReporting } = require('../lib/agent/zapi');
const { getAccountId } = require('../utils/async-local-storage');

const getLogger = require('../utils/logger/logger');

const logger = getLogger();

function isChangeIdrSupported(ontapVersion, objectStoreName) {
    if (ONTAP_VERSIONS_FEATURES_SUPPORT.ENABLE_COLD_DATA_REPORTING > ontapVersion) {
        return { idrState: 'not_supported', isSupported: false, reason: `Inactive data reporting (IDR) is not supported in ${ontapVersion} ONTAP version` };
    } if (objectStoreName) {
        return { idrState: 'read_only', isSupported: false, reason: 'Inactive data reporting (IDR) must remain enabled because data tiering is enabled on this aggregate' };
    }
    return { isSupported: true, idrState: 'editable' };
}

function isChangeTieringFullnessThersholdSupported(ontapVersion, objectStoreName) {
    if (ONTAP_VERSIONS_FEATURES_SUPPORT.TIERING_FULLNESS_THRESHOLD > ontapVersion) {
        return { tieringFullnessThresholdState: 'not_supported', isSupported: false, reason: `Tiering fullness threshold is not supported in ${ontapVersion} ONTAP version` };
    } if (!objectStoreName) {
        return { tieringFullnessThresholdState: 'not_supported', isSupported: false, reason: 'Tiering fullness threshold is not supported with aggregates that aren’t enabled for tiering' };
    }
    return { tieringFullnessThresholdState: 'editable', isSupported: true };
}

function getAggregateSupportedSettings(ontapVersion, objectStoreName) {
    // should return null if there is reason
    const isIdrSupported = isChangeIdrSupported(ontapVersion, objectStoreName);
    const isTieringFullnessThersholdSupported = isChangeTieringFullnessThersholdSupported(ontapVersion, objectStoreName);
    if (isIdrSupported.isSupported) {
        if (isTieringFullnessThersholdSupported.isSupported) {
            return {
                idrState: isIdrSupported.idrState,
                idr: ON_STATUS,
                tieringFullnessThresholdState: isTieringFullnessThersholdSupported.tieringFullnessThresholdState,
                tieringFullnessThreshold: 50
            };
        }
        return {
            idrState: isIdrSupported.idrState,
            idr: ON_STATUS,
            tieringFullnessThresholdState: isTieringFullnessThersholdSupported.tieringFullnessThresholdState,
            noTieringFullnessThreshold: isTieringFullnessThersholdSupported.reason
        };
    }
    if (isTieringFullnessThersholdSupported.isSupported) {
        return {
            idrState: isIdrSupported.idrState,
            tieringFullnessThresholdState: isTieringFullnessThersholdSupported.tieringFullnessThresholdState,
            noIdrReason: isIdrSupported.reason,
            tieringFullnessThreshold: 50
        };
    }
    return {
        idrState: isIdrSupported.idrState,
        tieringFullnessThresholdState: isTieringFullnessThersholdSupported.tieringFullnessThresholdState,
        noIdrReason: isIdrSupported.reason,
        noTieringFullnessThreshold: isTieringFullnessThersholdSupported.reason
    };
}

function adjustAggregateResponse(aggregate, ontapVersion) {
    const {
        name,
        totalCapacity: capacity, objectStoreName,
        isInactiveDataReportingEnabled,
        tieredCapacity, tieringRestriction,
        tieringFullnessThreshold, tierInactiveUserDataPercent, isComposite
    } = aggregate;
    const supportedSettings = getAggregateSupportedSettings(ontapVersion, objectStoreName);
    const usedPerformanceCapacity = get(aggregate, 'usedCapacity');
    const inactiveCapacityFromOccm = get(aggregate, 'tierInactiveUserData');
    const inactiveCapacity = isNil(inactiveCapacityFromOccm) ? round(usedPerformanceCapacity * SAVING_OPPORTUNITY_ESTIMATION_RATIO) : (inactiveCapacityFromOccm || 0);
    const idr = isInactiveDataReportingEnabled || objectStoreName ? ON_STATUS : 'OFF';
    const isEligibleForTiering = isComposite || !tieringRestriction.isTieringRestricted;
    const tieringRestrictionReason = isEligibleForTiering ? null : tieringRestriction.restrictionReason;
    const inactiveCapacityPercent = isNil(inactiveCapacityFromOccm) ? SAVING_OPPORTUNITY_ESTIMATION_RATIO * 100 : tierInactiveUserDataPercent;
    const isEstimatedInactiveCapacity = isNil(inactiveCapacityFromOccm);
    return {
        name,
        idr,
        capacity,
        tieredCapacity,
        tieringFullnessThreshold,
        supportedSettings,
        isEligibleForTiering,
        tieringRestrictionReason,
        inactiveCapacityPercent,
        isEstimatedInactiveCapacity,
        usedPerformanceCapacity,
        inactiveCapacity
    };
}

async function getAggregatesOperation(clusterId, agentIdParam) {
    const agentId = agentIdParam ?? await getAgentIdByClusterId(clusterId, getAccountId());
    const aggregates = await getAggregates(agentId, clusterId);
    const ontapVersion = await getOntapVersion(agentId, clusterId);
    return aggregates.map(aggregate => adjustAggregateResponse(aggregate, ontapVersion));
}

function verifyModifyAggregatesParams(clusterId, aggregatesProperties) {
    validateSchema(aggregatesProperties, aggregatesPropertiesSchema);
    const aggregatesByName = groupBy(aggregatesProperties, 'name');
    const duplicateAggregates = Object.values(aggregatesByName).filter(aggregateArr => aggregateArr.length > 1);
    if (duplicateAggregates.length) {
        logger.error('Received the same aggregate twice', JSON.stringify(duplicateAggregates));
        throw new ApolloError('You entered the same aggregate twice please try again', 400);
    }
}

async function modifyIdr(agentId, clusterId, aggregates, ontapVersion) {
    const errors = [];
    const data = [];
    await Promise.all(aggregates.map(async aggregate => {
        const {
            name, requestedIdr, objectStoreName, isInactiveDataReportingEnabled
        } = aggregate;
        try {
            const { isSupported, reason } = isChangeIdrSupported(ontapVersion, objectStoreName);
            if (isSupported) {
                const enable = requestedIdr === ON_STATUS;
                if (enable === isInactiveDataReportingEnabled) {
                    errors.push({ name, reason: `Inactive data reporting (IDR) is already ${requestedIdr}` });
                } else {
                    await enableColdDataReporting(agentId, clusterId, name, enable);
                    data.push({ name, idr: requestedIdr });
                }
            } else {
                errors.push({ name, reason });
            }
        } catch (err) {
            logger.error('Failed to enableColdData reporting', err);
            errors.push({ name, reason: `Failed to update Inactive Data Reporting (IDR) on aggregate ${name}` });
        }
    }));
    return { errors, data };
}

async function modifyTieringFullnessThresholdOperation(agentId, clusterId, aggregates, ontapVersion) {
    const errors = [];
    const data = [];
    await Promise.all(aggregates.map(async aggregate => {
        const {
            name, requestedTieringFullnessThreshold, objectStoreName
        } = aggregate;
        try {
            const { isSupported, reason } = isChangeTieringFullnessThersholdSupported(ontapVersion, objectStoreName);
            if (isSupported) {
                await modifyTieringFullnessThreshold(agentId, clusterId, name, objectStoreName, requestedTieringFullnessThreshold);
                data.push({ name, tieringFullnessThreshold: requestedTieringFullnessThreshold });
            } else {
                errors.push({ name, reason });
            }
        } catch (err) {
            logger.error('Failed to enableColdData reporting', err);
            errors.push({ name, reason: err.message });
        }
    }));
    return { errors, data };
}

async function modifyAggregatesOperation(clusterId, aggregatesProperties, agentIdParam) {
    // #1 validation
    // #2 get aggregates with filter
    // #3 add to each received aggregate the requested fields + seperate
    // #4 change idr
    // #5 change fullness threshold
    const accountId = getAccountId();
    const agentId = agentIdParam || await getAgentIdByClusterId(clusterId, accountId);
    verifyModifyAggregatesParams(clusterId, aggregatesProperties);
    const [ontapVersion, aggregates] = await Promise.all([
        getOntapVersion(agentId, clusterId, accountId, true),
        getAggregatesByAggregateNames(agentId, clusterId, aggregatesProperties.map(aggregate => aggregate.name), accountId, true)
    ]);
    const adjustedAggregateProperties = aggregatesProperties.map(aggregate => {
        const { name, idr, tieringFullnessThreshold } = aggregate;
        return { name, requestedIdr: idr, requestedTieringFullnessThreshold: tieringFullnessThreshold };
    });
    const aggregatesFullDetails = [...aggregates.concat(adjustedAggregateProperties).reduce((accumulator, aggr) => accumulator.set(aggr.name, Object.assign(accumulator.get(aggr.name) || {}, aggr)), new Map()).values()];
    const { data: successChangeIdr, errors: errorsChangeIdr } = await modifyIdr(agentId, clusterId, aggregatesFullDetails.filter(aggregate => aggregate.requestedIdr), ontapVersion);
    const { data: successChangeFullnessThreshold, errors: errorsChangeFullnessThreshold } = await modifyTieringFullnessThresholdOperation(agentId, clusterId, aggregatesFullDetails.filter(aggregate => aggregate.requestedTieringFullnessThreshold), ontapVersion, true);
    return {
        errors: errorsChangeIdr.concat(errorsChangeFullnessThreshold),
        data: successChangeIdr.concat(successChangeFullnessThreshold)
    };
}

module.exports = { modifyAggregatesOperation, getAggregatesOperation };
