const {
    get, cloneDeep, isEmpty, isUndefined
} = require('lodash');
const { validateAuditGroup } = require('../utils/schema-validation');
const hideSecretsHelper = require('../utils/hide-secrets');
const putRecord = require('../lib/aws/kinesis');
const { getAgent } = require('../lib/cloud-manager/agents');
const { Cluster } = require('../models/cluster');
const {
    getClusterId, getUserPortalId, getAccountId, getRequestId
} = require('../utils/async-local-storage');
const getLogger = require('../utils/logger/logger');

const logger = getLogger();

const CT_SERVICE_NAME = 'cloud-tiering';

const ACTION_NAME_MAP = {
    discover: 'Discover a cluster with agentId',
    autoDiscover: 'Discover a cluster',
    modifyVolumes: 'Modify volumes tiering settings',
    modifyAggregates: 'Modify aggregates settings',
    registerForSupport: 'Register for support',
    deregisterFromSupport: 'Deregister from support',
    sendAsup: 'Send asup from all your agents',
    removeCluster: 'Remove failed or unknown cluster',
    verifyPermissions: 'Verify aws and gcp permissions for tiering',
    createBucketG: 'Create bucket',
    changeTierLevelG: 'Change bucket tier level',
    setupClusterTieringG: 'Setup cluster tiering – create object store config',
    applyByolLicense: 'Apply BYOL license'
};

function getContextArgs(requestContext) {
    const contextArgs = cloneDeep(requestContext.context);
    Object.keys(contextArgs).forEach(key => {
        if (isUndefined(contextArgs[key])) {
            delete contextArgs[key];
        }
    });
    delete contextArgs._extensionStack;
    return contextArgs;
}
function getRequestData(requestContext) {
    const variables = cloneDeep(get(requestContext, 'request.variables', {}));
    const contextObj = getContextArgs(requestContext);
    hideSecretsHelper(Object.assign(variables, contextObj));
    return variables;
}

function getActionName(requestContext) {
    const operationName = get(requestContext, 'operation.selectionSet.selections[0].name.value'); // get resolver name
    return ACTION_NAME_MAP[operationName] ? ACTION_NAME_MAP[operationName] : 'operation description not found';
}

async function sendAuditGroupRecord(requestContext, isStart) {
    try {
        let status;
        let errors;
        if (isStart) {
            status = 'pending';
        } else {
            status = 'success';
            if (requestContext.errors) {
                status = 'failed';
                errors = requestContext.errors.map(err => {
                    try {
                        return err.message ? err.message : JSON.stringify(err);
                    } catch (error) {
                        logger.warn('Couldn\'t get errors', error);
                        return '';
                    }
                });
            }
        }

        const accountId = getAccountId();
        // build object
        const auditGroup = {
            accountId,
            actionName: getActionName(requestContext),
            status,
            requestId: getRequestId(),
            serviceName: CT_SERVICE_NAME,
            principalId: getUserPortalId(),
            errors: errors || null
        };

        if (isStart) {
            const reqDataObject = getRequestData(requestContext);
            auditGroup.requestData = !isEmpty(reqDataObject) ? JSON.stringify(reqDataObject) : null;
            const startTime = Date.now();
            requestContext.context.startTime = startTime;
            auditGroup.startTime = startTime;
        } else {
            auditGroup.startTime = requestContext.context.startTime;
            auditGroup.endTime = Date.now();

            try {
                // get cluster id
                const clusterId = getClusterId();
                auditGroup.resourceId = clusterId;
                // get cluster from db with clusterId
                const cluster = await Cluster.find({ clusterId }).exec();
                auditGroup.resourceName = get(cluster, '[0].clusterName') || null;
                // get agent name
                const { agent } = await getAgent(cluster.agentId, accountId);
                if (agent) {
                    auditGroup.agentName = agent.name;
                    auditGroup.agentId = agent.agentId;
                }
            } catch (error) {
                logger.warn('Couldn\'t get resource and agent for audit group record', error);
            }
        }
        // validate request...
        validateAuditGroup(auditGroup);
        // send to kinesis
        await putRecord({ auditGroup });
        logger.info('**** sent audit record successfully **** auditGroup:', auditGroup);
    } catch (err) {
        logger.error('Failed to send audit record to audit service', err);
    }
}

module.exports = { sendAuditGroupRecord };
