const Promise = require('bluebird');
const moment = require('moment');
const { ApolloError } = require('apollo-server');
const { sortBy } = require('lodash');
const { getAccountId } = require('../utils/async-local-storage');
const {
    AWS, AZURE, SGWS, GCP, HUBSPOT_FIELDS
} = require('../utils/consts');
const { updateHubspotContactByUserToken } = require('../lib/hubspot');
const { createGcpBucket, getGcpBuckets, setGcpStorageClass } = require('../lib/agent/gcp');
const { createSgwsBucket, getSgwsBuckets } = require('../lib/agent/sgws');
const { createAwsBucket, getAwsBuckets, setAwsTierLevel } = require('../lib/agent/aws');
const {
    setAzureStorageAccountTierLevel, createAzureContainer, getAzureStorageAccounts, getAzureContainers, getAzureStorageAccountKeys
} = require('../lib/agent/azure');
const { getAgentIdByClusterId } = require('./multi-tenancy-operation');
const { getRegionByRegionCode } = require('./regions-operations');
const getLogger = require('../utils/logger/logger');

const logger = getLogger();

// Supported storage accounts are those who support access tier on account level
const supportedStorageAccountsKind = ['StorageV2', 'BlobStorage'];

async function listAwsBuckets(clusterId, agentIdParam) {
    try {
        await updateHubspotContactByUserToken(HUBSPOT_FIELDS.list_buckets, moment().utc().startOf('day').valueOf());
        const agentId = agentIdParam ?? await getAgentIdByClusterId(clusterId, getAccountId());
        const buckets = await getAwsBuckets(agentId);
        return sortBy(buckets.filter(bucket => bucket.region).map(bucket => ({
            name: bucket.name,
            region: getRegionByRegionCode(AWS, bucket.region)
        })), 'name');
    } catch (err) { // The ui shouldnt get an error - the wizard gets stuck
        logger.error('Failed to list aws buckets', err);
        await updateHubspotContactByUserToken(HUBSPOT_FIELDS.failed, HUBSPOT_FIELDS.list_buckets);
        return [];
    }
}

async function listAzureStorageAccounts(clusterId, agentId, resourceGroupName) {
    try {
        await updateHubspotContactByUserToken(HUBSPOT_FIELDS.list_buckets, moment().utc().startOf('day').valueOf());
        const storageAccounts = await getAzureStorageAccounts(agentId, getAccountId(), resourceGroupName);
        return sortBy(storageAccounts.filter(storageAccount => supportedStorageAccountsKind.includes(storageAccount.kind)).map(storageAccount => {
            const { id } = storageAccount;
            const resourceGroup = id.split('/')[4];
            return {
                name: storageAccount.name,
                region: getRegionByRegionCode(AZURE, storageAccount.location),
                resourceGroup
            };
        }), 'name');
    } catch (err) {
        await updateHubspotContactByUserToken(HUBSPOT_FIELDS.failed, HUBSPOT_FIELDS.list_buckets);
        throw err;
    }
}

async function listAzureContainers(agentId, resourceGroupName) {
    try {
        await updateHubspotContactByUserToken(HUBSPOT_FIELDS.list_buckets, moment().utc().startOf('day').valueOf());
        const storageAccounts = await getAzureStorageAccounts(agentId, getAccountId(), resourceGroupName);
        const supportedStorageAccounts = storageAccounts.filter(storageAccount => supportedStorageAccountsKind.includes(storageAccount.kind));
        const containers = await Promise.map(supportedStorageAccounts, async storageAccount => {
            try {
                const { name, location } = storageAccount;
                const keys = await getAzureStorageAccountKeys(agentId, name, resourceGroupName);
                const storageAccountKey = keys.keys[0].value;
                const containersFound = await getAzureContainers(agentId, name, storageAccountKey);
                return containersFound.map(container => ({
                    name: container,
                    region: getRegionByRegionCode(AZURE, location),
                    storageAccount: name,
                    resourceGroup: resourceGroupName
                }));
            } catch (err) {
                logger.error('Failed to list containers of storage account', storageAccount.name, err);
                return [];
            }
        }, { concurrency: 20 });
        return containers.flat();
    } catch (err) { // The ui shouldn't get an error - the wizard gets stuck
        logger.error('Failed to list azure containers', err);
        await updateHubspotContactByUserToken(HUBSPOT_FIELDS.failed, HUBSPOT_FIELDS.list_buckets);
        return [];
    }
}

async function listGcpBuckets(clusterId, agentIdParam) {
    try {
        await updateHubspotContactByUserToken(HUBSPOT_FIELDS.list_buckets, moment().utc().startOf('day').valueOf());
        const agentId = agentIdParam ?? await getAgentIdByClusterId(clusterId, getAccountId());
        const buckets = await getGcpBuckets(agentId);
        return sortBy(buckets.map(bucket => ({
            name: bucket.name,
            region: getRegionByRegionCode(GCP, bucket.region)
        })), 'name');
    } catch (err) {
        // The ui shouldnt get an error - the wizard gets stuck
        logger.error('Failed to list gcp buckets', JSON.stringify(err));
        await updateHubspotContactByUserToken(HUBSPOT_FIELDS.failed, HUBSPOT_FIELDS.list_buckets);
        return [];
    }
}

async function listSgwsBucketsOperation(clusterId, host, port, accessKey, secretKey, agentIdParam) {
    try {
        await updateHubspotContactByUserToken(HUBSPOT_FIELDS.list_buckets, moment().utc().startOf('day').valueOf());
        const agentId = agentIdParam ?? await getAgentIdByClusterId(clusterId, getAccountId());
        const buckets = await getSgwsBuckets(agentId, host, port, accessKey, secretKey);
        return buckets.sort();
    } catch (err) {
        logger.error('Failed to list sgws buckets', err);
        await updateHubspotContactByUserToken(HUBSPOT_FIELDS.failed, HUBSPOT_FIELDS.list_buckets);
        throw err;
    }
}

// TODO after all the cleanning move the update hubspot here
async function listBucketsOperation(cloudProvider, options) {
    switch (cloudProvider) {
        case AWS: {
            const { aws: { clusterId, agentId } } = options;
            return listAwsBuckets(clusterId, agentId);
        }
        case GCP: {
            const { gcp: { clusterId, agentId } } = options;
            return listGcpBuckets(clusterId, agentId);
        }
        case AZURE: {
            const { azure: { agentId, resourceGroupName } } = options;
            return listAzureContainers(agentId, resourceGroupName);
        }
        case SGWS: {
            const {
                sgws: {
                    agentId, clusterId, host, port, credentials: { accessKey, secretKey }
                }
            } = options;
            return listSgwsBucketsOperation(clusterId, host, port, accessKey, secretKey, agentId);
        }
        default: {
            throw new ApolloError(`Received unknown type for list buckets operation ${cloudProvider}`, 400);
        }
    }
}

async function createAwsBucketOperation(clusterId, name, region, agentIdParam) {
    try {
        logger.info('create aws bucket operations', {
            clusterId, name, region
        });
        await updateHubspotContactByUserToken(HUBSPOT_FIELDS.create_bucket, moment().utc().startOf('day').valueOf());
        if (name.startsWith('fabric-pool')) {
            const agentId = agentIdParam ?? await getAgentIdByClusterId(clusterId, getAccountId());
            return await createAwsBucket(agentId, name, region);
        }
        throw new ApolloError('Bucket name must start with fabric-pool prefix');
    } catch (err) {
        await updateHubspotContactByUserToken(HUBSPOT_FIELDS.failed, HUBSPOT_FIELDS.create_bucket);
        throw err;
    }
}

async function createAzureContainerOperation(clusterId, containerName, resourceGroup, storageAccount, agentIdParam) {
    try {
        logger.info('create azure bucket operations', {
            clusterId, containerName, resourceGroup, storageAccount
        });
        await updateHubspotContactByUserToken(HUBSPOT_FIELDS.create_bucket, moment().utc().startOf('day').valueOf());
        const agentId = agentIdParam ?? await getAgentIdByClusterId(clusterId, getAccountId());
        const keys = await getAzureStorageAccountKeys(agentId, storageAccount, resourceGroup);
        const storageAccountKey = keys.keys[0].value;
        await createAzureContainer(agentId, containerName, storageAccount, storageAccountKey);
    } catch (err) {
        await updateHubspotContactByUserToken(HUBSPOT_FIELDS.failed, HUBSPOT_FIELDS.create_bucket);
        if (err.status === 409 || err.message.includes('ContainerAlreadyExists')) {
            throw new ApolloError('The specified container already exists. Please try again with a different name or choose an existing container', 400);
        }
        throw err;
    }
}

async function createGcpBucketOperation(clusterId, bucketName, location, agentIdParam) {
    try {
        logger.info('create gcp bucket operations', {
            clusterId, bucketName, location
        });
        await updateHubspotContactByUserToken(HUBSPOT_FIELDS.create_bucket, moment().utc().startOf('day').valueOf());
        const agentId = agentIdParam ?? await getAgentIdByClusterId(clusterId, getAccountId());
        return await createGcpBucket(agentId, bucketName, location);
    } catch (err) {
        await updateHubspotContactByUserToken(HUBSPOT_FIELDS.failed, HUBSPOT_FIELDS.create_bucket);
        throw err;
    }
}

async function createSgwsBucketOperation(clusterId, name, host, port, accessKey, secretKey, agentIdParam) {
    try {
        logger.info('create sgws bucket operations', {
            clusterId, host, port, accessKey, secretKey
        });
        await updateHubspotContactByUserToken(HUBSPOT_FIELDS.create_bucket, moment().utc().startOf('day').valueOf());
        const agentId = agentIdParam ?? await getAgentIdByClusterId(clusterId, getAccountId());
        return await createSgwsBucket(agentId, name, host, port, accessKey, secretKey);
    } catch (err) {
        await updateHubspotContactByUserToken(HUBSPOT_FIELDS.failed, HUBSPOT_FIELDS.create_bucket);
        throw err;
    }
}

async function createBucketOperation(cloudProvider, options) {
    logger.info('create bucket operations', JSON.stringify(options));
    switch (cloudProvider) {
        case AWS: {
            const {
                aws: {
                    agentId, clusterId, name, region
                }
            } = options;
            return createAwsBucketOperation(clusterId, name, region, agentId);
        }
        case GCP: {
            const {
                gcp: {
                    agentId, clusterId, name, region
                }
            } = options;
            return createGcpBucketOperation(clusterId, name, region, agentId);
        }
        case AZURE: {
            const {
                azure: {
                    agentId,
                    clusterId, name, resourceGroup, storageAccount
                }
            } = options;
            return createAzureContainerOperation(clusterId, name, resourceGroup, storageAccount, agentId);
        }
        case SGWS: {
            const {
                sgws: {
                    agentId,
                    clusterId, name, host, port, credentials: { accessKey, secretKey }
                }
            } = options;
            return createSgwsBucketOperation(clusterId, name, host, port, accessKey, secretKey, agentId);
        }
        default: {
            throw new ApolloError(`Received unknown type for create bucket operation ${cloudProvider}`, 400);
        }
    }
}

async function changeTierLevelOperation(cloudProvider, options) {
    logger.info('change tier level operation', { cloudProvider, options });
    switch (cloudProvider) {
        case AWS: {
            const {
                aws: {
                    bucket, clusterId, level, agentId: agentIdParam
                }
            } = options;
            const agentId = agentIdParam ?? await getAgentIdByClusterId(clusterId, getAccountId());
            return setAwsTierLevel(agentId, bucket, level);
        }
        case AZURE: {
            const {
                azure: {
                    agentId: agentIdParam,
                    clusterId, storageAccount, resourceGroup, level
                }
            } = options;
            const agentId = agentIdParam ?? await getAgentIdByClusterId(clusterId, getAccountId());
            return setAzureStorageAccountTierLevel(agentId, resourceGroup, storageAccount, level);
        }
        case GCP: {
            const {
                gcp: {
                    bucket, clusterId, level, agentId: agentIdParam
                }
            } = options;
            const agentId = agentIdParam ?? await getAgentIdByClusterId(clusterId, getAccountId());
            return setGcpStorageClass(agentId, bucket, level);
        }
        default: {
            throw new ApolloError(`Received unknown type for change tier level operation ${cloudProvider}`, 400);
        }
    }
}

module.exports = {
    changeTierLevelOperation,
    listAzureStorageAccounts,
    createBucketOperation,
    listBucketsOperation
};
