const { round, isNil, isNaN } = require('lodash');
const {
    BACKUP, ALL, SNAPSHOT_ONLY, SAVING_OPPORTUNITY_ESTIMATION_RATIO, CVO
} = require('../utils/consts');
const getLogger = require('../utils/logger/logger');

const logger = getLogger();

function volumeInactiveUserData(volume, policy) {
    switch (policy) {
        case BACKUP:
        case ALL: {
            return volume.usedSize;
        }
        case SNAPSHOT_ONLY: {
            return volume.snapshotsUsedSize;
        }
        default: {
            // default is AUTO tiering policy
            return isNil(volume.tierInactiveUserData) ? volume.usedSize * SAVING_OPPORTUNITY_ESTIMATION_RATIO : volume.tierInactiveUserData;
        }
    }
}

function calculatePerformanceCapacity(usedPerformanceCapacity, performanceInactiveCapacity, snapshotsUsedSize, isEstimatedInactiveCapacity) {
    return {
        inactiveCapacity: performanceInactiveCapacity,
        usedPerformanceCapacity,
        activeCapacity: usedPerformanceCapacity - performanceInactiveCapacity,
        isEstimatedInactiveCapacity,
        snapshotsUsedSize
    };
}

function findUsedSizeForTieringPolicy(volume, tieringPolicy) {
    return (tieringPolicy === SNAPSHOT_ONLY) ? (volume.snapshotsUsedSize || 0) : volume.usedSize;
}

function calculateUsedTieredAndColdData(volumes, availableForTieringVolumes, policy) {
    const tieredCapacity = volumes.reduce((acc, vol) => acc + (vol.capacityTierUsedSize || 0), 0);
    const performanceTierUsedCapacity = volumes.reduce((acc, vol) => acc + findUsedSizeForTieringPolicy(vol, policy), 0);
    const performanceInactiveCapacityNotTiered = availableForTieringVolumes.reduce((acc, vol) => acc + volumeInactiveUserData(vol, policy), 0);
    return { performanceInactiveCapacityNotTiered, tieredCapacity, performanceTierUsedCapacity };
}

// for savings - inactiveOrTieredCapacityGB is tieredCapacity for opportunities - inactiveCapacity
function savingsFunction(tieredAndUsedCapacityBytes, inactiveOrTieredCapacityBytes) {
    const TIER_LICENSE_PER_GB = 0.033;
    const S3_ESTIMATED_COST_PER_GB = 0.0152;
    const FIVE_YEAR_AFF_TCO_PER_ALLOCATED_GB = 3.042;
    const UTILIZATION = 0.4;
    const fiveYearTierTcoPerGb = (S3_ESTIMATED_COST_PER_GB + TIER_LICENSE_PER_GB) * 60;
    const fivYearAffTcpPerRawGb = FIVE_YEAR_AFF_TCO_PER_ALLOCATED_GB / UTILIZATION;
    const fiveYearWithTierTcoPerGB = (fivYearAffTcpPerRawGb * (tieredAndUsedCapacityBytes - inactiveOrTieredCapacityBytes) + fiveYearTierTcoPerGb * inactiveOrTieredCapacityBytes) / tieredAndUsedCapacityBytes;
    const fiveYearsSavingsPerGB = round(fivYearAffTcpPerRawGb - fiveYearWithTierTcoPerGB, 2);
    const result = round(fiveYearsSavingsPerGB * tieredAndUsedCapacityBytes / (1024 ** 3));
    return isNaN(result) ? 0 : result;
}

function calculateSavingsOpportunitiesAndSavings(performanceTierUsedCapacity, tieredCapacity, performanceInactiveCapacityNotTiered, clusterId, resourceType) {
    logger.info(`Calculating savings opportunities and savings received for cluster ${clusterId}`, {
        performanceTierUsedCapacity, tieredCapacity, performanceInactiveCapacityNotTiered, resourceType
    });
    const performanceAndTieredCapacity = performanceTierUsedCapacity + tieredCapacity;
    return {
        currentSavings: {
            cost: (resourceType !== CVO) ? savingsFunction(performanceAndTieredCapacity, tieredCapacity) : undefined,
            capacity: tieredCapacity
        },
        savingOpportunity: {
            cost: savingsFunction(performanceAndTieredCapacity, performanceInactiveCapacityNotTiered),
            capacity: performanceInactiveCapacityNotTiered
        }
    };
}

module.exports = {
    calculateUsedTieredAndColdData, calculateSavingsOpportunitiesAndSavings, calculatePerformanceCapacity, volumeInactiveUserData
};
