const { get } = require('lodash');
const { checkEnvironment } = require('../utils/utils');
const {
    OCCM_PROVIDER_TO_CLOUD_PROVIDER, AZURE, GCP, SGWS, FAS_AFF_NODE_MODELS,
    WHITELIST_VSIM_IP_E2E,
    WHITELIST_CVO_NAMES_PREFIX_E2E
} = require('../utils/consts');
const HttpError = require('../utils/errors');

const getLogger = require('../utils/logger/logger');

const logger = getLogger();

function volumeAggregates(volume) {
    return volume.aggregateName.split(',');
}

function volumeToString(volume) {
    const {
        name, svmName, aggregateName, tieringPolicy, volumeType
    } = volume;
    return JSON.stringify({
        name, svmName, aggregateName, tieringPolicy, volumeType
    });
}

function volumesTostring(volumes) {
    return volumes.map(vol => volumeToString(vol));
}

function isTieredVolume(volume, allAggregates) {
    const volAggregatesName = volumeAggregates(volume);
    const volAggregates = allAggregates.filter(aggregate => volAggregatesName.includes(aggregate.name));
    return volAggregates.some(aggregate => aggregate.objectStoreName
        && (volume.tieringPolicy ? volume.tieringPolicy !== 'none' : true));
}

function parseOntapVersion(ontapVersion) {
    if (ontapVersion) {
        const ontapVersionRegex = /(\d+\.\d*|\d+)/;
        const version = ontapVersionRegex.exec(ontapVersion)[1];
        return parseFloat(version);
    }
}

function findOntapVersion(clusterInformation) {
    const ontapVersion = get(clusterInformation, 'ontapClusterProperties.ontapVersion');
    return parseOntapVersion(ontapVersion);
}

function checkAggregatesRestrictions(aggregates) {
    const aggregateWithHardRestriction = aggregates.filter(aggregate => !aggregate.objectStoreName && aggregate.tieringRestriction.isTieringRestricted && !aggregate.isObjectStoreAttachEligible);
    // in order to support cluster without aggregates
    return !aggregates.length || aggregateWithHardRestriction.length !== aggregates.length;
}

function findMinimumOntapVersionRequiredForCloudProvider(cloudProvider) {
    switch (cloudProvider) {
        case SGWS:
        case AZURE: {
            return 9.4;
        }
        case GCP: {
            return 9.6;
        }
        default: {
            return 9.2;
        }
    }
}

function validateEligibleForTiering(ontapVersionParsed, agentProvider) {
    const eligibleVersionPerProvider = findMinimumOntapVersionRequiredForCloudProvider(agentProvider);
    if (ontapVersionParsed < eligibleVersionPerProvider) {
        throw new HttpError(`Tiering is not allowed. ONTAP Version ${ontapVersionParsed} is not supported`, 403);
    }
}

function clusterProviderType(cluster) {
    const providerType = get(cluster.capacityFeatures, 'objectStoreConfigExtended.objectStoreConfig.providerType');
    return OCCM_PROVIDER_TO_CLOUD_PROVIDER[providerType] || providerType;
}

// We want to disable in production the ability to discover CVO.
// We whitelisted: 1) CVO names for our E2E 2) Cluster IPs of our vSims
function checkIfCvoCluster(nodeModels, clusterName, clusterAddress) {
    logger.info('check if cluster is cvo', { nodeModels, clusterName, clusterAddress });
    if (checkEnvironment('production')) {
        const supportedNodeModel = nodeModels ? FAS_AFF_NODE_MODELS.some(nodeModel => nodeModels.includes(nodeModel)) : true;
        const whitelistCVO = clusterName ? WHITELIST_CVO_NAMES_PREFIX_E2E.some(cvoName => clusterName.startsWith(cvoName)) : true;
        const whitelistVsim = clusterAddress ? WHITELIST_VSIM_IP_E2E.includes(clusterAddress) : true;
        return !(supportedNodeModel || whitelistCVO || whitelistVsim);
    }
    return false;
}

function checkClusterEligibleNoReason(ontapVersion, agentCloudProvider, aggregates, nodeModels, clusterName, clusterAddress) {
    const eligibleVersionPerProvider = findMinimumOntapVersionRequiredForCloudProvider(agentCloudProvider);
    if (ontapVersion >= eligibleVersionPerProvider) {
        const aggregateRestrictionResult = checkAggregatesRestrictions(aggregates);
        if (aggregateRestrictionResult) {
            const isClusterIsCvo = checkIfCvoCluster(nodeModels, clusterName, clusterAddress);
            return { isEligibleForTiering: !isClusterIsCvo };
        }
        return { isEligibleForTiering: false };
    }
    return { isEligibleForTiering: false };
}

module.exports = {
    checkClusterEligibleNoReason,
    checkIfCvoCluster,
    volumesTostring,
    findOntapVersion,
    parseOntapVersion,
    isTieredVolume,
    volumeAggregates,
    findMinimumOntapVersionRequiredForCloudProvider,
    validateEligibleForTiering,
    clusterProviderType,
    checkAggregatesRestrictions

};
