const {
    SNAPSHOT_ONLY, BACKUP, ALL, AUTO, DP_VOLUME, NONE, TIERING_POLICIES_AND_MINIMUM_COOLING_DAYS, ONTAP_VERSIONS_FEATURES_SUPPORT
} = require('../utils/consts');
const { calculateSavingsOpportunitiesAndSavings, calculateUsedTieredAndColdData } = require('./calculations-operations');
const HttpError = require('../utils/errors');
const getLogger = require('../utils/logger/logger');

const logger = getLogger();

function tieringPolicyPackageResponse(policyName, availableVolumesLength, savingOpportunity, isEstimated, packageAvailability) {
    logger.debug('tieringPolicyPackageResponse', {
        policyName, availableVolumesLength, savingOpportunity, isEstimated, packageAvailability
    });
    return {
        name: policyName,
        count: availableVolumesLength,
        savingOpportunity,
        isEstimated,
        supported: packageAvailability
    };
}

function findTieringPoliciesByVersion(ontapVersion) {
    if (ontapVersion >= 9.2 && ontapVersion < 9.4) {
        return [AUTO, SNAPSHOT_ONLY, BACKUP];
    }
    if (ontapVersion >= 9.4 && ontapVersion < 9.6) {
        return [AUTO, SNAPSHOT_ONLY, BACKUP];
    }
    if (ontapVersion >= 9.6) {
        return [ALL, AUTO, SNAPSHOT_ONLY];
    }
    // default - if !ontapVersion or if ontapVersion<9.2
    return [];
}

function createSupportedSettingPerPolicy(tieringPolicy, ontapVersion) {
    if (ontapVersion >= ONTAP_VERSIONS_FEATURES_SUPPORT.MINIMUM_COOLING_DAYS) {
        const minimumCoolingDays = TIERING_POLICIES_AND_MINIMUM_COOLING_DAYS[tieringPolicy];
        return { tieringPolicy, minimumCoolingDays, noMinimumCoolingDaysReason: minimumCoolingDays ? '' : 'This policy doesn’t have a cooling period because it immediately moves all data (not including metadata) to object storage.' };
    }
    return { tieringPolicy, noMinimumCoolingDaysReason: 'Adjusting the cooling days requires ONTAP 9.4 or later' };
}

function supportedVolumeSettings(ontapVersion, isDpVolume) {
    const supportedPolicies = [createSupportedSettingPerPolicy(NONE)];
    if (ontapVersion >= 9.2 && ontapVersion < 9.6 && isDpVolume) {
        supportedPolicies.push(createSupportedSettingPerPolicy(BACKUP, ontapVersion));
    }
    if (ontapVersion >= 9.2) {
        supportedPolicies.push(createSupportedSettingPerPolicy(SNAPSHOT_ONLY, ontapVersion));
    }
    if (ontapVersion >= 9.4) {
        supportedPolicies.push(createSupportedSettingPerPolicy(AUTO, ontapVersion));
    }
    if (ontapVersion >= 9.6) {
        supportedPolicies.push(createSupportedSettingPerPolicy(ALL, ontapVersion));
    }
    return supportedPolicies;
}

function findIfPolicySupported(tieringPolicy, ontapVersion, dpVolumes) {
    switch (tieringPolicy) {
        case AUTO: {
            return ontapVersion >= 9.4;
        }
        case SNAPSHOT_ONLY: {
            return ontapVersion >= 9.2;
        }
        case ALL: {
            return ontapVersion >= 9.6;
        }
        case BACKUP: {
            return ontapVersion >= 9.2 && dpVolumes.length;
        }
        default: {
            throw new HttpError('Received unsupported tiering policy', tieringPolicy);
        }
    }
}

function tieringPackageAvailability(ontapVersion, tieringPolicy, availableVolumesLength, isPolicySupported) {
    if (tieringPolicy === BACKUP && !isPolicySupported) {
        return {
            isSupported: false,
            reason: `Tiering policy ${tieringPolicy} available on dp volumes only`
        };
    }
    if (!isPolicySupported) {
        return {
            isSupported: false,
            reason: `Tiering policy ${tieringPolicy} is not supported in ${ontapVersion} ONTAP version`
        };
    }
    if (availableVolumesLength === 0) {
        return {
            isSupported: false,
            reason: `No volumes eligible for ${tieringPolicy} tiering policy`
        };
    }
    return {
        isSupported: true
    };
}

function tieringPolicyPackage(tieringPolicy, ontapVersion, volumes, isEstimated) {
    const availableForTieringVolumes = volumes.filter(volume => volume.tieringPolicy !== tieringPolicy);
    const availableForTieringDpVolumes = availableForTieringVolumes.filter(volume => volume.volumeType === DP_VOLUME);
    const availableVolumesLength = (tieringPolicy === BACKUP) ? availableForTieringDpVolumes.length : availableForTieringVolumes.length;
    const relevantVolumes = (tieringPolicy === BACKUP) ? volumes.filter(volume => volume.volumeType === DP_VOLUME) : volumes;
    const { performanceInactiveCapacityNotTiered, tieredCapacity, performanceTierUsedCapacity } = calculateUsedTieredAndColdData(relevantVolumes, (tieringPolicy === BACKUP) ? availableForTieringDpVolumes : availableForTieringVolumes, tieringPolicy);
    const { savingOpportunity } = calculateSavingsOpportunitiesAndSavings(performanceTierUsedCapacity, tieredCapacity, performanceInactiveCapacityNotTiered);
    const packageAvailability = tieringPackageAvailability(ontapVersion, tieringPolicy, availableVolumesLength, findIfPolicySupported(tieringPolicy, ontapVersion, relevantVolumes));
    return tieringPolicyPackageResponse(tieringPolicy, availableVolumesLength, savingOpportunity, (tieringPolicy === AUTO) ? isEstimated : false, packageAvailability);
}

module.exports = {
    supportedVolumeSettings,
    findTieringPoliciesByVersion,
    tieringPolicyPackage
};
