const { get, isNil, round } = require('lodash');
const { getAgentIdByClusterId } = require('./multi-tenancy-operation');
const { getVolumesExtended, getOntapVersion } = require('./agent-zapi-operations');
const {
    SAVING_OPPORTUNITY_ESTIMATION_RATIO, DP_VOLUME, ONLINE_VOLUME, AUTO, SNAPSHOT_ONLY, TIERING_POLICIES_AND_MINIMUM_COOLING_DAYS
} = require('../utils/consts');
const { getAccountId } = require('../utils/async-local-storage');
const { calculatePerformanceCapacity } = require('./calculations-operations');
const { supportedVolumeSettings, findTieringPoliciesByVersion, tieringPolicyPackage } = require('./tiering-policy-operations');
const { volumeAggregates } = require('./cloud-manager-operations');
const getLogger = require('../utils/logger/logger');

const logger = getLogger();

function filterEligibleVolumes(eligibleVolumes, failedAggregates) {
    return eligibleVolumes.reduce((accumulator, volume) => {
        const failedAggregate = failedAggregates.find(aggregate => volumeAggregates(volume).includes(aggregate.name));
        if (failedAggregate) {
            accumulator.failedVolumes.push({ name: volume.name, svmName: volume.svmName, reason: failedAggregate.reason });
        } else {
            accumulator.volumesToChangeTieringPolicy.push(volume);
        }
        return accumulator;
    }, { failedVolumes: [], volumesToChangeTieringPolicy: [] });
}

function findMinimumCoolingDaysForAVolume(tieringPolicy, minimumCoolingDays) {
    if ([AUTO, SNAPSHOT_ONLY].includes(tieringPolicy)) {
        if (minimumCoolingDays) {
            return minimumCoolingDays;
        }
        return TIERING_POLICIES_AND_MINIMUM_COOLING_DAYS[tieringPolicy];
    }
}

function adjustVolumeResponse(volume, ontapVersion) {
    const {
        state, name, svmName, aggregateName, tieringPolicy, isTiered, tierInactiveUserDataPercent, volumeType, isEligibleForTiering, tieringRestrictionReason, minimumCoolingDays
    } = volume;
    const supportedSettings = supportedVolumeSettings(ontapVersion, volumeType === DP_VOLUME);
    const usedPerformanceCapacity = get(volume, 'usedSize');
    const inactiveCapacityFromOccm = get(volume, 'tierInactiveUserData');
    const isEstimatedInactiveCapacity = isNil(inactiveCapacityFromOccm);
    const inactiveCapacity = isEstimatedInactiveCapacity ? round(usedPerformanceCapacity * SAVING_OPPORTUNITY_ESTIMATION_RATIO) : (inactiveCapacityFromOccm || 0);
    const tieredCapacity = get(volume, 'capacityTierUsedSize', 0);
    const usedPerformanceAndTieredCapacity = usedPerformanceCapacity + tieredCapacity;
    const performance = calculatePerformanceCapacity(usedPerformanceCapacity, inactiveCapacity, volume.snapshotsUsedSize || 0, isEstimatedInactiveCapacity);

    return {
        name,
        svmName,
        aggregateName,
        state,
        tieringPolicy: isTiered ? tieringPolicy || 'none' : 'none',
        capacity: get(volume, 'size', 0),
        tieredCapacity,
        isEligibleForTiering,
        tieringRestrictionReason,
        usedPerformanceAndTieredCapacity,
        isTiered,
        supportedSettings,
        minimumCoolingDays: findMinimumCoolingDaysForAVolume(tieringPolicy, minimumCoolingDays),
        inactiveCapacityPercent: isEstimatedInactiveCapacity ? SAVING_OPPORTUNITY_ESTIMATION_RATIO * 100 : tierInactiveUserDataPercent,
        ...performance
    };
}

function findVolumesAggregates(volume, aggregates) {
    const volumeAggregatesNames = volumeAggregates(volume);
    return aggregates.filter(aggregate => volumeAggregatesNames.includes(aggregate.name));
}

function findRestrictedVolumes(volumes, aggregates) {
    return volumes.map(volume => {
        const requestedAggregates = findVolumesAggregates(volume, aggregates);
        return {
            volume,
            restrictedAggregate: requestedAggregates.find(aggregate => !aggregate.objectStoreName && aggregate.tieringRestriction.isTieringRestricted),
            requestedAggregates
        };
    }).filter(volumeAndRestrictedAggretgate => volumeAndRestrictedAggretgate.restrictedAggregate).map(volumeAfterFilter => {
        const { requestedAggregates, volume: { name, svmName }, restrictedAggregate } = volumeAfterFilter;
        const isHardRestricted = requestedAggregates.some(aggregate => !aggregate.objectStoreName && aggregate.tieringRestriction.isTieringRestricted && !aggregate.isObjectStoreAttachEligible);
        return {
            name,
            svmName,
            isHardRestricted,
            restrictionReason: restrictedAggregate.tieringRestriction.restrictionReason || 'unknown restriction reason'
        };
    });
}

function isVolumeEligibleForTiering(volume) {
    const {
        state, isHardRestricted, isRestricted, restrictionReason
    } = volume;
    let eligibleObj;
    if (state !== ONLINE_VOLUME) {
        eligibleObj = {
            isEligibleForTiering: false,
            tieringRestrictionReason: `Volume’s state is ${state}. To enable tiering, a volume’s state must be 'online'`
        };
    } else if (isRestricted) {
        if (!isHardRestricted) {
            eligibleObj = { isEligibleForTiering: false, tieringRestrictionReason: restrictionReason };
        }
    } else {
        eligibleObj = { isEligibleForTiering: true };
    }
    return Object.assign(volume, eligibleObj);
}

async function getSupportedVolumes(clusterId, agentIdParam) {
    try {
        logger.info('Finding supported (eligible + soft) volumes', { clusterId });
        const agentId = agentIdParam || await getAgentIdByClusterId(clusterId, getAccountId());
        const volumes = await getVolumesExtended(agentId, clusterId);
        const ontapVersion = await getOntapVersion(agentId, clusterId);
        return volumes.map(volume => isVolumeEligibleForTiering(volume))
            .filter(volumeWithEligibleObj => !isNil(volumeWithEligibleObj.isEligibleForTiering))
            .map(volume => adjustVolumeResponse(volume, ontapVersion));
    } catch (err) {
        logger.error('Failed to get supported volumes with error', err);
        throw err;
    }
}

function getTieredVolumesSummary(supportedVolumes, allVolumes, ontapVersion, isEstimated) {
    const availableTieringPolicies = findTieringPoliciesByVersion(ontapVersion);
    return {
        eligibleVolumes: supportedVolumes.length,
        tieredVolumes: supportedVolumes.filter(volume => volume.isTiered).length,
        totalVolumes: allVolumes ? allVolumes.length : 0,
        supportedPolicies: availableTieringPolicies.map(
            policy => tieringPolicyPackage(policy, ontapVersion, supportedVolumes, isEstimated)
        )
    };
}

module.exports = {
    findMinimumCoolingDaysForAVolume,
    getTieredVolumesSummary,
    getSupportedVolumes,
    findRestrictedVolumes,
    findVolumesAggregates,
    filterEligibleVolumes
};
