const express = require('express');
const { decode } = require('jsonwebtoken');
const { trimStart } = require('lodash');
const { getResourcesByAccount, deleteAccount } = require('../lib/cloud-manager/multi-tenancy');
const { getAgents, deleteAgent } = require('../lib/cloud-manager/agents');
const { Account } = require('../models/account');
const { Cluster } = require('../models/cluster');
const HttpError = require('../utils/errors');
const { removeClusterOperation } = require('../operations/cluster-operations');
const { collectChargingInfo } = require('../operations/charging-operations');
const { updateLicenses } = require('../operations/license-operations');
const { updateClusters } = require('../operations/clusters-collection-operations');
const { trialExpiryCheck } = require('../operations/subscription-operations');
const {
    AGENDA_JOB_NAME_TRIAL_EXPIRY_STATUS,
    AGENDA_JOB_NAME_INCREASE_LICENSE_CAPACITY, AGENDA_JOB_NAME_SAAS_METERING, AGENDA_JOB_NAME_UPDATE_CLUSTERS_COLLECTION
} = require('../utils/consts');
const { getCache } = require('../utils/got-utils');
const { getUserToken, setUserToken } = require('../utils/async-local-storage');
const getLogger = require('../utils/logger/logger');

const logger = getLogger();

const internalRoute = express.Router();

internalRoute.delete('/resources/:id', async (req, res, next) => {
    const { params: { id: accountId } } = req;
    try {
        // check if deletion is allowed - if the token has the field http://cloud.netapp.com/internal : NetAppTest it is a test-netapp user
        const payload = decode(trimStart(getUserToken(), 'Bearer').trim(), { complete: true });
        const httpInternal = payload.payload['http://cloud.netapp.com/internal'];
        logger.info('httpInternal inside delete', httpInternal);
        if (httpInternal === 'NetAppTest') {
            logger.info(`deleting all resources in account ${accountId}`);
            const allResourcesInAccount = await getResourcesByAccount(accountId);
            // for each resource delete
            await Promise.all(allResourcesInAccount.filter(resource => resource.resourceClass === 'ONPREM').map(async resource => {
                try {
                    await removeClusterOperation(resource.resourceIdentifier, resource.agentId);
                } catch (err) {
                    logger.error('Failed to remove cluster', resource);
                }
            }));
            // delete agents
            // find all agents in an account
            logger.info(`Deleting all agents in account ${accountId}`);
            const { agents } = await getAgents(accountId);
            await Promise.all(agents.map(async agent => {
                try {
                    await deleteAgent(agent.agentId, accountId);
                } catch (err) {
                    logger.info(`Failed to delete agent ${agent.agentId} from tenancy management. Skipping...`);
                }
            }));
            // delete account
            logger.info(`Deleting account ${accountId}`);
            try {
                await deleteAccount(accountId);
            } catch (err) {
                logger.info(`Failed to delete account ${accountId} from tenancy management. Skipping...`);
            }
            logger.info(`Deleting account ${accountId} from DB`);
            await Account.deleteOne({ accountId });
            res.json();
        } else {
            next(new Error('Delete user is not allowed for user '));
        }
    } catch (err) {
        logger.error(`Error deleting account ${accountId} with error ${JSON.stringify(err)}`);
        next(err);
    }
});

internalRoute.post('/activateInnerFunction', async (req, res, next) => {
    const { body: { operationName } } = req;
    try {
        logger.info('Received a POST request to activate inner function ', operationName);
        let result;
        switch (operationName) {
            case AGENDA_JOB_NAME_SAAS_METERING:
                result = await collectChargingInfo();
                break;
            case AGENDA_JOB_NAME_INCREASE_LICENSE_CAPACITY:
                result = await updateLicenses();
                break;
            case AGENDA_JOB_NAME_TRIAL_EXPIRY_STATUS:
                result = await trialExpiryCheck();
                break;
            case AGENDA_JOB_NAME_UPDATE_CLUSTERS_COLLECTION:
                if (!process.env.IS_SIMULATOR) {
                    logger.info('Update clusters collection, removing user token from context');
                    setUserToken(undefined);
                }
                result = await updateClusters();
                break;
            default: {
                logger.error('Received unrecognized operationName', operationName);
                next(new HttpError('Received unrecognized operation name', 403));
            }
        }
        res.json(result);
    } catch (err) {
        next(err);
    }
});

internalRoute.post('/clearCache', async (req, res, next) => {
    try {
        const cache = getCache();
        cache.clear();
        res.status(200).json();
    } catch (err) {
        next(err);
    }
});

internalRoute.post('/updateCluster', async (req, res, next) => {
    try {
        if (process.env.NODE_ENV !== 'production') {
            const { body: { tieredCapacity, clusterId, accountId } } = req;
            const updatedCluster = await Cluster.findOneAndUpdate({ accountId, clusterId }, { $set: { currentCapacity: tieredCapacity, sendNotification: false } }, { new: true });
            res.status(200).json(updatedCluster);
        }
        next(new HttpError('update cluster is not available on production environment', 403));
    } catch (err) {
        next(err);
    }
});

module.exports = internalRoute;
