const express = require('express');
const moment = require('moment');
const getLogger = require('../utils/logger/logger');
const { MpSubscription } = require('../models/mp-subscription');
const { Account } = require('../models/account');
const {
    API_PATH_SAAS_SUBSCRIPTION_NOTIFICATION, FAILED_SUBSCRIPTION,
    UNSUBSCRIBED, UPDATE_LICENSE_INFO_EVENT, PENDING_SUBSCRIPTION, SUBSCRIBED, HUBSPOT_FIELDS
} = require('../utils/consts');
const { subscriptionNotificationSchema } = require('../joi-schemas/subscription-schemas');
const { validateResponse } = require('../utils/schema-validation');
const {
    createSubscriptionsArray, calculateDaysLeftForTrial
} = require('../operations/subscription-operations');
const { updateHubspotContactByAccount } = require('../lib/hubspot');
const { updateClusterSubscription } = require('../operations/clusters-collection-operations');
const { updateMessages } = require('../graphql/mongo-async-iterator');

const saasSubscriptionWebHookRouter = express.Router();
const logger = getLogger();

async function sendGraphqlSubscriptionUpdates(account) {
    const { accountId, freeTrialExpiryDate } = account;
    const subscriptions = await createSubscriptionsArray(accountId);
    await updateMessages(accountId, UPDATE_LICENSE_INFO_EVENT, {
        licenseInfo: {
            accountId,
            subscriptions,
            daysLeftForTrial: calculateDaysLeftForTrial(freeTrialExpiryDate)

        }
    });
}

function validationMiddleware(req, res, next) {
    const { body } = req;
    logger.info('Received subscription notification - validation middleware', JSON.stringify(body));
    try {
        validateResponse(body, subscriptionNotificationSchema);
        const { body: { cloudProvider, accounts } } = req;
        req.body.accounts = accounts || [];
        req.body.cloudProvider = cloudProvider.toUpperCase();
        next();
    } catch (err) {
        logger.error('Error validation subscription notification');
        next(err);
    }
}

saasSubscriptionWebHookRouter.post(`${API_PATH_SAAS_SUBSCRIPTION_NOTIFICATION}/update`, validationMiddleware, async (req, res, next) => {
    try {
        const { body } = req;
        logger.info('Received subscription notification', JSON.stringify(body));
        const {
            notificationType, subscriptionStatus, accounts, subscriptionId,
            serviceName, cloudProvider, contract
        } = body;

        if (contract) {
            logger.info('Received notification about contract subscription');
            return res.status(200).send();
        }
        const status = [FAILED_SUBSCRIPTION, UNSUBSCRIBED, PENDING_SUBSCRIPTION].includes(subscriptionStatus) ? UNSUBSCRIBED : subscriptionStatus;
        // updating DB
        if (['add-subscription', 'status-change'].includes(notificationType)) {
            await Promise.all(accounts.map(async accountId => {
                const account = await Account.findOne({ accountId }).exec();
                if (account) {
                    await MpSubscription.updateOne({
                        subscriptionId,
                        accountId,
                        cloudProvider,
                        serviceName
                    }, { $set: { status } }, { upsert: true }).exec();
                    await updateClusterSubscription(accountId,
                        cloudProvider, serviceName);
                    // update UI
                    await sendGraphqlSubscriptionUpdates(account);
                    logger.info('before updating status for subscribed users', status);
                    if (status === SUBSCRIBED) {
                        logger.info('updating hubspot for subscribed users');
                        await updateHubspotContactByAccount(accountId, HUBSPOT_FIELDS.subscribe, moment().utc().startOf('day').valueOf());
                    }
                }
            }));
        }
        if (notificationType === 'remove-subscription') {
            await Promise.all(accounts.map(async accountId => {
                await MpSubscription.deleteOne({ subscriptionId, accountId }).exec();
                const account = await Account.findOne({ accountId }).exec();
                if (account) {
                    await updateClusterSubscription(accountId, cloudProvider, serviceName);
                    // update UI
                    await sendGraphqlSubscriptionUpdates(accountId);
                } else {
                    logger.error(`Error updating clusters for account - ${accountId} was not found`);
                }
            }));
        }

        res.status(200).send();
    } catch (err) {
        logger.error('Received error on subscription update webhook', err);
        next(err);
    }
});

module.exports = saasSubscriptionWebHookRouter;
