const Redis = require('ioredis');
const { deflate, unzip } = require('zlib');
const Router = require('find-my-way');
const { URL } = require('url');
const cacheRoutes = require('../resources/cache-routes');
const getLogger = require('./logger/logger');

const logger = getLogger('cache');

// logger.info('>>>>>>>>>>>>>>>>redis connection string', process.env.REDIS_CONNECTION_STRING);

const router = new Router();

async function compressString(value) {
    return new Promise((resolve, reject) => {
        deflate(value, (err, buffer) => {
            if (err) {
                logger.error('Failed to compress value', value);
                reject(err);
            }
            resolve(buffer.toString('base64'));
        });
    });
}

async function unzipString(value) {
    const buffer = Buffer.from(value, 'base64');
    return new Promise((resolve, reject) => {
        unzip(buffer, (err, result) => {
            if (err) {
                logger.error('Failed to unzip value', value);
                reject(err);
            }
            resolve(result.toString());
        });
    });
}

function createRoutes() {
    cacheRoutes.forEach(route => {
        const { method, path, 'paths-to-delete': pathsToDelete } = route;
        router.on(method, path, url => pathsToDelete.map(suffix => new URL(suffix, url.href).href));
    });
}

function findUrl(key) {
// for now the cache is relevant for zapi requests only
    const regexPattern = /cacheable-request:(.+):(http:\/\/.+:.+\/.+\/zapi\/.+)/;
    const groups = regexPattern.exec(key);
    if (groups) {
        const [, method, url] = groups;
        return { method, url: new URL(url) };
    }
}

function deleteRelatedEntreis(cache, method, url) {
    const foundRoute = router.find(method, url.pathname);
    if (foundRoute) {
        const keysToDelete = foundRoute.handler(url);
        keysToDelete.forEach(keyPath => {
            logger.info('DELETING key ', keyPath);
            cache.del(keyPath);
        });
    }
}

class ZapiCache {
    constructor() {
        try {
            createRoutes();
            this.cache = new Redis(`${process.env.REDIS_CONNECTION_STRING}/1`);
            logger.info('REDIS CONNECTED');
        } catch (err) {
            logger.error('FAILED to connect to redis', err);
        }
    }

    async set(key, value) {
        // add compression to the value
        logger.info('SET cache', key, value);
        const keyObject = findUrl(key);
        if (keyObject) {
            const { method, url } = keyObject;
            deleteRelatedEntreis(this.cache, method, url);
            if (method === 'GET') {
                logger.info('set url', url.href, value);
                const compressedValue = await compressString(value);
                await this.cache.setex(url.href, 300, compressedValue); // ttl in seconds
            }
        } else {
            logger.info('set - Not relevant for cache return undefined', key);
        }
    }

    async get(key) {
        logger.info('get from cache', key);
        // decompress value
        const keyObject = findUrl(key);
        if (keyObject) {
            const { url, method } = keyObject;
            if (method !== 'GET') {
                // deleting on get from cache because PUT is not cacheable request
                deleteRelatedEntreis(this.cache, method, url);
            } else {
                const isExists = await this.cache.exists(url.href);
                if (isExists) {
                    logger.info('GET from cache - exists', url.href);
                    const value = await this.cache.get(url.href);
                    if (value) {
                        return unzipString(value);
                    }
                }
                logger.info('Not found on cache', key, ' returning undefined');
            }
        } else {
            logger.info('get - Not relevant for cache return undefined', key);
        }
    }

    clear() {
        this.cache.flushdb();
    }
}

module.exports = ZapiCache;
