const config = require('config');
const mongoose = require('mongoose');
const moment = require('moment');
const { Account, findClustersToIncreaseLicense } = require('../../src/models/account');
const { Cluster } = require('../../src/models/cluster');
const { MpSubscription } = require('../../src/models/mp-subscription');
const { createAccount, createSubscription, createCluster } = require('../testUtils');

const getLogger = require('../../src/utils/logger/logger');
const { DB_CONNECTION_STRING, SUBSCRIBED, TIERING_SERVICE_NAME_SUBSCRIPTION } = require('../../src/utils/consts');

const logger = getLogger();

beforeAll(async () => {
    try {
        await mongoose.connect(DB_CONNECTION_STRING, { useNewUrlParser: true, dbName: 'cloudTiering' });
    } catch (e) {
        logger.error('Cannot connect to mongoDB');
    }
});

afterAll(async done => {
    try {
        await mongoose.connection.close();
        done();
    } catch (e) {
        logger.error('Error closing connection', e);
    }
});

describe('Account Model -CRUD', () => {
    const accountId = '1';
    const name = 'test';

    beforeEach(() => Account.deleteMany({}));

    test('create', async () => {
        const accountDocument = await Account.create({ name, accountId, support: { serialNumber: '2' } });
        expect(accountDocument.accountId).toEqual(accountId);
    });

    test('get', async () => {
        await Account.create({ name, accountId: '2', support: { serialNumber: '2' } });
        const foundDoc = await Account.findOne({ accountId: '2' });
        expect(foundDoc.accountId).toEqual('2');
    });

    test('update', async () => {
        await Account.create({ name, accountId, support: { serialNumber: '2' } });
        const freeTrialExpiryDate = moment().add(3, 'days');
        await Account.update({ accountId }, { $set: { freeTrialExpiryDate } });
        const updatedAccount = Account.findOne({ freeTrialExpiryDate });
        expect(updatedAccount).toBeTruthy();
    });

    test('delete', async () => {
        const doc = await Account.create({ name, accountId, support: { serialNumber: '2' } });
        await doc.remove();
        expect(doc.$isDeleted()).toEqual(true);
    });
});

describe('Account Model -Custom Methods', () => {
    const accountId = '1';
    const name = 'test';

    test('toJson', async () => {
        await Account.create({ name, accountId, support: { serialNumber: '2' } });
        const result = await Account.findOne({ accountId });
        expect(result.accountId).toEqual(accountId);
    });
});

describe('findClustersToIncreaseLicense', () => {
    beforeEach(async () => {
        await Account.deleteMany();
        await MpSubscription.deleteMany();
        await Cluster.deleteMany();
    });

    test('Account with no freeTrial and no subscription has cluster with no tiering, should return an empty array', async () => {
        const accountId = '12';
        await Account.create(createAccount(accountId));
        await Cluster.create(createCluster(accountId, '1', '1', 'aws', undefined,
            { serialNumber: '123', maxCapacity: 2199023255552 }, undefined, 1099511627776, new Date()));
        const clusters = await findClustersToIncreaseLicense();
        expect(clusters).toEqual([]);
    });

    test('Account subscribed with expired free trial and cluster with tiering, license capacity ratio is under the threshold, should return an empty array', async () => {
        const accountId = '19';
        const subscriptionId = '3';
        const clusterUuid = '1';

        await Account.create(createAccount(accountId, moment().subtract(3, 'days')));
        await MpSubscription.create(createSubscription(accountId, SUBSCRIBED, 'aws', subscriptionId, TIERING_SERVICE_NAME_SUBSCRIPTION));
        await Cluster.create(createCluster(accountId, '1', clusterUuid, 'aws', subscriptionId,
            undefined, { serialNumber: '123', maxCapacity: 10995116277760 }, 1073741824, new Date()));
        const clusters = await findClustersToIncreaseLicense();
        expect(clusters).toEqual([]);
    });

    test('Account subscribed with expired free trial and cluster with tiering, license capacity ratio is above the threshold, should return a cluster', async () => {
        const accountId = '19';
        const subscriptionId = '3';
        const clusterUuid = '1';

        await Account.create(createAccount(accountId, moment().subtract(3, 'days')));
        await MpSubscription.create(createSubscription(accountId, SUBSCRIBED, 'aws', subscriptionId, TIERING_SERVICE_NAME_SUBSCRIPTION));
        await Cluster.create(createCluster(accountId, '1', clusterUuid, 'aws', subscriptionId,
            undefined, { serialNumber: '123', maxCapacity: 10995116277760 }, 8796093022208, new Date()));
        const clusters = await findClustersToIncreaseLicense();
        expect(clusters).toEqual([
            {
                accountId: '19',
                clusterId: '1',
                licenseCapacityRatio: 0.8,
                maxCapacity: 10995116277760,
                currentCapacity: 8796093022208
            }
        ]);
    });

    test('Account with free trial and cluster with tiering - not subscribed, license capacity ratio is above the threshold, should return a cluster', async () => {
        const accountId = '19';
        const clusterUuid = '1';

        await Account.create(createAccount(accountId, moment().add(3, 'days')));
        await Cluster.create(createCluster(accountId, '1', clusterUuid, 'aws', undefined,
            undefined, { serialNumber: '123', maxCapacity: 10995116277760 }, 8796093022208, new Date()));
        const clusters = await findClustersToIncreaseLicense(config.get('increase-license-capacity-ratio'));
        expect(clusters).toEqual([
            {
                accountId: '19',
                clusterId: '1',
                licenseCapacityRatio: 0.8,
                maxCapacity: 10995116277760,
                currentCapacity: 8796093022208
            }
        ]);
    });

    // TODO CBB ADD test with tieringLicense and not just byol license
});
