const mongoose = require('mongoose');
const zapi = require('../../src/lib/agent/zapi');
const agentZapiOperations = require('../../src/operations/agent-zapi-operations');
const { Account } = require('../../src/models/account');
const { Cluster } = require('../../src/models/cluster');
const { MpSubscription } = require('../../src/models/mp-subscription');
const agent = require('../../src/lib/cloud-manager/agents');
const {
    createLicenseStatusResponse, createClusterInfoResponse, createCluster, createAccount, createSubscription, createAgent
} = require('../testUtils');
const {
    FAILED_STATUS, UNKNOWN_STATUS, ON_STATUS, ONPREM,
    DB_CONNECTION_STRING, SUBSCRIBED,
    TIERING_SERVICE_NAME_SUBSCRIPTION, GCP, AWS,
    AZURE, AGENT_INACTIVE_STATUS, AGENT_ACTIVE_STATUS
} = require('../../src/utils/consts');
const { findInformationAndUpdateCluster, updateClusters } = require('../../src/operations/clusters-collection-operations');
const getLogger = require('../../src/utils/logger/logger');

const logger = getLogger();

jest.mock('../../src/lib/cloud-manager/agents');
jest.mock('../../src/lib/cloud-manager/multi-tenancy');
jest.mock('../../src/lib/agent/zapi');
jest.mock('../../src/operations/multi-tenancy-operation');
jest.mock('../../src/operations/agent-zapi-operations');

beforeAll(async () => {
    try {
        await mongoose.connect(DB_CONNECTION_STRING, { useNewUrlParser: true, dbName: 'cloudTiering' });
    } catch (e) {
        logger.error('Cannot connect to mongoDB');
    }
});

afterAll(async done => {
    try {
        await mongoose.connection.close();
        done();
    } catch (e) {
        logger.error('Error closing connection', e);
    }
});

describe('findInformationAndUpdateCluster', () => {
    const accountId = '12';
    const clusterId = 'a';
    const clusterUuid = 'b';
    const subscriptionId = '1';
    const agentId = 'c';

    beforeEach(async () => {
        await Account.deleteMany();
        await MpSubscription.deleteMany();
        await Cluster.deleteMany();
        await Account.create(createAccount(accountId));
        await MpSubscription.create(createSubscription(accountId, SUBSCRIBED, AWS, subscriptionId, TIERING_SERVICE_NAME_SUBSCRIPTION));
        agentZapiOperations.clusterInfo.mockClear();
    });

    test('cluster has status FAILED- should create a document with minimum details', async () => {
        // mock clusterInfo - failed
        agentZapiOperations.clusterInfo.mockReturnValue(createClusterInfoResponse(clusterId, FAILED_STATUS));
        zapi.licenseStatus.mockReturnValue([]);
        await findInformationAndUpdateCluster({
            clusterId, accountId, agentId, agentLocation: AWS
        });
        const cluster = await Cluster.findOne({ accountId, clusterId }).exec();
        expect(cluster.agentLocation).toEqual(AWS);
        expect(cluster.lastSeen).toBeUndefined();
    });
    test('cluster has status UNKNOWN- should create a document with minimum details', async () => {
        agentZapiOperations.clusterInfo.mockReturnValue(createClusterInfoResponse(clusterId, UNKNOWN_STATUS));
        zapi.licenseStatus.mockReturnValue([]);
        await findInformationAndUpdateCluster({
            clusterId, accountId, agentId, agentLocation: AWS
        });
        const cluster = await Cluster.findOne({ accountId, clusterId }).exec();
        expect(cluster.agentLocation).toEqual(AWS);
        expect(cluster.lastSeen).toBeUndefined();
    });

    test('cluster has status ON but ONTAP version < 9.2 should create a document with subscriptionId but no license or tiering information', async () => {
        agentZapiOperations.clusterInfo.mockReturnValue(createClusterInfoResponse(clusterId, ON_STATUS, clusterUuid, false, AWS, undefined, '9.1'));
        zapi.licenseStatus.mockReturnValue([]);
        await findInformationAndUpdateCluster({
            clusterId, accountId, agentId, agentLocation: AWS
        });
        const cluster = await Cluster.findOne({ accountId, clusterId }).exec();
        expect(cluster.agentLocation).toEqual(AWS);
        expect(cluster.ontapVersion).toEqual('9.1');
        expect(cluster.clusterUuid).toEqual(clusterUuid);
        expect(cluster.lastSeen).toBeTruthy();
    });
    test('AFF cluster has status ON but no tiering license ( no tiering )  should create a document with subscriptionId but no license or tiering information', async () => {
        agentZapiOperations.clusterInfo.mockReturnValue(createClusterInfoResponse(clusterId, ON_STATUS, clusterUuid));
        zapi.licenseStatus.mockReturnValue([]);
        await findInformationAndUpdateCluster({
            clusterId, accountId, agentId, agentLocation: AWS
        });
        const cluster = await Cluster.findOne({ accountId, clusterId }).exec();
        expect(cluster.agentLocation).toEqual(AWS);
        expect(cluster.clusterUuid).toEqual(clusterUuid);
        expect(cluster.lastSeen).toBeTruthy();
    });
    test('CVO cluster has status ON but no tiering license  should create a document with subscriptionId but no license or tiering information', async () => {
        agentZapiOperations.clusterInfo.mockReturnValue(createClusterInfoResponse(clusterId, ON_STATUS, clusterUuid, AWS));
        zapi.licenseStatus.mockReturnValue([]);
        await findInformationAndUpdateCluster({
            clusterId, accountId, agentId, agentLocation: AWS
        });
        const cluster = await Cluster.findOne({ accountId, clusterId }).exec();
        expect(cluster.agentLocation).toEqual(AWS);
        expect(cluster.ontapVersion).toEqual('9.6');
        expect(cluster.clusterUuid).toEqual(clusterUuid);
        expect(cluster.lastSeen).toBeTruthy();
        expect(cluster.currentCapacity).toEqual(920243508);
    });
    test('cluster with byol license, should update byol license, trial should expired', async () => {
        agentZapiOperations.clusterInfo.mockReturnValue(createClusterInfoResponse(clusterId, ON_STATUS, clusterUuid, AWS));
        agentZapiOperations.getFabricpoolLicenses.mockReturnValue(createLicenseStatusResponse(true));
        await findInformationAndUpdateCluster({
            clusterId, accountId, agentId, agentLocation: AWS
        });
        const cluster = await Cluster.findOne({ accountId, clusterId }).exec();
        logger.info('CLUSTER', cluster);
        expect(cluster.agentLocation).toEqual(AWS);
        expect(cluster.ontapVersion).toEqual('9.6');
        expect(cluster.clusterUuid).toEqual(clusterUuid);
        expect(cluster.lastSeen).toBeTruthy();
        expect(cluster.currentCapacity).toEqual(920243508);
        expect(cluster.byolLicense.serialNumber).toEqual('360000099');
        expect(cluster.byolLicense.maxCapacity).toEqual(2199023255552);
        const account = await Account.findOne({ accountId }).exec();
        expect(account.freeTrialExpiryDate.getTime()).toBeLessThan(new Date().getTime());
    });
    test('cluster with tiering license (PAYGO), should update tiering license', async () => {
        agentZapiOperations.clusterInfo.mockReturnValue(createClusterInfoResponse(clusterId, ON_STATUS, clusterUuid, AWS));
        agentZapiOperations.getFabricpoolLicenses.mockReturnValue(createLicenseStatusResponse(false));
        await findInformationAndUpdateCluster({
            clusterId, accountId, agentId, agentLocation: AWS
        });
        const cluster = await Cluster.findOne({ accountId, clusterId }).exec();
        expect(cluster.agentLocation).toEqual(AWS);
        expect(cluster.ontapVersion).toEqual('9.6');
        expect(cluster.clusterUuid).toEqual(clusterUuid);
        expect(cluster.lastSeen).toBeTruthy();
        expect(cluster.currentCapacity).toEqual(920243508);
        expect(cluster.tieringLicense.serialNumber).toEqual('22654892');
        expect(cluster.tieringLicense.maxCapacity).toEqual(2199023255552);
    });
    test('cluster already exists with byol license now it has paygo license, should not delete byol license and add tiering license', async () => {
        await Cluster.create(createCluster(accountId, clusterId, clusterUuid, AWS, subscriptionId, null, {
            serialNumber: '360000099',
            maxCapacity: 1099511627776
        }, 10, new Date()));
        agentZapiOperations.clusterInfo.mockReturnValue(createClusterInfoResponse(clusterId, ON_STATUS, clusterUuid, AWS));
        agentZapiOperations.getFabricpoolLicenses.mockReturnValue(createLicenseStatusResponse(false));
        await findInformationAndUpdateCluster({
            clusterId, accountId, agentId, agentLocation: AWS
        });
        const cluster = await Cluster.findOne({ accountId, clusterId }).exec();
        expect(cluster.agentLocation).toEqual(AWS);
        expect(cluster.ontapVersion).toEqual('9.6');
        expect(cluster.clusterUuid).toEqual(clusterUuid);
        expect(cluster.lastSeen).toBeTruthy();
        expect(cluster.currentCapacity).toEqual(920243508);
        expect(cluster.byolLicense.serialNumber).toEqual('360000099');
        expect(cluster.byolLicense.maxCapacity).toEqual(1099511627776);
        expect(cluster.tieringLicense.serialNumber).toEqual('22654892');
        expect(cluster.tieringLicense.maxCapacity).toEqual(2199023255552);
    });
    test('cluster with gcp tiering, and no subscriptions, should save the db but without subscriptionId', async () => {
        agentZapiOperations.clusterInfo.mockReturnValue(createClusterInfoResponse(clusterId, ON_STATUS, clusterUuid, true, GCP));
        zapi.licenseStatus.mockReturnValue(createLicenseStatusResponse(false));
        await findInformationAndUpdateCluster({
            clusterId, accountId: '16', agentId, agentLocation: GCP
        });
        const cluster = await Cluster.findOne({ accountId: '16', clusterId }).exec();
        expect(cluster.agentLocation).toEqual(GCP);
        expect(cluster.ontapVersion).toEqual('9.6');
        expect(cluster.clusterUuid).toEqual(clusterUuid);
        expect(cluster.cloudProvider).toEqual(GCP);
        expect(cluster.lastSeen).toBeTruthy();
    });
    test('cluster with ONPREM agent and no tiering, should update subscriptionId', async () => {
        agentZapiOperations.clusterInfo.mockReturnValue(createClusterInfoResponse(clusterId, ON_STATUS, clusterUuid, false));
        zapi.licenseStatus.mockReturnValue(createLicenseStatusResponse(false));
        await findInformationAndUpdateCluster({
            clusterId, accountId, agentId, agentLocation: ONPREM
        });
        const cluster = await Cluster.findOne({ accountId, clusterId }).exec();
        expect(cluster.agentLocation).toEqual(ONPREM);
        expect(cluster.ontapVersion).toEqual('9.6');
        expect(cluster.clusterUuid).toEqual(clusterUuid);
        expect(cluster.lastSeen).toBeTruthy();
    });
    test('cluster with AZURE tiering, AZURE subscription, should update subscriptionId', async () => {
        const azureSubscription = 'azureSubscription';
        await MpSubscription.create(createSubscription(accountId, SUBSCRIBED, AZURE, azureSubscription, TIERING_SERVICE_NAME_SUBSCRIPTION));
        agentZapiOperations.clusterInfo.mockReturnValue(createClusterInfoResponse(clusterId, ON_STATUS, clusterUuid, true, AZURE));
        agentZapiOperations.getFabricpoolLicenses.mockReturnValue(createLicenseStatusResponse(false));
        await findInformationAndUpdateCluster({
            clusterId, accountId, agentId, agentLocation: AZURE
        });
        const cluster = await Cluster.findOne({ accountId, clusterId }).exec();
        expect(cluster.agentLocation).toEqual(AZURE);
        expect(cluster.cloudProvider).toEqual(AZURE);
        expect(cluster.ontapVersion).toEqual('9.6');
        expect(cluster.clusterUuid).toEqual(clusterUuid);
        expect(cluster.lastSeen).toBeTruthy();
        expect(cluster.tieringLicense.serialNumber).toEqual('22654892');
        expect(cluster.tieringLicense.maxCapacity).toEqual(2199023255552);
    });
    test('cluster with no tiering but agentLocation is AZURE, should update subscriptionId', async () => {
        const azureSubscription = 'azureSubscription';
        await MpSubscription.create(createSubscription(accountId, SUBSCRIBED, AZURE, azureSubscription, TIERING_SERVICE_NAME_SUBSCRIPTION));
        agentZapiOperations.clusterInfo.mockReturnValue(createClusterInfoResponse(clusterId, ON_STATUS, clusterUuid, false));
        zapi.licenseStatus.mockReturnValue(createLicenseStatusResponse(false));
        await findInformationAndUpdateCluster({
            clusterId, accountId, agentId, agentLocation: AZURE
        });
        const cluster = await Cluster.findOne({ accountId, clusterId }).exec();
        expect(cluster.agentLocation).toEqual(AZURE);
        expect(cluster.ontapVersion).toEqual('9.6');
        expect(cluster.clusterUuid).toEqual(clusterUuid);
        expect(cluster.lastSeen).toBeTruthy();
    });
    test('update cluster with isEligibleForTiering true, make sure isEligibleForTiering is not overrides', async () => {
        await Cluster.create(createCluster(accountId, clusterId, clusterUuid, AWS, subscriptionId, null, {
            serialNumber: '360000099',
            maxCapacity: 1099511627776
        }, 10, new Date(), true));
        agentZapiOperations.clusterInfo.mockReturnValue(createClusterInfoResponse(clusterId, ON_STATUS, clusterUuid, AWS));
        agentZapiOperations.getFabricpoolLicenses.mockReturnValue(createLicenseStatusResponse(false));
        await findInformationAndUpdateCluster({
            clusterId, accountId, agentId, agentLocation: AWS, isEligibleForTiering: true
        });
        const cluster = await Cluster.findOne({ accountId, clusterId }).exec();
        expect(cluster.agentLocation).toEqual(AWS);
        expect(cluster.ontapVersion).toEqual('9.6');
        expect(cluster.clusterUuid).toEqual(clusterUuid);
        expect(cluster.lastSeen).toBeTruthy();
        expect(cluster.currentCapacity).toEqual(920243508);
        expect(cluster.byolLicense.serialNumber).toEqual('360000099');
        expect(cluster.byolLicense.maxCapacity).toEqual(1099511627776);
        expect(cluster.tieringLicense.serialNumber).toEqual('22654892');
        expect(cluster.tieringLicense.maxCapacity).toEqual(2199023255552);
        expect(cluster.isEligibleForTiering).toEqual(true);
    });

    test('update active agents clusters', async () => {
        await Cluster.create(createCluster(accountId, 'a', clusterUuid, AWS, subscriptionId, null, null, null, new Date(), true, 'agentId1'));
        await Cluster.create(createCluster(accountId, 'aa', clusterUuid, AWS, subscriptionId, null, null, null, new Date(), true, 'agentId2'));
        await Cluster.create(createCluster('13', 'aaa', clusterUuid, AWS, subscriptionId, null, null, null, new Date(), true, 'agentId2'));

        const inactiveAgent = createAgent('inactiveAgent', AGENT_INACTIVE_STATUS, 'agentId1');
        const activeAgent = createAgent('activeAgent', AGENT_ACTIVE_STATUS, 'agentId2');

        agent.getAgentsInternal.mockReturnValue({ agents: [inactiveAgent, activeAgent] });

        await updateClusters();
        expect(agentZapiOperations.clusterInfo).toHaveBeenCalledTimes(1);
    });
});
