const mongoose = require('mongoose');
const clusterOperations = require('../../src/operations/cluster-operations');
const agentZapiOperations = require('../../src/operations/agent-zapi-operations');
const { createClusterInfoResponse } = require('../testUtils');
const {
    DB_CONNECTION_STRING,
    OBJECT_STORE_UNAVAILABLE_STATE, AWS, ON_STATUS, FAILED_STATUS,
    OBJECT_STORE_AVAILABLE_STATE, CLOUD_PROVIDERS, AZURE
} = require('../../src/utils/consts');
const { Cluster } = require('../../src/models/cluster');
const getLogger = require('../../src/utils/logger/logger');

const logger = getLogger();

jest.mock('../../src/operations/multi-tenancy-operation');
jest.mock('../../src/operations/agent-zapi-operations');
jest.mock('../../src/lib/cloud-manager/agents');

const aggregatesUnavailabeTiering = [
    {
        name: 'aggr1',
        state: 'online',
        objectStoreName: 'object-store-1',
        isInactiveDataReportingEnabled: true,
        isObjectStoreAttachEligible: false,
        tieringRestriction: {
            isTieringRestricted: false,
            restrictionReason: null
        },
        availabilityState: OBJECT_STORE_UNAVAILABLE_STATE
    },
    {
        name: 'aggr2',
        state: 'online',
        objectStoreName: 'object-store-1',
        isInactiveDataReportingEnabled: true,
        isObjectStoreAttachEligible: false,
        tieringRestriction: {
            isTieringRestricted: false,
            restrictionReason: null
        },
        availabilityState: OBJECT_STORE_UNAVAILABLE_STATE
    }
];
const clusterFailedStatus = createClusterInfoResponse('cluster1',
    FAILED_STATUS, 'cluster1');
const nonCompositeAggregates = [
    {
        name: 'aggr1',
        state: 'online',
        objectStoreName: null,
        isInactiveDataReportingEnabled: true,
        isObjectStoreAttachEligible: false,
        tieringRestriction: {
            isTieringRestricted: false,
            restrictionReason: null
        }
    },
    {
        name: 'aggr2',
        state: 'online',
        objectStoreName: null,
        isInactiveDataReportingEnabled: true,
        isObjectStoreAttachEligible: false,
        tieringRestriction: {
            isTieringRestricted: false,
            restrictionReason: null
        }
    }
];
const clusterWithTiering = createClusterInfoResponse('cluster1', ON_STATUS,
    'cluster1', true);
const compositeAggregates = [
    {
        name: 'aggr1',
        state: 'online',
        objectStoreName: 'object-store-1',
        isInactiveDataReportingEnabled: true,
        isObjectStoreAttachEligible: false,
        tieringRestriction: {
            isTieringRestricted: false,
            restrictionReason: null
        },
        availabilityState: OBJECT_STORE_AVAILABLE_STATE
    },
    {
        name: 'aggr2',
        state: 'online',
        objectStoreName: 'object-store-1',
        isInactiveDataReportingEnabled: true,
        isObjectStoreAttachEligible: false,
        tieringRestriction: {
            isTieringRestricted: false,
            restrictionReason: null
        },
        availabilityState: OBJECT_STORE_AVAILABLE_STATE
    }
];
const clusterInfoNoTiering = createClusterInfoResponse('clusterId', ON_STATUS, 'clusterId', false, AWS, undefined, 9.5);
const eligibleAggregate = {
    name: 'aggr-2',
    state: 'up',
    objectStoreName: 'object-store-3',
    isInactiveDataReportingEnabled: true,
    isObjectStoreAttachEligible: true,
    usedCapacity: {
        size: 1,
        unit: 'TB'
    },
    tieringRestriction: {
        isTieringRestricted: false,
        restrictionReason: null
    },
    availabilityState: 'available'
};
describe('tieringStatus', () => {
    test('should return ON when cluster and connector is on and tiering health is ok', () => {
        const expectedResult = {
            status: 'ON',
            message: undefined
        };
        expect(clusterOperations.tieringStatus(clusterWithTiering, compositeAggregates)).toEqual(expectedResult);
    });

    test('should return ON when cluster and connector is on and no tiering health', () => {
        const expectedResult = {
            status: 'ON',
            message: undefined
        };
        expect(clusterOperations.tieringStatus(clusterInfoNoTiering, nonCompositeAggregates)).toEqual(expectedResult);
    });

    test('should return ON when cluster and connector is on and at least one of aggregates\' tiering health is available', () => {
        const expectedResult = {
            status: 'ON',
            message: undefined
        };
        const aggregateUnavailableHealth = nonCompositeAggregates;
        nonCompositeAggregates[0].availabilityState = OBJECT_STORE_UNAVAILABLE_STATE;
        expect(clusterOperations.tieringStatus(clusterWithTiering, aggregateUnavailableHealth)).toEqual(expectedResult);
    });

    test('should return FAILED when cluster is off', () => {
        const expectedResult = {
            status: 'FAILED',
            message: 'Invalid ONTAP credentials'
        };

        expect(clusterOperations.tieringStatus(clusterFailedStatus, compositeAggregates)).toEqual(expectedResult);
    });

    test('should return NO_TIERING when all tiered aggregates have unavailable state', () => {
        const expectedResult = {
            status: 'NO_TIERING',
            message: 'No connection to the object storage'
        };
        expect(clusterOperations.tieringStatus(clusterWithTiering, nonCompositeAggregates.concat(aggregatesUnavailabeTiering))).toEqual(expectedResult);
    });
});
describe('regionByS3Endpoint', () => {
    test('should get region by s3 endpoint when North Virginia', () => {
        expect(clusterOperations.regionByS3Endpoint('s3.amazonaws.com')).toEqual('US East (N. Virginia)');
    });
    test('should get region by s3 endpoint', () => {
        expect(clusterOperations.regionByS3Endpoint('s3-ap-northeast-1.amazonaws.com')).toEqual('Asia Pacific (Tokyo)');
    });
});
describe('updateClusterColdDataReporting', () => {
    beforeEach(() => {
        agentZapiOperations.enableColdDataReportingOperation.mockClear();
    });

    beforeAll(async () => {
        try {
            await mongoose.connect(DB_CONNECTION_STRING, { useNewUrlParser: true, dbName: 'cloudTiering' });
        } catch (e) {
            logger.error('Cannot connect to mongoDB');
        }
    });

    afterAll(async done => {
        try {
            await mongoose.connection.close();
            done();
        } catch (e) {
            logger.error('Error closing connection', e);
        }
    });

    test('should call enableDataReporting if supported ONTAP version', async () => {
        await clusterOperations.updateClusterColdDataReporting(9.4, 'test456', 'agentId', 'accountId');
        expect(agentZapiOperations.enableColdDataReportingOperation).toHaveBeenCalled();
    });

    test('should skip enableDataReporting if non-supported ONTAP version', async () => {
        await clusterOperations.updateClusterColdDataReporting(9.3, 'clusterId', 'agentId', 'accountId');
        expect(agentZapiOperations.enableColdDataReportingOperation).not.toHaveBeenCalled();
    });
});
describe('findTieringStats', () => {
    beforeAll(async () => {
        try {
            await mongoose.connect(DB_CONNECTION_STRING, { useNewUrlParser: true, dbName: 'cloudTiering' });
        } catch (e) {
            logger.error('Cannot connect to mongoDB');
        }
    });

    afterAll(async done => {
        try {
            await mongoose.connection.close();
            done();
        } catch (e) {
            logger.error('Error closing connection', e);
        }
    });

    beforeEach(() => Cluster.deleteMany({}));

    test('should get tieredCapacity 0- no clusters with tiering', async () => {
        await Cluster.create({
            lastSeen: new Date().getTime(),
            accountId: 'account',
            clusterId: 'ba82ebe6-a366-415b-a051-dbc8bc09bd5f',
            agentLocation: 'AWS',
            cloudProvider: 'AWS',
            clusterName: 'Cluster',
            clusterUuid: '8ced6c36-cdca-4a86-91ae-6cab4e56a172',
            currentCapacity: 0,
            ontapVersion: '9.4'
        });
        expect(await clusterOperations.findTieringStats('account')).toEqual({ tieredCapacity: 0, numberOfClusters: 1 });
    });

    test('should get tieredCapacity - 2 clusters with tiering', async () => {
        await Cluster.create({
            lastSeen: new Date().getTime(),
            accountId: 'account',
            clusterId: 'ba82ebe6-a366-415b-a051-dbc8bc09bd5f',
            agentLocation: 'AWS',
            cloudProvider: 'AWS',
            clusterName: 'Cluster',
            clusterUuid: '8ced6c36-cdca-4a86-91ae-6cab4e56a172',
            currentCapacity: 20,
            ontapVersion: '9.4'
        });
        await Cluster.create({
            lastSeen: new Date().getTime(),
            accountId: 'account',
            clusterId: 'ba82ebe6-a366-415b-a051-dbc8bc09bd51',
            agentLocation: 'AWS',
            cloudProvider: 'AWS',
            clusterName: 'Cluster',
            clusterUuid: '8ced6c36-cdca-4a86-91ae-6cab4e56a172',
            currentCapacity: 10485740,
            ontapVersion: '9.4'
        });

        expect(await clusterOperations.findTieringStats('account')).toEqual({ tieredCapacity: 10485760, numberOfClusters: 2 });
    });

    test('should get tieredCapacity 0 - no clusters at all', async () => {
        expect(await clusterOperations.findTieringStats('account')).toEqual({ tieredCapacity: 0, numberOfClusters: 0 });
    });

    test('should get tieredCapacity - 2 clusters, one with tiering', async () => {
        await Cluster.create({
            lastSeen: new Date().getTime(),
            accountId: 'account',
            clusterId: 'ba82ebe6-a366-415b-a051-dbc8bc09bd5f',
            agentLocation: 'AWS',
            cloudProvider: 'AWS',
            clusterName: 'Cluster',
            clusterUuid: '8ced6c36-cdca-4a86-91ae-6cab4e56a172',
            currentCapacity: 5242880,
            ontapVersion: '9.4'
        });
        await Cluster.create({
            lastSeen: new Date().getTime(),
            accountId: 'account',
            clusterId: 'ba82ebe6-a366-415b-a051-dbc8bc09bd51',
            agentLocation: 'AWS',
            cloudProvider: 'AWS',
            clusterName: 'Cluster',
            clusterUuid: '8ced6c36-cdca-4a86-91ae-6cab4e56a172',
            currentCapacity: 0,
            ontapVersion: '9.4'
        });

        expect(await clusterOperations.findTieringStats('account')).toEqual({ tieredCapacity: 5242880, numberOfClusters: 2 });
    });
});
describe('isClusterEligibleForTiering', () => {
    beforeEach(() => {
        process.env.NODE_ENV = 'test';
    });
    const supportedNodeModels = ['AFF-A800', 'blabla'];
    const unSupportedNodeModels = ['A701S', 'blabla'];
    const supportedIp = '10.68.128.225';
    const unsupportedIp = '1.1.1.1';
    const unsupportedClusterName = 'clusterName';
    const supportedClusterName = 'gcpcvotieringproduction-1';
    const clusterUuid = '123456';
    it('cluster has an old version and agent provider type is AWS  should not be available for tiering', () => {
        expect(clusterOperations.isClusterEligibleForTiering(AWS, [], { nodeModels: [], ontapVersion: 9.1 })).toEqual({ isEligibleForTiering: false, reason: 'You need to upgrade ONTAP to version 9.4 or later' });
    });
    it('cluster has an old version and agent provider type is AZURE should not be available for tiering', () => {
        expect(clusterOperations.isClusterEligibleForTiering(AZURE, [], { nodeModels: [], ontapVersion: 9.3 })).toEqual({ isEligibleForTiering: false, reason: 'You need to upgrade ONTAP to version 9.4 or later' });
    });
    it('cluster has good version and provider type AWS but only hard restricted aggregates should not be availbale for tiering', () => {
        expect(clusterOperations.isClusterEligibleForTiering(AWS, [{
            name: 'hard_restricted_aggr',
            state: 'up',
            objectStoreName: null,
            isInactiveDataReportingEnabled: true,
            isObjectStoreAttachEligible: false,
            usedCapacity: {
                size: 1,
                unit: 'TB'
            },
            tieringRestriction: {
                isTieringRestricted: true,
                restrictionReason: null
            },
            availabilityState: 'available'
        }], { ontapVersion: 9.5, nodeModels: supportedNodeModels }, 'cluster-A')).toEqual({ isEligibleForTiering: false, reason: 'Tiering data from HDDs isn’t currently supported' });
    });
    it('cluster has good version and not good aggregates should not be availbale for tiering', () => {
        expect(clusterOperations.isClusterEligibleForTiering(CLOUD_PROVIDERS.AWS, [{
            name: 'aggr-2',
            state: 'up',
            objectStoreName: null,
            isInactiveDataReportingEnabled: true,
            isObjectStoreAttachEligible: false,
            usedCapacity: {
                size: 1,
                unit: 'TB'
            },
            tieringRestriction: {
                isTieringRestricted: true,
                restrictionReason: 'reason'
            },
            availabilityState: 'available'
        }], { nodeModels: supportedNodeModels, ontapVersion: 9.5 }, supportedClusterName)).toEqual({ isEligibleForTiering: false, reason: 'Tiering data from HDDs isn’t currently supported' });
    });
    it('cluster doesnt have a version - it has status failed should return true - available for tiering', () => {
        expect(clusterOperations.isClusterEligibleForTiering(CLOUD_PROVIDERS.AWS, [], undefined)).toEqual({ isEligibleForTiering: true });
    });

    it('cluster has one node that is part of supported node model list', () => {
        process.env.NODE_ENV = 'production';
        expect(clusterOperations.isClusterEligibleForTiering(CLOUD_PROVIDERS.AWS, [eligibleAggregate], { ontapVersion: 9.5, nodeModels: supportedNodeModels, clusterAddress: unsupportedIp }, unsupportedClusterName)).toEqual({ isEligibleForTiering: true });
    });

    it('cluster has supported name prefix', () => {
        process.env.NODE_ENV = 'production';
        expect(clusterOperations.isClusterEligibleForTiering(CLOUD_PROVIDERS.AWS,
            [eligibleAggregate], {
                nodeModels: unSupportedNodeModels,
                clusterAddress: unsupportedIp,
                ontapVersoin: 9.5
            }, supportedClusterName)).toEqual({ isEligibleForTiering: true });
    });

    it('cluster has supported IP', () => {
        process.env.NODE_ENV = 'production';
        expect(clusterOperations.isClusterEligibleForTiering(CLOUD_PROVIDERS.AWS, [eligibleAggregate],
            { nodeModels: unSupportedNodeModels, ontapVersion: 9.5, clusterAddress: supportedIp }, unsupportedClusterName)).toEqual({ isEligibleForTiering: true });
    });

    it('cluster has unsupported name prefix, unsupportedName and unsupportedNodeModels', () => {
        process.env.NODE_ENV = 'production';
        expect(clusterOperations
            .isClusterEligibleForTiering(CLOUD_PROVIDERS.AWS, [eligibleAggregate],
                {
                    nodeModels: unSupportedNodeModels, ontapVersion: 9.5, clusterAddress: unsupportedIp, clusterUuid
                }, unsupportedClusterName)).toEqual({ isEligibleForTiering: false, reason: 'Data tiering isn’t supported with this on-prem cluster. Refer to the docs for details about supported systems' });
    });

    it('cluster has CVO attributes but staging environment', () => {
        expect(clusterOperations.isClusterEligibleForTiering(CLOUD_PROVIDERS.AWS, [eligibleAggregate],
            {
                nodeModels: unSupportedNodeModels, ontapVersion: 9.5, clusterAddress: unsupportedIp, clusterUuid
            }, unsupportedClusterName))
            .toEqual({ isEligibleForTiering: true });
    });

    it('cluster has undefined parameters', () => {
        process.env.NODE_ENV = 'production';
        expect(clusterOperations.isClusterEligibleForTiering(CLOUD_PROVIDERS.AWS,
            [eligibleAggregate], { ontapVersion: 9.5, clusterUuid }, undefined))
            .toEqual({ isEligibleForTiering: true });
    });
});
