const { getVolumesExtended } = require('../../src/operations/agent-zapi-operations');
const {
    ALL, BACKUP, AUTO, SNAPSHOT_ONLY
} = require('../../src/utils/consts');

jest.mock('../../src/operations/multi-tenancy-operation');
jest.mock('../../src/operations/agent-zapi-operations');

const volumeOperations = require('../../src/operations/volume-operations');

describe('getSupportedVolumes', () => {
    const VOLUME = {
        name: 'vol1',
        aggregateName: 'aggr1',
        svmName: 'svm_onprem',
        size: 3221225472,
        usedSize: 2147483648,
        state: 'online',
        constituentsAggregates: []
    };

    const VOLUME_RESTRICTED = {
        name: 'vol1',
        aggregateName: 'aggr1',
        svmName: 'svm_onprem',
        size: 3221225472,
        usedSize: 2147483648,
        state: 'online',
        constituentsAggregates: [],
        isHardRestricted: true,
        isRestricted: true,
        restrictionReason: 'Hybrid aggregate was detected'
    };

    beforeEach(() => {
        getVolumesExtended.mockClear();
    });

    test('no volumes returns - no volumes', async () => {
        getVolumesExtended.mockReturnValue([]);
        expect(await volumeOperations.getSupportedVolumes('a', 'clusterId')).toEqual([]);
    });

    test('no volumes returns - hard restriction', async () => {
        getVolumesExtended.mockReturnValue([VOLUME_RESTRICTED]);
        expect(await volumeOperations.getSupportedVolumes('a', 'clusterId')).toEqual([]);
    });

    test('return volumes - soft restrictions restriction', async () => {
        getVolumesExtended.mockReturnValue([VOLUME]);
        expect(await volumeOperations.getSupportedVolumes('a', 'clusterId')).toHaveLength(1);
    });

    test('return volumes - no restrictions', async () => {
        getVolumesExtended.mockReturnValue([VOLUME]);
        expect(await volumeOperations.getSupportedVolumes('a', 'clusterId')).toHaveLength(1);
    });

    test('return volumes - with soft restriction - state is offline', async () => {
        VOLUME.state = 'offline';
        getVolumesExtended.mockReturnValue([VOLUME]);
        const volumesWithRestriction = await volumeOperations.getSupportedVolumes('a', 'clusterId');
        expect(volumesWithRestriction).toHaveLength(1);
        expect(volumesWithRestriction[0].tieringRestrictionReason).toBe('Volume’s state is offline. To enable tiering, a volume’s state must be \'online\'');
    });
});

describe('getTieredVolumesSummary', () => {
    const VOLUME = {
        name: 'vol1',
        aggregateName: 'aggr1',
        svmName: 'svm_onprem',
        size: 5368709120,
        usedSize: 7645041786,
        snapshotsUsedSize: 1073741824,
        capacityTierUsedSize: 0,
        tieringPolicy: 'auto',
        constituentsAggregates: [],
        isTiered: true
    };

    const VOLUME2 = {
        name: 'vol2',
        aggregateName: 'aggr2',
        svmName: 'svm_onprem',
        size: 3221225472,
        usedSize: 7645041786,
        snapshotsUsedSize: 1073741824,
        tierInactiveUserData: 3006477107,
        tieringPolicy: 'none',
        constituentsAggregates: []
    };

    const DP_VOLUME = {
        name: 'vol3',
        aggregateName: 'aggr2',
        svmName: 'svm_onprem',
        size: 3221225472,
        usedSize: 7645041786,
        snapshotsUsedSize: 1073741824,
        tierInactiveUserData: 3006477107,
        tieringPolicy: 'none',
        constituentsAggregates: [],
        volumeType: 'dp'
    };

    beforeEach(() => {
        getVolumesExtended.mockClear();
    });

    test('all volumes eligible', () => {
        expect(volumeOperations.getTieredVolumesSummary([VOLUME2, VOLUME2], [VOLUME2, VOLUME2], '9.6', false)).toEqual({
            eligibleVolumes: 2,
            tieredVolumes: 0,
            totalVolumes: 2,
            supportedPolicies: [{
                count: 2, isEstimated: false, name: ALL, savingOpportunity: { capacity: 15290083572, cost: 67 }, supported: { isSupported: true }
            },
            {
                count: 2, isEstimated: false, name: AUTO, savingOpportunity: { capacity: 6012954214, cost: 26 }, supported: { isSupported: true }
            },
            {
                count: 2, isEstimated: false, name: SNAPSHOT_ONLY, savingOpportunity: { capacity: 2147483648, cost: 9 }, supported: { isSupported: true }
            }]
        });
    });

    test('all volumes eligible, some are tiered', () => {
        expect(volumeOperations.getTieredVolumesSummary([VOLUME, VOLUME2, DP_VOLUME], [VOLUME, VOLUME2, DP_VOLUME], '9.4', false)).toEqual({
            eligibleVolumes: 3,
            tieredVolumes: 1,
            totalVolumes: 3,
            supportedPolicies: [{
                count: 2, isEstimated: false, name: AUTO, savingOpportunity: { capacity: 6012954214, cost: 26 }, supported: { isSupported: true }
            },
            {
                count: 3, isEstimated: false, name: SNAPSHOT_ONLY, savingOpportunity: { capacity: 3221225472, cost: 14 }, supported: { isSupported: true }
            },
            {
                count: 1, isEstimated: false, name: BACKUP, savingOpportunity: { capacity: 7645041786, cost: 34 }, supported: { isSupported: true }
            }]

        });
    });
});
