const { URL } = require('url');
const { nth } = require('lodash');
const nock = require('nock');
const {
    AGENT_ENDPOINT, AGENT_API_PREFIX, AGENT_LIST_SGWS_BUCKETS,
    AGENT_CREATE_SGWS_BUCKET, AGENT_SGWS_PATH
} = require('../../../src/utils/consts');
const { USERS, createScenarioForUser } = require('../simulator-state');
const { createSgwsBucketRequestSchema } = require('../../../src/joi-schemas/agent-schemas');
const { validateRequest } = require('../../../src/utils/schema-validation');
const { findUserId, escapeForwardSlash } = require('../simulator-utils');

const sgwsAgentScope = nock(AGENT_ENDPOINT)
// SGWS buckets
    .get(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_LIST_SGWS_BUCKETS}`))
    .delay(100)
    .reply(function (uri) {
        const requestURL = new URL(uri, AGENT_ENDPOINT);
        const host = requestURL.searchParams.get('host');
        if (host === 'failToList.netapp.com') {
            return [504, { statusCode: 504, message: 'no access to the host' }];
        }
        const userId = findUserId(this.req.headers.authorization);

        return USERS[userId] ? [200, USERS[userId].BUCKETS.SGWS] : [400, 'user not found'];
    })
// create bucket
    .post(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_CREATE_SGWS_BUCKET}`), body => {
        try {
            validateRequest(body, createSgwsBucketRequestSchema);
            return true;
        } catch (err) {
            return false;
        }
    })
    .delay(100)
    .reply(function (uri, body) {
        const requestUri = new URL(uri, AGENT_ENDPOINT);
        const accountId = nth(requestUri.pathname.split('/'), 2);
        const userId = findUserId(this.req.headers.authorization);
        if (!USERS[userId]) {
            USERS[userId] = createScenarioForUser(userId, accountId);
        }
        // body has name and region
        const { name, host } = body;
        if (host === 'failToCreate.netapp.com') {
            return [403, { statusCode: 403, message: 'The AWS Access Key Id you provided does not exist in our records' }];
        }
        const bucket = USERS[userId].BUCKETS.SGWS.find(bucketObj => bucketObj.name === name);
        if (bucket) {
            return [400, { message: `A bucket with name ${name} already exist. please try again`, statusCode: 400 }];
        }
        USERS[userId].BUCKETS.SGWS.push({ name });
        return [200, body];
    })
// get object
    .get(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_SGWS_PATH}.+/.+`))
    .query(query => query.host)
    .delay(100)
    .reply(function () {
        const userId = findUserId(this.req.headers.authorization);
        if (USERS[userId]) {
            const { accessKey } = this.req.headers;
            if (accessKey === 'failToGet') {
                return [403, { statusCode: 403, message: 'Unauthorized' }];
            }
            return [200, { body: '' }];
        }
        return [400, 'user not found'];
    })
// delete object
    .delete(escapeForwardSlash(`${AGENT_API_PREFIX}.+/api${AGENT_SGWS_PATH}.+/.+`))
    .query(query => query.host)
    .delay(100)
    .reply(function () {
        const userId = findUserId(this.req.headers.authorization);
        if (USERS[userId]) {
            const { accessKey } = this.req.headers;
            if (accessKey === 'failToDelete') {
                return [403, { statusCode: 403, message: 'Unauthorized' }];
            }
            return [200];
        }
        return [400, 'user not found'];
    })
    // put object
    .put(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_SGWS_PATH}.+/.+`))
    .delay(100)
    .reply(function (uri, body) {
        const userId = findUserId(this.req.headers.authorization);
        if (USERS[userId]) {
            const { accessKey } = body;
            if (accessKey === 'failToPut') {
                return [403, { statusCode: 403, message: 'Unauthorized' }];
            }
            return [200];
        }
        return [400, 'user not found'];
    });

sgwsAgentScope.persist(true);
