const { URL } = require('url');
const {
    get, nth, set, isNil, round
} = require('lodash');
const nock = require('nock');
const {
    AGENT_ZAPI_PATH, AGENT_MODIFY_TIERING_FULLNESS_THRESHOLD_PATH,
    AGENT_DESCRIBE_CLUSTER_VOLUME_FOOTPRINT_PATH, AGENT_DESCRIBE_VOLUME_MINIMUM_COOLING_DAYS_PATH,
    AGENT_DESCRIBE_OBJECT_STORES_PATH,
    AGENT_DESCRIBE_AGGREGATES_PATH, CLOUD_PROVIDER_TO_OCCM_PROVIDER,
    AGENT_DESCRIBE_LIFS_PATH, AGENT_DESCRIBE_SVMS_PATH, AGENT_DESCRIBE_OBJECTSTORE_CONFIG_PATH,
    AGENT_ENDPOINT, AGENT_API_PREFIX, AGENT_API_PROVIDERS_PREFIX,
    AGENT_CLUSTER_INFO_PATH, AGENT_DESCRIBE_NODES_PATH, AGENT_DESCRIBE_SVM_VOLUMES_PATH, AGENT_ENABLE_COLD_DATA_REPORTING_PATH,
    AGENT_MODIFY_VOLUME_MINIMUM_COOLING_DAYS_PATH,
    AGENT_DESCRIBE_ONTAP_VERSION_PATH, NONE, AUTO, SNAPSHOT_ONLY, BACKUP, ALL, SAVING_OPPORTUNITY_ESTIMATION_RATIO, AGENT_LICENSES_PATH
} = require('../../../src/utils/consts');
const { USERS } = require('../simulator-state');
const {
    enableColdDataReportingRequestSchema, modifyTieringFullnessThresholdRequestSchema,
    volumeSchema, aggregateSchema, createObjectStoreConfigRequestSchema,
    attachObjectStoreSchema, modifyVolumeRequestSchema, applyLicenseRequestSchema
} = require('../../../src/joi-schemas/agent-schemas');
const { validateSchema, validateRequest } = require('../../../src/utils/schema-validation');
const { findUserId, escapeForwardSlash } = require('../simulator-utils');
const getLogger = require('../../../src/utils/logger/logger');
const { createScenarioForUser } = require('../simulator-state');

const logger = getLogger('simulator');

const zapiAgentScope = nock(AGENT_ENDPOINT)
    // GET ontapVersion
    .get(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_API_PROVIDERS_PREFIX}${AGENT_ZAPI_PATH}.+${AGENT_DESCRIBE_ONTAP_VERSION_PATH}`))
    .delay(300)
    .reply(function (uri) {
        const requestUri = new URL(uri, AGENT_ENDPOINT);
        const clusterId = nth(requestUri.pathname.split('/'), 7);
        const accountId = nth(requestUri.pathname.split('/'), 2);
        const userId = findUserId(this.req.headers.authorization);
        if (!USERS[userId]) {
            // create new user in case there is no user- get cluster is another entry point to the system
            USERS[userId] = createScenarioForUser(userId, accountId);
        }
        let cluster = Object.values(USERS[userId].CLUSTERS).find(x => x.cluster.publicId === clusterId);
        if (cluster) {
            if (['AFF-FAILED-1', 'CVO-FAILED'].includes(get(cluster, 'cluster.clusterInfo.clusterName'))) {
                return [401, 'Invalid ONTAP credentials'];
            }
            return [200, cluster.cluster.ontapVersion];
        } if (process.env.DEMO) {
            cluster = Object.values(USERS[userId].CLUSTERS).find(x => x.cluster.clusterInfo.clusterName === 'AFF-2');
            const clusterIdToDelete = cluster.cluster.publicId;
            set(USERS, `${userId}.CLUSTERS.${clusterId}`, cluster);
            delete USERS[userId].CLUSTERS[clusterIdToDelete];
            set(cluster, 'cluster.publicId', clusterId);
            return [200, cluster.cluster.ontapVersion];
        }
        return [404, 'cluster not found'];
    })
    // describe cluster info with publicId
    .get(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_API_PROVIDERS_PREFIX}/zapi/.+${AGENT_CLUSTER_INFO_PATH}`))
    .delay(100)
    .reply(function (uri) {
        const requestUri = new URL(uri, AGENT_ENDPOINT);
        const clusterId = nth(requestUri.pathname.split('/'), 7);
        const userId = findUserId(this.req.headers.authorization);
        const cluster = Object.values(USERS[userId].CLUSTERS).find(obj => obj.cluster.publicId === clusterId);
        return [200, cluster.cluster.clusterInfo];
    })
    // describe nodes with publicId
    .get(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_API_PROVIDERS_PREFIX}/zapi/.+${AGENT_DESCRIBE_NODES_PATH}`))
    .reply(function (uri) {
        const requestUri = new URL(uri, AGENT_ENDPOINT);
        const clusterId = nth(requestUri.pathname.split('/'), 7);
        const userId = findUserId(this.req.headers.authorization);
        if (USERS[userId]) {
            const cluster = Object.values(USERS[userId].CLUSTERS).find(obj => obj.cluster.publicId === clusterId);
            return cluster ? [200, cluster.cluster.nodes] : [404, 'cluster was not found'];
        }
        return [404, 'user not found'];
    })
    // modify minimum cooling days
    .put(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_API_PROVIDERS_PREFIX}.+${AGENT_MODIFY_VOLUME_MINIMUM_COOLING_DAYS_PATH}`))
    .reply(function (uri, body) {
        const requestUri = new URL(uri, AGENT_ENDPOINT);
        const clusterId = nth(requestUri.pathname.split('/'), 7);
        const userId = findUserId(this.req.headers.authorization);
        const { volumeName, days } = body;
        const cluster = Object.values(USERS[userId].CLUSTERS).find(obj => obj.cluster.publicId === clusterId);
        if (cluster) {
            const volume = cluster.volumes.find(vol => vol.name === volumeName);
            set(volume, 'minimumCoolingDays', days);
            return [200];
        }
        return [404, 'volume was not found'];
    })
    // enable cold data reporting
    .put(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_API_PROVIDERS_PREFIX}.+${AGENT_ENABLE_COLD_DATA_REPORTING_PATH}`), body => {
        try {
            validateRequest(body, enableColdDataReportingRequestSchema);
            return true;
        } catch (err) {
            return false;
        }
    })
    .reply(function (uri, body) {
        const requestUri = new URL(uri, AGENT_ENDPOINT);
        const clusterId = nth(requestUri.pathname.split('/'), 7);
        const aggregateName = nth(requestUri.pathname.split('/'), 9);
        const userId = findUserId(this.req.headers.authorization);
        const { enable } = body;
        const cluster = Object.values(USERS[userId].CLUSTERS).find(obj => obj.cluster.publicId === clusterId);
        if (cluster) {
            const aggregate = cluster.aggregates.find(aggr => aggr.name === aggregateName);
            if (aggregate) {
                if (aggregateName === 'aggr-failed-to-change-idr') {
                    return [403, { statusCode: 403, error: 'Forbidden', message: `Failed to modify enable_cold_data_reporting of aggregate ${aggregateName}` }];
                }
                if (aggregate.isInactiveDataReportingEnabled === enable) {
                    return [403, { statusCode: 403, error: 'Forbidden', message: `Failed to modify enable_cold_data_reporting of aggregate ${aggregateName}. Reason: ${aggregateName} has inactive data reporting already turned on. Fields that were successfully modified: none` }];
                }
                const objectStore = cluster.objectStores.find(objectStoreObj => objectStoreObj.aggregateName === aggregateName);
                if (objectStore) {
                    return [403, { statusCode: 403, error: 'Forbidden', message: `Failed to modify enable_cold_data_reporting of aggregate ${aggregateName}. Reason: This operation is not allowed on a composite aggregate. Fields that were successfully modified: none` }];
                }
                set(aggregate, 'isInactiveDataReportingEnabled', enable);
                return [200];
            }
            return [404, 'aggregate not found'];
        }
        return [404, 'cluster was not found'];
    })
    // modify tiering fullness threshold
    .put(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_API_PROVIDERS_PREFIX}.+${AGENT_MODIFY_TIERING_FULLNESS_THRESHOLD_PATH}`), body => {
        try {
            validateRequest(body, modifyTieringFullnessThresholdRequestSchema);
            return true;
        } catch (err) {
            return false;
        }
    })
    .reply(function (uri, body) {
        const requestUri = new URL(uri, AGENT_ENDPOINT);
        const clusterId = nth(requestUri.pathname.split('/'), 7);
        const userId = findUserId(this.req.headers.authorization);
        const { aggregateName, objectStoreName, tieringFullnessThreshold } = body;
        const cluster = Object.values(USERS[userId].CLUSTERS).find(obj => obj.cluster.publicId === clusterId);
        if (cluster) {
            const aggregate = cluster.aggregates.find(aggr => aggr.name === aggregateName);
            if (aggregate) {
                if (aggregateName === 'aggr-failed-to-tiering fullness threshold') {
                    return [403, { statusCode: 403, error: 'Forbidden', message: `Failed to modify tiering fullness threshold of aggregate ${aggregateName}` }];
                }
                const objectStore = cluster.objectStores.find(objectStoreObj => objectStoreObj.name === objectStoreName);
                if (objectStore) {
                    set(objectStore, 'tieringFullnessThreshold', tieringFullnessThreshold);
                    return [200];
                }
                return [403, { statusCode: 403, error: 'Forbidden', message: `Failed to modify tiering fullness threshold of aggregate ${aggregateName}. Reason: This operation is not allowed on a composite aggregate. Fields that were successfully modified: none` }];
            }
            return [404, 'aggregate not found'];
        }
        return [404, 'cluster was not found'];
    })
    // describe lifs
    .get(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_API_PROVIDERS_PREFIX}.+${AGENT_DESCRIBE_LIFS_PATH}`))
    .reply(function (uri) {
        const requestUri = new URL(uri, AGENT_ENDPOINT);
        const clusterId = nth(requestUri.pathname.split('/'), 7);
        const userId = findUserId(this.req.headers.authorization);
        const cluster = Object.values(USERS[userId].CLUSTERS).find(x => x.cluster.publicId === clusterId);
        return (cluster) ? [200, cluster.cluster.lifs] : [404, 'cluster was not found'];
    })
    // describe svms
    .get(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_API_PROVIDERS_PREFIX}.+${AGENT_DESCRIBE_SVMS_PATH}`))
    .reply(function (uri) {
        const requestUri = new URL(uri, AGENT_ENDPOINT);
        const clusterId = nth(requestUri.pathname.split('/'), 7);
        const userId = findUserId(this.req.headers.authorization);
        const cluster = Object.values(USERS[userId].CLUSTERS).find(x => x.cluster.publicId === clusterId);
        if (['AFF-FAILED-1', 'CVO-FAILED'].includes(get(cluster, 'cluster.clusterInfo.clusterName'))) {
            return [401, 'Invalid ONTAP credentials'];
        }
        return cluster ? [200, cluster.svms] : [504, 'cluster was not found'];
    })
    // describe object store config
    .get(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_API_PROVIDERS_PREFIX}.+${AGENT_DESCRIBE_OBJECTSTORE_CONFIG_PATH}`))
    .reply(function (uri) {
        const requestUri = new URL(uri, AGENT_ENDPOINT);
        const clusterId = nth(requestUri.pathname.split('/'), 7);
        const userId = findUserId(this.req.headers.authorization);
        const cluster = Object.values(USERS[userId].CLUSTERS).find(x => x.cluster.publicId === clusterId);
        return cluster ? [200, cluster.objectStoresConfig] : [404, 'cluster was not found'];
    })
    // describe aggregates
    .get(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_API_PROVIDERS_PREFIX}.+${AGENT_DESCRIBE_AGGREGATES_PATH}`))
    .reply(function (uri) {
        const requestUri = new URL(uri, AGENT_ENDPOINT);
        const clusterId = nth(requestUri.pathname.split('/'), 7);
        const userId = findUserId(this.req.headers.authorization);
        const cluster = USERS[userId].CLUSTERS[clusterId];
        return cluster ? [200, cluster.aggregates] : [404, 'cluster was not found'];
    }) // describe object stores
    .get(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_API_PROVIDERS_PREFIX}.+${AGENT_DESCRIBE_OBJECT_STORES_PATH}`))
    .reply(function (uri) {
        const requestUri = new URL(uri, AGENT_ENDPOINT);
        const clusterId = nth(requestUri.pathname.split('/'), 7);
        const userId = findUserId(this.req.headers.authorization);
        const cluster = USERS[userId].CLUSTERS[clusterId];
        return cluster ? [200, cluster.objectStores] : [404, 'cluster was not found'];
    })
    // get volumes
    .get(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_API_PROVIDERS_PREFIX}.+${AGENT_DESCRIBE_SVM_VOLUMES_PATH}`))
    .reply(function (uri) {
        const requestUri = new URL(uri, AGENT_ENDPOINT);
        const clusterId = nth(requestUri.pathname.split('/'), 7);
        const svmName = nth(requestUri.pathname.split('/'), 8);
        const userId = findUserId(this.req.headers.authorization);
        const cluster = USERS[userId].CLUSTERS[clusterId];
        return (cluster) ? [200, cluster.volumes.filter(volume => volume.svmName === svmName)] : [404, 'cluster was not found'];
    })
    // get volumes footprints
    .get(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_API_PROVIDERS_PREFIX}.+${AGENT_DESCRIBE_CLUSTER_VOLUME_FOOTPRINT_PATH}`))
    .reply(function (uri) {
        const requestUri = new URL(uri, AGENT_ENDPOINT);
        const clusterId = nth(requestUri.pathname.split('/'), 7);
        const userId = findUserId(this.req.headers.authorization);
        const cluster = USERS[userId].CLUSTERS[clusterId];
        return cluster ? [200, cluster.volumeFootprints] : [404, 'cluster was not found'];
    })
    // get volumes minimum cooling days
    .get(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_API_PROVIDERS_PREFIX}.+${AGENT_DESCRIBE_VOLUME_MINIMUM_COOLING_DAYS_PATH}`))
    .reply(function (uri) {
        const requestUri = new URL(uri, AGENT_ENDPOINT);
        const clusterId = nth(requestUri.pathname.split('/'), 7);
        const userId = findUserId(this.req.headers.authorization);
        const cluster = USERS[userId].CLUSTERS[clusterId];
        const { volumes } = cluster;
        const cliOutput = volumes.reduce((accumulator, volume) => {
            const { minimumCoolingDays, svmName, name } = volume;
            const minimumCoolingDaysToPrint = isNil(minimumCoolingDays) ? '-' : minimumCoolingDays;
            accumulator += `${svmName}\t${name}\t${minimumCoolingDaysToPrint}\n`;
            return accumulator;
        }, 'vserver\tvolume\ttiering-minimum-cooling-days\n------- ----------------------------\n');
        return cluster ? [200, { cliOutput }] : [404, 'cluster was not found'];
    })
    // attach object store
    .post(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_API_PROVIDERS_PREFIX}.+${AGENT_DESCRIBE_OBJECT_STORES_PATH}`), body => {
        try {
            validateSchema(body, attachObjectStoreSchema);
            return true;
        } catch (err) {
            return false;
        }
    })
    .reply(function (uri, body) {
        const requestUri = new URL(uri, AGENT_ENDPOINT);
        const clusterId = nth(requestUri.pathname.split('/'), 7);
        const userId = findUserId(this.req.headers.authorization);
        if (USERS[userId]) {
            const {
                checkOnly, objectStoreName, aggregateName
            } = body;
            const cluster = get(USERS, `${userId}.CLUSTERS.${clusterId}`);
            if (cluster) {
                const { aggregates, objectStores, objectStoresConfig } = cluster;
                const aggregate = aggregates.find(aggr => aggr.name === aggregateName);
                if (!aggregate) {
                    return [404, `aggregate ${aggregateName} was not found`];
                }
                if (aggregate.name === 'hard_restricted_aggr') {
                    return [200, {
                        isAggregateRestricted: true,
                        reason: 'Hard restricted aggregate'
                    }];
                }
                if (aggregate.name === 'soft_restricted_aggr') {
                    return [200, {
                        isAggregateRestricted: true,
                        reason: 'Hybrid aggregate was detected'
                    }];
                }
                if (aggregate.name === 'aggr_soft_restricted_flexgroup') {
                    return [200, {
                        isAggregateRestricted: true,
                        reason: 'Volume guarantee other than none'
                    }];
                }
                if (!checkOnly) {
                    if (aggregate.attachObjectStoreFailure) {
                        return [403, { message: `Failed to attach object store to aggregate ${aggregateName}`, statusCode: 403 }];
                    }
                    const objectStoreConfig = objectStoresConfig.find(objectStoreConfigObj => objectStoreConfigObj.objectStoreName === objectStoreName);
                    if (objectStoreConfig) {
                        objectStores.push({
                            availabilityState: 'available',
                            aggregateName,
                            name: objectStoreName,
                            usedSpace: objectStoreConfig.usedSpace,
                            tieringFullnessThreshold: 50
                        });
                        set(USERS, `${userId}.CLUSTERS.${clusterId}.objectStores`, objectStores);
                        aggregate.isObjectStoreAttachEligible = false;
                        aggregate.isComposite = true;
                        return [200];
                    }
                    return [404, { message: 'object store was not found', statusCode: 404 }];
                }
                logger.info('Check only');
                return [200];
            }
        } else {
            return [404, { message: 'user not found', statusCode: 404 }];
        }
    })
    // create object store config
    .post(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_API_PROVIDERS_PREFIX}.+${AGENT_DESCRIBE_OBJECTSTORE_CONFIG_PATH}`), body => {
        try {
            validateSchema(body, createObjectStoreConfigRequestSchema);
            return true;
        } catch (err) {
            return false;
        }
    })
    .reply(function (uri, requestBody) {
        const requestUri = new URL(uri, AGENT_ENDPOINT);
        const clusterId = nth(requestUri.pathname.split('/'), 7);
        const userId = findUserId(this.req.headers.authorization);
        const cluster = Object.values(USERS[userId].CLUSTERS).find(x => x.cluster.publicId === clusterId);
        const {
            objectStoreName,
            objectStoreProviderType: providerType,
            server,
            s3Name,
            azureAccount
        } = requestBody;
        if (cluster) {
            if (get(cluster, 'cluster.clusterInfo.clusterUuid') === '10bec0b3-6d82-454f-878a-434cf5158b90') {
                return [403, 'Cannot configure fabric pool resources'];
            }
            const objectStoreConfig = {
                objectStoreName,
                s3Name,
                usedSpace: 0,
                providerType,
                server: (providerType === CLOUD_PROVIDER_TO_OCCM_PROVIDER.AZURE) ? `${azureAccount}.${server}` : server
            };
            USERS[userId].CLUSTERS[clusterId].objectStoresConfig.push(objectStoreConfig);
            return [200];
        }
        return [404, 'Cluster for fabric pool not found'];
    })
    // modify volume
    .put(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_API_PROVIDERS_PREFIX}.+${AGENT_DESCRIBE_SVM_VOLUMES_PATH}`), body => {
        try {
            validateSchema(body, modifyVolumeRequestSchema);
            return true;
        } catch (err) {
            return false;
        }
    })
    .reply(function (uri, requestBody) {
        const requestUri = new URL(uri, AGENT_ENDPOINT);
        const userId = findUserId(this.req.headers.authorization);
        const clusterId = nth(requestUri.pathname.split('/'), 7);
        const svmName = nth(requestUri.pathname.split('/'), 8);
        const volumeName = nth(requestUri.pathname.split('/'), 10);
        if (USERS[userId]) {
            const { tieringPolicy } = requestBody;
            if (get(USERS, `${userId}.CLUSTERS.${clusterId}.volumes`)) {
                const indexOfVolumeToModify = USERS[userId].CLUSTERS[clusterId].volumes.findIndex(volume => volume.name === volumeName && volume.svmName === svmName);
                const indexOfVolumeFootPrintToModify = USERS[userId].CLUSTERS[clusterId].volumeFootprints.findIndex(volume => volume.name === volumeName && volume.svmName === svmName);
                if (indexOfVolumeToModify > -1 && indexOfVolumeFootPrintToModify > -1) {
                    const volumeToModify = USERS[userId].CLUSTERS[clusterId].volumes[indexOfVolumeToModify];
                    const volumeFootPrintToModify = USERS[userId].CLUSTERS[clusterId].volumeFootprints[indexOfVolumeFootPrintToModify];
                    // modify tiering policy failure
                    if (tieringPolicy !== NONE && volumeToModify.modifyVolumeFailure) {
                        return [403, { message: 'failed to change volume tiering policy', statusCode: 403 }];
                    }
                    // changing tieringpolicy
                    volumeToModify.tieringPolicy = tieringPolicy;
                    const objectStoreConfig = get(USERS, `${userId}.CLUSTERS.${clusterId}.objectStoresConfig[0]`);
                    const objectStoreExtendedTieredCapacityBytes = get(USERS, `${userId}.CLUSTERS.${clusterId}.objectStoresConfig[0].usedSpace`, 0);
                    const usedSize = get(volumeFootPrintToModify, 'performanceTierFootprint', volumeToModify.parentVolumeName ? volumeToModify.usedSize : volumeToModify.physicalUsed);
                    const license = get(USERS, `${userId}.CLUSTERS.${clusterId}.license`);
                    // modify volume tiered capacity

                    if (tieringPolicy === AUTO) {
                        // changing tierSize to be cold data
                        volumeFootPrintToModify.capacityTierFootprint = isNil(volumeToModify.tierInactiveUserData) ? (usedSize * SAVING_OPPORTUNITY_ESTIMATION_RATIO) : volumeToModify.tierInactiveUserData;
                        const volumeTierInactiveUserDataBytes = volumeToModify.tierInactiveUserData ? volumeToModify.tierInactiveUserData : usedSize * SAVING_OPPORTUNITY_ESTIMATION_RATIO;
                        // adding usedSpace to object store config

                        set(USERS, `${userId}.CLUSTERS.${clusterId}.objectStoresConfig[0].usedSpace`, objectStoreExtendedTieredCapacityBytes + volumeTierInactiveUserDataBytes);

                        // modify license
                        if (license) {
                            logger.info('license', license);
                            USERS[userId].CLUSTERS[clusterId].license[0].currentCapacity = license[0].currentCapacity + (volumeTierInactiveUserDataBytes / (1024 ** 3));
                        }
                        // modify volume cold data - now cold data is 0
                        volumeToModify.tierInactiveUserData = 0;
                        // modify minimum cooling days - putting the default on 31
                        volumeToModify.minimumCoolingDays = 31;
                    } else if (tieringPolicy === SNAPSHOT_ONLY) {
                        // modify volume tiered capacity
                        volumeFootPrintToModify.capacityTierFootprint = volumeToModify.snapshotsUsedSize;
                        // modify volume cold data

                        volumeToModify.tierInactiveUserData = (volumeToModify.tierInactiveUserData || 0) - (volumeToModify.snapshotsUsedSize || 0);

                        if (volumeToModify.tierInactiveUserData < 0) {
                            volumeToModify.tierInactiveUserData = 0;
                        }
                        // modify object store on cluster
                        set(USERS, `${userId}.CLUSTERS.${clusterId}.objectStoresConfig[0].usedSpace`, objectStoreExtendedTieredCapacityBytes + (volumeToModify.snapshotsUsedSize || 0));

                        // modify license
                        if (license) {
                            // modify object store on cluster
                            USERS[userId].CLUSTERS[clusterId].license[0].currentCapacity = license[0].currentCapacity + ((volumeToModify.snapshotsUsedSize / (1024 ** 3)) || 0);
                        }
                        // // modify volume snapshot used size - ontap changes automatically
                        indexOfVolumeToModify.minimumCoolingDays = 2;
                    } else if ([BACKUP, ALL].includes(tieringPolicy)) {
                        // modify volume tiered capacity
                        volumeFootPrintToModify.capacityTierFootprint = usedSize;

                        // modify volume cold data
                        volumeToModify.tierInactiveUserData = 0;
                        // modify license
                        if (objectStoreConfig) {
                            set(USERS, `${userId}.CLUSTERS.${clusterId}.objectStoresConfig[0].usedSpace`, objectStoreExtendedTieredCapacityBytes + usedSize);
                        }
                        if (license) {
                            USERS[userId].CLUSTERS[clusterId].license[0].currentCapacity = license[0].currentCapacity + (volumeToModify.usedSize / (1024 ** 3));
                        }
                    }

                    volumeToModify.tierInactiveUserDataPercent = (round(volumeToModify.tierInactiveUserData / volumeToModify.sizeTotal * 100)) || 0;

                    USERS[userId].CLUSTERS[clusterId].volumes[indexOfVolumeToModify] = volumeToModify;
                    USERS[userId].CLUSTERS[clusterId].volumeFootprints[indexOfVolumeFootPrintToModify] = volumeFootPrintToModify;

                    return [200];
                }

                return [404, 'volume not found'];
            }
        } else {
            return [404, 'user not found'];
        }
    })
    // get licenses status
    .get(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_API_PROVIDERS_PREFIX}.+${AGENT_LICENSES_PATH}`))
    .reply(function (uri) {
        const requestUri = new URL(uri, AGENT_ENDPOINT);
        const clusterId = nth(requestUri.pathname.split('/'), 7);
        const userId = findUserId(this.req.headers.authorization);
        const cluster = Object.values(USERS[userId].CLUSTERS).find(x => x?.cluster?.publicId === clusterId);
        if (cluster) {
            if (cluster.license) {
                return [200, cluster.license];
            }
            return [200, []];
        }
        return [404, 'cluster was not found'];
    })

    // applyLicense
    .put(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_API_PROVIDERS_PREFIX}.+${AGENT_LICENSES_PATH}`), body => {
        try {
            validateSchema(body, applyLicenseRequestSchema);
            return true;
        } catch (err) {
            return false;
        }
    })
    .delay(100)
    .reply(function (uri, requestBody) {
        const requestUri = new URL(uri, AGENT_ENDPOINT);
        const clusterId = nth(requestUri.pathname.split('/'), 7);
        const userId = findUserId(this.req.headers.authorization);
        const body = requestBody;
        if (!userId.includes('clients')) { // user triggered
            const cluster = Object.values(USERS[userId].CLUSTERS).find(x => x.cluster.publicId === clusterId);
            if (cluster) {
                if (cluster.license) {
                    cluster.license[0].serialNumber = body.statusResp.serialNumber;
                    cluster.license[0].maxCapacity = body.statusResp.licenses.capacity * 1024;
                } else {
                    const license = [{
                        serialNumber: body.statusResp.serialNumber,
                        maxCapacity: body.statusResp.licenses.capacity * 1024,
                        currentCapacity: 0,
                        package: 'fabricpool'
                    }];
                    set(USERS, `${userId}.CLUSTERS.${clusterId}.license`, license);
                }
                return [200];
            }
            return [404, 'cluster was not found'];
        }
        const allClustersOfAllUsers = Object.values(USERS).map(user => user.CLUSTERS); // offline operation

        const clustersWithRequestedCluster = allClustersOfAllUsers.find(clusters => clusters[clusterId]);
        if (clustersWithRequestedCluster) {
            let { license } = clustersWithRequestedCluster[clusterId];
            if (license) {
                license[0].maxCapacity = body.statusResp.licenses.capacity * 1024;
                license[0].serialNumber = body.statusResp.serialNumber;
            } else {
                license = [{
                    serialNumber: body.statusResp.serialNumber,
                    maxCapacity: body.statusResp.licenses.capacity * 1024,
                    currentCapacity: 0,
                    package: 'fabricpool'
                }];
            }
            clustersWithRequestedCluster[clusterId].license = license;
            return [200];
        }
        return [404];
    })
// simulator
// create aggregate
    .post(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_API_PROVIDERS_PREFIX}.+${AGENT_DESCRIBE_AGGREGATES_PATH}`), body => {
        try {
            validateRequest(body, aggregateSchema);
            return true;
        } catch (err) {
            return false;
        }
    })
    .reply(function (uri, body) {
        const requestUri = new URL(uri, AGENT_ENDPOINT);
        const clusterId = nth(requestUri.pathname.split('/'), 7);
        const userId = findUserId(this.req.headers.authorization);
        const cluster = USERS[userId].CLUSTERS[clusterId];
        if (cluster) {
            cluster.aggregates.push(body);
            return [200];
        }
        return [404, 'cluster was not found'];
    })
    // create volume
    .post(escapeForwardSlash(`${AGENT_API_PREFIX}.+${AGENT_API_PROVIDERS_PREFIX}.+${AGENT_DESCRIBE_SVM_VOLUMES_PATH}`), body => {
        try {
            validateRequest(body, volumeSchema);
            return true;
        } catch (err) {
            return false;
        }
    })
    .reply(function (uri, body) {
        const requestUri = new URL(uri, AGENT_ENDPOINT);
        const clusterId = nth(requestUri.pathname.split('/'), 7);
        const userId = findUserId(this.req.headers.authorization);
        const cluster = USERS[userId].CLUSTERS[clusterId];
        if (cluster) {
            cluster.volumes.push(body);
            cluster.volumeFootprints.push({ name: body.name, svmName: body.svmName });
            return [200];
        }
        return [404, 'cluster was not found'];
    })
    .defaultReplyHeaders({
        'Cache-Control': 'public,max-age=10,smax-age=10'
    });
zapiAgentScope.persist(true);
