const { URL } = require('url');
const nock = require('nock');
const {
    last, get
} = require('lodash');
const { validateResponse } = require('../../../src/utils/schema-validation');
const {
    clusterResponseSchema
} = require('../../../src/joi-schemas/cloud-manager-schamas');
const {
    CLOUD_MANAGER_ENDPOINT, CLOUD_MANAGER_API_PREFIX, CLOUD_MANAGER_CLUSTERS_PATH_PREFIX
} = require('../../../src/utils/consts');
const { escapeForwardSlash, findUserId } = require('../simulator-utils');
const { reset, USERS, createScenarioForUser } = require('../simulator-state');
const { capacityToBytes } = require('../../../src/utils/utils');

const clustersScope = nock(CLOUD_MANAGER_ENDPOINT)
    // create cluster + discover- once we dont have standalone each cluster we create exists on the workspace
    .post(`${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, body => {
        try {
            validateResponse(body, clusterResponseSchema);
            return true;
        } catch (err) {
            return false;
        }
    })
    .delay(100)
    .reply(200, function (uri, body) {
        const {
            publicId: clusterId, ontapClusterProperties, capacityFeatures, name, agentId, resourceClass, accountId, workspaceId
        } = body;
        const userId = findUserId(this.req.headers.authorization);
        if (!USERS[userId]) {
            USERS[userId] = createScenarioForUser(userId, accountId);
        }
        const clusterInfo = {
            clusterUuid: ontapClusterProperties.clusterUuid,
            clusterName: name
        };
        const clustermanagementip = ontapClusterProperties.clusterAddress;
        const lifs = get(ontapClusterProperties, 'lifs', []);
        const version = ontapClusterProperties.ontapVersion.split('.');
        const ontapVersion = { generation: version[0], major: version[1], minor: version[2] || '0' };
        lifs.push({
            interfaceName: 'management',
            address: clustermanagementip,
            status: 'up',
            role: 'cluster_mgmt',
            ipSpace: 'ipSpace'
        });
        const objectStoreConfig = get(capacityFeatures, 'objectStoreConfigExtended.objectStoreConfig');
        if (objectStoreConfig) {
            objectStoreConfig.usedSpace = capacityToBytes(objectStoreConfig.usedSpace);
        }
        USERS[userId].CLUSTERS[clusterId] = {
            cluster: {
                ontapVersion,
                clusterInfo,
                lifs,
                publicId: clusterId,
                clustermanagementip,
                nodes: get(ontapClusterProperties, 'nodeModels', [{
                    nodeName: 'aff-01',
                    model: 'AFF8040',
                    isNodeHealthy: true,
                    isClusterEligible: true
                }, {
                    nodeName: 'aff-02', model: 'AFF8040', isNodeHealthy: true, isClusterEligible: true
                }])
            },
            volumes: [],
            volumeFootprints: [],
            aggregates: [],
            svms: [],
            objectStoresConfig: objectStoreConfig ? [objectStoreConfig] : [],
            objectStores: [],
            workspaceId,
            agentId,
            resourceClass
        };
        return body;
    })

    // delete/remove cluster
    .delete(escapeForwardSlash(`${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/.+`))
    .delay(100)
    .reply(function (uri) {
        const requestURL = new URL(uri, CLOUD_MANAGER_ENDPOINT);
        const publicId = last(requestURL.pathname.split('/'));
        const userId = findUserId(this.req.headers.authorization);
        if (USERS[userId]) {
            if (USERS[userId].CLUSTERS[publicId]) {
                delete USERS[userId].CLUSTERS[publicId];
                return [200];
            }
            return [404,
                {
                    message: `Cluster ${publicId} was not found`,
                    causeMessage:
                        `Cluster ${publicId} was not found`
                }];
        }
        return [404, 'USER was not found'];
    })

    // delete/remove all clusters
    .delete(escapeForwardSlash(`${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`))
    .delay(100)
    .reply(200, function () {
        if (this.req.headers.authorization) {
            const userId = findUserId(this.req.headers.authorization);
            reset('CLUSTERS', userId);
            return [200];
        }
        reset('CLUSTERS');
        return [200];
    });

clustersScope.persist(true);
