const { URL } = require('url');
const hyperid = require('hyperid');
const nock = require('nock');
const {
    last, nth, flatten, get
} = require('lodash');
const {
    MULTI_TENANCY_ENDPOINT, API_PATH_ACCOUNTS, API_PATH_RESOURCES,
    API_PATH_RESOURCES_INTERNAL, API_PATH_USERS_OF_ACCOUNT_INTERNAL
} = require('../../../src/utils/consts');
const { escapeForwardSlash, findUserId } = require('../simulator-utils');
const getLogger = require('../../../src/utils/logger/logger');

const logger = getLogger('simulator');
const {
    USERS, createScenarioForUser, overrideWorkspaceId, overrideAccountId
} = require('../simulator-state');

const multiTenancyScope = nock(MULTI_TENANCY_ENDPOINT)
    // get accounts
    .get(API_PATH_ACCOUNTS)
    .delay(100)
    .reply(function () {
        const userId = findUserId(this.req.headers['x-user-token']);
        const accountId = this.req.headers['x-account-id'];
        const createEmptyScenario = this.req.headers['empty-scenario'];
        if (!USERS[userId] && !userId.includes('clients')) {
            USERS[userId] = createEmptyScenario ? {
                ACCOUNTS: { },
                CLUSTERS: {},
                AGENTS: {},
                SUBSCRIPTIONS: {}
            } : createScenarioForUser(userId, accountId);
        }

        if (accountId) {
            overrideAccountId(userId, accountId);
        }
        const accounts = [];
        Object.keys(USERS[userId].ACCOUNTS).forEach(accountKey => accounts.push(USERS[userId].ACCOUNTS[accountKey].account));
        const shouldFail = accounts.some(account => account.shouldFail);
        return shouldFail ? [504, 'Timeout exception'] : [200, JSON.stringify(accounts)];
    })

// create workspace
    .post(escapeForwardSlash(`${API_PATH_ACCOUNTS}/.+/.+`))
    .delay(100)
    .reply(function (uri) {
        const requestUri = new URL(uri, MULTI_TENANCY_ENDPOINT);
        const accountId = nth(requestUri.pathname.split('/'), 4);
        const userId = findUserId(this.req.headers['x-user-token']);
        if (USERS[userId] && USERS[userId].ACCOUNTS[accountId]) {
            const workspaceName = nth(requestUri.pathname.split('/'), 5);
            const workspace = {
                workspacePublicId: hyperid().uuid,
                workspaceName
            };

            if (USERS[userId].ACCOUNTS[accountId].workspaces) {
                USERS[userId].ACCOUNTS[accountId].workspaces.push(workspace);
            } else {
                USERS[userId].ACCOUNTS[accountId].workspaces = [workspace];
            }
            return [200, workspace];
        }
        return [404, 'No account found'];
    })

    // create account
    .post(escapeForwardSlash(`${API_PATH_ACCOUNTS}/.+`))
    .delay(100)
    .reply(200, function (uri) {
        const userId = findUserId(this.req.headers['x-user-token']);
        const createEmptyScenario = this.req.headers['empty-scenario'];
        const accountId = hyperid().uuid;
        if (!USERS[userId] && !userId.includes('clients')) {
            USERS[userId] = createEmptyScenario ? {
                ACCOUNTS: { },
                CLUSTERS: {},
                AGENTS: {},
                SUBSCRIPTIONS: {}
            } : createScenarioForUser(userId, accountId);
        }
        const requestUri = new URL(uri, MULTI_TENANCY_ENDPOINT);
        const account = {
            accountName: last(requestUri.pathname.split('/')),
            accountPublicId: accountId
        };
        if (USERS[userId].ACCOUNTS) {
            USERS[userId].ACCOUNTS[account.accountPublicId] = { account, workspaces: [] };
        } else {
            USERS[userId].ACCOUNTS = { [account.accountPublicId]: { account, workspaces: [] } };
        }
        return { accountPublicId: account.accountPublicId };
    })

    //  account users
    .get(escapeForwardSlash(`${API_PATH_USERS_OF_ACCOUNT_INTERNAL}`))
    .query(query => query.account_id && query.only_admins)
    .delay(100)
    .reply(200, JSON.stringify([
        {
            name: 'a',
            email: 'a@netapp.com',
            company: 'NetApp'
        }
    ]))

    // get workspaces
    .get(escapeForwardSlash(`${API_PATH_ACCOUNTS}/.+/workspace`))
    .delay(100)
    .reply(200, function (uri) {
        const requestUri = new URL(uri, MULTI_TENANCY_ENDPOINT);
        const accountId = nth(requestUri.pathname.split('/'), 4);
        const userId = findUserId(this.req.headers['x-user-token']);
        if (USERS[userId].ACCOUNTS[accountId]) {
            return JSON.stringify(USERS[userId].ACCOUNTS[accountId].workspaces ? USERS[userId].ACCOUNTS[accountId].workspaces : []);
        }
        return [404, 'account not found'];
    })

    // get resources
    .get(API_PATH_RESOURCES)
    .query(query => query.account)
    .delay(100)
    .reply(function (uri) {
        const requestURL = new URL(uri, MULTI_TENANCY_ENDPOINT);
        const tenantId = requestURL.searchParams.get('workspace');
        const accountId = requestURL.searchParams.get('account');
        const resourceClass = requestURL.searchParams.get('resource-class');
        const userId = findUserId(this.req.headers['x-user-token']);
        overrideAccountId(userId, accountId);
        if (get(USERS, `${userId}.ACCOUNTS.${accountId}.workspaces`)) {
            if (tenantId) {
                overrideWorkspaceId(userId, accountId, tenantId);
                if (USERS[userId].ACCOUNTS[accountId].workspaces.find(workspace => workspace.workspacePublicId === tenantId)) {
                    const clusters = Object.values(USERS[userId].CLUSTERS).filter(data => data.workspaceId === tenantId);
                    const shouldFail = clusters.some(cluster => cluster.cluster.clustermanagementip === '100.100.100.100');
                    if (shouldFail) {
                        return [504, 'timoutException'];
                    }
                    return [200, clusters.filter(cluster => {
                        if (resourceClass) {
                            return cluster.resourceClass === resourceClass && cluster.agentId;
                        }
                        return cluster.agentId;
                    }).map(cluster => ({
                        resourceIdentifier: cluster.cluster.publicId,
                        agentId: cluster.agentId,
                        resourceClass: cluster.resourceClass
                    }))];
                }
                return [404, 'workspace not found'];
            }
            logger.info('get resources by account', accountId, USERS[userId].ACCOUNTS[accountId]);
            const workspaces = USERS[userId].ACCOUNTS[accountId].workspaces.map(workspace => workspace.workspacePublicId);
            const clusters = Object.values(USERS[userId].CLUSTERS).filter(data => workspaces.includes(data.workspaceId));
            const shouldFail = clusters.some(cluster => cluster.cluster.lifs.find(lif => lif.address === '100.100.100.100'));
            if (shouldFail) {
                return [504, 'timoutException'];
            }
            return [200, clusters.filter(cluster => {
                if (resourceClass) {
                    return cluster.resourceClass === resourceClass && cluster.agentId;
                }
                return cluster.agentId;
            }).map(cluster => ({
                resourceIdentifier: cluster.cluster.publicId,
                agentId: cluster.agentId,
                resourceClass: cluster.resourceClass
            }))];
        }
        return [404];
    })
    // get resources
    .get(API_PATH_RESOURCES_INTERNAL)
    .query(query => query.account)
    .delay(100)
    .reply(200, uri => {
        const requestURL = new URL(uri, MULTI_TENANCY_ENDPOINT);
        const accountId = requestURL.searchParams.get('account');
        const allUsersWithAccounts = Object.values(USERS).filter(user => user.ACCOUNTS).filter(user => Object.keys(user.ACCOUNTS).includes(accountId));
        const resourcesOfAccount = flatten(allUsersWithAccounts.map(user => Object.values(user.CLUSTERS)));
        return JSON.stringify(resourcesOfAccount
            .filter(cluster => cluster.agentId)
            .map(cluster => ({
                resourceIdentifier: cluster.cluster.publicId,
                agentId: cluster.agentId,
                resourceClass: cluster.resourceClass
            })));
    })
    // delete/remove resource

    .delete(escapeForwardSlash(`${API_PATH_RESOURCES}/.+`))
    .delay(100)
    .reply(200, function (uri) {
        const requestURL = new URL(uri, MULTI_TENANCY_ENDPOINT);
        const publicId = last(requestURL.pathname.split('/'));
        const userId = findUserId(this.req.headers['x-user-token']);
        if (USERS[userId]) {
            if (USERS[userId].CLUSTERS[publicId]) {
                delete USERS[userId].CLUSTERS[publicId];
                return [200];
            }
            return [404,
                {
                    message: `Cluster ${publicId} was not found`,
                    causeMessage:
                        `Cluster ${publicId} was not found`
                }];
        }
        return [404, 'USER was not found'];
    })

    // delete/remove account
    .delete(escapeForwardSlash(`${API_PATH_ACCOUNTS}/.+`))
    .delay(100)
    .reply(200, function (uri) {
        const requestURL = new URL(uri, MULTI_TENANCY_ENDPOINT);
        const publicId = last(requestURL.pathname.split('/'));
        const userId = findUserId(this.req.headers['x-user-token']);
        if (USERS[userId]) {
            if (USERS[userId].ACCOUNTS[publicId]) {
                delete USERS[userId].ACCOUNTS[publicId];
                return [200];
            }
            return [404,
                `Account ${publicId} was not found`];
        }
        return [404, 'USER was not found'];
    });

multiTenancyScope.persist(true);
