const { cloneDeep } = require('lodash');
const got = require('got');
const hyperid = require('hyperid');
const { reset } = require('../simulator-state');
const {
    CLOUD_MANAGER_ENDPOINT, CLOUD_MANAGER_API_PREFIX,
    CLOUD_MANAGER_CLUSTERS_PATH_PREFIX, AGENT_ENDPOINT, AGENT_API_PREFIX, AGENT_API_PROVIDERS_PREFIX, AGENT_ZAPI_PATH, AGENT_DESCRIBE_ONTAP_VERSION_PATH
} = require('../../../src/utils/consts');
const { signJwtToken } = require('../simulator-utils');

require('../scopes/clusters-scope');
require('../scopes/agents-scope');
require('../scopes/multi-tenancy-scope');
require('../scopes/agent-zapi-scope');

const cluster = {
    name: hyperid().uuid,
    publicId: hyperid().uuid,
    status: { status: 'ON' },
    ontapClusterProperties: {
        clusterUuid: hyperid().uuid,
        ontapVersion: '9.4',
        clusterAddress: '192.168.0.1'
    },
    capacityFeatures: {
        objectStoreConfigExtended: {
            objectStoreConfig: {
                objectStoreName: 'objectStoreConfig-1',
                s3Name: 'bucket',
                providerType: 'AWS-S3',
                usedSpace: {
                    size: 20,
                    unit: 'GB'
                },
                server: 's3.amazonaws.com'
            },
            tierLevel: 'normal'
        },
        performanceTierUsedCapacity: {
            size: 1004,
            unit: 'GB'
        },
        rawCapacity: {
            size: 5,
            unit: 'GB'
        },
        inactiveUserData: {
            size: 20,
            unit: 'GB'
        }
    }
};

describe('cloud manager scope', () => {
    beforeEach(() => reset('CLUSTERS'));

    test('create single cluster', async () => {
        const data = await got.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            json: cluster,
            headers: {
                authorization: signJwtToken()
            }
        });

        expect(data.ontapClusterProperties).toEqual(cluster.ontapClusterProperties);
    });

    test('delete single cluster', async () => {
        const data = await got.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`,
            {
                json: cluster,
                resolveBodyOnly: true,
                responseType: 'json',
                headers: {
                    authorization: signJwtToken()
                }
            });
        expect(data.publicId).toBe(cluster.publicId);
        await got.delete(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/${cluster.publicId}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                authorization: signJwtToken()
            }
        });

        try {
            await got.get(`${AGENT_ENDPOINT}${AGENT_API_PREFIX}accountId${AGENT_API_PROVIDERS_PREFIX}${AGENT_ZAPI_PATH}${cluster.publicId}${AGENT_DESCRIBE_ONTAP_VERSION_PATH}`, {
                resolveBodyOnly: true,
                responseType: 'json',
                headers: {
                    authorization: signJwtToken()
                }
            });
        } catch (err) {
            expect(err.response.statusCode).toBe(404);
        }
    });

    test('delete a cluster of a different user- should fail', async () => {
        const data = await got.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`,
            {
                json: cluster,
                resolveBodyOnly: true,
                responseType: 'json',
                headers: {
                    authorization: signJwtToken()
                }
            });
        expect(data.publicId).toBe(cluster.publicId);
        try {
            await got.delete(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/${cluster.publicId}`, {
                resolveBodyOnly: true,
                responseType: 'json',
                headers: {
                    authorization: signJwtToken('user2')
                }
            });
        } catch (err) {
            expect(err.response.statusCode).toBe(404);
        }
    });

    test('delete all clusters', async () => {
        jest.setTimeout(10000);
        const cluster1 = cluster;
        const cluster2 = cloneDeep(cluster);
        cluster2.publicId = hyperid().uuid;
        const res1 = await got.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`,
            {
                json: cluster1,
                resolveBodyOnly: true,
                responseType: 'json',
                headers: {
                    authorization: signJwtToken()
                }
            });

        const res2 = await got.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`,
            {
                resolveBodyOnly: true,
                responseType: 'json',
                json: cluster2,
                headers: {
                    authorization: signJwtToken()
                }
            });
        expect(res1.publicId).toBe(cluster1.publicId);
        expect(res2.publicId).toBe(cluster2.publicId);
        await got.delete(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`,
            {
                resolveBodyOnly: true,
                responseType: 'json',
                headers: {
                    authorization: signJwtToken()
                }
            });

        try {
            await got.get(`${AGENT_ENDPOINT}${AGENT_API_PREFIX}accountId${AGENT_API_PROVIDERS_PREFIX}${AGENT_ZAPI_PATH}${cluster1.publicId}${AGENT_DESCRIBE_ONTAP_VERSION_PATH}`, {
                resolveBodyOnly: true,
                responseType: 'json',
                headers: {
                    authorization: signJwtToken()
                }
            });
        } catch (err) {
            expect(err.response.statusCode).toBe(404);
        }
        try {
            await got.get(`${AGENT_ENDPOINT}${AGENT_API_PREFIX}accountId${AGENT_API_PROVIDERS_PREFIX}${AGENT_ZAPI_PATH}${cluster2.publicId}${AGENT_DESCRIBE_ONTAP_VERSION_PATH}`,
                {
                    resolveBodyOnly: true,
                    responseType: 'json',
                    headers: {
                        authorization: signJwtToken()
                    }
                });
        } catch (err) {
            expect(err.response.statusCode).toBe(404);
        }
    });
});
