const got = require('got');
const hyperid = require('hyperid');
const { reset } = require('../simulator-state');
const {
    MULTI_TENANCY_ENDPOINT, API_PATH_ACCOUNTS, CLOUD_MANAGER_ENDPOINT, CLOUD_MANAGER_API_PREFIX, CLOUD_MANAGER_CLUSTERS_PATH_PREFIX, ONPREM_DISCOVER_LOCATION,
    API_PATH_RESOURCES, API_PATH_RESOURCES_INTERNAL
} = require('../../../src/utils/consts');
const { signJwtToken } = require('../simulator-utils');
require('../scopes/multi-tenancy-scope');

require('../scopes/clusters-scope');

const UUID_REGEX = /^[\d|\w]{8}-[\d\w]{4}-[\d\w]{4}-[\d\w]{4}-[\d\w]{12}$/;
const ACCOUNT_1_NAME = 'account-1';
const ACCOUNT_2_NAME = 'account-2';
const WORKSPACE_1_NAME = 'workspace-1';

const WORKSPACE_2_NAME = 'workspace-2';

describe('multi tenancy scope', () => {
    beforeEach(() => reset());

    test('accounts - create single', async () => {
        const data = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken(),
                emptyScenario: true
            }
        });
        expect(data).toMatchObject({ accountPublicId: expect.stringMatching(UUID_REGEX) });
    });

    test('accounts - get single', async () => {
        const account = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken(),
                'empty-scenario': true
            }
        });
        const accounts = await got.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken(),
                'empty-scenario': true
            }
        });

        expect(accounts).toEqual([{
            accountName: ACCOUNT_1_NAME,
            accountPublicId: account.accountPublicId
        }]);
    });

    test('accounts - get empty', async () => {
        const data = await got.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken(),
                'empty-scenario': true
            }
        });

        expect(data).toEqual([]);
    });

    test('accounts - create multiple with the same user', async () => {
        const account1 = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken(),
                'empty-scenario': true
            }
        });
        const account2 = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_2_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken(),
                'empty-scenario': true
            }
        });
        const accounts = await got.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken(),
                'empty-scenario': true
            }
        });
        expect(accounts).toEqual([
            {
                accountName: ACCOUNT_1_NAME,
                accountPublicId: account1.accountPublicId
            }, {
                accountName: ACCOUNT_2_NAME,
                accountPublicId: account2.accountPublicId
            }
        ]);
    });

    test('accounts - create multiple with the 2 different users', async () => {
        const account1 = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken(),
                'empty-scenario': true
            }
        });
        const account2 = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_2_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken('user2'),
                authorization: signJwtToken('user2'),
                'empty-scenario': true
            }
        });
        const accountsUser1 = await got.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken(),
                'empty-scenario': true
            }
        });
        const accountsUser2 = await got.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken('user2'),
                authorization: signJwtToken('user2'),
                'empty-scenario': true
            }
        });
        expect(accountsUser1).toEqual([
            {
                accountName: ACCOUNT_1_NAME,
                accountPublicId: account1.accountPublicId
            }]);

        expect(accountsUser2).toEqual([{
            accountName: ACCOUNT_2_NAME,
            accountPublicId: account2.accountPublicId
        }]);
    });

    test('workspaces - create single', async () => {
        const { accountPublicId } = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const data = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${accountPublicId}/${WORKSPACE_1_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        expect(data).toEqual({
            workspaceName: WORKSPACE_1_NAME,
            workspacePublicId: expect.stringMatching(UUID_REGEX)
        });
    });

    test('workspaces - create for non existing account', async () => {
        try {
            await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${hyperid().uuid}/${WORKSPACE_1_NAME}`, {
                headers: {
                    'X-User-Token': signJwtToken(),
                    authorization: signJwtToken()
                }
            });
        } catch (err) {
            expect(err.response.statusCode).toBe(404);
        }
    });

    test('workspaces - get single', async () => {
        const { accountPublicId } = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const workspace = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${accountPublicId}/${WORKSPACE_1_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const workspaces = await got.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${accountPublicId}/workspace`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        expect(workspaces).toEqual([workspace]);
    });

    test('workspaces - get non existent', async () => {
        const { accountPublicId } = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${accountPublicId}/${WORKSPACE_1_NAME}`, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        try {
            await got.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${hyperid().uuid}/workspace`, {
                headers: {
                    'X-User-Token': signJwtToken(),
                    authorization: signJwtToken()
                }
            });
        } catch (err) {
            expect(err.response.statusCode).toEqual(404);
        }
    });

    test('workspaces - create multiple workspaces in same account', async () => {
        const { accountPublicId } = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const workspace1 = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${accountPublicId}/${WORKSPACE_1_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const workspace2 = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${accountPublicId}/${WORKSPACE_2_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const workspaces = await got.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${accountPublicId}/workspace`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        expect(workspaces).toEqual([workspace1, workspace2]);
    });

    test('workspaces - create multiple workspaces in different accounts', async () => {
        const { accountPublicId: account1PublicId } = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { accountPublicId: account2PublicId } = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_2_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const workspace1 = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${account1PublicId}/${WORKSPACE_1_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const workspace2 = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${account2PublicId}/${WORKSPACE_2_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const workspacesAccount1 = await got.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${account1PublicId}/workspace`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const workspacesAccount2 = await got.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${account2PublicId}/workspace`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        expect(workspacesAccount1).toEqual([workspace1]);
        expect(workspacesAccount2).toEqual([workspace2]);
    });

    const clusterAddress = '192.168.0.1';

    const cluster = {
        name: hyperid().uuid,
        publicId: hyperid().uuid,
        ontapClusterProperties: {
            clusterUuid: hyperid().uuid,
            ontapVersion: '9.4',
            clusterAddress
        },
        capacityFeatures: {
            objectStoreConfigExtended: {
                objectStoreConfig: {
                    objectStoreName: 'objectStoreConfig-1',
                    s3Name: 'bucket',
                    providerType: 'AWS-S3',
                    usedSpace: {
                        size: 20,
                        unit: 'GB'
                    },
                    server: 's3.amazonaws.com'
                },
                tierLevel: 'normal'
            },
            performanceTierUsedCapacity: {
                size: 1004,
                unit: 'GB'
            },
            rawCapacity: {
                size: 5,
                unit: 'GB'
            },
            inactiveUserData: {
                size: 20,
                unit: 'GB'
            }
        }
    };

    test('resources - get resources of existing workspace with one cluster', async () => {
        // create account
        // create workspace
        // create cluster
        // discover inside workspace
        const { accountPublicId: account1PublicId } = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const workspace1 = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${account1PublicId}/${WORKSPACE_1_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        cluster.workspaceId = workspace1.workspacePublicId;
        cluster.agentId = 'agent-1';
        cluster.resourceClass = ONPREM_DISCOVER_LOCATION;

        await got.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            json: cluster,
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        const getResourcesResult = await got.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_RESOURCES}?workspace=${workspace1.workspacePublicId}&account=${account1PublicId}&resource-class=${ONPREM_DISCOVER_LOCATION}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        //
        expect(getResourcesResult).toEqual([{ resourceIdentifier: cluster.publicId, agentId: 'agent-1', resourceClass: ONPREM_DISCOVER_LOCATION }]);
    });

    test('resources - get resources of existing account with one cluster', async () => {
        // create account
        // create workspace
        // create cluster
        // discover inside workspace
        const { accountPublicId: account1PublicId } = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const workspace1 = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${account1PublicId}/${WORKSPACE_1_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        cluster.workspaceId = workspace1.workspacePublicId;
        cluster.agentId = 'agent-1';
        cluster.resourceClass = ONPREM_DISCOVER_LOCATION;

        await got.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            json: cluster,
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const getResourcesResult = await got.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_RESOURCES}?&account=${account1PublicId}&resource-class=${ONPREM_DISCOVER_LOCATION}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        expect(getResourcesResult).toEqual([{ resourceIdentifier: cluster.publicId, agentId: 'agent-1', resourceClass: ONPREM_DISCOVER_LOCATION }]);
    });

    test.skip('resources - get resources of  non-existing workspace - should receive status 404', async () => {
        try {
            await got.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_RESOURCES}?workspace=workspacePublicId&account=account1PublicId&resource-class=${ONPREM_DISCOVER_LOCATION}`, {
                headers: {
                    'X-User-Token': signJwtToken(),
                    authorization: signJwtToken()
                }
            });
        } catch (err) {
            expect(err.response.statusCode).toBe(404);
        }
    });

    test('resources - internal - get resources of existing workspace with one cluster', async () => {
        // create account
        // create workspace
        // create cluster
        const { accountPublicId: account1PublicId } = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const workspace1 = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${account1PublicId}/${WORKSPACE_1_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        cluster.workspaceId = workspace1.workspacePublicId;
        cluster.agentId = 'agent-1';
        cluster.resourceClass = ONPREM_DISCOVER_LOCATION;

        await got.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            json: cluster,
            headers: {
                authorization: signJwtToken()
            }
        });

        const getResourcesResult = await got.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_RESOURCES_INTERNAL}?workspace=${workspace1.workspacePublicId}&account=${account1PublicId}&resource-class=${ONPREM_DISCOVER_LOCATION}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        //
        expect(getResourcesResult).toEqual([{ resourceIdentifier: cluster.publicId, agentId: 'agent-1', resourceClass: ONPREM_DISCOVER_LOCATION }]);
    });

    test('resources - internal - get resources of existing account with one cluster', async () => {
        // create account
        // create workspace
        // create cluster
        // discover inside workspace
        const { accountPublicId: account1PublicId } = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const workspace1 = await got.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${account1PublicId}/${WORKSPACE_1_NAME}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        await got.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            json: cluster,
            headers: {
                authorization: signJwtToken()
            }
        });
        cluster.workspaceId = workspace1.workspacePublicId;
        cluster.agentId = 'agent-1';
        cluster.resourceClass = ONPREM_DISCOVER_LOCATION;

        const getResourcesResult = await got.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_RESOURCES_INTERNAL}?&account=${account1PublicId}&resource-class=${ONPREM_DISCOVER_LOCATION}`, {
            resolveBodyOnly: true,
            responseType: 'json',
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        expect(getResourcesResult).toEqual([{ resourceIdentifier: cluster.publicId, agentId: 'agent-1', resourceClass: ONPREM_DISCOVER_LOCATION }]);
    });

    test('resources - internal - get resources of  non-existing workspace - should receive status 404', async () => {
        try {
            await got.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_RESOURCES_INTERNAL}?workspace=workspacePublicId&account=account1PublicId&resource-class=${ONPREM_DISCOVER_LOCATION}`, {
                headers: {
                    'X-User-Token': signJwtToken(),
                    authorization: signJwtToken()
                }
            });
        } catch (err) {
            expect(err.response.status).toBe(404);
        }
    });
});
