import React from 'react';
import _ from 'lodash';
import {connect} from 'react-redux';
import {withRouter, Route, Switch, Redirect} from 'react-router-dom';
import {withApollo} from '@apollo/client/react/hoc'
import ContentWrapper from './components/ContentWrapper';
import TabsHeader from './components/header/TabsHeader';
import {
    GET_LAZY_CLUSTERS_QUERY,
    GET_LICENSE_INFO_QUERY,
    UPDATE_CLUSTER_INFO,
    UPDATE_LICENSE_INFO
} from './graphql/queries';
import WizardStarter from './components/wizard/WizardStarter';
import WizardsHeader from './components/wizard/WizardsHeader';
import LoadingIcon from './components/widgets/LoadingIcon';
import MoreInfo from './components/tabs/MoreInfo';
import ClustersDashboard from './components/tabs/ClustersDashboardNew';
import Oops from './components/Oops';
import LicensePage from './components/tabs/license/LicensePage';
import SetupCompleteStep from './components/setupClusterWizard/SetupCompleteStep';
import Isvg from 'react-inlinesvg'
import {STATUS_ERROR, UPDATE_INITIAL_QUERY_PARAMS} from './actions';
import {NOTIFICATION_ERROR_CODE} from './utils/consts';
import ChooseProvider from './components/setupClusterWizard/ChooseProvider'
import SourceVolumesStep from './components/tierVolumes/SourceVolumesStep';
import ManageAggregates from './components/aggregates/ManageAggregates';
import './app.scss';
import useRunOnce from "./hooks/useRunOnce";
import {useQuery, useSubscription} from "@apollo/client";
import {parse} from "query-string";
import * as actions from "./actions";
import OCCMIntegrationEntryPoint from "./occmIntegration/OCCMIntegrationEntryPoint";
import useRunOnceWhenTruthy from "./hooks/useRunOnceWhenTruthy";
import {updateLocalStorage} from "./utils/utils";


const AuthenticatedRoute = props => props.auth.authenticated ? <Route {...props}/> : <OCCMIntegrationNeedLogin/>;

const OCCMIntegrationNeedLogin = () => {
    return <div>
        <div className="wizards-header-need-login">
            <div className="title no-wrap">
                <Isvg src={require('./assets/svgs/tiering-icon.svg')}/>
                Cloud Tiering
            </div>
            <div className='refresh-browser'>
                <div className="inner-title">Reload this page from your web browser to log in again.</div>
                <div className="inner-info">This is required because you were logged out of Cloud Central's single
                    sign-on service from another browser tab.
                </div>
            </div>
        </div>
    </div>
};

const whereTo = (error, errorCode) => {
    const notificationError = _.isEqual(errorCode, NOTIFICATION_ERROR_CODE);
    if (error && !notificationError) {
        return '/errorPage';
    } else {
        return '/clusters';
    }
};

const AppRoutes = ({auth, error, errorCode}) => {
    return (
        <Switch>
            <AuthenticatedRoute auth={auth} path="/clusters" exact
                                render={() => <ContentWrapper content={ClustersDashboard}
                                                              headerComponent={TabsHeader}/>}/>
            {/*setup wizard storage grid flow*/}
            <AuthenticatedRoute auth={auth} path="/setup/:clusterId/:clusterName/:agentId/:ontapVersion/choose-provider"
                                render={() => <ContentWrapper content={ChooseProvider}
                                                              headerComponent={WizardsHeader}
                                                              hideGlobalNotifications={true}
                                                              data={{wizardType: 'setup'}}/>}/>
            <AuthenticatedRoute auth={auth} path="/setup-sg/:clusterId/:clusterName/:agentId/:provider/setup-server"
                                render={() => <ContentWrapper content={WizardStarter}
                                                              headerComponent={WizardsHeader}
                                                              hideGlobalNotifications={true}
                                                              data={{wizardType: 'setup-sg'}}/>}/>

            <AuthenticatedRoute auth={auth} path="/setup-sg/:clusterId/:clusterName/setup-complete"
                                render={() => <ContentWrapper content={SetupCompleteStep}
                                                              headerComponent={WizardsHeader}
                                                              hideGlobalNotifications={true}
                                                              data={{wizardType: 'setup-sg'}}/>}/>
            {/*setup wizard aws flow*/}
            <AuthenticatedRoute auth={auth} path="/setup/:clusterId/:clusterName/:agentId/:provider/setup-bucket"
                                render={() => <ContentWrapper content={WizardStarter}
                                                              headerComponent={WizardsHeader}
                                                              hideGlobalNotifications={true}
                                                              data={{wizardType: 'setup'}}/>}/>
            <AuthenticatedRoute auth={auth} path="/setup/:clusterId/:clusterName/setup-complete"
                                render={() => <ContentWrapper content={SetupCompleteStep}
                                                              headerComponent={WizardsHeader}
                                                              hideGlobalNotifications={true}
                                                              data={{wizardType: 'setup'}}/>}/>
            {/*setup wizard azure flow*/}
            <AuthenticatedRoute auth={auth}
                                path="/setup-azure/:clusterId/:clusterName/:agentId/:provider/resource-group"
                                render={() => <ContentWrapper content={WizardStarter}
                                                              headerComponent={WizardsHeader}
                                                              hideGlobalNotifications={true}
                                                              data={{wizardType: 'setup-azure'}}/>}/>
            <AuthenticatedRoute auth={auth} path="/setup-azure/:clusterId/:clusterName/setup-complete"
                                render={() => <ContentWrapper content={SetupCompleteStep}
                                                              headerComponent={WizardsHeader}
                                                              hideGlobalNotifications={true}
                                                              data={{wizardType: 'setup'}}/>}/>
            {/*modify volumes wizard flow*/}
            <AuthenticatedRoute auth={auth} path="/tier-volumes/:clusterId/:clusterName/volumes"
                                render={() => <ContentWrapper content={SourceVolumesStep}
                                                              headerComponent={WizardsHeader}
                                                              hideGlobalNotifications={true}
                                                              data={{wizardType: 'tier-volumes'}}/>}/>
            <AuthenticatedRoute auth={auth} path="/:clusterId/:clusterName/info"
                                render={() => <ContentWrapper content={MoreInfo}
                                                              headerComponent={TabsHeader}/>}/>
            <AuthenticatedRoute auth={auth} path="/:clusterId/:clusterName/aggregates"
                                render={() => <ContentWrapper content={ManageAggregates}
                                                              hideGlobalNotifications={true}
                                                              headerComponent={WizardsHeader}
                                                              data={{wizardType: 'manage-aggregates'}}/>}/>
            <AuthenticatedRoute auth={auth} path="/errorPage" exact render={() => <ContentWrapper content={Oops}
                                                                                                  hideGlobalNotifications={true}
                                                                                                  data={{error}}/>}/>
            <AuthenticatedRoute auth={auth} path="/license" exact
                                render={() => <ContentWrapper content={LicensePage}
                                                              headerComponent={TabsHeader}/>}/>
            <AuthenticatedRoute auth={auth} path="/occm-integration-setup/:clusterId/" exact
                                render={() => <ContentWrapper content={OCCMIntegrationEntryPoint}
                                                              data={{entryType: "setup"}}/>}/>
            <AuthenticatedRoute auth={auth} path="/occm-integration-tier/:clusterId/" exact
                                render={() => <ContentWrapper content={OCCMIntegrationEntryPoint}
                                                              data={{entryType: "tier"}}/>}/>
            <AuthenticatedRoute auth={auth} path="/occm-integration-dashboard/" exact
                                render={() => <ContentWrapper content={OCCMIntegrationEntryPoint}
                                                              data={{dashboard: true, entryType: "dashboard"}}/>}/>

            <Redirect to={whereTo(error, errorCode)}/>
        </Switch>
    );
};


const App = ({auth, error, errorCode}) => {
    return (<AppRoutes auth={auth} error={error} errorCode={errorCode}/>)
}


const WithRouterApp = withRouter(App);

const mapStateToProps = state => {
    return {
        auth: state.auth,
        routing: state.router,
        agentId: state.general.agentId,
        accountId: state.general.accountId,
        workspaceId: state.general.workspaceId,
    };
};

const isAuthenticationError = (error) => {
    return (_.get(error, 'networkError.result.errors[0].extensions.code') === 401 || (_.get(error, 'graphQLErrors[0].extensions.code') === 401));
}

const useInitialQueryParams = (dispatch) => {
    useRunOnce(() => {
        const search = parse(window.location.search);

        let accountId = _.get(search, 'accountId', null);
        let agentId = _.get(search, 'agentId', null);
        let workspaceId = _.get(search, 'workspaceId', null);

        if (process.env.REACT_APP_DEVELOPMENT_MODE === 'true') {
            // In case when there are no query params, set defaults
            accountId = updateLocalStorage('accountId', accountId, `default-accountId-development`);
            agentId = updateLocalStorage('agentId', agentId, '5dc29e61-dcb6-492e-b900-2f267f2731b7'); // aws default
            workspaceId = updateLocalStorage('workspaceId', workspaceId, 'default-workspaceId-development');
        }

        if (accountId && agentId && workspaceId) {
            dispatch({
                type: UPDATE_INITIAL_QUERY_PARAMS,
                payload: {
                    agentId,
                    accountId,
                    workspaceId
                }
            })
        } else {
            dispatch({
                type: actions.GENERAL_ERROR,
                code: actions.ERROR_BAD_REQUEST,
                payload: "One or more of the following properties are missing in the query params: accountId, agentId or workSpaceId"
            });
        }

    })
}


const useLicenseSubscription = (auth, accountId) => {
    const {loading, data, subscribeToMore} = useQuery(GET_LICENSE_INFO_QUERY, {
        variables: {
            accountId,
        },
        skip: !auth.authenticated
    })

    useRunOnceWhenTruthy(() => {
        subscribeToMore({
                document: UPDATE_LICENSE_INFO,
                variables: {accountId},
                updateQuery: (prev, {subscriptionData}) => {
                    const {data} = subscriptionData;
                    if (!data) return prev;
                    return data
                },
            }
        );
    }, data)

    return {
        loadingLicenseInfo: loading,
    }
}

const InitialQuery = (props) => {
    useRunOnce(() => {
        console.log(`%c APP clearing store due to first time navigation`, 'font-weight:bold; color:purple');
        props.client.clearStore();

    })

    const {auth, dispatch, agentId, accountId, workspaceId} = props;
    useInitialQueryParams(dispatch)

    const skip = !auth.authenticated || !accountId
    const {loading: loadingClusters, error: errorClusters, data: dataClusters} = useQuery(GET_LAZY_CLUSTERS_QUERY, {
        variables: {
            accountId,
            agentId,
            workspaceId
        },
        skip
    })
    const {loadingLicenseInfo} = useLicenseSubscription(auth, accountId)

    useSubscription(UPDATE_CLUSTER_INFO, {
            variables: {accountId, agentId, workspaceId},
            skip
        }
    )


    if (auth && !auth.authenticated) {
        if (auth.status === STATUS_ERROR) { //make sure not to show it on status progress. otherwise it will always be there before showing the clusters
            return <OCCMIntegrationNeedLogin/>;
        }
        return <div/>;
    } else {
        if (loadingClusters || loadingLicenseInfo) {
            return (<LoadingIcon text={'Cloud Tiering'}/>);
        }
        const errorMessage = errorClusters ? (isAuthenticationError(errorClusters) ? null : errorClusters.message) : null;
        const errorCode = _.get(errorClusters, 'graphQLErrors[0].extensions.code');
        const clusters = !errorClusters && dataClusters?.lazyClusters ? dataClusters.lazyClusters : [];
        if (!clusters) {
            console.log(`APP unable to get clusters error ${JSON.stringify(errorClusters)} data ${JSON.stringify(dataClusters)}`);
        }
        return (<>
            {(clusters || errorMessage) &&
            <WithRouterApp auth={auth} error={errorMessage}
                           errorCode={errorCode}/>}
            {!clusters && !errorMessage && <LoadingIcon text={'Clusters'}/>}
        </>)
    }
};

export default connect(mapStateToProps)(withApollo(InitialQuery));
