import React, { useState, useMemo } from 'react';
import { GET_AGGREGATES_QUERY, GET_CLUSTER_QUERY } from '../../graphql/queries';
import { ERROR_SERVER_ERROR, GENERAL_ERROR, routesObject } from '../../actions';
import LoadingIcon from '../widgets/LoadingIcon';
import classNames from 'classnames';
import SearchableSelectableTable from '../table/SearchableSelectableTable';
import { MANAGE_AGGREGATES_INFO, MAX_VOLUMES_IN_PAGE, NOTIFICATION_TYPES } from '../../utils/consts';
import OutsideClickHandler from 'react-outside-click-handler';
import _ from 'lodash';
import { useQuery } from '@apollo/client'
import { buildCapacityObject } from '../../utils/utils';
import Isvg from 'react-inlinesvg';
import EditAggregatePanel from './EditAggregatePanel';
import InfoPopover from '../reactPopover/InfoPopover';
import NotificationWidget from '../widgets/NotificationWidget';
import Breadcrumbs from "../widgets/Breadcrumbs";
import useQueryParam from "../../hooks/useQueryParam";
import {connect} from "react-redux";

require('./manageAggregates.scss');

const returnToDashboard = () => routesObject.routes.push(`/clusters`);

const ManageAggregates = (props) => {
    const {dispatch, agentId, accountId, workspaceId} = props;
    const clusterId = props.match.params.clusterId;
    const clusterName = props.match.params.clusterName;
    const origin = useQueryParam('origin')
    const [sort, setSort] = useState(null)

    const [editAggregateOpen, setEditAggregateOpen] = useState(false);
    const [editAggregate, setEditAggregate] = useState(null);
    const [editable, setEditable] = useState(true);
    const {loading: clusterLoading, error: clusterError, data: clusterData } = useQuery(GET_CLUSTER_QUERY, {
        skip: !clusterId,
        variables: {clusterId, accountId, agentId, workspaceId}, fetchPolicy: "cache-first"});

    useMemo(() => {
        if (!clusterLoading && !clusterError && clusterData) {
            setEditable(clusterData.cluster.resourceType === 'ON_PREM');
        }
    }, [clusterLoading, clusterError, clusterData]);


    const { loading, error, data } = useQuery(GET_AGGREGATES_QUERY, {
        skip: !clusterId,
        variables: {
            accountId,
            agentId,
            clusterId,
            workspaceId
        },
        fetchPolicy: 'cache-first',
        notifyOnNetworkStatusChange: true
    })

    const aggregates = useMemo(()=> {
        if (!loading && !error && data) {
            let updatedAggregates = _.map(data.aggregates, aggregate => {
                const capacityObject = buildCapacityObject(_.get(aggregate, 'capacity', 0));
                const usedPerformanceCapacityObject = buildCapacityObject(_.get(aggregate, 'usedPerformanceCapacity', 0));
                const inactiveCapacityObject = buildCapacityObject(_.get(aggregate, 'inactiveCapacity', 0));
                const tieredCapacityObject = buildCapacityObject(_.get(aggregate, 'tieredCapacity', 0));

                return {
                    capacityText: `${capacityObject.value} ${capacityObject.unit}`,
                    usedPerformanceCapacityText: `${usedPerformanceCapacityObject.value} ${usedPerformanceCapacityObject.unit}`,
                    inactiveCapacityText: `${inactiveCapacityObject.value} ${inactiveCapacityObject.unit} ${aggregate.isEstimatedInactiveCapacity ? '(Estimated)' : ''}`,
                    inactiveCapacityPercentText: `${aggregate.inactiveCapacityPercent} %`,
                    tieredCapacityText: `${tieredCapacityObject.value} ${tieredCapacityObject.unit}`,
                    ...aggregate
                };

            })
            if (sort) {
                const sortedAggregates = _.sortBy(updatedAggregates, [sort.column])
                updatedAggregates = sort.ascending ? sortedAggregates : _.reverse(sortedAggregates)
            }

            return updatedAggregates
        }
    }, [loading, error, data, sort])

    const applySort = (column, ascending) => {
        setSort({ascending, column});
    };


    const getTableHeader = () => {
        return [
            {
                className: 'check'
            },
            {
                name: 'name',
                text: 'Aggregate Name',
                sort: applySort
            },
            {
                name: 'capacity',
                text: 'Total Size',
                sort: applySort
            },
            {
                name: 'usedPerformanceCapacity',
                text: 'Used Size',
                sort: applySort,
                info: {
                    text: MANAGE_AGGREGATES_INFO.USED_SIZE,
                    width: '500px'
                }
            },
            {
                name: 'inactiveCapacity',
                className: 'cold-data',
                text: 'Cold Data',
                sort: applySort,
                info: {
                    text: MANAGE_AGGREGATES_INFO.COLD_DATA,
                    width: '470px'
                }
            },
            {
                name: 'tieredCapacity',
                text: 'Tiered Size',
                sort: applySort
            },
            {
                name: 'idr',
                text: 'IDR',
                sort: applySort,
                info: {
                    text: MANAGE_AGGREGATES_INFO.IDR,
                    width: '460px'
                }
            },
            {
                name: 'tieringFullnessThreshold',
                className: 'fullness-threshold',
                text: 'Fullness Threshold',
                sort: applySort,
                info: {
                    text: MANAGE_AGGREGATES_INFO.FULLNESS_THRESHOLD,
                    width: '460px'
                }
            },
            {
                className: 'pencil',
                name: '',
                text: ''
            }
        ];
    };

    const openEditAggregatePanel = (aggregate) => {
        setEditAggregateOpen(true);
        setEditAggregate(aggregate);
    };


    const renderRow = (aggregate) => {
        const {
            name, isEligibleForTiering, tieringRestrictionReason, capacityText, usedPerformanceCapacityText, inactiveCapacityText,
            tieredCapacityText, idr, tieringFullnessThreshold
        } = aggregate;
        return (
            <div className="table-rows ">
                <td className="check"/>
                <td className="aggregate-name">
                    {!isEligibleForTiering &&
                    <InfoPopover wrapperClassName="not-eligible-info" popoverClassName="not-eligible-info"
                                 popoverText={tieringRestrictionReason} contentStyle={{ width: '460px' }}/>}
                    <div title={name} className="no-wrap">{name}</div>
                </td>
                <td className="aggregate-size">{capacityText}</td>
                <td className="used-size">{usedPerformanceCapacityText}</td>
                <td className="cold-data">{inactiveCapacityText}</td>
                <td className="tiered-size"><span className="no-wrap"
                                                  title={tieredCapacityText}>{tieredCapacityText}</span></td>
                <td className="idr">{idr}</td>
                <td className="fullness">{tieringFullnessThreshold ? `${tieringFullnessThreshold} %` : '--'}</td>
                <td className="pencil" onClick={() => editable ? openEditAggregatePanel(aggregate) : _.noop}>
                    {editable && <div className="pencil"><Isvg src={require('./../../assets/svgs/pencil.svg')}/></div>}
                </td>
            </div>
        );
    };

    const closeEditAggregate = () => {
        setTimeout(() => {
            setEditAggregateOpen(false);
        });
    };


    if (error) {
        dispatch({
            type: GENERAL_ERROR,
            code: ERROR_SERVER_ERROR,
            payload: error
        });
        returnToDashboard()
        return <div></div>;
    } //in case of an error, return to dashboard
    if (!aggregates || loading ) {
        return (<LoadingIcon text={'Aggregates'} className="loading-aggregates"/>);
    }
    return (
        <div className={classNames('manage-aggregates new-wizard-step-full-screen')}>
            <div className="step-title">
                <div className="left-side">
                    {origin === 'dashboard' && <Breadcrumbs levels={[{
                                text: `Cluster Dashboard | ${clusterName}`,
                        onClick: returnToDashboard
                            }, {text: `${editable ? 'Manage' : 'View'} Aggregates`}]}/>}
                </div>
                <h1>{editable ? 'Manage' : 'View'} Aggregates</h1>
                <div className="right-side"></div>
            </div>
            <form className="aggregates-form">
                <div className="aggregates">
                    <SearchableSelectableTable
                        Table={{
                            getTableHeader,
                            renderRow
                        }}
                        data={aggregates}
                        searchKeys={['name', 'capacityText', 'usedPerformanceCapacityText', 'inactiveCapacityText', 'tieredCapacity', 'idr', 'tieringFullnessThreshold']}
                        containerOptions={{
                            singularTitle: 'Aggregate',
                            pluralTitle: 'Aggregates',
                            maxElementsOnPage: MAX_VOLUMES_IN_PAGE,
                            rowHeight: 55,
                            info: {
                                text: MANAGE_AGGREGATES_INFO.DISPLAYED_AGGREGATES,
                                width: '500px'
                            }

                        }}>
                    </SearchableSelectableTable>
                </div>
                {editAggregateOpen && <div className='slider-mask show-mask'/>}
                <OutsideClickHandler
                    onOutsideClick={() => editAggregateOpen ? closeEditAggregate() : _.noop}>
                    <EditAggregatePanel
                        isOpen={editAggregateOpen}
                        close={() => closeEditAggregate()}
                        editAggregate={editAggregate}
                        clusterId={clusterId}
                    />
                </OutsideClickHandler>
                {!editable && <div className="notification-widget-container">
                    <NotificationWidget variant={NOTIFICATION_TYPES.WARNING}>
                        {clusterData && clusterData.cluster.resourceType === 'CVO' &&
                        <div>This table enables you to view information about your aggregates.
                            You can manage tiering settings for aggregates from the Cloud Volumes ONTAP
                            CLI.</div>}
                    </NotificationWidget>
                </div>}
            </form>
        </div>
    );

};

const mapStateToProps = state => {
    return {
        agentId: state.general.agentId,
        accountId: state.general.accountId,
        workspaceId: state.general.workspaceId,
    };
};

export default connect(mapStateToProps)(ManageAggregates);
