import React, {useMemo, useState} from 'react';
import ButtonsPanelWithError from '../widgets/ButtonsPanelWithError';
import classNames from 'classnames';
import {reduxForm, SubmissionError} from 'redux-form';
import InlineSVG from 'react-inlinesvg';
import {PROVIDER_TYPE_SGWS, PROVIDERS} from '../../utils/consts';
import Button from "../widgets/Button";
import {useQuery} from "@apollo/client";
import {GET_LAZY_CLUSTERS_QUERY} from "../../graphql/queries";
import {connect} from "react-redux";
import get from 'lodash/get';
import noop from 'lodash/noop';
import useQueryParam from "../../hooks/useQueryParam";

require('./chooseProvider.scss');

const ProviderBox = ({selected, setSelectedProvider, ribbon, providerIcon, provider, disabled, disabledReason}) => {
    return (<div className="provider-column">
        <div className={classNames('provider', {
            selected,
            disabled
        })} onClick={() => disabled ? noop() : setSelectedProvider(provider)}>
            <div className="ribbon"><InlineSVG src={require(`../../assets/svgs/${ribbon}`)}/></div>
            {selected &&
            <div className="selected-icon"><InlineSVG src={require('../../assets/svgs/selected.svg')}/></div>}
            <div className={`provider-icon ${provider === 'AWS' ? 'aws' : 'sg'}`}><InlineSVG
                src={require(`../../assets/svgs/${providerIcon}`)}/></div>
            <div className="provider-name">{PROVIDERS[provider].displayName}</div>
        </div>
        {disabled && <div className="not-supported">{disabledReason}</div>}
    </div>);
};

const ChooseProvider = (props) => {
    const clusterId = props.match.params.clusterId;
    const agentId = props.match.params.agentId;
    const clusterName = props.match.params.clusterName;
    const ontapVersion = props.match.params.ontapVersion;
    const origin = useQueryParam('origin')

    const [selectedProvider, setSelectedProvider] = useState(() => ontapVersion < 9.4 ? "AWS" : null);
    const submit = () => {
        if (!selectedProvider) {
            const error = {_error: `In order to continue please select a provider`};
            throw new SubmissionError(error);
        } else {
            if (selectedProvider === PROVIDER_TYPE_SGWS.name)
                props.history.push(`/setup-sg/${clusterId}/${clusterName}/${agentId}/${selectedProvider}/setup-server?origin=${origin}`);
            else props.history.push(`/setup/${clusterId}/${clusterName}/${agentId}/${selectedProvider}/setup-bucket?origin=${origin}`);
        }
    };
    const {handleSubmit, error: formError, accountId, workspaceId} = props;
    const {data} = useQuery(GET_LAZY_CLUSTERS_QUERY, {
        fetchPolicy: 'cache-only',
        variables: {accountId, agentId, workspaceId},
    })
    const cluster = useMemo(() => data?.lazyClusters?.find(cluster => cluster.clusterId === clusterId), [clusterId, data])
    const shouldEnforce = get(cluster, 'licenseInfo.shouldEnforce', null);

    return (<div className="choose-provider new-wizard-step-full-screen">
        <div className="step-main-title">
            <div className="main-title-icon"><InlineSVG src={require('../../assets/svgs/choose-provider-icon.svg')}/>
            </div>
            <h1>Choose Provider</h1>
        </div>
        <form onSubmit={handleSubmit(submit)}>
            <div className="providers-container">
                <ProviderBox selected={selectedProvider === PROVIDER_TYPE_SGWS.name}
                             setSelectedProvider={setSelectedProvider}
                             ribbon="private-cloud.svg"
                             providerIcon="choose-sg-provider.svg"
                             provider={PROVIDER_TYPE_SGWS.name}
                             disabled={ontapVersion < 9.4}
                             disabledReason={'ONTAP 9.4 or later is required'}/>
                <ProviderBox selected={selectedProvider === 'AWS'} setSelectedProvider={setSelectedProvider}
                             disabled={shouldEnforce}
                             disabledReason={'In order to tier data to AWS, you need to subscribe to the AWS Marketplace or activate a license (BYOL)'}
                             ribbon="public-cloud.svg" providerIcon="choose-aws-provider.svg" provider="AWS"/>
            </div>
            <div className="wizard-button">
                <div className="wizard-button">
                    <ButtonsPanelWithError error={formError}>
                        <Button type="submit">Continue</Button>
                    </ButtonsPanelWithError>
                </div>
            </div>
        </form>
    </div>);
};

const ChooseProviderForm = reduxForm({
    form: 'chooseProvider'
})(ChooseProvider);

const mapStateToProps = state => ({
    accountId: state.general.accountId,
    workspaceId: state.general.workspaceId,
});

export default connect(mapStateToProps)(ChooseProviderForm);