import React, {useCallback, useMemo} from 'react';
import Validator from "validatorjs";
import {Field, reduxForm, SubmissionError} from "redux-form";

import {connect} from "react-redux";
import mapStateToProps from "../wizard/wizardMapStateToProps";
import stepinize from "../wizard/stepinize";
import SetupWizardDiagram from "./SetupWizardDiagram";
import {useQuery} from "@apollo/client";
import {GET_RESOURCE_GROUP} from "../../graphql/queries";
import LoadingIcon from "../widgets/LoadingIcon";
import InternalWizardHeader from "../wizard/InternalWizardHeader";
import {ControlledSelectField} from "../forms/forms";
import get from "lodash/get";
import ButtonsPanelWithError from "../widgets/ButtonsPanelWithError";
import Button from "../widgets/Button";
import {calculateNextStep, getNextStepComponent} from "../wizard/WizardStarter";
import {Promise} from "q";
import {Route, Switch} from "react-router";
import formValueSelector from "redux-form/lib/formValueSelector";
import useRunOnceWhenTruthy from "../../hooks/useRunOnceWhenTruthy";
import {extractError, sendMixPanelEvent} from "../../utils/utils";

require('./resourceGroupStep.scss');

const rules = {
    resourceGroup: 'required'
};

const validate = (values) => {
    let errors = {};
    const validator = new Validator(values, rules);
    validator.setAttributeNames({resourceGroup: 'Resource Group'});
    validator.passes();
    errors = validator.errors.all();
    return errors;
};

const useResourceGroup = ({wizard, history, change, accountId}) => {
    const agentId = get(wizard.selectedWizardParams, ['agentId'], null);

    const {loading, error: rawEror, data} = useQuery(GET_RESOURCE_GROUP, {
        fetchPolicy: 'network-only',
        skip: !agentId,
        variables: {accountId, agentId}
    })
    const error = useMemo(() => rawEror && extractError(rawEror), [rawEror])

    const resourceGroupOptions = useMemo(() => data?.resourceGroups.map(({name}) => ({
        value: name,
        label: name
    })), [data])

    useRunOnceWhenTruthy(() => {
        const selectedResourceGroup = get(wizard.selectedWizardParams, ['resourceGroup'], null);
        if (selectedResourceGroup && resourceGroupOptions) {
            const selectedOption = resourceGroupOptions.find(({value}) => value === selectedResourceGroup)
            change('resourceGroup', selectedOption)
        }
    }, resourceGroupOptions)

    const handleContinue = useCallback(({resourceGroup}) => {
        const selectedResourceGroup = resourceGroup?.value

        if (!selectedResourceGroup) {
            throw new SubmissionError({_error: `Before you can proceed, select a resource group.`});
        }
        sendMixPanelEvent('ResourceGroup');
        const nextStep = calculateNextStep(wizard);
        const nextStepUrl = `/${selectedResourceGroup}/${nextStep}`;
        return Promise.resolve().then(() => {
            wizard.nextStep(nextStepUrl, history);
        });
    }, [wizard, history])

    return {loading, error, resourceGroupOptions, handleContinue}
}

const ResourceGroupStep = (props) => {
    const {loading, error, resourceGroupOptions, handleContinue} = useResourceGroup(props)
    const {handleSubmit} = props;
    return <div className="new-wizard-step resource-group-step">
        <div className="panels-container">
            <div className="left-panel">
                <form className="pane center" onSubmit={handleSubmit(handleContinue)}>
                    <h1>Choose Resource Group</h1>
                    <InternalWizardHeader {...props}/>
                    <div className="step-selector">
                        <div className="step-description">
                            Select a resource group where an existing container is managed, or where you would like to
                            create a new container for tiered data.
                        </div>
                        <div className="step-form">
                            {(loading || !resourceGroupOptions) && !error ?
                                <LoadingIcon text={'Resource Groups'} className="loading-static"/> : (
                                    <Field component={ControlledSelectField} label="Resource Group"
                                           className={"resource-group"}
                                           name="resourceGroup" options={resourceGroupOptions}
                                           placeholder={'Select a Resource Group'}/>
                                )}
                        </div>
                    </div>
                    <div className="wizard-button">
                        <ButtonsPanelWithError error={error}>
                            <Button type="submit">Continue</Button>
                        </ButtonsPanelWithError>
                    </div>
                </form>
            </div>
            <SetupWizardDiagram/>
        </div>
    </div>
}

const nextRoutes = ({match, wizard}) => {
    const nextStep = calculateNextStep(wizard, 'Resource Group');
    const NextStepComp = getNextStepComponent(nextStep);
    return (
        <Switch>
            <Route path={`${match.url}/:resourceGroup/${nextStep}`}
                   render={(props) => <NextStepComp {...props}/>}/>
        </Switch>
    );
};

const ResourceGroupForm = reduxForm({
    form: 'resourceGroupForm',
    validate
})(ResourceGroupStep);


const selector = formValueSelector('resourceGroupForm');

const ResourceGroupFormWithState = connect(state => ({
    selectedResourceGroup: get(selector(state, 'resourceGroup'), 'value'),
}))(ResourceGroupForm);

export default connect(mapStateToProps)(stepinize(ResourceGroupFormWithState, {
    name: () => `Choose Resource group`,
    nextRoutes
}));

