import React, { useCallback, useMemo, useState } from 'react';
import {calculateNextStep, getNextStepComponent} from '../wizard/WizardStarter';
import {Route, Switch} from 'react-router';
import {Field, formValueSelector, reduxForm, stopSubmit, SubmissionError} from 'redux-form';
import mapStateToProps from '../wizard/wizardMapStateToProps';
import {connect} from 'react-redux';
import stepinize from '../wizard/stepinize';
import classNames from 'classnames';
import SetupWizardDiagram from './SetupWizardDiagram';
import ButtonsPanelWithError from '../widgets/ButtonsPanelWithError';
import { CREATE_BUCKET_G, GET_BUCKETS_G_QUERY, GET_REGIONS_QUERY } from '../../graphql/queries';
import LoadingIcon from '../widgets/LoadingIcon';
import Isvg from 'react-inlinesvg';
import { TIERING_SETUP_TITLES } from '../../utils/consts';
import _ from 'lodash';
import {extractError, sendMixPanelEvent} from '../../utils/utils';
import { InputField, InputFieldWithPrefix, RadioInputs, ControlledSelectField } from '../forms/forms';
import Validator from 'validatorjs';
import SearchableSelectableTable, {HEADER_TYPES} from '../table/SearchableSelectableTable';
import {Promise} from 'q';
import InternalWizardHeader from '../../components/wizard/InternalWizardHeader';
import Button from "../widgets/Button";
import { useMutation, useQuery } from '@apollo/client'
import useRunOnceWhenTruthy from '../../hooks/useRunOnceWhenTruthy'


const fabric_pool = 'fabric-pool'
const rules = (provider) => {
    if (provider === 'AWS') {
        return {
            bucketName: ['required', 'only-prefix', 'regex:/(?=^.{3,63}$)(?!^(\\d+\\.)+\\d+$)(^(([a-z0-9]|[a-z0-9][a-z0-9\\-]*[a-z0-9])\\.)*([a-z0-9]|[a-z0-9][a-z0-9\\-]*[a-z0-9])$)/'],
            region: 'required'
        };
    } else {
        return {
            bucketName: ['required'],
            region: 'required'
        };
    }
};


const validate = (values, props) => {
    const {match: {params: {provider}}} = props;

    let errors = {};

    Validator.register('only-prefix', (value) => {
        return value !== fabric_pool;
    }, `A value is required after the "${fabric_pool}" prefix.`);
    const validator = new Validator(values, rules(provider));
    switch (provider) {
        case 'AWS' : {
            validator.setAttributeNames({
                bucketName: 'S3 Bucket Name',
                region: 'S3 Bucket Region'
            });
            break;
        }
        case 'GCP' : {
            validator.setAttributeNames({
                bucketName: 'Bucket Name',
                region: 'Location'
            });
            break;
        }
        default: {
            break;
        }
    }
    validator.passes();
    errors = validator.errors.all();
    return errors;
};

const handleError = (error, action, setError) => {
    const errorMessage = extractError(error, 'Bucket Step SG');
    sendMixPanelEvent(action, errorMessage);
    setError(errorMessage);
}

const getRegionUXByProvider = (provider, regionOptions) => {
    switch (provider) {
        case 'AWS':
            return (
                <>
                    <div className={`pane center bucket-name-field ${provider?.toLowerCase()}`}>
                        <Field name="bucketName" component={InputFieldWithPrefix} label="Bucket Name"
                               parse={addFabricPoolPrefix}
                               format={removeFabricPoolPrefix}
                               placeholder={' Bucket Name'}
                               type="text"
                               prefixVariant={fabric_pool}/>
                    </div>
                    <Field component={ControlledSelectField} label={'Bucket Region'}
                           name="region" options={regionOptions}
                           placeholder={'Region'}
                           className="region" noSearch={true}/>
                </>
            );
        case 'GCP': {
            return (
                <>
                    <div className={`pane center bucket-name-field`}>
                        <Field name="bucketName" component={InputField} label="Bucket Name"
                               type="text"/>
                    </div>
                    <Field component={ControlledSelectField} label={'Bucket Region'}
                           name="region" options={regionOptions}
                           placeholder={'Region'}
                           className="region" noSearch={true}/>
                </>
            );
        }
        default : {
            return <div>Unknown Provider {provider}</div>;
        }
    }
}


const useCreateBucket = (props) => {
    const {wizard, history, agentId, accountId} = props;
    const clusterId = _.get(wizard.selectedWizardParams, 'clusterId', null);
    const provider =  _.get(props, 'routingInfo.urlParams.decodedParams.provider', null)

    const [error, setError] = useState(null)
    const [bucketName, setBucketName] = useState(null)
    const [region, setRegion] = useState(null)

    const nextStep = () => {
        const nextStep = calculateNextStep(wizard);
        sendMixPanelEvent("ContinueBucket");
        wizard.nextStep(`/${region}/${bucketName}/${nextStep}`, history);
    }

    const [createBucketMutation, {loading:loadingCreateBucket}] = useMutation(CREATE_BUCKET_G, {
        onCompleted: nextStep,
        onError: (error) => handleError(error, "CreateBucket-Error", setError)
    })

    const createBucket = useCallback((values) => {
        const { bucketName, region } = values;
        setBucketName(bucketName)
        setRegion(region.value)
        sendMixPanelEvent('CreateBucketSG');
        return createBucketMutation({
            variables:
                {
                    cloudProvider: provider,
                    [provider.toLowerCase()]: {
                        accountId,
                        agentId,
                        region: region.value,
                        name: bucketName,
                        clusterId
                    }
                }
        });
    }, [accountId, agentId, clusterId, provider, createBucketMutation]);
    return {createBucket, loadingCreateBucket, error}
}
const SetupNewBucket = (props) => {
    const {wizard, accountId, agentId, change, handleSubmit} = props
    const clusterId = _.get(wizard.selectedWizardParams, 'clusterId', null);
    const provider =  _.get(props, 'routingInfo.urlParams.decodedParams.provider', null)

    const [error, setError] = useState(null)
    const {createBucket, loadingCreateBucket, error: mutationError} = useCreateBucket(props)

    const {loading, data} = useQuery(GET_REGIONS_QUERY, {
        variables: {
            accountId,
            agentId,
            cloudProvider: provider,
            clusterId
        },
        onError: (error) => handleError(error, 'getRegions', setError),
        skip: !provider || !clusterId
    })

    const regionOptions = useMemo(() => data?.regions.map(region => ({
        label: region.name,
        value: region.code
    })), [data])

    useRunOnceWhenTruthy(()=> {
        const selectedRegion = _.get(wizard.selectedWizardParams, ['region'], null);
        if (selectedRegion) {
            const selectedOption = _.find(regionOptions, {value: selectedRegion})
            change('region', selectedOption)
        }
        const selectedBucketName = _.get(wizard.selectedWizardParams, ['bucketName'], null)
        if (selectedBucketName){
            change('bucketName', selectedBucketName)
        }

    }, regionOptions)

    if (loading && !error && !mutationError) {
        return (<LoadingIcon text={'Regions'} className="loading-static"/>);
    }
    return (
        <>
            {TIERING_SETUP_TITLES[provider].title1}
            <form onSubmit={handleSubmit(createBucket)}>
                <div>
                    {regionOptions && <div className="step-content">
                        {getRegionUXByProvider(provider, regionOptions)}
                    </div>}
                    <div className="wizard-button">
                        <ButtonsPanelWithError error={error || mutationError} submitting={loadingCreateBucket}>
                            <Button type="submit"
                                    disabled={loadingCreateBucket}>Continue</Button>
                        </ButtonsPanelWithError>
                    </div>
                </div>
            </form>
        </>
    );
}

const SetupExistingBucket = (props) => {
    const {wizard, accountId, agentId, handleSubmit, error: formError, dispatch, history } = props
    const provider = _.get(wizard.selectedWizardParams, ['provider'], null);
    const bucketName = _.get(wizard.selectedWizardParams, ['bucketName'], null)
    const [selectedBucket, setSelectedBucket] = useState(null)
    const [state, setState] = useState({})
    const [error, setError] = useState(null)

    const queryCompleted = (data) => {
        if (data) {
            const buckets = data.bucketsG;
            const bucketsRegions = _.uniq(_.map(buckets, 'region'));
            const countedRegions = _.countBy(buckets, (bucket) => bucket.region.name);
            const regionFilters = _.map(bucketsRegions, (region) => {
                return {
                    name: region.name,
                    text: region.name,
                    count: countedRegions[region.name],
                    checked: true
                };
            });
            setState((prevState) => ({
                ...prevState,
                allBuckets: buckets,
                buckets,
                regions: bucketsRegions,
                regionFilters
            }));
            if (bucketName && buckets) {
                const selectedBucket = _.find(buckets, {name: bucketName});
                if (selectedBucket) {
                    const updatedBuckets = _.without(buckets, selectedBucket);
                    updatedBuckets.unshift(selectedBucket);
                    setSelectedBucket(selectedBucket)
                    setState((prev) => ({
                        ...prev,
                        buckets: updatedBuckets,
                    }));
                }
            }
        }
    }

    const { loading} = useQuery(GET_BUCKETS_G_QUERY, {
        fetchPolicy: 'network-only',
        onCompleted: queryCompleted,
        variables: {
            request: {
                cloudProvider: provider,
                [provider.toLowerCase()]: {
                    accountId,
                    agentId,
                }
            }
        },
        onError: (error) => handleError(error, "GetBuckets", setError),
        skip: !provider || !agentId
    })


    const applyFilter = (filters, sort) => {
        const updatedRegionFilters = state.regionFilters.map(value => {
            const filter = _.find(filters, {name: value.name});
            value.checked = filter.checked;
            return value;
        });
        const getBucketsByRegion = (region) => {
            return _.compact(_.map(state.allBuckets, (bucket) => bucket.region.name === region ? bucket : null));
        };
        const buckets = _.reduce(filters, (buckets, filter) => {
            if (filter.checked) {
                return [...buckets, ...getBucketsByRegion(filter.name)];
            } else {
                return buckets;
            }
        }, []);
        if (!sort) {
            setState((prev) => ({
                ...prev,
                regionFilters: updatedRegionFilters,
                buckets
            }));
        } else {
            setState((prev) => ({...prev, regionFilters: updatedRegionFilters})); //the sort will update the buckets
            applySort(sort.column, sort.ascending, buckets);
        }
    }

    const applySort = (column, ascending, buckets) => {
        const sortedBuckets = _.sortBy(buckets ? buckets : state.buckets, [column]);
        setState((prev) => ({...prev, buckets: ascending ? sortedBuckets : _.reverse(sortedBuckets)}));
    }

    const renderRow = (row) => {
        return (<div className='table-rows'>
            <td className="check clickable">
                <div><Isvg className="clickable"
                           src={require('./../../assets/svgs/checkmark-fill-circle-icon.svg')}></Isvg></div>
            </td>
            <td className='bucket-name'>
                {row.name}
            </td>
            <td className='region-name'>
                {row.region.name}
            </td>
        </div>);
    };


    const getTableHeader = () => {
        const {provider} = state;
        const sort = applySort;
        return [
            {
                className: 'check'
            },
            {
                name: 'name',
                text: 'Bucket Name',
                sort
            },
            {
                name: 'region.name',
                text: provider === 'AWS' ? 'Region' : 'Location',
                type: HEADER_TYPES.FILTER,
                filterOptions: {
                    filterName: 'region',
                    filter: applyFilter,
                    filterValues: state.regionFilters
                },
                sort //must give the sort method to allow sorting after the filter is applied


            }
        ];
    };

    const handleSelect = (selectedBucket) => {
        dispatch(stopSubmit('setupBucketForm', {})); //clear errors
        setSelectedBucket(selectedBucket);
    }


    const nextStep = () => {
        if (!selectedBucket) {
            throw new SubmissionError({_error: `Before you can proceed, select a bucket.`});
        }
        return Promise.resolve(true)
            .then(() => {
                sendMixPanelEvent("ContinueBucket");
                const nextStep = calculateNextStep(wizard);
                wizard.nextStep(`/${selectedBucket.region.code}/${selectedBucket.name}/${nextStep}`, history);
            });
    }

    const {buckets} = state;

    if (loading) {
        return (<LoadingIcon text={'Buckets'} className="loading-static"/>)
    }
    return (
        <div className="step-content">
            {TIERING_SETUP_TITLES[provider].title1existing}
            <form onSubmit={handleSubmit(nextStep)}>
                {buckets && <SearchableSelectableTable
                    height={'600px'}
                    Table={{
                        getTableHeader: getTableHeader,
                        renderRow: renderRow
                    }}
                    data={buckets}
                    searchKeys={['name', 'region.name']}
                    selectionOptions={{
                        selector: handleSelect,
                        selectedRows: selectedBucket ? [selectedBucket] : [],
                        clearAll: () => setSelectedBucket(null),
                        getKey: (bucket) => {
                            return {name: bucket.name};
                        },
                        noCountWidget: true
                    }}
                    containerOptions={{
                        singularTitle: 'Bucket',
                        pluralTitle: 'Buckets',
                        maxElementsOnPage: 7,
                        rowHeight: 52
                    }}>
                </SearchableSelectableTable>}
                <div className="wizard-button">
                    <ButtonsPanelWithError error={error || formError}>
                        <Button type="submit">Continue</Button>
                    </ButtonsPanelWithError>
                </div>
            </form>
        </div>
    );

}

const SetupBucketStep = (props) => {
    const {wizard} = props
    const [step, setStep] = useState('newBucket')
    const provider = _.get(wizard.selectedWizardParams, ['provider'], null)
    const bucketName = _.get(wizard.selectedWizardParams, ['bucketName'], null)
    useRunOnceWhenTruthy(() => {
        if (bucketName) {
            setStep('selectBucket')
        }
    }, bucketName)


    if (!provider) return <div/>

    return (
        <div className={classNames('setup-bucket-step new-wizard-step')}>
            <div className="panels-container">
                <div className="left-panel">
                    <div className="pane center">
                        <h1>Tiering Setup</h1>
                        <InternalWizardHeader {...props}/>
                        <div className="step-selector">
                            <Field component={RadioInputs}
                                   changeSelection={(index) => setStep(index === 0 ? 'newBucket' : 'selectBucket')}
                                   checkedIndex={step === 'newBucket' ? 0 : 1}
                                   name="stepSelector"
                                   inputs={[`Add a new ${provider === 'AWS' ? 'S3 bucket' : 'bucket'}`, `Select an existing ${provider === 'AWS' ? 'S3 bucket' : 'bucket'}`]}
                                   className="step-selector-radio"/>
                        </div>
                        {step === 'newBucket' && <SetupNewBucket {...props}/>}
                        {step === 'selectBucket' && <SetupExistingBucket{...props}/>}
                    </div>
                </div>
                <SetupWizardDiagram/>
            </div>
        </div>
    );
}

const nextRoutes = ({match, wizard}) => {
    const nextStep = calculateNextStep(wizard, 'Setup Bucket');
    const NextStepComp = getNextStepComponent(nextStep);
    return (
        <Switch>
            <Route path={`${match.url}/:region/:bucketName/${nextStep}`}
                   render={(props) => <NextStepComp {...props}/>}/>
        </Switch>
    );
};

const addFabricPoolPrefix = (value) => fabric_pool + value;
const removeFabricPoolPrefix = (value) => value?.replace(fabric_pool, '');

let BucketStepForm = reduxForm({
    form: 'setupBucketForm',
    validate
})(SetupBucketStep);

const selector = formValueSelector('setupBucketForm');
const combineStateObjects = (state, ownProps) => {
    const bucketName = selector(state, 'bucketName');
    const region = selector(state, 'region');
    return Object.assign({}, mapStateToProps(state, ownProps), {
        bucketName,
        region,
        accountId: state.general.accountId,
        agentId: state.general.agentId
    });
};
export default connect(combineStateObjects)(stepinize(BucketStepForm, {
    name: () => `Setup Bucket`,
    nextRoutes
}));

