import React, { useState, useCallback, useMemo} from 'react'
import { calculateNextStep, getNextStepComponent } from '../wizard/WizardStarter';
import { Route, Switch } from 'react-router';
import { Field, reduxForm, stopSubmit, SubmissionError } from 'redux-form';
import mapStateToProps from '../wizard/wizardMapStateToProps';
import { connect } from 'react-redux';
import stepinize from '../wizard/stepinize';
import classNames from 'classnames';
import SetupWizardDiagram from './SetupWizardDiagram';
import ButtonsPanelWithError from '../widgets/ButtonsPanelWithError';
import Isvg from 'react-inlinesvg'
import {
    CREATE_BUCKET_G,
    GET_CONTAINERS,
    GET_STORAGE_ACCOUNTS
} from '../../graphql/queries';
import LoadingIcon from '../widgets/LoadingIcon';
import { PROVIDER_TYPE_AZURE, TIERING_SETUP_TITLES } from '../../utils/consts';
import _ from 'lodash';
import { extractError, sendMixPanelEvent } from '../../utils/utils';
import { ControlledSelectField, InputField} from '../forms/forms';
import Validator from 'validatorjs';
import SearchableSelectableTable, { HEADER_TYPES } from '../table/SearchableSelectableTable';
import { Promise } from 'q';
import InternalWizardHeader from '../../components/wizard/InternalWizardHeader';
import { RadioInputs } from '../forms/forms';
import Button from '../widgets/Button';
import { useQuery, useMutation} from '@apollo/client'
import useRunOnceWhenTruthy from '../../hooks/useRunOnceWhenTruthy'

const rules = {
    containerName: ['required', 'regex:/^[a-z0-9](?!.*--)[a-z0-9-]{3,63}[a-z0-9]$/'],
    storageAccount: 'required'
};


const validate = (values) => {
    let errors = {};
    const validator = new Validator(values, rules);
    validator.setAttributeNames({
        containerName: 'Azure Blob Name',
        storageAccount: 'Azure Blob Storage Account'
    });
    validator.passes();
    errors = validator.errors.all();
    return errors;
};

const handleError = (error, action, setError) => {
    if (error) {
        const errorMessage = extractError(error, 'SetupBucketStepAzure');
        sendMixPanelEvent(`${action}-Error`, errorMessage);
        setError(errorMessage)
    }
}

const useCreateContainer = (props) => {
    const {wizard, history, agentId, accountId} = props
    const [error, setError] = useState('')
    const [storageAccount, setStorageAccount] = useState(null)
    const [containerName, setContainerName] = useState(null)

    const nextStep = () => {
        const nextStep = calculateNextStep(wizard);
        wizard.nextStep(`/${storageAccount.name}/${storageAccount.region.code}/${containerName}/${nextStep}`, history);
    }

    const [createAzureContainer, { loading: loadingCreateAzureContainer }] = useMutation(CREATE_BUCKET_G, {
        onCompleted: nextStep,
        onError: (error) => handleError(error, 'createAzureContainer', setError),
    })

    const createContainer = useCallback((values) => {
        const { storageAccount, containerName } = values;
        setContainerName(containerName)
        setStorageAccount(storageAccount.value)
        return createAzureContainer({
            variables: {
                cloudProvider: PROVIDER_TYPE_AZURE.name,
                azure: {
                    accountId,
                    agentId,
                    storageAccount: storageAccount.value.name,
                    name: containerName,
                    resourceGroup: storageAccount.value.resourceGroup
                }
            }
        });
    }, [ agentId, accountId, createAzureContainer]);

    return {loadingCreateAzureContainer, createContainer, error}
}

const SetupNewContainer = (props) => {
    const {loadingCreateAzureContainer, createContainer, error: mutationError} = useCreateContainer(props)
    const { wizard, change, handleSubmit, accountId, agentId } = props;
    const provider = _.get(wizard.selectedWizardParams, ['provider'], null);
    const resourceGroup = _.get(wizard.selectedWizardParams, ['resourceGroup'], null);

    const [error, setError] = useState('')

    const { loading, data } = useQuery(GET_STORAGE_ACCOUNTS, {
        fetchPolicy: 'cache-first',
        variables: {
            accountId,
            agentId,
            resourceGroupName: resourceGroup
        },
        onError: (error) => handleError(error, 'getStorageAccounts', setError),
        skip: !provider || !agentId || !resourceGroup
    });

    const storageAccountOptions = useMemo(() => data?.storageAccounts.map(storageAccount => ({
        label: storageAccount.name,
        value: storageAccount
    })), [data])

    useRunOnceWhenTruthy(()=> {
        const selectedStorageAccount = _.get(wizard.selectedWizardParams, ['storageAccount'], null);
        if (selectedStorageAccount) {
            const selectedOption = _.find(storageAccountOptions, {label: selectedStorageAccount})
            change('storageAccount', selectedOption)
        }
        const selectedContainerName = _.get(wizard.selectedWizardParams, ['containerName'], null)
        if (selectedContainerName){
            change('containerName', selectedContainerName)
        }

    }, storageAccountOptions)


    if (loading) {
        return (<LoadingIcon text={'Storage Accounts'} className="loading-static"/>);
    }


    return (<form
        onSubmit={handleSubmit(createContainer)}>
        <div>
            {storageAccountOptions && <div className="step-content">
                <div className="pane center">
                    <Field name="containerName" component={InputField} label="Container Name"
                           type="text"></Field>
                    <Field component={ControlledSelectField} label="Storage Account"
                           name="storageAccount" options={storageAccountOptions}
                           placeholder='Select a Storage Account'
                           isSearchable={true}
                           className="storageAccount"></Field>
                </div>
            </div>}
            <div className="wizard-button">
                <ButtonsPanelWithError error={error || mutationError}
                                       submitting={loadingCreateAzureContainer}>
                    <Button type="submit"
                            disabled={loadingCreateAzureContainer}>Continue
                    </Button>
                </ButtonsPanelWithError>
            </div>
        </div>
    </form>);
}


const SetupExistingContainer = (props) => {
    const {wizard, accountId, agentId, handleSubmit, error: formError } = props
    const provider = _.get(wizard.selectedWizardParams, ['provider'], null);
    const resourceGroup = _.get(wizard.selectedWizardParams, ['resourceGroup'], null);
    const containerName = _.get(wizard.selectedWizardParams, ['containerName'], null)
    const [selectedContainer, setSelectedContainer] = useState(null)
    const [state, setState] = useState({})

    const buildTableInfo = (data) =>{
        if (data) {
            const containers = data.bucketsG;
            const containersStorageAccounts = _.uniq(_.map(containers, 'storageAccount'));
            const countedStorageAccounts = _.countBy(containers, (container) => container.storageAccount);
            const storageAccountFilters = _.map(containersStorageAccounts, (storageAccount) => {
                return {
                    name: storageAccount,
                    text: storageAccount,
                    count: countedStorageAccounts[storageAccount],
                    checked: true
                };
            });
            setState((prevState) => ({
                ...prevState,
                allContainers: containers,
                containers,
                storageAccountFilters
            }));
            //now that the table is filled, select the existing container name
            if (containerName) {
                const containerObj = _.find(containers, {name: containerName})
                setSelectedContainer(containerObj)
            }
        }
    }

    const { loading, error } = useQuery(GET_CONTAINERS, {
        fetchPolicy: 'network-only',
        onCompleted: buildTableInfo,
        variables: {
            request: {
                cloudProvider: PROVIDER_TYPE_AZURE.name,
                azure: {
                    accountId,
                    agentId,
                    resourceGroupName: resourceGroup
                }
            }
        },
        onError: _.noop,
        skip: !provider || !agentId || !resourceGroup
    })

    const nextStep = () => {
        if (!selectedContainer) {
            throw new SubmissionError({ _error: `Before you can proceed, select a container.` });
        }
        const nextStep = calculateNextStep(props.wizard);
        const nextStepUrl = `/${selectedContainer.storageAccount}/${selectedContainer.region.code}/${selectedContainer.name}/${nextStep}`;
        return Promise.resolve(true)
            .then(() => {
                props.wizard.nextStep(nextStepUrl, props.history);
            });
    }


    const renderRow = (row) => {
        return (<div className='table-rows'>
            <td className="check clickable">
                <div><Isvg className="clickable"
                           src={require('./../../assets/svgs/checkmark-fill-circle-icon.svg')}></Isvg></div>
            </td>
            <td className='container-name'>
                {row.name}
            </td>
            <td className='storageAccount-name no-wrap' title={row.storageAccount}>
                {row.storageAccount}
            </td>
        </div>);
    }
    const applyFilter = (filters, sort) => {
        const updatedStroageAccountFilters = state.storageAccountFilters.map(value => {
            const filter = _.find(filters, { name: value.name });
            value.checked = filter.checked;
            return value;
        });
        const getContainersByStroageAccount = (storageAccount) => {
            return _.compact(_.map(state.allContainers, (container) => container.storageAccount === storageAccount ? container : null));
        };
        const containers = _.reduce(filters, (containers, filter) => {
            if (filter.checked) {
                return [...containers, ...getContainersByStroageAccount(filter.name)];
            } else {
                return containers;
            }
        }, []);
        if (!sort) {
            setState((prevState) => ({
                ...prevState,
                storageAccountFilters: updatedStroageAccountFilters,
                containers
            }));
        } else {
            setState((prevState)=>({...prevState, storageAccountFilters: updatedStroageAccountFilters })); //the sort will update the containers
            applySort(sort.column, sort.ascending, containers);
        }
    }
    const applySort = (column, ascending, containers) => {
        const sortedContainers = _.sortBy(containers ? containers : state.containers, [column]);
        setState((prevState) => ({...prevState, containers: ascending ? sortedContainers : _.reverse(sortedContainers) }));
    }
    const getTableHeader = () => {
        const sort = applySort;
        return [
            {
                className: 'check'
            },
            {
                name: 'name',
                text: 'Container Name',
                sort
            },
            {
                name: 'storageAccount',
                text: 'Storage Account',
                type: HEADER_TYPES.FILTER,
                filterOptions: {
                    filterName: 'storageAccount',
                    filter: applyFilter,
                    filterValues: state.storageAccountFilters
                },
                sort

            }
        ];
    }

    const handleSelect = (selectedContainer) => {
        props.dispatch(stopSubmit('setupContainerForm', {})); //clear errors
        setSelectedContainer(selectedContainer)
    }


    const {containers} = state
    if ((loading || !containers) && !error) {//due to skip there might not be data yet
        return (<LoadingIcon text={'Containers'} className="loading-static"/>);
    }
    return (
        <div className="step-content">
            <form onSubmit={handleSubmit(nextStep)}>
                <SearchableSelectableTable
                    height={'600px'}
                    Table={{
                        getTableHeader: getTableHeader,
                        renderRow: renderRow
                    }}
                    data={containers}
                    searchKeys={['name', 'storageAccount', 'resourceGroup']}
                    selectionOptions={{
                        selector: handleSelect,
                        selectedRows: selectedContainer ? [selectedContainer] : [],
                        clearAll: () => setSelectedContainer(null),
                        getKey: (container) => {
                            return {
                                name: container.name,
                                storageAccount: container.storageAccount
                            };
                        },
                        noCountWidget: true
                    }}
                    containerOptions={{
                        singularTitle: 'Container',
                        pluralTitle: 'Containers',
                        maxElementsOnPage: 7,
                        rowHeight: 52
                    }}>
                </SearchableSelectableTable>
                <div className="wizard-button">
                    <ButtonsPanelWithError error={error || formError}>
                        <Button type="submit">Continue</Button>
                    </ButtonsPanelWithError>
                </div>
            </form>
        </div>
    );
}

const SetupContainerStep = (props) => {
    const {wizard} = props
    const containerName = _.get(wizard.selectedWizardParams, ['containerName'], null)
    const [step, setStep] = useState('newContainer')

    useRunOnceWhenTruthy(() => {
        if (containerName) {
            setStep('selectContainer')
        }
    }, containerName)


    return (
        <div className={classNames('setup-bucket-step new-wizard-step')}>
            <div className="panels-container">
                <div className="left-panel">
                    <div className="pane center">
                        <h1>Tiering Setup</h1>
                        <InternalWizardHeader {...props}/>
                        <div className="step-selector">
                            <Field component={RadioInputs}
                                   changeSelection={(index) => setStep(index === 0 ? 'newContainer' : 'selectContainer')}
                                   checkedIndex={step === 'newContainer' ? 0 : 1} name="stepSelector"
                                   inputs={['Add a new container', 'Select an existing container']}
                                   className="step-selector-radio"/>
                        </div>
                        {TIERING_SETUP_TITLES['AZURE'].title1}
                        {step === 'newContainer' && <SetupNewContainer {...props}/>}
                        {step === 'selectContainer' && <SetupExistingContainer{...props}/>}
                    </div>
                </div>
                <SetupWizardDiagram/>
            </div>
        </div>
    );

}

const nextRoutes = ({ match, wizard }) => {
    const nextStep = calculateNextStep(wizard, 'Setup Bucket Azure');
    const NextStepComp = getNextStepComponent(nextStep);
    return (
        <Switch>
            <Route path={`${match.url}/:storageAccount/:region/:containerName/${nextStep}`}
                   render={(props) => <NextStepComp {...props}/>}/>
        </Switch>
    );
};

let ContainerStepForm = reduxForm({
    form: 'setupContainerForm',
    validate
})(SetupContainerStep);

export default connect(mapStateToProps)(stepinize(ContainerStepForm, {
    name: () => `Setup Bucket Azure`,
    nextRoutes
}));

