import React, { useState, useCallback} from 'react';

import InternalWizardHeader from '../wizard/InternalWizardHeader';
import { Field, reduxForm, stopSubmit, SubmissionError } from 'redux-form';
import { InputField, RadioInputs } from '../forms/forms';
import ButtonsPanelWithError from '../widgets/ButtonsPanelWithError';
import classNames from 'classnames';
import SetupWizardDiagram from './SetupWizardDiagram';
import stepinize from '../wizard/stepinize';
import mapStateToProps from '../wizard/wizardMapStateToProps';
import { calculateNextStep, getNextStepComponent } from '../wizard/WizardStarter';
import { Route, Switch } from 'react-router';
import { connect } from 'react-redux';
import { CREATE_BUCKET_G, GET_BUCKETS_G_QUERY } from '../../graphql/queries';
import { decodeAll } from '../../utils/encoders';
import localStorage from '../../utils/localStorage';
import { extractError, sendMixPanelEvent } from '../../utils/utils';
import _ from 'lodash';
import { PROVIDER_TYPE_SGWS, TIERING_SETUP_TITLES } from '../../utils/consts';
import SearchableSelectableTable from '../table/SearchableSelectableTable';
import Isvg from 'react-inlinesvg';
import { Promise } from 'q';
import LoadingIcon from '../widgets/LoadingIcon';
import Validator from 'validatorjs';
import Button from "../widgets/Button";
import useRunOnce from '../../hooks/useRunOnce'
import useRunOnceWhenTruthy from '../../hooks/useRunOnceWhenTruthy'
import { useMutation, useQuery } from '@apollo/client'


require('./setupBucketStep.scss');

const rules = {
    bucketName: ['required']
};


const validate = (values) => {
    let errors = {};
    const validator = new Validator(values, rules);
    validator.setAttributeNames({
        bucketName: 'StorageGRID Bucket Name'
    });
    validator.passes();
    errors = validator.errors.all();
    return errors;
};

const handleError = (error, action, setError) => {
    const errorMessage = extractError(error, 'Bucket Step SG');
    sendMixPanelEvent(action, errorMessage);
    setError(errorMessage);
}


const SetupExistingBucket = (props) => {
    const {wizard, agentId, accountId, history, dispatch, handleSubmit, keys: {accessKey, secretKey}} = props
    const provider = _.get(wizard.selectedWizardParams, 'provider', null);
    const server = decodeAll(_.get(wizard.selectedWizardParams, 'sg', null));
    const portFromURL = _.get(wizard.selectedWizardParams, 'port', null);
    const port = portFromURL !== "undefined" && portFromURL !== "null" ? parseInt(portFromURL) : null
    const bucketName = _.get(wizard.selectedWizardParams, 'bucketName', null);

    const [error, setError] = useState(null)
    const [selectedBucket, setSelectedBucket] = useState(null)
    const [showBuckets, setShowBuckets] = useState(null)

    const queryCompleted = (data) => {
        if (data) {
            const buckets = data.bucketsG;
            if (bucketName && buckets) {
                const selectedBucket = _.find(buckets, { name: bucketName });
                if (selectedBucket) {
                    const updatedBuckets = _.without(buckets, selectedBucket);
                    updatedBuckets.unshift(selectedBucket);
                    setShowBuckets(updatedBuckets)
                    setSelectedBucket(selectedBucket)
                }
            }
            else {
                setShowBuckets(buckets)
            }
        }
    }

    const {loading} = useQuery(GET_BUCKETS_G_QUERY, {
        fetchPolicy: 'network-only',
        onCompleted: queryCompleted,
        variables: {
            request: {
                cloudProvider: PROVIDER_TYPE_SGWS.name,
                sgws: {
                    accountId,
                    agentId,
                    host: server,
                    port,
                    credentials: {
                        accessKey,
                        secretKey
                    }
                }
            }
        },
        onError: (error) => handleError(error, 'CreateBucketSG', setError),
        skip: !agentId || !server || !accessKey || !secretKey
    })

    const nextStep = () => {
        if (!selectedBucket) {
            throw new SubmissionError({ _error: `Before you can proceed, select a bucket.` });
        }
        return Promise.resolve(true)
            .then(() => {
                sendMixPanelEvent("ContinueBucketSG");
                const nextStep = calculateNextStep(wizard);
                wizard.nextStep(`/${selectedBucket.name}/${nextStep}`, history);
            });
    }

    const applySort = (column, ascending) => {
        const sortedBuckets = _.sortBy(showBuckets, [column]);
        setShowBuckets(ascending ? sortedBuckets : _.reverse(sortedBuckets));
    }

    const renderRow = (row) => {
        return (<div className='table-rows'>
            <td className="check clickable">
                <div><Isvg className="clickable"
                           src={require('./../../assets/svgs/checkmark-fill-circle-icon.svg')}></Isvg></div>
            </td>
            <td className='bucket-name'>
                {row.name}
            </td>
        </div>);
    };

    const getTableHeader = () => {
        return [
            {
                className: 'check'
            },
            {
                name: 'name',
                text: 'Bucket Name',
                sort: applySort
            }
        ];
    };

    const handleSelect = (selectedBucket) => {
        dispatch(stopSubmit('setupBucketSGForm', {})); //clear errors
        setSelectedBucket(selectedBucket);
    }

    if ((loading || !showBuckets) && !error) {//due to skip there might not be data yet
        return (<LoadingIcon text={'Buckets'} className="loading-regions"/>);
    }

    return (
        <div className="step-content">
            {TIERING_SETUP_TITLES[provider].title1existing}
            <form onSubmit={handleSubmit(nextStep)}>
                {showBuckets && <SearchableSelectableTable
                    height={'600px'}
                    Table={{
                        getTableHeader,
                        renderRow
                    }}
                    data={showBuckets}
                    searchKeys={['name']}
                    selectionOptions={{
                        selector: handleSelect,
                        selectedRows: selectedBucket ? [selectedBucket] : [],
                        clearAll: () => setSelectedBucket(null),
                        getKey: (bucket) => {
                            return { name: bucket.name };
                        },
                        noCountWidget: true
                    }}
                    containerOptions={{
                        singularTitle: 'Bucket',
                        pluralTitle: 'Buckets',
                        maxElementsOnPage: 7,
                        rowHeight: 52
                    }}>
                </SearchableSelectableTable>}
                <div className="wizard-button">
                    <ButtonsPanelWithError error={error}>
                        <Button type="submit">Continue</Button>
                    </ButtonsPanelWithError>
                </div>
            </form>
        </div>
    );

}
const useCreateBucket = (props) => {
    const {wizard, agentId, accountId, history, keys: {accessKey, secretKey}} = props
    const server = decodeAll(_.get(wizard.selectedWizardParams, 'sg', null));
    const portFromURL = _.get(wizard.selectedWizardParams, 'port', null);
    const port = portFromURL !== "undefined" && portFromURL !== "null" ? parseInt(portFromURL) : undefined

    const [error, setError] = useState(null)
    const [bucketName, setBucketName] = useState(null)

    const nextStep = () => {
        const nextStep = calculateNextStep(wizard);
        sendMixPanelEvent("ContinueBucketSG");
        wizard.nextStep(`/${bucketName}/${nextStep}`, history);
    }

    const [createBucketMutation, {loading:loadingCreateBucket}] = useMutation(CREATE_BUCKET_G, {
        onCompleted: nextStep,
        onError: (error) => handleError(error, "CreateBucketSG-Error", setError)
    })

    const createBucket = useCallback((values) => {
        const { bucketName } = values;
        setBucketName(bucketName)
        sendMixPanelEvent("CreateBucketSG");
        return createBucketMutation({
            variables: {
                cloudProvider: PROVIDER_TYPE_SGWS.name,
                sgws: {
                    accountId,
                    agentId,
                    host: server,
                    port,
                    name: bucketName,
                    credentials: {
                        accessKey,
                        secretKey
                    }
                }
            }
        });
    }, [accountId, agentId, server, port, accessKey, secretKey, createBucketMutation]);

    return {createBucket, loadingCreateBucket, error}
}

const SetupNewBucket = (props) => {
    const {wizard, change, handleSubmit} = props
    const server = decodeAll(_.get(wizard.selectedWizardParams, 'sg', null));
    const bucketName = _.get(wizard.selectedWizardParams, 'bucketName', null);
    const {createBucket, loadingCreateBucket, error: mutationError} = useCreateBucket(props)

    useRunOnceWhenTruthy(() => {
        change('bucketName',bucketName);
    }, bucketName)

    if (!server) {
        return <div></div>
    }
    return (<>
        {TIERING_SETUP_TITLES[PROVIDER_TYPE_SGWS.name].title1}
        <form onSubmit={handleSubmit(createBucket)}>
            <div>
                <div className="step-content">
                    <div className="pane center">
                        <Field name="bucketName" component={InputField} label="Bucket Name"
                               type="text"></Field>
                    </div>
                </div>
                <div className="wizard-button">
                    <ButtonsPanelWithError error={mutationError} submitting={loadingCreateBucket}>
                        <Button type="submit" disabled={loadingCreateBucket}>Continue</Button>
                    </ButtonsPanelWithError>
                </div>
            </div>
        </form>
        ;
    </>)

}


const SetupBucketStepSG = (props) => {
    const {wizard} = props
    const [step, setStep] = useState('newBucket')
    const [accessKey, setAccessKey] = useState(null)
    const [secretKey, setSecretKey] = useState(null)
    const bucketName = _.get(wizard.selectedWizardParams, 'bucketName', null);

    useRunOnceWhenTruthy(() => {
        setStep('selectBucket')
    }, bucketName)

    useRunOnce(() => {
        const {accessKey, secretKey} = localStorage.get('selectedKeys');
        setAccessKey(accessKey);
        setSecretKey(secretKey)
    })



    return (
        <div className={classNames('setup-bucket-step new-wizard-step')}>
            <div className="panels-container">
                <div className="left-panel">
                    <div className="pane center">
                        <h1>Tiering Setup</h1>
                        <InternalWizardHeader {...props}/>
                        <div className="step-selector">
                            <Field component={RadioInputs}
                                   changeSelection={(index) => setStep(index === 0 ? 'newBucket' : 'selectBucket')}
                                   checkedIndex={step === 'newBucket' ? 0 : 1} name="stepSelector"
                                   inputs={['Add a new bucket', 'Select an existing bucket']}
                                   className="step-selector-radio"/>
                        </div>
                        {step === 'newBucket' && <SetupNewBucket keys={{accessKey, secretKey}} {...props} />}
                        {step === 'selectBucket' && <SetupExistingBucket keys={{accessKey, secretKey}} {...props} />}
                    </div>
                </div>
                <SetupWizardDiagram/>
            </div>
        </div>
    );
}


const nextRoutes = ({ match, wizard }) => {
    const nextStep = calculateNextStep(wizard, 'Setup Bucket SG');
    const NextStepComp = getNextStepComponent(nextStep);
    return (
        <Switch>
            <Route path={`${match.url}/:bucketName/${nextStep}`} render={(props) => <NextStepComp {...props}/>}/>
        </Switch>
    );
};


const SetupBucketStepSGForm = reduxForm({
    form: 'SetupBucketStepSGForm',
    validate
})(SetupBucketStepSG);

export default connect(mapStateToProps)(stepinize(SetupBucketStepSGForm, {
    name: () => `Setup Bucket SG`,
    nextRoutes
}));




