import React, {useState} from 'react';
import {calculateNextStep, getNextStepComponent} from '../wizard/WizardStarter';
import {Route, Switch} from 'react-router';
import {Field, reduxForm} from 'redux-form';
import mapStateToProps from '../wizard/wizardMapStateToProps';
import {connect} from 'react-redux';
import stepinize from '../wizard/stepinize';
import classNames from 'classnames';
import ButtonsPanelWithError from '../widgets/ButtonsPanelWithError';
import SetupWizardDiagram from './SetupWizardDiagram';
import {TIERING_SETUP_TITLES} from '../../utils/consts';
import {VERIFY_PERMISSIONS} from '../../graphql/queries';
import localStorage from '../../utils/localStorage';
import _ from 'lodash';
import {extractError, sendMixPanelEvent} from '../../utils/utils';
import Validator from 'validatorjs';
import {InputField} from '../forms/forms';
import InternalWizardHeader from '../wizard/InternalWizardHeader';
import Button from "../widgets/Button";
import useRunOnce from '../../hooks/useRunOnce'
import { useMutation } from '@apollo/client'

require('./setupCredentialsStep.scss');

const rules = {
    accessKey: 'required',
    secretKey: 'required'
};


const validate = (values, props) => {
    const provider = _.get(props, 'routingInfo.urlParams.decodedParams.provider', null);
    let errors = {};
    const validator = new Validator(values, rules);
    switch (provider) {
        case "AWS" : {
            validator.setAttributeNames({
                accessKey: "AWS Access Key",
                secretKey: "AWS Secret Key",
                region: 'S3 Bucket Region'
            });
            break;
        }
        case "GCP" : {
            validator.setAttributeNames({
                accessKey: "GCP Access Key",
                secretKey: "GCP Secret Key",
                region: 'Bucket Region'
            });
            break;
        }
        default: {
            break;
        }
    }
    validator.passes();
    errors = validator.errors.all();
    return errors;
};

const handleError = (error, action, setError) => {
    const errorMessage = extractError(error, 'Storage Class Step');
    sendMixPanelEvent(action, errorMessage);
    setError(error)
}

const SetupCredentialsStep  = (props) =>{
    const {wizard, history, change, accountId, handleSubmit} = props;
    const region = _.get(wizard.selectedWizardParams, "region", null)
    const bucketName =  _.get(wizard.selectedWizardParams, "bucketName", null)
    const provider =  _.get(props, 'routingInfo.urlParams.decodedParams.provider', null)
    const [error, setError] = useState(null)

    useRunOnce(() => {
        const keys = localStorage.get("selectedKeys");
        if (keys) {
            change("accessKey", keys.accessKey);
            change("secretKey", keys.secretKey);
        }
    })

    const nextStep = () => {
        const nextStep = calculateNextStep(wizard);
        sendMixPanelEvent("ContinueCredentials");
        wizard.nextStep(`/next/${nextStep}`, history);
    }
    const [verifyPermissionsMutation, {loading:loadingConfigureBucket}] = useMutation(VERIFY_PERMISSIONS, {
        onCompleted: nextStep,
        onError: (error) => handleError(error, 'ContinueCredentials-Error', setError),
        errorPolicy: 'all'
    })

    const verifyPermissions = (values) => {
        const accessKey = values.accessKey;
        const secretKey = values.secretKey;
        localStorage.set("selectedKeys", {accessKey, secretKey});

        sendMixPanelEvent("VerifyPermissions");
        return verifyPermissionsMutation({
            variables: {
                accountId,
                bucket: bucketName,
                cloudProvider: provider,
                region,
                credentials: {accessKey, secretKey}
            }
        });
    }



    const getContentUX = (provider) => {
        switch (provider) {
            case "AWS":
            case "GCP": {
                return (
                    <>
                        <div className="pane center">
                            <div className="">
                                <Field name="accessKey" component={InputField} label={`${provider} Access Key`}
                                       type="text"></Field>
                            </div>
                            <div className="">
                                <Field name="secretKey" component={InputField} label={`${provider} Secret Key`}
                                       type="password"></Field>
                            </div>
                        </div>
                    </>
                )
            }
            default : {
                return <div>Unknown Provider {provider}</div>
            }
        }
    }


    if (!provider) {
        return <div/>;
    }
    return (
        <div className={classNames('setup-credentials-step new-wizard-step')}>
            <div className="panels-container">
                <div className="left-panel">
                    <div className="pane center">
                        <h1>Tiering Setup</h1>
                        <InternalWizardHeader {...props}/>
                        {TIERING_SETUP_TITLES[provider].title3}
                    </div>
                    <form
                        onSubmit={handleSubmit(verifyPermissions)}>
                        <div>
                            <div className="step-content">
                                {getContentUX(provider)}
                            </div>
                            <div className="wizard-button">
                                <div className="wizard-button">
                                    <ButtonsPanelWithError error={error}
                                                           submitting={loadingConfigureBucket}>
                                        <Button type="submit"
                                                disabled={loadingConfigureBucket}>Continue</Button>
                                    </ButtonsPanelWithError>
                                </div>
                            </div>
                        </div>
                    </form>
                </div>
                <SetupWizardDiagram flowStep="cluster-discovered"/>
            </div>
        </div>
    );

}

const nextRoutes = ({match, wizard}) => {
    const nextStep = calculateNextStep(wizard, 'Setup Credentials');
    const NextStepComp = getNextStepComponent(nextStep);
    return (
        <Switch>
            <Route path={`${match.url}/:credentials/${nextStep}`} render={(props) => <NextStepComp {...props}/>}/>
        </Switch>
    );
};

let CredentialsStepForm = reduxForm({
    form: 'setupCredentialsForm',
    validate
})(SetupCredentialsStep);

export default connect(mapStateToProps)(stepinize(CredentialsStepForm, {
    name: () => `Setup Credentials`,
    nextRoutes
}));

