import React, { useState } from 'react';
import { useLazyQuery } from '@apollo/client';
import SetupWizardDiagram from './SetupWizardDiagram';
import { Field, reduxForm, formValueSelector } from 'redux-form';
import ButtonsPanelWithError from '../widgets/ButtonsPanelWithError';
import { connect } from 'react-redux';
import stepinize from '../wizard/stepinize';
import { calculateNextStep, getNextStepComponent } from '../wizard/WizardStarter';
import { Route, Switch } from 'react-router';
import { sendMixPanelEvent } from '../../utils/utils';
import mapStateToProps from '../wizard/wizardMapStateToProps';
import InternalWizardHeader from '../wizard/InternalWizardHeader';
import { InputField } from '../forms/forms';
import Validator from 'validatorjs';
import { decodeAll, encodeAll } from '../../utils/encoders';
import _ from 'lodash';
import localStorage from '../../utils/localStorage';
import { HOST_NAME_REG_EXP, PROVIDER_TYPE_SGWS } from '../../utils/consts';
import { GET_BUCKETS_G_QUERY } from '../../graphql/queries';
import { KEYS_REG_EXP } from '../../utils/consts';
import Button from "../widgets/Button";

require('./storageGridStep.scss');

const rules = {
    server: ['required', 'max:256', `regex:${HOST_NAME_REG_EXP}`],
    port: ['numeric', 'between:1,65535'],
    accessKey: ['required', `regex:${KEYS_REG_EXP}`],
    secretKey: ['required', `regex:${KEYS_REG_EXP}`]
};


const validate = (values) => {
    const validator = new Validator(values, rules);
    validator.setAttributeNames({
        port: 'Port',
        accessKey: `${PROVIDER_TYPE_SGWS.displayName} Access Key`,
        secretKey: `${PROVIDER_TYPE_SGWS.displayName} Secret Key`,
        server: `${PROVIDER_TYPE_SGWS.displayName} Server Name (FQDN)`
    });
    validator.passes();
    return validator.errors.all();
};


const StorageGridStep = (props) => {
    const { handleSubmit, change, error: formError, wizard, accountId } = props;
    const agentId = props.match.params.agentId;
    const keys = localStorage.get('selectedKeys');

    const [server, setServer] = useState(() => {
        const selectedServer = decodeAll(_.get(wizard.selectedWizardParams, ['sg'], null));
        change('server', selectedServer);
        return selectedServer;
    });
    const [port, setPort] = useState(() => {
        const selectedPort = _.get(wizard.selectedWizardParams, ['port'], null);
        if (selectedPort !== "undefined" && selectedPort !== "null") {
            change('port', selectedPort);
            return selectedPort;
        }
        return undefined;
    });
    const [accessKey, setAccessKey] = useState(() => {
        if (keys) {
            const { accessKey } = keys;
            change('accessKey', accessKey);
            return accessKey;
        }
        return null;
    });
    const [secretKey, setSecretKey] = useState(() => {
        if (keys) {
            const { secretKey } = keys;
            change('secretKey', secretKey);
            return secretKey;
        }
        return null;

    });
    const [validateServerError, setValidateServerError] = useState(null);

    const next = () => {
        const nextStep = calculateNextStep(wizard);
        sendMixPanelEvent('ContinueStorageGrid');
        wizard.nextStep(`/${encodeAll(server)}/${port}/${nextStep}`, props.history);
    };
    const [getSGBuckets, { loading, error }] = useLazyQuery(GET_BUCKETS_G_QUERY, {
        variables: {
            request: {
                cloudProvider: PROVIDER_TYPE_SGWS.name,
                sgws: {
                    accountId,
                    agentId,
                    host: server,
                    port: parseInt(port),
                    credentials: {
                        accessKey,
                        secretKey
                    }
                }
            }
        },
        notifyOnNetworkStatusChange: true,
        fetchPolicy: 'network-only',
        onCompleted: next,
        onError: (err) => setValidateServerError(err)
    });


    const validateServer = (values) => {
        setValidateServerError(null); //clear errors
        const server = values.server;
        const port = values.port;
        const accessKey = values.accessKey;
        const secretKey = values.secretKey;
        localStorage.set('selectedKeys', {
            accessKey,
            secretKey
        });
        setServer(server);
        setPort(port);
        setAccessKey(accessKey);
        setSecretKey(secretKey);
        getSGBuckets();
    };

    return (
        <div className="storage-grid-step new-wizard-step">
            <div className="panels-container">
                <div className="left-panel">
                    <div className="pane center">
                        <h1>Tiering Setup</h1>
                        <InternalWizardHeader {...props}/>
                        <form onSubmit={handleSubmit(validateServer)}>
                            <div className="">
                                <Field name="server" component={InputField} label="Server Name (FQDN)"
                                       type="text"></Field>
                            </div>
                            <div className="">
                                <Field name="port" component={InputField} label="Port (Optional)"
                                       type="text"></Field>
                            </div>
                            <div className="">
                                <Field name="accessKey" component={InputField} label="Access Key"
                                       type="text"></Field>
                            </div>
                            <div className="">
                                <Field name="secretKey" component={InputField} label="Secret Key"
                                       type="password"></Field>
                            </div>

                            <div className="wizard-button">
                                <ButtonsPanelWithError error={formError || error || validateServerError ? 'Unable to access the defined StorageGRID' : null} submitting={loading}>
                                    <Button type="submit" >Continue</Button>
                                </ButtonsPanelWithError>
                            </div>

                        </form>
                    </div>
                </div>
                <SetupWizardDiagram flowStep="storage-grid"/>
            </div>
        </div>
    );
};

const nextRoutes = ({ match, wizard }) => {
    const nextStep = calculateNextStep(wizard, 'Setup Storage Grid');
    const NextStepComp = getNextStepComponent(nextStep);
    return (
        <Switch>
            <Route path={`${match.url}/:sg/:port/${nextStep}`} render={(props) => <NextStepComp {...props}/>}/>
        </Switch>
    );
};


const StorageGridStepForm = reduxForm({
    form: 'storageGrid',
    validate
})(StorageGridStep);

const selector = formValueSelector('storageGrid');
const combineStateObjects = (state, ownProps) => {
    return Object.assign({}, mapStateToProps(state, ownProps), {
        formServer: selector(state, 'server'),
        formPort: selector(state, 'port'),
        formAccessKey: selector(state, 'accessKey'),
        formSecretKey: selector(state, 'secretKey')
    });
};


export default connect(combineStateObjects)(stepinize(StorageGridStepForm, {
    name: () => `Setup Storage Grid`,
    nextRoutes
}));

