import React, {Component, Fragment} from "react"
import SearchWidget from "./../widgets/SearchWidget"
import ItemCountWidget from "./../widgets/ItemCountWidget"
import SelectedCountWidget from "./../widgets/SelectedCountWidget"
import PaginationWidget from "./../widgets/PaginationWidget"
import {defaultFilter, defaultFilterWithKeys} from "../../utils/utils";
import _ from "lodash";
import {CSSTransition, TransitionGroup} from "react-transition-group";
import classNames from "classnames";
import FilterWidget from '../widgets/FilterWidget'
import Isvg from "react-inlinesvg";
import InfoPopover from '../reactPopover/InfoPopover';

require("./searchableSelectableTable.scss");


export const HEADER_TYPES = {
    "ALL": "all",
    "FILTER": "filter",
    "TD": "td"
};

export class SelectableTable extends Component {
    constructor(props, context) {
        super(props, context);
        this.state = {mask: false};
    }

    applyFilter(filterMethod, filterName, filters) {
        const filtersCount = _.reduce(filters, (count, value) => count + (value.checked ? 1 : 0), 0);
        this.setState({[filterName]: {filterOpen: false, filtersCount}, mask: false});
        filterMethod(filters, this.state.sort);
        const {fromStart} = this.props;
        fromStart(); //to clear pagination
    }

    handleSort(sortMethod, column) {
        //if columns was not sorted yet, we start with ascending
        const ascending = this.state.sort && this.state.sort.column === column ? !this.state.sort.ascending : true;
        this.setState({sort: {column, ascending}});
        sortMethod(column, ascending);
        const {fromStart} = this.props;
        fromStart(); //to clear pagination
    }

    render() {
        const {Table, data, selectionOptions} = this.props;
        const headerElement = Table.getTableHeader ? Table.getTableHeader().map((cell, index) => {
            const type = cell.type ? cell.type : HEADER_TYPES.TD;
            const filtersCountFromState = cell.filterOptions ? _.get(this.state, `${cell.filterOptions.filterName}.filtersCount`, undefined) : undefined;
            const filtersCount = cell.filterOptions ? (filtersCountFromState ? filtersCountFromState : _.reduce(cell.filterOptions.filterValues, (count, value) => count + (value.checked ? 1 : 0), 0)) : undefined;
            const ascSort = cell.sort && this.state.sort && this.state.sort.column === cell.name ? this.state.sort.ascending : null;
            return (
                <Fragment key={index}>
                    {type === HEADER_TYPES.ALL &&
                    <td key={index}
                        className={classNames(cell.className, "all-selector", {selected: cell.allOptions.allSelected})}
                        onClick={() => cell.allOptions.toggleAll()}>
                        <div>
                            <Isvg
                                src={require("./../../assets/svgs/checkmark-transparent-circle-icon.svg")}></Isvg>
                        </div>
                    </td>}
                    {type === HEADER_TYPES.FILTER &&
                    <td key={index} className={`${cell.className} filter-header`}>
                        <div className="with-right-border">
                            <span>{cell.text}</span>
                            <button type="button" onClick={() => this.setState({
                                [cell.filterOptions.filterName]: {
                                    filterOpen: true,
                                    filtersCount
                                },
                                mask: true
                            })}>
                                <Isvg src={require('./../../assets/svgs/filter.svg')}></Isvg>
                            </button>
                            {cell.filterOptions && this.state[cell.filterOptions.filterName] && this.state[cell.filterOptions.filterName].filterOpen &&
                            <FilterWidget filterValues={cell.filterOptions.filterValues}
                                          allCount={cell.filterOptions.allCount}
                                          close={() => this.setState({
                                              [cell.filterOptions.filterName]: {
                                                  filterOpen: false,
                                                  filtersCount
                                              },
                                              mask: false
                                          })}
                                          apply={this.applyFilter.bind(this, cell.filterOptions.filter, cell.filterOptions.filterName)}
                                          fixed={false}/>}</div>
                    </td>}
                    {type === HEADER_TYPES.TD &&
                    <td key={index} className={cell.className}>
                        <div className="flex-td with-right-border">
                            {cell.info &&
                            <InfoPopover wrapperClassName="table-header-info"
                                         popoverClassName={`table-header-info ${cell.info.className}`}
                                         popoverText={cell.info.text} contentStyle={{width: cell.info.width}}/>
                            }
                            <div className="column-name">
                                {cell.text}
                            </div>
                            {cell.sort &&
                            <div className="sort">
                                <button type="button" className={classNames("sort-button", {
                                    none: _.isNull(ascSort) && cell.sort,
                                    asc: ascSort === true,
                                    dsc: ascSort === false,
                                })}
                                        onClick={() => this.handleSort(cell.sort, cell.name)}>
                                </button>
                            </div>}
                        </div>
                    </td>}
                </Fragment>
            )
        }) : null;

        const d = data.map((row, index) => {
                const rowElement = Table.renderRow(row);
                const key = selectionOptions && selectionOptions.getKey ? selectionOptions.getKey(row) : {id: index}; //in case no get key was defined, use id
                const disabled = row.isDisabled;
                const selected = selectionOptions && _.find(selectionOptions.selectedRows, key) !== undefined;
                return (
                    <CSSTransition classNames="tr" timeout={1000} key={JSON.stringify(key)}>
                        <tr key={JSON.stringify(key)}
                            onClick={(event) => selectionOptions && !disabled ? selectionOptions.selector(row, event.target) : ""}
                            className={classNames(rowElement.props.className, {selected, disabled})}>
                            {rowElement.props.children}
                        </tr>
                    </CSSTransition>)
            }
        );

        const classes = classNames("standard", {selectable: !!selectionOptions}, Table.className);
        const tableHeight = this.props.height ? {height: this.props.height} : {flex: 1}

        return (
            <div className="selectable-table-wrapper" style={tableHeight}>
                {headerElement &&
                <table className={classNames(classes, "header", "fixed")}>
                    <thead>
                    <tr>
                        {headerElement}
                    </tr>
                    </thead>
                </table>}
                <div className="table-scrollarea">
                    <table className={classNames(classes, {mask: this.state.mask})}>
                        <TransitionGroup component="tbody">{d}</TransitionGroup>
                    </table>
                </div>
            </div>
        )
    }
};


//SearchableSelectableTable
export default class extends Component {
    constructor(props, context) {
        super(props, context);
        const {containerOptions: {startFromPage}} = this.props;
        this.state = {filter: null, page: startFromPage || 0};
    }

    setFilter(filter) {
        this.setState({filter, page: 0});
    }

    handleNext() {
        const {containerOptions: {keepPage}} = this.props;
        const page = this.state.page + 1;
        this.setState({page});
        if (keepPage) keepPage(page);
    }

    handlePrev() {
        const {containerOptions: {keepPage}} = this.props;
        const page = this.state.page - 1;
        this.setState({page});
        if (keepPage) keepPage(page);
    }

    render() {
        const {containerOptions: {singularTitle, pluralTitle, maxElementsOnPage, info}, Table, data, selectionOptions, searchKeys} = this.props;
        const filterFunction = searchKeys ? defaultFilterWithKeys : defaultFilter;
        const filteredItems = this.state.filter ? data.filter(item => filterFunction(item, this.state.filter, searchKeys)) : data;
        //handle pagination if required
        const firstItem = maxElementsOnPage ? (this.state.page * maxElementsOnPage) + 1 : undefined;
        const lastItem = maxElementsOnPage ? Math.min(((this.state.page + 1) * maxElementsOnPage), filteredItems.length) : undefined;
        const lastPage = maxElementsOnPage ? Math.ceil(filteredItems.length / maxElementsOnPage) - 1 : undefined;
        const pageItems = maxElementsOnPage ? _.slice(filteredItems, this.state.page * maxElementsOnPage, lastItem) : filteredItems;
        // console.log(`%c pagination\nfirstItem: ${firstItem}\nlastItem: ${lastItem}\nlastPage: ${lastPage}\nnumOfElements: ${filteredItems.length}`, 'font-weight: bold; color: blue');

        return (
            <div className="searchable-selectable-table-container">
                <div className={classNames('widgets', 'fixed')}>
                    {!maxElementsOnPage &&
                    <ItemCountWidget className="float-left" singularTitle={singularTitle} pluralTitle={pluralTitle}
                                     itemCount={data.length} filteredItemCount={filteredItems.length}
                                     info={info}></ItemCountWidget>}

                    {maxElementsOnPage && <PaginationWidget first={firstItem}
                                                            last={lastItem}
                                                            total={filteredItems.length} title={pluralTitle}
                                                            handleNext={this.handleNext.bind(this)}
                                                            handlePrev={this.handlePrev.bind(this)}
                                                            firstPage={this.state.page === 0}
                                                            lastPage={this.state.page === lastPage} info={info}
                    />}
                    {selectionOptions && !selectionOptions.noCountWidget &&
                    <SelectedCountWidget count={selectionOptions.selectedRows.length}
                                         handleClear={selectionOptions.clearAll}/>}
                    <SearchWidget className="float-right" items={data}
                                  setFilter={this.setFilter.bind(this)}></SearchWidget>
                </div>
                <SelectableTable Table={Table} data={pageItems} selectionOptions={selectionOptions}
                                 height={this.props.height}
                                 fromStart={() => this.setState({page: 0})}/>
            </div>
        )
    }
}
