import React, {useEffect, useState, useMemo} from 'react';
import {useQuery} from '@apollo/client';
import ClusterStatus from './ClusterStatus';
import {Link} from 'react-router-dom';
import {GET_LAZY_CLUSTERS_QUERY} from '../../graphql/queries';
import {CLEAR_ERRORS, routesObject} from '../../actions';
import Isvg from 'react-inlinesvg';
import ItemCountWidget from '../widgets/ItemCountWidget';
import SearchWidget from '../widgets/SearchWidget';
import {
    calculateSavings,
    defaultFilterWithKeys,
    getWizardRoute,
} from './../../utils/utils';
import {getEnforcementInfo, checkShouldEnforce} from './../../utils/enforcmentUtil'
import {connect} from 'react-redux';
import LoadingIcon from '../widgets/LoadingIcon';
import classNames from 'classnames';
import {
    CLUSTER_STATUS,
    DASHBOARD_FILTER_TYPE,
    DASHBOARD_TITLES,
    TIER_BUTTON_TEXT
} from '../../utils/consts';
import localStorage from '../../utils/localStorage';
import routingInfoDispatcher from '../../dispatchers/routingInfoDispatcher';
import {CardItem, Estimated, ValuesContainer} from './CardComponents';
import _ from 'lodash';
import ReactPopover from '../reactPopover/ReactPopover';
import {openDialog} from '../dialog/Dialog';
import RemoveClusterDialog from './RemoveClusterDialog';
import Select from 'react-select';
import useRunOnce from '../../hooks/useRunOnce';
import Button from '../widgets/Button'
import SkeletonCluster from "../skeletons/SkeletonCluster";
import SpinnerLoader from "../widgets/SpinnerLoader"
import ContextMenuButton from '../widgets/ContextMenuButton';
import {addNotification} from './../../dispatchers/generalDispatcher'

require('./clustersDashboardNew.scss');

export const Summary = ({clustersByResourceType, filterClusters, initialFilter}) => {
    const total = clustersByResourceType['all'].length;
    const filterOptions = [{
        label: 'All clusters',
        value: 'all'
    }, {
        label: 'On-prem clusters',
        value: 'ON_PREM',
        isDisabled: !clustersByResourceType['ON_PREM']
    }, {
        label: 'Cloud Volumes ONTAP',
        value: 'CVO',
        isDisabled: !clustersByResourceType['CVO']
    }];

    const [selectedFilter, setSelectedFilter] = useState(initialFilter);
    const handleFilterSelection = (option) => {
        setSelectedFilter(option.value);
        filterClusters(option.value);
    };

    return (
        <div className="dashboard-summary-container flex-container flex-dir-column">
            <div className="summary-widgets flex-container">
                <div className="summary-filter-part flex-container">
                    <label>Filter by:</label>
                    <Select className="Select summary-filter"
                            classNamePrefix="Select"
                            options={filterOptions}
                            isSearchable={false}
                            isClearable={false}
                            onChange={handleFilterSelection}
                            value={_.find(filterOptions, {value: selectedFilter})}
                    /></div>
            </div>
            <div className="dashboard-summary flex-container">
                <div className="summary-part flex-container">
                    <div className="total">{total}</div>
                    <div className="summary-title">Total clusters</div>
                </div>
                <div className="summary-part flex-container">
                    <div className="on-prem">{clustersByResourceType['ON_PREM']?.length || 0}</div>
                    <div className="summary-title">On-prem clusters</div>
                </div>
                <div className="summary-part flex-container">
                    <div className="cvo">{clustersByResourceType['CVO']?.length || 0}</div>
                    <div className="summary-title">Cloud Volumes ONTAP</div>
                </div>
            </div>
        </div>
    );
};

const ClusterName = ({cluster, exitRight, hideMoreInfo, dispatch, addNotification, shouldEnforce}) => {
    const {clusterId, name, tierInformation, status: {status}, isEligibleForTiering, resourceType} = cluster;
    //view or tier volumes
    const isResourceCvo = resourceType === 'CVO';
    const showMenu = isEligibleForTiering || isResourceCvo;
    const allowMore = (status === CLUSTER_STATUS.ON || status === CLUSTER_STATUS.NO_TIERING) && !hideMoreInfo;
    const isRemoveEnabled = _.includes([CLUSTER_STATUS.FAILED, CLUSTER_STATUS.UNKNOWN], status) && resourceType !== 'CVO';
    const isActionsDisabled = _.includes([CLUSTER_STATUS.NO_TIERING, CLUSTER_STATUS.FAILED, CLUSTER_STATUS.UNKNOWN], status) || (!isEligibleForTiering && resourceType === 'ON_PREM') || isRemoveEnabled || shouldEnforce; //for CVO showAggregates and showVolumes is enabled

    return (
        <div className="card-top">
            <div className="name-container flex-container">
                {resourceType === 'ON_PREM' &&
                <Isvg className="on-prem" src={require('../../assets/svgs/on-prem-icon.svg')}/>}
                {resourceType === 'CVO' && <Isvg className="cvo" src={require('../../assets/svgs/cvo-icon.svg')}/>}
                <div className="flex-container flex-dir-column">
                    <div className={classNames('name no-wrap', {italic: cluster.name === 'Unknown Cluster'})}
                         title={cluster.name}>{cluster.name}</div>
                    <div
                        className="name smaller">{resourceType === 'CVO' ? 'Cloud Volumes ONTAP' : 'On-prem cluster'}</div>
                </div>
            </div>
            {showMenu && <ReactPopover popoverClass="card-menu" contentStyle={{width: '183px'}}
                                       container={<ContextMenuButton/>}>
                <ul className="no-bullet">
                    <li><Link
                        className={isActionsDisabled ? 'disabled-link' : ''}
                        to={`/${clusterId}/${name}/aggregates?origin=dashboard`}>{resourceType === 'CVO' ? 'View' : 'Manage'} aggregates</Link>
                    </li>
                    <li><Link
                        className={isActionsDisabled ? 'disabled-link' : ''}
                        to={`/tier-volumes/${clusterId}/${name}/volumes?origin=dashboard`}>{_.get(tierInformation, 'bucketName') && resourceType === 'ON_PREM' ? 'Tier' : 'View'} volumes</Link>
                    </li>
                    <li>
                        <div className={`menu-item ${!allowMore ? 'disabled-link' : ''}`} onClick={() => {
                            exitRight();
                            setTimeout(() => {
                                dispatch({type: CLEAR_ERRORS});
                                routesObject.routes.push(`/${cluster.clusterId}/${cluster.name}/info`);
                            }, 200);
                        }}>Cluster info
                        </div>
                    </li>
                    <li>
                        <div className={`menu-item ${isRemoveEnabled ? '' : 'disabled-link'}`} onClick={() => {
                            openDialog(<RemoveClusterDialog className="remove-cluster-dialog"
                                                            addNotification={addNotification}
                                                            clusterId={clusterId}/>, undefined, true);
                        }}>Remove cluster
                        </div>
                    </li>
                </ul>
            </ReactPopover>}
        </div>
    );
};

const ClusterCard = ({cluster, handleTierCluster, exitRight, dispatch, addNotification}) => {
    if (!cluster.status) {
        return <SkeletonCluster/>
    }

    const {volumes, capacityInformation, licenseInfo, tierInformation, isEligibleForTiering, isEligibleForTieringReason, resourceType, clusterId, agentCloudProvider} = cluster;
    const buttonType = _.get(tierInformation, 'bucketName') ? 'volumes' : 'setup';
    const eligibleVolumes = _.get(volumes, 'eligibleVolumes', 0);
    const tieredVolumes = _.get(volumes, 'tieredVolumes', 0);
    const showNotEligibleStatus = isEligibleForTiering || resourceType === 'CVO';

    const estimated = _.get(capacityInformation, 'performanceCapacity.isEstimatedInactiveCapacity', false);
    const {currentSavings, additionalSavings} = calculateSavings(cluster);

    const enforcementInfo = getEnforcementInfo(licenseInfo);
    const shouldEnforce = checkShouldEnforce(cluster);
    return (
        <div className={classNames('cluster-card', {enforce: shouldEnforce})}>
            <ClusterName cluster={cluster} exitRight={exitRight} addNotification={addNotification}
                         hideMoreInfo={!isEligibleForTiering && resourceType === 'ON_PREM'}
                         dispatch={dispatch} shouldEnforce={shouldEnforce}/>
            <div className="items-container">
                {showNotEligibleStatus && <CardItem title="Operational Health" icon={<Isvg
                    src={require(`./../../assets/svgs/health.svg`)}/>}>
                    <ClusterStatus cluster={cluster}/>
                </CardItem>}
                {!showNotEligibleStatus && <CardItem title="Not Eligible For Tiering"
                                                     icon={<Isvg src={require('./../../assets/svgs/warning.svg')}/>}
                                                     className="not-eligible">
                    <div className="not-eligible-reason">{isEligibleForTieringReason}</div>

                </CardItem>}
                <div className="vertical-line"></div>
                <CardItem title="Current Savings" icon={<Isvg
                    src={require(`./../../assets/svgs/current-savings.svg`)}/>}>
                    <ValuesContainer values={currentSavings}/>
                </CardItem>
                <div className="vertical-line"></div>
                <CardItem title={`${tieredVolumes > 0 ? 'Additional' : ''} Saving Opportunities`}
                          className={estimated ? 'estimated' : ''}
                          icon={<Isvg src={require(`./../../assets/svgs/additional-saving-opportunity.svg`)}/>}>
                    {estimated && <Estimated text={'Estimated Amount'} popoverText={DASHBOARD_TITLES.estimated}/>}
                    <ValuesContainer values={additionalSavings}/>
                    {eligibleVolumes > 0 && isEligibleForTiering && <Button
                        disabled={cluster.status.status === CLUSTER_STATUS.NO_TIERING || shouldEnforce}
                        title={cluster.status.status === CLUSTER_STATUS.NO_TIERING ? cluster.status.message : shouldEnforce ? 'no valid license or subscription' : ''}
                        styleClass="tier-button"
                        onClick={handleTierCluster}>{TIER_BUTTON_TEXT[buttonType]}
                    </Button>
                    }
                </CardItem>
            </div>
            {shouldEnforce && <div className="enforcement">
                <div className="text">
                    {enforcementInfo.reason}
                </div>
                <div className="action">
                    <enforcementInfo.ActionComp agentCloudProvider={agentCloudProvider} clusterId={clusterId}/>
                </div>

            </div>}
        </div>
    );
};

const ClustersDashboard = ({dispatch, history, clearUrlParams, agentId, accountId, workspaceId, addNotification}) => {

    const [state, setState] = useState(() => {
        const initialFilter = localStorage.get('dashboardFilter') || 'all';
        return {
            filter: null,
            exitRight: false,
            showClustersType: initialFilter
        }
    })

    const {loading, error, data} = useQuery(GET_LAZY_CLUSTERS_QUERY, {
        fetchPolicy: 'cache-only',
        variables: {accountId, agentId, workspaceId},
        notifyOnNetworkStatusChange: true
    });

    useEffect(() => {
        const clusters = data ? data.lazyClusters.map(cluster => {
            if (!cluster.name) {
                return {...cluster, name: 'Unknown Cluster'}
            }
            return cluster;
        }) : null;

        let clustersByResourceType = null;
        let updatedShowClustersType = state.showClustersType;
        if (data) {
            // console.log(`%c got data for clusters ${JSON.stringify(clusters[0], null, 2)}`, 'font-weight:bold; color:purple');
            clustersByResourceType = _.groupBy(clusters, 'resourceType'); //group from all the clusters and not just what was filtered.
            clustersByResourceType['all'] = clusters;
            if (!clustersByResourceType[state.showClustersType]) {
                updatedShowClustersType = 'all';
            }
        }
        setState(prevState => ({
            ...prevState,
            clusters,
            clustersByResourceType,
            showClustersType: updatedShowClustersType
        }));
    }, [data, history, state.showClustersType]);

    useRunOnce(() => {
        //clear all selections of last wizard
        localStorage.remove('selectedVolumes');
        localStorage.remove('volumesPolicies');
        localStorage.remove('selectedKeys');
        localStorage.remove('clusterInfo');
        clearUrlParams();
    })

    useEffect(() => {
        localStorage.set('dashboardFilter', state.showClustersType);
    }, [state.showClustersType])

    const exitRight = () => {
        setState(prevState => ({
            ...prevState,
            exitRight: true
        }));
    }

    const setFilter = (filter) => {
        setState(prevState => ({
            ...prevState,
            filter
        }));
    }

    const tierCluster = (cluster) => {
        const {clusterId, tierInformation, name, agentCloudProvider, ontapClusterProperties, agentId} = cluster;
        const wizardRoute = getWizardRoute(tierInformation, clusterId, name, agentCloudProvider, ontapClusterProperties, agentId);
        const nextRoute = `${wizardRoute}?origin=dashboard`;
        dispatch({type: CLEAR_ERRORS});
        routesObject.routes.push(nextRoute);
    }

    const filterClustersFromSummary = (type) => {
        setState(prevState => ({
            ...prevState,
            showClustersType: type
        }))
    }

    const areClustersStillMissing = useMemo(() => state.clusters?.some(({status}) => !status), [state.clusters])

    if (loading || !state.clustersByResourceType) {
        return (<LoadingIcon className="loading-clusters" text={'Clusters'}/>);
    }
    const errorMessage = error ? error.message : null;
    const clusters = state.clustersByResourceType[state.showClustersType];
    const filteredItems = state.filter ? clusters.filter(item => defaultFilterWithKeys(item, state.filter, ['name'])) : clusters;

    return <div className={`clusters ${state.exitRight ? 'exit-right' : ''}`}>
        {!errorMessage && clusters.length > 0 && <div className="widgets-cards-container">
            <Summary clustersByResourceType={state.clustersByResourceType}
                     filterClusters={filterClustersFromSummary} initialFilter={state.showClustersType}/>
            <div className="widgets-container flex-container flex-dir-column">
                <h4>{`${DASHBOARD_FILTER_TYPE[state.showClustersType]} ${state.showClustersType !== 'CVO' ? 'clusters' : ''}`}</h4>
                <div className="inner-widgets">
                    <ItemCountWidget className="item-count" singularTitle={'Cluster'} pluralTitle={'Clusters'}
                                     itemCount={clusters.length}
                                     filteredItemCount={filteredItems.length}></ItemCountWidget>
                    {state.filter && areClustersStillMissing &&
                    <div className="still-fetching-loader"><SpinnerLoader variant={'stand-alone'}/></div>}
                    <SearchWidget placeholder="You can search by name" className="search-widget"
                                  setFilter={setFilter}></SearchWidget>
                </div>
            </div>
            <div className="card-container">{filteredItems.map(cluster => (
                <ClusterCard cluster={cluster} key={cluster.clusterId}
                             exitRight={exitRight}
                             handleTierCluster={() => tierCluster(cluster)}
                             dispatch={dispatch}
                             addNotification={addNotification}
                />
            ))}
            </div>
        </div>}
        {!errorMessage && clusters.length === 0 &&
        <div className="empty-dashboard-occm">
            <div className="icon">
                <Isvg src={require('../../assets/svgs/logo.svg')}/>
            </div>
            <div>
                There are no existing On-Prem clusters suitable for tiering.
                You can discover On-Prem clusters from the Working Environment Gallery.
            </div>
        </div>}
    </div>
}

const mapStateToProps = state => ({
    agentId: state.general.agentId,
    accountId: state.general.accountId,
    workspaceId: state.general.workspaceId,
});

const mapDispatchToProps = dispatch => ({
    ...routingInfoDispatcher(dispatch),
    addNotification: addNotification(dispatch),
    dispatch
})

export default connect(mapStateToProps, mapDispatchToProps)(ClustersDashboard);

