import React, {useState} from 'react';
import {Link} from 'react-router-dom';
import Isvg from 'react-inlinesvg';
import _ from 'lodash';
import {CLUSTER_STATUS, DASHBOARD_TITLES, PROVIDER_TYPE_SGWS, PROVIDERS, TIER_BUTTON_TEXT} from '../../utils/consts';
import {CardItem, ValuesContainer} from './CardComponents';
import {Doughnut} from 'react-chartjs-2';
import {
    calculateSavings,
    buildCapacityObject,
    extractByType,
    getObjectStorageName,
    getWizardRoute,
    getStorageClassNameByProvider
} from '../../utils/utils';
import {Estimated} from './CardComponents';
import classNames from 'classnames';
import InfoPopover from '../reactPopover/InfoPopover';
import localStorage from '../../utils/localStorage';
import {useQuery} from '@apollo/client';
import {GET_CLUSTER_QUERY} from '../../graphql/queries';
import LoadingIcon from '../widgets/LoadingIcon';
import Button from "../widgets/Button";
import useRunOnce from "../../hooks/useRunOnce";
import Breadcrumbs from "../widgets/Breadcrumbs";
import {routesObject} from "../../actions";
import {connect} from "react-redux";

require('./moreInfo.scss');

export const InfoHeader = (props) => {
    const {match: {params: {clusterId}}, data: {clusters}} = props;
    const selectedCluster = _.find(clusters, {clusterId});
    const clusterName = selectedCluster ? selectedCluster.name : '';
    return <div className="more-info-header">
        <div className="name">{clusterName}</div>
        <Link className="close" to={`/clusters`}><Isvg
            src={require('./../../assets/svgs/x.svg')}></Isvg></Link>
    </div>;
};

const InsideGraphInfo = ({info: {insideGraph: {value, unit, subTitle}}, icon}) => {
    return (
        <div className="item">
            <div className="icon">{icon}</div>
            <div className="size">{value}{unit && <span>{` ${unit}`}</span>}</div>
            <div className="sub-title">{subTitle}</div>
        </div>
    );
};

const BottomInfoItem = ({item}) => {
    return <div className="bottom-item">
        <div className="size">{item.value}{item.unit && <span>{` ${item.unit}`}</span>}</div>
        <div className="rectangle float-left" style={{'backgroundColor': item.color}}/>
        <div className="bottom-title float-left">{item.bottomTitle}</div>
    </div>;
};

const InfoGraphCard = ({delay, insideGraph, icon, bottomInfoItems, estimated, primaryTitle, children, className}) => {
    const [showGraph, setShowGraph] = useState(false)
    useRunOnce(() => setTimeout(() => setShowGraph(true), delay))

    return <div className={`info-graph-card ${className}`}>
        <div className="primary-title title">{primaryTitle}
            {estimated && <Estimated text={'Estimated Values'} popoverText={DASHBOARD_TITLES.estimated}/>}
        </div>
        <div className="graph">
            {showGraph &&
            <Doughnut options={{
                preserveAspectRatio: false,
                cutoutPercentage: 85,
                tooltips: {enabled: false}
            }}
                      legend={{display: false}} data={{
                labels: bottomInfoItems.map(item => item.bottomTitle),
                datasets: [{
                    data: bottomInfoItems.map(item => item.bytes),
                    backgroundColor: bottomInfoItems.map(item => item.color)
                }]
            }
            }/>}

            <div className="inside-graph">
                {insideGraph && <InsideGraphInfo info={insideGraph} icon={icon}/>}
            </div>
            <div className="bottom">
                {bottomInfoItems.map((bottomItem, index) => {
                    return <BottomInfoItem item={bottomItem} key={index}/>;
                })}
            </div>
        </div>
        <div className="gray-info title">
            {children}
        </div>
    </div>;
}

const MoreInfoRow = ({row}) => {
    return <div className="info-row">
        {!row.popoverText && <div className="info-key">{row.key}</div>}
        {row.popoverText &&
        <InfoPopover trigger={'mouseover-exit-click'} wrapperClassName="info-key" popoverClassName="info-key"
                     textElement={row.key} popoverText={row.popoverText} contentStyle={{width: '545px'}}/>}
        <div className={classNames('info-value', {'no-wrap': !row.icon})}
             title={row.icon ? '' : row.value}>{row.icon ? row.icon : row.value}</div>
    </div>;
};

const MoreInfoCard = ({title, infoRows, noInfoRowsText, className}) => {
    return <div className={`more-info-card ${className}`}>
        <div className="title">{title}</div>
        {infoRows && <div className="rows-container">
            {infoRows.map((row, index) => {
                return <MoreInfoRow row={row} key={index}/>;
            })}
        </div>}
        {noInfoRowsText && <div className="no-info italic">{noInfoRowsText}</div>}
    </div>;
};

const returnToDashboard = () => routesObject.routes.push(`/clusters`);

const MoreInfo = (props) => {
    const {accountId, workspaceId, agentId, history, match: {params: {clusterId, clusterName}}} = props;
    useRunOnce(() => {
        //since we can now return from wizard to MoreInfo we need to clear all wizard params kept on the local storage here as well. and not only in dashboard
        localStorage.remove('selectedVolumes');
        localStorage.remove('selectedKeys');
    })

    const handleError = () => {
        history.push('/');
    }

    const {loading, error, data} = useQuery(GET_CLUSTER_QUERY, {
        variables: {
            clusterId,
            accountId,
            agentId,
            workspaceId
        },
        fetchPolicy: 'cache-first',
        onError: handleError
    })


    if (loading) {
        return (<LoadingIcon className="loading-clusters" text={'Cluster\'s Info'}/>);
    }
    if (error) {
        handleError();
        return <div></div>;
    } else {
        const cluster = extractByType(data.cluster);
        if (!cluster || cluster.status.status === CLUSTER_STATUS.UNKNOWN || cluster.status.status === CLUSTER_STATUS.FAILED) {
            handleError();
            return <div></div>;
        }

        const {volumes: {eligibleVolumes, tieredVolumes}, ontapClusterProperties, tierInformation, capacityInformation, resourceType} = cluster;

        const {currentSavings, additionalSavings} = calculateSavings(cluster);

        const clusterInfo = ontapClusterProperties ? [
            {
                key: 'IP',
                value: ontapClusterProperties.clusterAddress
            },
            {
                key: 'ONTAP Version',
                value: ontapClusterProperties.ontapVersion
            },
            {
                key: 'Tiered Volumes',
                value: `${tieredVolumes} / ${eligibleVolumes}`,
                popoverText: <span>The total number of volumes that Cloud Tiering displays might be less than the number of volumes on your ONTAP system. That's because some volumes can't be tiered. For example, you can't tier data from SnapLock volumes or from MetroCluster configurations. Refer to
                                            <a href="https://docs.netapp.com/ontap-9/index.jsp?topic=%2Fcom.netapp.doc.dot-cm-psmg%2FGUID-8E421CC9-1DE1-492F-A84C-9EB1B0177807.html"
                                               target="_blank" rel="noopener noreferrer"> ONTAP Documentation
                                            </a>
                                            for features not supported by FabricPool.
                                        </span>
            }
        ] : [];

        const cloudProvider = _.get(cluster, ['tierInformation', 'providerType'], _.get(cluster, 'agentCloudProvider'));
        const objectStorageInfo = tierInformation ? [
            {
                key: 'Provider',
                icon: PROVIDERS[cloudProvider].icon
            },
            {
                key: cloudProvider === 'AWS' ? 'S3 Bucket' : cloudProvider === 'GCP' ? 'GCP Bucket' : cloudProvider === PROVIDER_TYPE_SGWS.name ? 'Bucket' : 'Azure Blob',
                value: tierInformation.bucketName
            },
            {
                key: 'Storage Class',
                value: getStorageClassNameByProvider(tierInformation.providerType, tierInformation.tierLevel)
            }
        ] : null;


        const tierCapacity = buildCapacityObject(_.get(tierInformation, 'currentSavings.capacity', 0));
        const performanceUsedCapacity = buildCapacityObject(_.get(capacityInformation, 'performanceCapacity.usedPerformanceCapacity', 0));
        const totalCapacity = buildCapacityObject(_.get(capacityInformation, 'usedPerformanceAndTieredCapacity', 0));
        const coldData = buildCapacityObject(_.get(capacityInformation, 'performanceCapacity.inactiveCapacity', 0));
        const hotData = buildCapacityObject(_.get(capacityInformation, 'performanceCapacity.activeCapacity', 0));

        // console.log(`%c id ${clusterId} tierCapacity: ${JSON.stringify(tierCapacity)} performanceUsedCapacity ${JSON.stringify(performanceUsedCapacity)} total ${JSON.stringify(totalCapacity)} cold ${JSON.stringify(coldData)} hot ${JSON.stringify(hotData)}`, 'font-weight: bold; color:green');

        const totalCapacityInfo = {
            insideGraph: {
                ...totalCapacity,
                subTitle: 'Total Used Capacity'
            }
        };

        const usedCapacityInfo = {
            insideGraph: {
                ...performanceUsedCapacity,
                subTitle: 'Used Capacity'
            }
        };

        const cameFrom = window.location.pathname;
        const buttonType = _.get(tierInformation, 'bucketName') ? 'volumes' : 'setup';
        const wizardRoute = getWizardRoute(tierInformation, clusterId, clusterName, cloudProvider, null, agentId);
        const nextRoute = `/${wizardRoute}?origin=moreInfo&cameFrom=${cameFrom}`;
        const disableAction = cluster.status.status === CLUSTER_STATUS.NO_TIERING;
        const shouldEnforce = cluster.shouldEnforce && cluster.agentCloudProvider !== "ONPREM";

        return (<div className="more-info">
            <div className="more-info-header">
                <Breadcrumbs levels={[{
                    text: `Cluster Dashboard | ${clusterName}`,
                    onClick: returnToDashboard
                }, {text: `Cluster Info`}]}/>
            </div>
            <div className="info-cards-container">
                <InfoGraphCard primaryTitle="Total Used Capacity" insideGraph={totalCapacityInfo}
                               className="enter-1 total-used-capacity"
                               icon={<Isvg
                                   src={require('../../assets/svgs/total-used-capacity.svg')}/>}
                               delay={800}
                               bottomInfoItems={[
                                   {
                                       ...tierCapacity,
                                       bottomTitle: `${getObjectStorageName(cloudProvider)} Tiered Capacity`,
                                       color: '#3bc5d5'
                                   },
                                   {
                                       ...performanceUsedCapacity,
                                       bottomTitle: 'Cluster Used Capacity',
                                       color: '#2c6bcc'
                                   }
                               ]}>
                    <CardItem className="gray-card-item" title="Current Savings"
                              icon={<Isvg
                                  src={require(`./../../assets/svgs/current-savings.svg`)}/>}>
                        <ValuesContainer values={currentSavings}/>
                    </CardItem>
                </InfoGraphCard>
                <InfoGraphCard primaryTitle="Cluster Used Capacity" insideGraph={usedCapacityInfo}
                               className="enter-2"
                               icon={<Isvg
                                   src={require('./../../assets/svgs/cluster-used-capacity.svg')}/>}
                               estimated={_.get(capacityInformation, 'performanceCapacity.isEstimatedInactiveCapacity', false)}
                               delay={1200}
                               bottomInfoItems={[
                                   {
                                       ...hotData,
                                       bottomTitle: 'Hot Data',
                                       color: '#c9004d'
                                   },
                                   {
                                       ...coldData,
                                       bottomTitle: 'Cold Data',
                                       color: '#2c6bcc'
                                   }
                               ]}>

                    <CardItem className="gray-card-item" title="Cold Data Saving Opportunity"
                              icon={<Isvg
                                  src={require(`./../../assets/svgs/additional-saving-opportunity.svg`)}/>}>
                        <ValuesContainer values={additionalSavings}/>
                        {eligibleVolumes > 0 && !disableAction && resourceType === "ON_PREM" && !shouldEnforce &&
                        <Link to={nextRoute}>
                            <Button styleClass="tier-button">{TIER_BUTTON_TEXT[buttonType]}</Button>
                        </Link>}
                        {eligibleVolumes > 0 && disableAction && resourceType === "ON_PREM" && !shouldEnforce &&
                        <Button styleClass="tier-button" title={cluster.status.message}
                                disabled={disableAction}>{TIER_BUTTON_TEXT[buttonType]}</Button>
                        }
                    </CardItem>
                </InfoGraphCard>
                <div className="more-info-cards-container">
                    <MoreInfoCard title="Cluster Info" infoRows={clusterInfo} className="enter-3"/>
                    <MoreInfoCard title="Object Storage Info" infoRows={objectStorageInfo}
                                  className="enter-4"
                                  noInfoRowsText={!objectStorageInfo ? DASHBOARD_TITLES.notTiered : ''}/>
                </div>
            </div>
        </div>);
    }
}

const mapStateToProps = state => {
    return {
        accountId: state.general.accountId,
        workspaceId: state.general.workspaceId,
        agentId: state.general.agentId,
    };
};

export default connect(mapStateToProps)(MoreInfo);

