import React, { useState, useEffect} from 'react';
import classNames from 'classnames';
import Isvg from 'react-inlinesvg/lib/index';
import {CSSTransition} from 'react-transition-group';
import {extractError, sendMixPanelEvent} from '../../utils/utils';
import {MODIFY_VOLUMES} from '../../graphql/queries';
import { useMutation } from '@apollo/client';
import CloudSlider from '../widgets/CloudSlider';
import pluralize from 'pluralize';
import _ from 'lodash';
import ButtonWithError from '../widgets/ButtonWithError';
import { COOLING_DAYS_INFO, MAX_COOLING_DAYS, MIN_COOLING_DAYS, NOTIFICATION_TYPES } from '../../utils/consts';
import InfoPopover from '../reactPopover/InfoPopover';
import NotificationWidget from "../widgets/NotificationWidget";
import {connect} from "react-redux";


require('./editVolumePanel.scss');

const getCoolingDaysInfo = (supportedSettings, policy) => {
    const setting = _.find(supportedSettings, { tieringPolicy: policy });
    const defaultCoolingDays = setting.minimumCoolingDays ? setting.minimumCoolingDays : null;
    const noCoolingDaysReason = setting.minimumCoolingDays ? null : setting.noMinimumCoolingDaysReason;
    return {
        defaultCoolingDays,
        noCoolingDaysReason
    };
}

const PolicyButton = ({policy: {description, name, icon}, selectPolicy, isSelected}) => {
    return (<div className={classNames('policy-button', {selected: isSelected})} onClick={() => selectPolicy(name)}>
        <Isvg className={name} src={require(`./../../assets/svgs/${icon ? icon : 'no-tiering-policy.svg'}`)}/>
        <div className="policy-name">{description}</div>
    </div>);
};

const EditVolumePanel = (props) => {
    const [state, setState] = useState({firstTimeInfoSet: false})
    const { editVolume, isOpen, close, accountId, agentId, workspaceId } = props;
    const [modifyVolumes, { loading }] = useMutation(MODIFY_VOLUMES, {
        onError: () => handleError('EditVolume'),
        errorPolicy: 'all'
    })

    useEffect(() => {
        const fields = ['name', 'svmName', 'minimumCoolingDays', 'tieringPolicy'];
        if (isOpen && !state.firstTimeInfoSet) {
            const coolingDaysInfo = getCoolingDaysInfo(editVolume.supportedSettings, editVolume.tieringPolicy);
            setState((prevState) => ({
                ...prevState,
                firstTimeInfoSet: true,
                originalVolume: _.pick(editVolume, fields),
                coolingDays: editVolume.minimumCoolingDays ? editVolume.minimumCoolingDays : null,
                selectedPolicy: editVolume.tieringPolicy ? editVolume.tieringPolicy : null,
                ...coolingDaysInfo,
                error: null
            }));
        }
        if (!isOpen && state.firstTimeInfoSet) { //clear the state on close of the panel
            setState((prevState) => ({
                ...prevState,
                firstTimeInfoSet: false,
                error: null,
                originalVolume: null,
                coolingDays: null,
                selectedPolicy: null,
                defaultCoolingDays: null,
                noCoolingDaysReason: null
            }));
        }
    }, [state.firstTimeInfoSet, editVolume, isOpen])


    const handleSelectPolicy = (name) => {
        const { editVolume: { supportedSettings } } = props;
        const coolingDaysInfo = getCoolingDaysInfo(supportedSettings, name);
        setState((prevState) => ({
            ...prevState,
            error: null,
            selectedPolicy: name,
            ...coolingDaysInfo,
            coolingDays: coolingDaysInfo.defaultCoolingDays
        })); //clear the last selected cooling days
    }

    const handleError = (action, error) => {
        const extractedError = extractError(error);
        sendMixPanelEvent(`${action}-Error`, extractedError);
        setState((prevState) => ({
            ...prevState,
            error: extractedError
        }));
    }

    const modifyComplete = (store, { data: { modifyVolumes: { errors } } }) => {
        if (errors && errors.length > 0) {
            setState((prevState) => ({
                ...prevState,
                error: errors[0].reason
            }));
        }
        if (!errors || errors.length === 0) { //close only if there are no errors at all.
            close();
        }
    }

    const submit = (modifyVolumes) => {
        const { clusterId, editVolume } = props;
        const { selectedPolicy, coolingDays } = state;
        const variables = {
            accountId,
            agentId,
            workspaceId,
            clusterId,
            volumeProperties: [{
                name: editVolume.name,
                svmName: editVolume.svmName
            }]
        };
        if (selectedPolicy) {
            variables.volumeProperties[0].tieringPolicy = selectedPolicy;
        }
        if (coolingDays) {
            variables.volumeProperties[0].minimumCoolingDays = coolingDays;
        }
        sendMixPanelEvent('ModifyVolume');
        setState((prevState) => ({
            ...prevState,
            error: null
        }));
        return modifyVolumes({
            variables,
            update: modifyComplete
        });
    }

    const somethingChanged = () => {
        const { selectedPolicy, coolingDays, originalVolume } = state;
        // console.log(`%c selectedPolicy: ${selectedPolicy}\ncoolingDays: ${coolingDays}\n originalVolume: ${JSON.stringify(_.pick(originalVolume, ["tieringPolicy", 'minimumCoolingDays']))}`, 'font-weight:bold; color:green');
        if ((selectedPolicy && !_.isEqual(selectedPolicy, originalVolume.tieringPolicy)) || (coolingDays && !_.isEqual(coolingDays, originalVolume.minimumCoolingDays))) {
            return true; //something changed.
        }
        return false; //nothing changed
    }


    const { error, originalVolume, defaultCoolingDays, coolingDays, noCoolingDaysReason, selectedPolicy } = state;
    if (!editVolume || !originalVolume) {
        return (<CSSTransition in={isOpen} classNames="slide-from-right-edit-volume" timeout={400} unmountOnExit>
            <div className="slide-from-right-edit-volume-parent">
                <div className={classNames('edit-volume-panel')}>
                </div>
            </div>
        </CSSTransition>);
    }
    const showCoolingDaysSlider = defaultCoolingDays || coolingDays;
    const showCoolingDaysEmptyBox = selectedPolicy === 'none';

    return (
        <CSSTransition in={isOpen} classNames="slide-from-right-edit-volume" timeout={400} unmountOnExit>
            <div className="slide-from-right-edit-volume-parent">
                <div className={classNames('edit-volume-panel')}>
                        <div className="close-edit-volume-panel" onClick={close}><Isvg
                            src={require('../../assets/svgs/back-arrow.svg')}/></div>
                    <div className="name-part">
                        <div className="name no-wrap" title={editVolume.name}><Isvg
                                src={require('./../../assets/svgs/volume-icon.svg')}/>{editVolume.name}</div>
                        <div
                                className={classNames('volume-status', {error: editVolume.state.toUpperCase() === 'OFFLINE'})}>
                                <Isvg
                                    src={require('../../assets/svgs/service-connector-status-ok.svg')}/>{editVolume.state.toUpperCase()}
                            </div>
                    </div>
                    <>
                        <div className="panel-content">
                            <div className="policy-part box">
                                <div className="title">SELECT TIERING POLICY</div>
                                <div className="policies-container">
                                    {editVolume.supportedPolicies.map(policy => {
                                        const isSelected = selectedPolicy ? selectedPolicy === policy.name : originalVolume.tieringPolicy === policy.name;
                                        return <PolicyButton policy={policy} key={policy.name}
                                                                             selectPolicy={handleSelectPolicy}
                                                                             isSelected={isSelected}/>;
                                    })}
                                </div>

                            </div>
                            <div className="cooling-days-part box">
                                {!showCoolingDaysEmptyBox && <div className="title">ADJUST COOLING DAYS
                                                    <InfoPopover wrapperClassName="cooling-days-info"
                                                                 popoverClassName="cooling-days-info"
                                                                 popoverText={COOLING_DAYS_INFO}
                                                                 contentStyle={{width: '250px'}}/>
                                                </div>}
                                                {(showCoolingDaysSlider && !showCoolingDaysEmptyBox) &&
                                                <div className="cooling-days-slider">
                                    <div className="slider-min slider-value">{MIN_COOLING_DAYS}</div>
                                    <div className="slider-comp">
                                        <CloudSlider
                                            min={MIN_COOLING_DAYS}
                                            max={MAX_COOLING_DAYS}
                                            valueString={(valueNow) => {
                                                pluralize.addIrregularRule('Day', 'Days');
                                                return `${valueNow} ${pluralize('Day', valueNow)}`;
                                            }}
                                            updateChange={(value) => setState((prevState) => ({
                                                ...prevState,
                                                coolingDays: value,
                                                error: null
                                            }))}
                                            value={coolingDays || defaultCoolingDays}
                                        />
                                    </div>
                                    <div className="slider-max slider-value">{MAX_COOLING_DAYS}</div>
                                </div>}
                                                {!showCoolingDaysSlider && !showCoolingDaysEmptyBox &&
                                                <div className='no-cooling-days'>
                                    {noCoolingDaysReason}
                                </div>}
                            </div>
                        </div>
                        {error && <NotificationWidget className="error-modify-aggr"
                                                                      variant={NOTIFICATION_TYPES.ERROR}>{error}</NotificationWidget>}
                        <ButtonWithError submitting={!!loading} disable={!somethingChanged()}
                                         type="button" onClick={() => submit(modifyVolumes)}
                                                         title={!somethingChanged() ? 'No change detected' : ''}>
                            APPLY
                        </ButtonWithError>
                    </>
                </div>
            </div>
        </CSSTransition>
    );
}

const mapStateToProps = state => ({
    agentId: state.general.agentId,
    accountId: state.general.accountId,
    workspaceId: state.general.workspaceId,
});

export default connect(mapStateToProps)(EditVolumePanel);




