import React from 'react';
import _ from 'lodash';
import moment from 'moment';
import {
    CLUSTER_STATUS, DASHBOARD_TITLES, DEFAULT_STORAGE_CLASS, FREE_TRIAL, FREE_TRIAL_EXPIRED, PROVIDER_TYPE_SGWS, STORAGE_CLASSES, SUBSCRIBED
} from './consts';
import numeral from 'numeral';
import bytes from 'bytes';
import ReactPopover from '../components/reactPopover/ReactPopover';
import Isvg from 'react-inlinesvg';
import {checkShouldEnforce} from "./enforcmentUtil";

export const defaultFilter = (item, filterString) => {
    if (_.isFunction(item)) return false;
    return _.values(item).find(value => {
        if (_.isEmpty(value) || !value) {
            return false;
        } else if (_.isObject(value)) {
            return defaultFilter(value, filterString);
        } else if (_.isArray(value)) {
            return false;
        } else return _.toString(value).toLowerCase().indexOf(filterString.toLowerCase()) !== -1;
    });
};

export const defaultFilterWithKeys = (item, filterString, keys) => {
    if (_.isFunction(item)) return false;
    return _.keys(item).find(key => {
        if (!_.includes(keys, key)) return false; //not one of the keys to check
        const value = item[key];
        if (_.isEmpty(value) || !value) {
            return false;
        } else if (_.isObject(value)) {
            return defaultFilterWithKeys(value, filterString, keys);
        } else if (_.isArray(value)) {
            return false;
        } else return _.toString(value).toLowerCase().indexOf(filterString.toLowerCase()) !== -1;
    });
};

export const calculateSavings = (cluster) => {
    const {status, tierInformation, capacityInformation, resourceType} = cluster;
    const clusterIsOff = status && (status.status === CLUSTER_STATUS.FAILED || status.status === CLUSTER_STATUS.UNKNOWN);
    const isResourceCvo = resourceType === 'CVO';

    const currentSavingsCapacity = buildCapacityObject(_.get(tierInformation, "currentSavings.capacity", 0));

    const currentSavings = [];
    if (clusterIsOff) currentSavings.push({italicText: DASHBOARD_TITLES.dataNotAvailable});
    else if (currentSavingsCapacity.bytes > 0) {
        currentSavings.push({...currentSavingsCapacity});
        if (!isResourceCvo) {
            currentSavings.push({value: `$ ${numeral(_.get(tierInformation, "currentSavings.cost", 0)).format('0,0')}`});
        }
    } else currentSavings.push({italicText: DASHBOARD_TITLES.notTiered});
    const savingOpportunityCapacity = buildCapacityObject(_.get(capacityInformation, "savingOpportunity.capacity", 0));
    const additionalSavings = [];
    if (clusterIsOff) additionalSavings.push({italicText: DASHBOARD_TITLES.dataNotAvailable});
    else {
        const {volumes: {eligibleVolumes, tieredVolumes}} = cluster;
        if (savingOpportunityCapacity.bytes > 0) additionalSavings.push({...savingOpportunityCapacity});
        else if (tieredVolumes === 0) additionalSavings.push({italicText: DASHBOARD_TITLES.notTiered});
        else if (tieredVolumes && eligibleVolumes && tieredVolumes === eligibleVolumes) additionalSavings.push({italicText: DASHBOARD_TITLES.allTiered});
        else additionalSavings.push({italicText: DASHBOARD_TITLES.dataNotAvailable});
    }
    return {currentSavings, additionalSavings};
};

export const buildCapacityObject = (bytesInput) => {
    if (bytesInput === null) return bytesInput;
    const converted = bytes(bytesInput, {unitSeparator: " "});
    const convertedArray = converted ? converted.split(" ") : ["0", "B"];
    // console.log(`%c converted1 ${converted} convertedArray ${convertedArray}`, 'font-weight:bold; color:blue');
    return {
        bytes: bytesInput,
        value: convertedArray[0],
        unit: convertedArray[1]
    };
};

export const extractONTAPVersion = (versionString) => {
    const regex = /(\d+\.\d*|\d+)/;
    const version = versionString.match(regex)[0];
    return parseFloat(version);
};

export const extractError = (error, step) => {
    console.log(`%c ${step} Got GraphQL error ${JSON.stringify(error)}`, "font-weight:bold; color: red");
    if (error.networkError && error.networkError.result && error.networkError.result.errors) return error.networkError.result.errors[0].message;
    else if (error.graphQLErrors) return error.graphQLErrors[0].message;
    return `extarctError got UNKNOWN GraphQL error ${JSON.stringify(error, null, 2)}`;
};

export const extractByType = (object) => {
    return _.isArray(object) ? object[0] : object;
};

/*eslint-disable */
export const bootGoogleAnalytics = () => {
    (function (i, s, o, g, r, a, m) {
        i['GoogleAnalyticsObject'] = r;
        i[r] = i[r] || function () {
            (i[r].q = i[r].q || []).push(arguments)
        }, i[r].l = 1 * new Date();
        a = s.createElement(o),
            m = s.getElementsByTagName(o)[0];
        a.async = 1;
        a.src = g;
        m.parentNode.insertBefore(a, m)
    })(window, document, 'script', 'https://www.google-analytics.com/analytics.js', 'ga');

    ga('create', 'UA-3177638-12', 'auto');
    ga('set', 'transport', 'beacon');
    ga('send', 'pageview');
};


export const sendAnalyticsEvent = (gaAction, gaLabel, gaCategory) => {
    window.ga && window.ga('send', {
        hitType: 'event',
        eventCategory: gaCategory,
        eventAction: gaAction,
        eventLabel: gaLabel
    });
};

export const bootMixPanel = (userId, email, company) => {
    (function (e, a) {
        if (!a.__SV) {
            var b = window;
            try {
                var c, l, i, j = b.location, g = j.hash;
                c = function (a, b) {
                    return (l = a.match(RegExp(b + "=([^&]*)"))) ? l[1] : null
                };
                g && c(g, "state") && (i = JSON.parse(decodeURIComponent(c(g, "state"))), "mpeditor" === i.action && (b.sessionStorage.setItem("_mpcehash", g), history.replaceState(i.desiredHash || "", e.title, j.pathname + j.search)))
            } catch (m) {
            }
            var k, h;
            window.mixpanel = a;
            a._i = [];
            a.init = function (b, c, f) {
                function e(b, a) {
                    var c = a.split(".");
                    2 == c.length && (b = b[c[0]], a = c[1]);
                    b[a] = function () {
                        b.push([a].concat(Array.prototype.slice.call(arguments,
                            0)))
                    }
                }

                var d = a;
                "undefined" !== typeof f ? d = a[f] = [] : f = "mixpanel";
                d.people = d.people || [];
                d.toString = function (b) {
                    var a = "mixpanel";
                    "mixpanel" !== f && (a += "." + f);
                    b || (a += " (stub)");
                    return a
                };
                d.people.toString = function () {
                    return d.toString(1) + ".people (stub)"
                };
                k = "disable time_event track track_pageview track_links track_forms register register_once alias unregister identify name_tag set_config reset opt_in_tracking opt_out_tracking has_opted_in_tracking has_opted_out_tracking clear_opt_in_out_tracking people.set people.set_once people.unset people.increment people.append people.union people.track_charge people.clear_charges people.delete_user".split(" ");
                for (h = 0; h < k.length; h++) e(d, k[h]);
                a._i.push([b, c, f])
            };
            a.__SV = 1.2;
            b = e.createElement("script");
            b.type = "text/javascript";
            b.async = !0;
            b.src = "undefined" !== typeof MIXPANEL_CUSTOM_LIB_URL ? MIXPANEL_CUSTOM_LIB_URL : "file:" === e.location.protocol && "//cdn4.mxpnl.com/libs/mixpanel-2-latest.min.js".match(/^\/\//) ? "https://cdn4.mxpnl.com/libs/mixpanel-2-latest.min.js" : "//cdn4.mxpnl.com/libs/mixpanel-2-latest.min.js";
            c = e.getElementsByTagName("script")[0];
            c.parentNode.insertBefore(b, c)
        }
    })(document, window.mixpanel || [])
    window.mixpanel.init(process.env.REACT_APP_MIXPANEL_TOKEN, {
        track_pageview: false,
        loaded: function (mixpanel) {
            mixpanel.identify(userId);
            const userInfo = {email, company};
            mixpanel.people.set(userInfo);
            mixpanel.register(userInfo);
        }
    });
};

export const sendMixPanelEvent = (action, properties) => {
    window.mixpanel && window.mixpanel.track(action, properties);
};

export const fromNowNoFuture = (time) => {
    const d = moment(time);
    if (d.diff(moment()) > 0) return "Just now";
    else return _.capitalize(d.fromNow());
};

export const getObjectAsValuesString = (obj, delimiter = ":") => {
    let stringObjectValues = "";
    _.each(_.values(obj), value => {
        stringObjectValues = `${stringObjectValues}${value}${delimiter}`;
        return stringObjectValues;
    });
    return stringObjectValues
};

export const getObjectStorageName = (provider) => {
    switch (provider) {
        case "AWS" :
            return "S3";
        case "AZURE":
            return "Blob";
        case "GCP" :
            return "";
        default :
            return "";
    }
};
const NoLicenseNeeded = ({tooltipText}) => {
    return (<div>No License Needed
        <ReactPopover popoverClass="sg-license-info"
                      trigger="mouseover"
                      contentStyle={{padding: '10px', width: '173px'}}
                      container={<button className="sg-license-info" type="button"><Isvg
                          src={require('./../assets/svgs/license-page-red-info.svg')}/></button>}>
            <div>{tooltipText}</div>
        </ReactPopover>
    </div>);
}

export const getClusterLicenseInfo = ({licenseInfo, resourceType}) => {
    if (resourceType === 'CVO') {
        return <NoLicenseNeeded tooltipText={'A license isn’t required to tier data to Cloud Volumes ONTAP'}/>;
    }
    if (!licenseInfo) return "";
    const {freeTrialStatus, hasBYOL, subscriptionStatus, byolCapacity: maxCapacity, currentCapacity} = licenseInfo;
    const subscribed = subscriptionStatus === SUBSCRIBED;
    if (freeTrialStatus === FREE_TRIAL) return "Free trial";
    // else if (freeTrialStatus === PRE_FREE_TRIAL) return "";
    if (licenseInfo.provider === PROVIDER_TYPE_SGWS.name) {
        return <NoLicenseNeeded tooltipText={'A license isn’t required to tier data to StorageGRID'}/>;
    }
    if (!hasBYOL) {
        if (subscribed) return "PAYGO";
        else if (freeTrialStatus === FREE_TRIAL_EXPIRED) return "Free trial is over"
    } else {
        const maxCapacityObject = buildCapacityObject(maxCapacity);
        const BYOLString = `${maxCapacityObject.value}${maxCapacityObject.unit} BYOL`;
        if (currentCapacity <= maxCapacity) return BYOLString;
        else { //BYOL exceeded
            if (subscribed) {
                const paygoObject = buildCapacityObject(currentCapacity - maxCapacity);
                const paygoString = `${paygoObject.value}${paygoObject.unit} PayGo`;
                return `${BYOLString} + ${paygoString}`;
            } else {
                return `${BYOLString} exceeded`;
            }
        }
    }
};

export const getClusterNameTooltip = (cluster) => {
    const {status, licenseInfo} = cluster;
    if (status) {
        const {status: clusterStatus, message} = status
        const dataAvailable = (clusterStatus === CLUSTER_STATUS.ON || clusterStatus === CLUSTER_STATUS.NO_TIERING);
        if (!dataAvailable) {
            return `Unable to show license info, cluster is unreachable (${message || 'Unknown Error'})`
        }
    }
    if (!licenseInfo) return null;
    if (checkShouldEnforce(cluster)) {
        return `This cluster has no license, please subscribe to MP or activate BYOL license`;
    }
    return null
};

export const getWizardRoute = (tierInformation, clusterId, clusterName, provider, ontapClusterProperties, agentId) => {
    const azureSetupBucketUrl = `${provider}/resource-group`;
    const ontapVersion = ontapClusterProperties ? extractONTAPVersion(ontapClusterProperties.ontapVersion) : null;
    const setupBucketUrl = provider === 'ONPREM' ? `${ontapVersion}/choose-provider` : `${provider}/setup-bucket`;


    const clusterSetupComplete = _.get(tierInformation, "bucketName");
    const routePrefix = clusterSetupComplete ? 'tier-volumes' : provider === 'AZURE' ? 'setup-azure' : 'setup';
    const calculatedRoute = `${routePrefix}/${clusterId}/${clusterName}/${!clusterSetupComplete ? `${agentId}/` : ''}${clusterSetupComplete ? 'volumes' : provider === 'AZURE' ? azureSetupBucketUrl : setupBucketUrl}`;
    console.log(`%c getWizardRoute\n${calculatedRoute}`, 'font-weight:bold;color:purple');
    return calculatedRoute;
};

export const getStorageClassNameByProvider = (provider, storageClass) => {
    if (provider === PROVIDER_TYPE_SGWS.name) return 'Standard';
    return storageClass ? _.find(STORAGE_CLASSES[provider], {storageClass}).storageClassName : STORAGE_CLASSES[provider][DEFAULT_STORAGE_CLASS].storageClassName;
};

export const IsInIframe = () => {
    try {
        return window.self !== window.top;
    } catch (e) {
        return true;
    }
}

export const updateLocalStorage = (attr, value, defaultValue) => {
    if (!value) {
        return localStorage.getItem(attr) || defaultValue
    } else {
        localStorage.setItem(attr, value)
        return value
    }
}